cecal.o cecal.d : \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/cecal.cpp \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/utypes.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/umachine.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/ptypes.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/platform.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uconfig.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uvernum.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/urename.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uversion.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/cecal.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/calendar.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uobject.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/locid.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/bytestream.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/std_string.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/localpointer.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/strenum.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/unistr.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/char16ptr.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/rep.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/stringpiece.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/putil.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uloc.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uenum.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/timezone.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/ures.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/ucal.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/umisc.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/gregoimp.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/putilimp.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/utypes.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/putil.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/cstring.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/cmemory.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/localpointer.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/uassert.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uobject.h
