calendar.o calendar.d : \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/calendar.cpp \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/utypeinfo.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/utypes.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/umachine.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/ptypes.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/platform.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uconfig.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uvernum.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/urename.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uversion.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/gregocal.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/calendar.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uobject.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/locid.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/bytestream.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/std_string.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/localpointer.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/strenum.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/unistr.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/char16ptr.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/rep.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/stringpiece.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/putil.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uloc.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uenum.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/timezone.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/ures.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/ucal.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/umisc.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/basictz.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/tzrule.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/dtrule.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/tztrans.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/simpletz.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/basictz.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/rbtz.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/vtzone.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/gregoimp.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/calendar.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/putilimp.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/utypes.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/putil.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/buddhcal.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/taiwncal.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/japancal.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/islamcal.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/hebrwcal.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/persncal.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/indiancal.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/iso8601cal.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/timezone.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/chnsecal.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/coptccal.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/cecal.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/dangical.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/ethpccal.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/cpputils.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/unistr.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/cmemory.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/localpointer.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/uassert.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uobject.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/servloc.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/locid.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/strenum.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/hash.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/uhash.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/uelement.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/uvector.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/uarrsort.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/serv.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/umisc.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/servnotf.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/mutex.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/umutex.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uclean.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/putilimp.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/locutil.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/ucln_in.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/ucln.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/cstring.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/locbased.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/uresimp.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/ures.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/uresdata.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/udata.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/udataswp.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/resource.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/restrace.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/ustrenum.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uenum.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/uassert.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/olsontz.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/umutex.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/sharedcalendar.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/sharedobject.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unifiedcache.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/utypeinfo.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/sharedobject.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/cstring.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/ustr_imp.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/utf8.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/utf.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/ulocimp.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/bytestream.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uloc.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/charstr.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/charstr.h
