translit.o translit.d : \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/translit.cpp \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/utypeinfo.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/utypes.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/umachine.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/ptypes.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/platform.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uconfig.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uvernum.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/urename.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uversion.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/putil.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/translit.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uobject.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/unistr.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/char16ptr.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/rep.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/std_string.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/stringpiece.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/bytestream.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/parseerr.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/utrans.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/urep.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uenum.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/localpointer.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uset.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uchar.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/stringoptions.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/ucpmap.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/strenum.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/locid.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uloc.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/msgfmt.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/format.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/fmtable.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/uformattable.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/fieldpos.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/fpositer.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/umisc.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/parsepos.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/messagepattern.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/plurfmt.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/numfmt.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/unum.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/ucurr.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/udisplaycontext.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/ufieldpositer.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/unumberoptions.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/curramt.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/measure.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/currunit.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/measunit.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/plurrule.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/upluralrules.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/resbund.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/ures.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/unifilt.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/unifunct.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/unimatch.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uniset.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uscript.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/utf16.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/utf.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/cpdtrans.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/nultrans.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/rbt_data.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uclean.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/rbt_set.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/utrans.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/uvector.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/utypes.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uobject.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/cmemory.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/localpointer.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/uassert.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/uarrsort.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/uelement.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/hash.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/unistr.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/uhash.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/rbt_pars.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/unorm.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uiter.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/unorm2.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/rbt.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/udata.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/transreg.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/name2uni.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/nortrans.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/normalizer2.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/remtrans.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/titletrn.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/ucase.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uset.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/putilimp.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/putil.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/uset_imp.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/udataswp.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/utrie2.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/utf8.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/utf.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/casetrn.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/tolowtrn.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/toupptrn.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/uni2name.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/brktrans.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/esctrn.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unesctrn.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/tridpars.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/anytrans.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/uhash.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/util.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/charstr.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/mutex.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/umutex.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uclean.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/ucln_in.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/ucln.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/uassert.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/cmemory.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/cstring.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/uinvchar.h
