number_modifiers.o number_modifiers.d : \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/number_modifiers.cpp \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/utypes.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/umachine.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/ptypes.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/platform.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uconfig.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uvernum.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/urename.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uversion.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/umutex.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/utypes.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uclean.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uobject.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/putilimp.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/putil.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/ucln_cmn.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/ucln.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/ucln_in.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/ucln.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/number_modifiers.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uniset.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/ucpmap.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/unifilt.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/unifunct.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uobject.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/unimatch.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/unistr.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/char16ptr.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/rep.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/std_string.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/stringpiece.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/bytestream.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uset.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uchar.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/stringoptions.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/localpointer.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/simpleformatter.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/standardplural.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/formatted_string_builder.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/cstring.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/cmemory.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/localpointer.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/uassert.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/uassert.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/fphdlimp.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/fieldpos.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/fpositer.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/fieldpos.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/umisc.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/formattedvalue.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/appendable.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/fpositer.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/uformattedvalue.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/ufieldpositer.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/number_types.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/decimfmt.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/dcfmtsym.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/locid.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/strenum.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/putil.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uloc.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uenum.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/numsys.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/format.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/fmtable.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/uformattable.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/parsepos.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/parseerr.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/unum.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/ucurr.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/udisplaycontext.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/unumberoptions.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/numfmt.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/curramt.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/measure.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/currunit.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/measunit.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/enumset.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/unum.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/numsys.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/numberformatter.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/displayoptions.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/udisplayoptions.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/nounit.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/plurrule.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/upluralrules.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/unumberformatter.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/uformattednumber.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/formattednumber.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/formattedvalue.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/utf16.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/utf.h
