icupkg.o icupkg.d : \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/tools/icupkg/icupkg.cpp \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/utypes.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/umachine.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/ptypes.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/platform.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uconfig.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uvernum.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/urename.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uversion.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/utf.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/utf8.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/utf16.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/utf_old.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/putil.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/cstring.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/utypes.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/cmemory.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/localpointer.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/uassert.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uobject.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/tools/icupkg/../toolutil/toolutil.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/errorcode.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uobject.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/umutablecptrie.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/ucpmap.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/ucptrie.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/localpointer.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/tools/icupkg/../toolutil/uoptions.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/tools/icupkg/../toolutil/uparse.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/tools/icupkg/../toolutil/filestrm.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/tools/icupkg/../toolutil/package.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/tools/icupkg/../toolutil/pkg_icu.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/tools/icupkg/../toolutil/package.h
