servls.o servls.d : \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/servls.cpp \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/utypes.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/umachine.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/ptypes.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/platform.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uconfig.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uvernum.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/urename.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uversion.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/resbund.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/utypes.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uobject.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/ures.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/char16ptr.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uloc.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uenum.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/localpointer.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/unistr.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/rep.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/std_string.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/stringpiece.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/bytestream.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/locid.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/strenum.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/putil.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/uresimp.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/ures.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/uresdata.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/udata.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/putilimp.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/putil.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/udataswp.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/resource.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/unistr.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/restrace.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/cmemory.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/localpointer.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/uassert.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uobject.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/servloc.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/locid.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/strenum.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/hash.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/uhash.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/uelement.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/uvector.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/uarrsort.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/serv.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/umisc.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/servnotf.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/mutex.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/umutex.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uclean.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/locutil.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/ustrfmt.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/charstr.h
