/* Generated automatically. */
static const char configuration_arguments[] = "/root/mxe/tmp-gcc-x86_64-w64-mingw32.static.posix/gcc-13.3.0/configure ac_cv_prog_HAVE_DOXYGEN=false --enable-doc=no --enable-gtk-doc=no --enable-gtk-doc-html=no --enable-gtk-doc-pdf=no --docdir=/root/mxe/tmp-gcc-x86_64-w64-mingw32.static.posix/gcc-13.3.0.build_.sink --infodir=/root/mxe/tmp-gcc-x86_64-w64-mingw32.static.posix/gcc-13.3.0.build_.sink --mandir=/root/mxe/tmp-gcc-x86_64-w64-mingw32.static.posix/gcc-13.3.0.build_.sink --with-html-dir=/root/mxe/tmp-gcc-x86_64-w64-mingw32.static.posix/gcc-13.3.0.build_.sink --disable-doxygen --target=x86_64-w64-mingw32.static.posix --build=x86_64-pc-linux-gnu --prefix=/usr/lib/mxe/usr --libdir=/usr/lib/mxe/usr/lib --with-sysroot=/usr/lib/mxe/usr/x86_64-w64-mingw32.static.posix --enable-languages=c,c++,objc,fortran --enable-version-specific-runtime-libs --with-gcc --with-gnu-ld --with-gnu-as --disable-nls --disable-shared --disable-multilib --without-x --disable-win32-registry --enable-threads=posix --enable-default-ssp --enable-libgomp --with-gmp=/usr/lib/mxe/usr/x86_64-pc-linux-gnu --with-isl=/usr/lib/mxe/usr/x86_64-pc-linux-gnu --with-mpc=/usr/lib/mxe/usr/x86_64-pc-linux-gnu --with-mpfr=/usr/lib/mxe/usr/x86_64-pc-linux-gnu --with-as=/usr/lib/mxe/usr/bin/x86_64-w64-mingw32.static.posix-as --with-ld=/usr/lib/mxe/usr/bin/x86_64-w64-mingw32.static.posix-ld --with-nm=/usr/lib/mxe/usr/bin/x86_64-w64-mingw32.static.posix-nm --enable-libstdcxx-time --with-zstd=/usr/lib/mxe/usr/x86_64-pc-linux-gnu";
static const char thread_model[] = "posix";

static const struct {
  const char *name, *value;
} configure_default_options[] = { { "cpu", "generic" }, { "arch", "x86-64" } };
