brktrans.o brktrans.d : \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/brktrans.cpp \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/utypes.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/umachine.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/ptypes.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/platform.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uconfig.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uvernum.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/urename.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uversion.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/brkiter.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uobject.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/unistr.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/char16ptr.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/rep.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/std_string.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/stringpiece.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/bytestream.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/chariter.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/locid.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/localpointer.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/strenum.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/putil.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uloc.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uenum.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/ubrk.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/utext.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uchar.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/stringoptions.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/ucpmap.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/parseerr.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/umisc.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/unifilt.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/unifunct.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/unimatch.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uniset.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uset.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/brktrans.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/translit.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/i18n/unicode/utrans.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/urep.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/cmemory.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/utypes.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/localpointer.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/uassert.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uobject.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/mutex.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/umutex.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uclean.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/putilimp.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/putil.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/uprops.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/uset.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/uset_imp.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/udataswp.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/uinvchar.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/unicode/unistr.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/util.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/charstr.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/cmemory.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/uvectr32.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/uhash.h \
 /root/mxe/tmp-icu4c-x86_64-pc-linux-gnu/icu/source/common/uelement.h
