/* Generated automatically by the program `gencodes'
   from the machine description file `md'.  */

#ifndef GCC_INSN_CODES_H
#define GCC_INSN_CODES_H

enum insn_code {
  CODE_FOR_nothing = 0,
  CODE_FOR_x86_sahf_1 = 43,
  CODE_FOR_swapsi = 94,
  CODE_FOR_swapdi = 95,
  CODE_FOR_insvhi_1 = 116,
  CODE_FOR_insvsi_1 = 117,
  CODE_FOR_insvdi_1 = 118,
  CODE_FOR_zero_extendditi2 = 150,
  CODE_FOR_zero_extendqidi2 = 152,
  CODE_FOR_zero_extendhidi2 = 153,
  CODE_FOR_zero_extendqisi2_and = 154,
  CODE_FOR_zero_extendhisi2_and = 155,
  CODE_FOR_zero_extendqihi2_and = 158,
  CODE_FOR_extendsidi2_1 = 161,
  CODE_FOR_extendditi2 = 162,
  CODE_FOR_extendqidi2 = 163,
  CODE_FOR_extendhidi2 = 164,
  CODE_FOR_extendhisi2 = 165,
  CODE_FOR_extendqisi2 = 167,
  CODE_FOR_extendqihi2 = 169,
  CODE_FOR_extendbfsf2_1 = 179,
  CODE_FOR_truncdfsf2 = 182,
  CODE_FOR_truncxfsf2 = 183,
  CODE_FOR_truncxfdf2 = 184,
  CODE_FOR_truncsfbf2 = 187,
  CODE_FOR_fix_trunchfsi2 = 188,
  CODE_FOR_fixuns_trunchfsi2 = 189,
  CODE_FOR_fix_trunchfdi2 = 190,
  CODE_FOR_fixuns_trunchfdi2 = 191,
  CODE_FOR_fixuns_truncsfdi2 = 192,
  CODE_FOR_fixuns_truncdfdi2 = 193,
  CODE_FOR_fixuns_truncsfsi2_avx512f = 194,
  CODE_FOR_fixuns_truncdfsi2_avx512f = 195,
  CODE_FOR_fix_truncsfsi_sse = 201,
  CODE_FOR_fix_truncsfdi_sse = 202,
  CODE_FOR_fix_truncdfsi_sse = 203,
  CODE_FOR_fix_truncdfdi_sse = 204,
  CODE_FOR_fix_trunchi_i387_fisttp = 205,
  CODE_FOR_fix_truncsi_i387_fisttp = 206,
  CODE_FOR_fix_truncdi_i387_fisttp = 207,
  CODE_FOR_fix_truncdi_i387 = 211,
  CODE_FOR_fix_trunchi_i387 = 212,
  CODE_FOR_fix_truncsi_i387 = 213,
  CODE_FOR_x86_fnstcw_1 = 214,
  CODE_FOR_floathisf2 = 215,
  CODE_FOR_floathidf2 = 216,
  CODE_FOR_floathixf2 = 217,
  CODE_FOR_floatsixf2 = 218,
  CODE_FOR_floatdixf2 = 219,
  CODE_FOR_floatsihf2 = 224,
  CODE_FOR_floatunssihf2 = 225,
  CODE_FOR_floatdihf2 = 226,
  CODE_FOR_floatunsdihf2 = 227,
  CODE_FOR_floatdisf2_i387_with_xmm = 230,
  CODE_FOR_floatdidf2_i387_with_xmm = 231,
  CODE_FOR_floatdixf2_i387_with_xmm = 232,
  CODE_FOR_floatunssisf2_i387_with_xmm = 237,
  CODE_FOR_floatunssidf2_i387_with_xmm = 238,
  CODE_FOR_floatunssixf2_i387_with_xmm = 239,
  CODE_FOR_addsi_1_zext = 248,
  CODE_FOR_addvqi4_1 = 284,
  CODE_FOR_addvhi4_1 = 285,
  CODE_FOR_addvsi4_1 = 286,
  CODE_FOR_addvdi4_1 = 287,
  CODE_FOR_subvqi4_1 = 344,
  CODE_FOR_subvhi4_1 = 345,
  CODE_FOR_subvsi4_1 = 346,
  CODE_FOR_subvdi4_1 = 347,
  CODE_FOR_addqi3_carry = 368,
  CODE_FOR_addhi3_carry = 369,
  CODE_FOR_addsi3_carry = 370,
  CODE_FOR_adddi3_carry = 371,
  CODE_FOR_addcarrysi = 383,
  CODE_FOR_addcarrydi = 384,
  CODE_FOR_subqi3_carry = 387,
  CODE_FOR_subhi3_carry = 388,
  CODE_FOR_subsi3_carry = 389,
  CODE_FOR_subdi3_carry = 390,
  CODE_FOR_subsi3_carry_ccc = 402,
  CODE_FOR_subdi3_carry_ccc = 403,
  CODE_FOR_subsi3_carry_ccgz = 406,
  CODE_FOR_subdi3_carry_ccgz = 407,
  CODE_FOR_subborrowsi = 408,
  CODE_FOR_subborrowdi = 409,
  CODE_FOR_smulsi3_highpart = 492,
  CODE_FOR_umulsi3_highpart = 493,
  CODE_FOR_smuldi3_highpart = 494,
  CODE_FOR_umuldi3_highpart = 495,
  CODE_FOR_divmodsi4_1 = 504,
  CODE_FOR_divmoddi4_1 = 505,
  CODE_FOR_udivmodsi4_1 = 506,
  CODE_FOR_udivmoddi4_1 = 507,
  CODE_FOR_divmodsi4_zext_1 = 508,
  CODE_FOR_udivmodsi4_zext_1 = 509,
  CODE_FOR_divmodsi4_zext_2 = 510,
  CODE_FOR_udivmodsi4_zext_2 = 511,
  CODE_FOR_divmodhiqi3 = 537,
  CODE_FOR_udivmodhiqi3 = 538,
  CODE_FOR_ashldi3_doubleword = 791,
  CODE_FOR_ashlti3_doubleword = 792,
  CODE_FOR_x86_64_shld = 793,
  CODE_FOR_x86_64_shld_1 = 794,
  CODE_FOR_x86_shld = 797,
  CODE_FOR_x86_shld_1 = 798,
  CODE_FOR_lshrdi3_doubleword = 843,
  CODE_FOR_ashrdi3_doubleword = 844,
  CODE_FOR_lshrti3_doubleword = 845,
  CODE_FOR_ashrti3_doubleword = 846,
  CODE_FOR_x86_64_shrd = 847,
  CODE_FOR_x86_64_shrd_1 = 848,
  CODE_FOR_x86_shrd = 851,
  CODE_FOR_x86_shrd_1 = 852,
  CODE_FOR_ashrsi3_cvt = 855,
  CODE_FOR_ashrdi3_cvt = 856,
  CODE_FOR_ix86_rotldi3_doubleword = 919,
  CODE_FOR_ix86_rotlti3_doubleword = 920,
  CODE_FOR_ix86_rotrdi3_doubleword = 921,
  CODE_FOR_ix86_rotrti3_doubleword = 922,
  CODE_FOR_rotl32di2_doubleword = 923,
  CODE_FOR_rotr32di2_doubleword = 924,
  CODE_FOR_rotl64ti2_doubleword = 925,
  CODE_FOR_rotr64ti2_doubleword = 926,
  CODE_FOR_setcc_sf_sse = 992,
  CODE_FOR_setcc_df_sse = 993,
  CODE_FOR_setcc_hf_mask = 994,
  CODE_FOR_jump = 996,
  CODE_FOR_blockage = 1023,
  CODE_FOR_prologue_use = 1025,
  CODE_FOR_simple_return_internal = 1026,
  CODE_FOR_interrupt_return = 1027,
  CODE_FOR_simple_return_internal_long = 1028,
  CODE_FOR_simple_return_pop_internal = 1029,
  CODE_FOR_nop = 1032,
  CODE_FOR_nops = 1033,
  CODE_FOR_pad = 1034,
  CODE_FOR_set_got_rex64 = 1037,
  CODE_FOR_set_rip_rex64 = 1038,
  CODE_FOR_set_got_offset_rex64 = 1039,
  CODE_FOR_eh_return_internal = 1040,
  CODE_FOR_split_stack_return = 1043,
  CODE_FOR_ffssi2_no_cmove = 1044,
  CODE_FOR_ctzsi2 = 1051,
  CODE_FOR_ctzdi2 = 1052,
  CODE_FOR_bsr_rex64 = 1061,
  CODE_FOR_bsr_rex64_1 = 1062,
  CODE_FOR_bsr_rex64_1_zext = 1063,
  CODE_FOR_bsr = 1064,
  CODE_FOR_bsr_1 = 1065,
  CODE_FOR_bsr_zext_1 = 1066,
  CODE_FOR_clzsi2_lzcnt = 1069,
  CODE_FOR_clzdi2_lzcnt = 1070,
  CODE_FOR_tzcnt_si = 1079,
  CODE_FOR_lzcnt_si = 1080,
  CODE_FOR_tzcnt_di = 1081,
  CODE_FOR_lzcnt_di = 1082,
  CODE_FOR_tzcnt_hi = 1087,
  CODE_FOR_lzcnt_hi = 1088,
  CODE_FOR_bmi_bextr_si = 1089,
  CODE_FOR_bmi_bextr_di = 1090,
  CODE_FOR_bmi2_pdep_si3 = 1119,
  CODE_FOR_bmi2_pdep_di3 = 1120,
  CODE_FOR_bmi2_pext_si3 = 1121,
  CODE_FOR_bmi2_pext_di3 = 1122,
  CODE_FOR_tbm_bextri_si = 1123,
  CODE_FOR_tbm_bextri_di = 1124,
  CODE_FOR_popcountsi2 = 1143,
  CODE_FOR_popcountdi2 = 1144,
  CODE_FOR_popcounthi2 = 1153,
  CODE_FOR_bswaphi_lowpart = 1159,
  CODE_FOR_parityhi2_cmp = 1160,
  CODE_FOR_parityqi2_cmp = 1161,
   CODE_FOR_tls_initial_exec_64_sun = CODE_FOR_nothing,
  CODE_FOR_rcphf2 = 1192,
  CODE_FOR_truncxfsf2_i387_noop_unspec = 1217,
  CODE_FOR_truncxfdf2_i387_noop_unspec = 1218,
  CODE_FOR_sqrtxf2 = 1219,
  CODE_FOR_rsqrthf2 = 1221,
  CODE_FOR_sqrthf2 = 1222,
  CODE_FOR_x86_fnstsw_1 = 1225,
  CODE_FOR_fpremxf4_i387 = 1226,
  CODE_FOR_fprem1xf4_i387 = 1227,
  CODE_FOR_sinxf2 = 1228,
  CODE_FOR_cosxf2 = 1229,
  CODE_FOR_sincosxf3 = 1230,
  CODE_FOR_fptanxf4_i387 = 1231,
  CODE_FOR_atan2xf3 = 1232,
  CODE_FOR_fyl2xxf3_i387 = 1233,
  CODE_FOR_fyl2xp1xf3_i387 = 1234,
  CODE_FOR_fxtractxf3_i387 = 1235,
  CODE_FOR_fscalexf4_i387 = 1237,
  CODE_FOR_avx512f_scalefsf2 = 1238,
  CODE_FOR_avx512f_scalefdf2 = 1239,
  CODE_FOR_sse4_1_roundhf2 = 1240,
  CODE_FOR_sse4_1_roundsf2 = 1241,
  CODE_FOR_sse4_1_rounddf2 = 1242,
  CODE_FOR_rintxf2 = 1243,
  CODE_FOR_lrintxfdi2 = 1244,
  CODE_FOR_lrintxfhi2 = 1245,
  CODE_FOR_lrintxfsi2 = 1246,
  CODE_FOR_frndintxf2_roundeven = 1247,
  CODE_FOR_frndintxf2_floor = 1248,
  CODE_FOR_frndintxf2_ceil = 1249,
  CODE_FOR_frndintxf2_trunc = 1250,
  CODE_FOR_frndintxf2_roundeven_i387 = 1251,
  CODE_FOR_frndintxf2_floor_i387 = 1252,
  CODE_FOR_frndintxf2_ceil_i387 = 1253,
  CODE_FOR_frndintxf2_trunc_i387 = 1254,
  CODE_FOR_fistdi2_floor = 1261,
  CODE_FOR_fistdi2_ceil = 1262,
  CODE_FOR_fisthi2_floor = 1263,
  CODE_FOR_fisthi2_ceil = 1264,
  CODE_FOR_fistsi2_floor = 1265,
  CODE_FOR_fistsi2_ceil = 1266,
  CODE_FOR_fxamsf2_i387 = 1267,
  CODE_FOR_fxamdf2_i387 = 1268,
  CODE_FOR_fxamxf2_i387 = 1269,
  CODE_FOR_movmsk_df = 1270,
  CODE_FOR_cld = 1271,
  CODE_FOR_movhf_mask = 1319,
  CODE_FOR_smaxsf3 = 1325,
  CODE_FOR_sminsf3 = 1326,
  CODE_FOR_smaxdf3 = 1327,
  CODE_FOR_smindf3 = 1328,
  CODE_FOR_smaxhf3 = 1329,
  CODE_FOR_sminhf3 = 1330,
  CODE_FOR_pro_epilogue_adjust_stack_add_si = 1361,
  CODE_FOR_pro_epilogue_adjust_stack_add_di = 1362,
  CODE_FOR_pro_epilogue_adjust_stack_sub_si = 1363,
  CODE_FOR_pro_epilogue_adjust_stack_sub_di = 1364,
  CODE_FOR_allocate_stack_worker_probe_si = 1365,
  CODE_FOR_allocate_stack_worker_probe_di = 1366,
  CODE_FOR_probe_stack_1_si = 1367,
  CODE_FOR_probe_stack_1_di = 1368,
  CODE_FOR_adjust_stack_and_probe_si = 1369,
  CODE_FOR_adjust_stack_and_probe_di = 1370,
  CODE_FOR_probe_stack_range_si = 1371,
  CODE_FOR_probe_stack_range_di = 1372,
  CODE_FOR_trap = 1373,
  CODE_FOR_ud2 = 1374,
  CODE_FOR_prefetchi = 1378,
  CODE_FOR_stack_protect_set_1_si = 1379,
  CODE_FOR_stack_protect_set_1_di = 1380,
  CODE_FOR_stack_protect_test_1_si = 1384,
  CODE_FOR_stack_protect_test_1_di = 1385,
  CODE_FOR_sse4_2_crc32qi = 1386,
  CODE_FOR_sse4_2_crc32hi = 1387,
  CODE_FOR_sse4_2_crc32si = 1388,
  CODE_FOR_sse4_2_crc32di = 1389,
  CODE_FOR_rdpmc = 1390,
  CODE_FOR_rdpmc_rex64 = 1391,
  CODE_FOR_rdtsc = 1392,
  CODE_FOR_rdtsc_rex64 = 1393,
  CODE_FOR_rdtscp = 1394,
  CODE_FOR_rdtscp_rex64 = 1395,
  CODE_FOR_fxsave = 1396,
  CODE_FOR_fxsave64 = 1397,
  CODE_FOR_fxrstor = 1398,
  CODE_FOR_fxrstor64 = 1399,
  CODE_FOR_xsave = 1400,
  CODE_FOR_xsaveopt = 1401,
  CODE_FOR_xsavec = 1402,
  CODE_FOR_xsaves = 1403,
  CODE_FOR_xsave_rex64 = 1404,
  CODE_FOR_xsaveopt_rex64 = 1405,
  CODE_FOR_xsavec_rex64 = 1406,
  CODE_FOR_xsaves_rex64 = 1407,
  CODE_FOR_xsave64 = 1408,
  CODE_FOR_xsaveopt64 = 1409,
  CODE_FOR_xsavec64 = 1410,
  CODE_FOR_xsaves64 = 1411,
  CODE_FOR_xrstor = 1412,
  CODE_FOR_xrstors = 1413,
  CODE_FOR_xrstor_rex64 = 1414,
  CODE_FOR_xrstors_rex64 = 1415,
  CODE_FOR_xrstor64 = 1416,
  CODE_FOR_xrstors64 = 1417,
  CODE_FOR_xsetbv = 1418,
  CODE_FOR_xsetbv_rex64 = 1419,
  CODE_FOR_xgetbv = 1420,
  CODE_FOR_xgetbv_rex64 = 1421,
  CODE_FOR_fnstenv = 1422,
  CODE_FOR_fldenv = 1423,
  CODE_FOR_fnstsw = 1424,
  CODE_FOR_fnclex = 1425,
  CODE_FOR_lwp_llwpcbsi = 1426,
  CODE_FOR_lwp_llwpcbdi = 1427,
  CODE_FOR_lwp_slwpcbsi = 1428,
  CODE_FOR_lwp_slwpcbdi = 1429,
  CODE_FOR_lwp_lwpvalsi = 1430,
  CODE_FOR_lwp_lwpvaldi = 1431,
  CODE_FOR_lwp_lwpinssi = 1432,
  CODE_FOR_lwp_lwpinsdi = 1433,
  CODE_FOR_rdfsbasesi = 1434,
  CODE_FOR_rdgsbasesi = 1435,
  CODE_FOR_rdfsbasedi = 1436,
  CODE_FOR_rdgsbasedi = 1437,
  CODE_FOR_wrfsbasesi = 1438,
  CODE_FOR_wrgsbasesi = 1439,
  CODE_FOR_wrfsbasedi = 1440,
  CODE_FOR_wrgsbasedi = 1441,
  CODE_FOR_ptwritesi = 1442,
  CODE_FOR_ptwritedi = 1443,
  CODE_FOR_rdrandhi = 1444,
  CODE_FOR_rdrandsi = 1445,
  CODE_FOR_rdranddi = 1446,
  CODE_FOR_rdseedhi = 1447,
  CODE_FOR_rdseedsi = 1448,
  CODE_FOR_rdseeddi = 1449,
  CODE_FOR_rdsspsi = 1451,
  CODE_FOR_rdsspdi = 1452,
  CODE_FOR_incsspsi = 1453,
  CODE_FOR_incsspdi = 1454,
  CODE_FOR_saveprevssp = 1455,
  CODE_FOR_rstorssp = 1456,
  CODE_FOR_wrsssi = 1457,
  CODE_FOR_wrssdi = 1458,
  CODE_FOR_wrusssi = 1459,
  CODE_FOR_wrussdi = 1460,
  CODE_FOR_setssbsy = 1461,
  CODE_FOR_clrssbsy = 1462,
  CODE_FOR_nop_endbr = 1463,
  CODE_FOR_xbegin_1 = 1464,
  CODE_FOR_xend = 1465,
  CODE_FOR_xabort = 1466,
  CODE_FOR_xtest_1 = 1467,
  CODE_FOR_clwb = 1468,
  CODE_FOR_clflushopt = 1469,
  CODE_FOR_mwaitx = 1470,
  CODE_FOR_monitorx_si = 1471,
  CODE_FOR_monitorx_di = 1472,
  CODE_FOR_clzero_si = 1473,
  CODE_FOR_clzero_di = 1474,
  CODE_FOR_rdpid = 1477,
  CODE_FOR_rdpid_rex64 = 1478,
  CODE_FOR_wbinvd = 1479,
  CODE_FOR_wbnoinvd = 1480,
  CODE_FOR_movdirisi = 1481,
  CODE_FOR_movdiridi = 1482,
  CODE_FOR_movdir64b_si = 1483,
  CODE_FOR_movdir64b_di = 1484,
  CODE_FOR_xsusldtrk = 1485,
  CODE_FOR_xresldtrk = 1486,
  CODE_FOR_enqcmd_si = 1487,
  CODE_FOR_enqcmds_si = 1488,
  CODE_FOR_enqcmd_di = 1489,
  CODE_FOR_enqcmds_di = 1490,
  CODE_FOR_clui = 1491,
  CODE_FOR_stui = 1492,
  CODE_FOR_testui = 1493,
  CODE_FOR_senduipi = 1494,
  CODE_FOR_umwait = 1495,
  CODE_FOR_umwait_rex64 = 1496,
  CODE_FOR_umonitor_si = 1497,
  CODE_FOR_umonitor_di = 1498,
  CODE_FOR_tpause = 1499,
  CODE_FOR_tpause_rex64 = 1500,
  CODE_FOR_cldemote = 1501,
  CODE_FOR_speculation_barrier = 1502,
  CODE_FOR_serialize = 1503,
  CODE_FOR_patchable_area = 1504,
  CODE_FOR_hreset = 1505,
  CODE_FOR_ldtilecfg = 1506,
  CODE_FOR_sttilecfg = 1507,
  CODE_FOR_sse_movntq = 1537,
  CODE_FOR_mmx_ieee_maxv2sf3 = 1546,
  CODE_FOR_mmx_ieee_minv2sf3 = 1547,
  CODE_FOR_mmx_rcpv2sf2 = 1548,
  CODE_FOR_mmx_rcpit1v2sf3 = 1549,
  CODE_FOR_mmx_rcpit2v2sf3 = 1550,
  CODE_FOR_sqrtv2sf2 = 1551,
  CODE_FOR_mmx_rsqrtv2sf2 = 1552,
  CODE_FOR_mmx_rsqit1v2sf3 = 1553,
  CODE_FOR_mmx_hsubv2sf3 = 1556,
  CODE_FOR_vec_addsubv2sf3 = 1559,
  CODE_FOR_mmx_gtv2sf3 = 1561,
  CODE_FOR_mmx_gev2sf3 = 1562,
  CODE_FOR_mmx_blendvps = 1566,
  CODE_FOR_andv2sf3 = 1568,
  CODE_FOR_iorv2sf3 = 1569,
  CODE_FOR_xorv2sf3 = 1570,
  CODE_FOR_fmav2sf4 = 1571,
  CODE_FOR_fmsv2sf4 = 1572,
  CODE_FOR_fnmav2sf4 = 1573,
  CODE_FOR_fnmsv2sf4 = 1574,
  CODE_FOR_mmx_fix_truncv2sfv2si2 = 1575,
  CODE_FOR_fixuns_truncv2sfv2si2 = 1576,
  CODE_FOR_mmx_floatv2siv2sf2 = 1577,
  CODE_FOR_floatunsv2siv2sf2 = 1578,
  CODE_FOR_mmx_pf2iw = 1579,
  CODE_FOR_mmx_pi2fw = 1580,
  CODE_FOR_mmx_pswapdv2sf2 = 1581,
  CODE_FOR_lrintv2sfv2si2 = 1593,
  CODE_FOR_addv4hf3 = 1594,
  CODE_FOR_subv4hf3 = 1595,
  CODE_FOR_mulv4hf3 = 1596,
  CODE_FOR_divv4hf3 = 1597,
  CODE_FOR_addv2hf3 = 1598,
  CODE_FOR_subv2hf3 = 1599,
  CODE_FOR_mulv2hf3 = 1600,
  CODE_FOR_divv2hf3 = 1601,
  CODE_FOR_negv2qi2 = 1602,
  CODE_FOR_addv4qi3 = 1611,
  CODE_FOR_subv4qi3 = 1612,
  CODE_FOR_addv2hi3 = 1613,
  CODE_FOR_subv2hi3 = 1614,
  CODE_FOR_addv2qi3 = 1615,
  CODE_FOR_subv2qi3 = 1616,
  CODE_FOR_mulv2hi3 = 1638,
  CODE_FOR_smulv2hi3_highpart = 1641,
  CODE_FOR_umulv2hi3_highpart = 1642,
  CODE_FOR_smaxv8qi3 = 1646,
  CODE_FOR_sminv8qi3 = 1647,
  CODE_FOR_smaxv2si3 = 1648,
  CODE_FOR_sminv2si3 = 1649,
  CODE_FOR_smaxv4qi3 = 1652,
  CODE_FOR_sminv4qi3 = 1653,
  CODE_FOR_smaxv2qi3 = 1654,
  CODE_FOR_sminv2qi3 = 1655,
  CODE_FOR_smaxv2hi3 = 1656,
  CODE_FOR_sminv2hi3 = 1657,
  CODE_FOR_umaxv4hi3 = 1658,
  CODE_FOR_uminv4hi3 = 1659,
  CODE_FOR_umaxv2si3 = 1660,
  CODE_FOR_uminv2si3 = 1661,
  CODE_FOR_umaxv4qi3 = 1664,
  CODE_FOR_uminv4qi3 = 1665,
  CODE_FOR_umaxv2qi3 = 1666,
  CODE_FOR_uminv2qi3 = 1667,
  CODE_FOR_umaxv2hi3 = 1668,
  CODE_FOR_uminv2hi3 = 1669,
  CODE_FOR_ssse3_absv8qi2 = 1670,
  CODE_FOR_ssse3_absv4hi2 = 1671,
  CODE_FOR_ssse3_absv2si2 = 1672,
  CODE_FOR_absv4qi2 = 1673,
  CODE_FOR_absv2qi2 = 1674,
  CODE_FOR_absv2hi2 = 1675,
  CODE_FOR_mmx_ashrv4hi3 = 1676,
  CODE_FOR_mmx_ashrv2si3 = 1677,
  CODE_FOR_mmx_ashlv4hi3 = 1678,
  CODE_FOR_mmx_lshrv4hi3 = 1679,
  CODE_FOR_mmx_ashlv2si3 = 1680,
  CODE_FOR_mmx_lshrv2si3 = 1681,
  CODE_FOR_mmx_ashlv1di3 = 1682,
  CODE_FOR_mmx_lshrv1di3 = 1683,
  CODE_FOR_mmx_ashlv1si3 = 1684,
  CODE_FOR_mmx_lshrv1si3 = 1685,
  CODE_FOR_ashlv2hi3 = 1686,
  CODE_FOR_lshrv2hi3 = 1687,
  CODE_FOR_ashrv2hi3 = 1688,
  CODE_FOR_ashlv2qi3 = 1689,
  CODE_FOR_lshrv2qi3 = 1690,
  CODE_FOR_ashrv2qi3 = 1691,
  CODE_FOR_mmx_gtv8qi3 = 1698,
  CODE_FOR_mmx_gtv4hi3 = 1699,
  CODE_FOR_mmx_gtv2si3 = 1700,
  CODE_FOR_mmx_pblendvb_v8qi = 1716,
  CODE_FOR_mmx_pblendvb_v4qi = 1717,
  CODE_FOR_mmx_pblendvb_v2qi = 1718,
  CODE_FOR_mmx_pblendvb_v2hi = 1719,
  CODE_FOR_mmx_ppermv64 = 1727,
  CODE_FOR_mmx_ppermv32 = 1728,
  CODE_FOR_one_cmplv4qi2 = 1729,
  CODE_FOR_one_cmplv2qi2 = 1730,
  CODE_FOR_one_cmplv2hi2 = 1731,
  CODE_FOR_mmx_andnotv8qi3 = 1732,
  CODE_FOR_mmx_andnotv4hi3 = 1733,
  CODE_FOR_mmx_andnotv2si3 = 1734,
  CODE_FOR_mmx_packsswb = 1756,
  CODE_FOR_mmx_packuswb = 1757,
  CODE_FOR_mmx_packssdw = 1758,
  CODE_FOR_mmx_packusdw = 1759,
  CODE_FOR_mmx_punpckhbw = 1760,
  CODE_FOR_mmx_punpckhbw_low = 1761,
  CODE_FOR_mmx_punpcklbw = 1762,
  CODE_FOR_mmx_punpcklbw_low = 1763,
  CODE_FOR_mmx_punpckhwd = 1764,
  CODE_FOR_mmx_punpcklwd = 1765,
  CODE_FOR_mmx_punpckhdq = 1766,
  CODE_FOR_mmx_punpckldq = 1767,
  CODE_FOR_sse4_1_sign_extendv4qiv4hi2 = 1768,
  CODE_FOR_sse4_1_zero_extendv4qiv4hi2 = 1769,
  CODE_FOR_sse4_1_sign_extendv2hiv2si2 = 1770,
  CODE_FOR_sse4_1_zero_extendv2hiv2si2 = 1771,
  CODE_FOR_sse4_1_sign_extendv2qiv2hi2 = 1772,
  CODE_FOR_sse4_1_zero_extendv2qiv2hi2 = 1773,
  CODE_FOR_mmx_pshufbv8qi3 = 1784,
  CODE_FOR_mmx_pshufbv4qi3 = 1785,
  CODE_FOR_mmx_pshufw_1 = 1786,
  CODE_FOR_mmx_pswapdv2si2 = 1790,
  CODE_FOR_uavgv4qi3_ceil = 1814,
  CODE_FOR_uavgv2qi3_ceil = 1815,
  CODE_FOR_uavgv2hi3_ceil = 1816,
  CODE_FOR_mmx_pmovmskb = 1818,
  CODE_FOR_movv64qi_internal = 1823,
  CODE_FOR_movv32qi_internal = 1824,
  CODE_FOR_movv16qi_internal = 1825,
  CODE_FOR_movv32hi_internal = 1826,
  CODE_FOR_movv16hi_internal = 1827,
  CODE_FOR_movv8hi_internal = 1828,
  CODE_FOR_movv16si_internal = 1829,
  CODE_FOR_movv8si_internal = 1830,
  CODE_FOR_movv4si_internal = 1831,
  CODE_FOR_movv8di_internal = 1832,
  CODE_FOR_movv4di_internal = 1833,
  CODE_FOR_movv2di_internal = 1834,
  CODE_FOR_movv4ti_internal = 1835,
  CODE_FOR_movv2ti_internal = 1836,
  CODE_FOR_movv1ti_internal = 1837,
  CODE_FOR_movv32hf_internal = 1838,
  CODE_FOR_movv16hf_internal = 1839,
  CODE_FOR_movv8hf_internal = 1840,
  CODE_FOR_movv32bf_internal = 1841,
  CODE_FOR_movv16bf_internal = 1842,
  CODE_FOR_movv8bf_internal = 1843,
  CODE_FOR_movv16sf_internal = 1844,
  CODE_FOR_movv8sf_internal = 1845,
  CODE_FOR_movv4sf_internal = 1846,
  CODE_FOR_movv8df_internal = 1847,
  CODE_FOR_movv4df_internal = 1848,
  CODE_FOR_movv2df_internal = 1849,
  CODE_FOR_avx512f_movhf_mask = 1898,
  CODE_FOR_avx512f_movsf_mask = 1899,
  CODE_FOR_avx512f_movdf_mask = 1900,
  CODE_FOR_avx512f_storehf_mask = 1904,
  CODE_FOR_avx512f_storesf_mask = 1905,
  CODE_FOR_avx512f_storedf_mask = 1906,
  CODE_FOR_avx512f_blendmv16si = 1907,
  CODE_FOR_avx512vl_blendmv8si = 1908,
  CODE_FOR_avx512vl_blendmv4si = 1909,
  CODE_FOR_avx512f_blendmv8di = 1910,
  CODE_FOR_avx512vl_blendmv4di = 1911,
  CODE_FOR_avx512vl_blendmv2di = 1912,
  CODE_FOR_avx512f_blendmv16sf = 1913,
  CODE_FOR_avx512vl_blendmv8sf = 1914,
  CODE_FOR_avx512vl_blendmv4sf = 1915,
  CODE_FOR_avx512f_blendmv8df = 1916,
  CODE_FOR_avx512vl_blendmv4df = 1917,
  CODE_FOR_avx512vl_blendmv2df = 1918,
  CODE_FOR_avx512bw_blendmv64qi = 1919,
  CODE_FOR_avx512vl_blendmv16qi = 1920,
  CODE_FOR_avx512vl_blendmv32qi = 1921,
  CODE_FOR_avx512bw_blendmv32hi = 1922,
  CODE_FOR_avx512vl_blendmv16hi = 1923,
  CODE_FOR_avx512vl_blendmv8hi = 1924,
  CODE_FOR_avx512bw_blendmv32hf = 1925,
  CODE_FOR_avx512vl_blendmv16hf = 1926,
  CODE_FOR_avx512fp16_blendmv8hf = 1927,
  CODE_FOR_avx512bw_blendmv32bf = 1928,
  CODE_FOR_avx512vl_blendmv16bf = 1929,
  CODE_FOR_avx512vl_blendmv8bf = 1930,
  CODE_FOR_movdi_to_sse = 1957,
  CODE_FOR_avx_lddqu256 = 1958,
  CODE_FOR_sse3_lddqu = 1959,
  CODE_FOR_sse2_movntisi = 1960,
  CODE_FOR_sse2_movntidi = 1961,
  CODE_FOR_avx512f_movntv16sf = 1962,
  CODE_FOR_avx_movntv8sf = 1963,
  CODE_FOR_sse_movntv4sf = 1964,
  CODE_FOR_avx512f_movntv8df = 1965,
  CODE_FOR_avx_movntv4df = 1966,
  CODE_FOR_sse2_movntv2df = 1967,
  CODE_FOR_avx512f_movntv8di = 1968,
  CODE_FOR_avx_movntv4di = 1969,
  CODE_FOR_sse2_movntv2di = 1970,
  CODE_FOR_kandqi = 1971,
  CODE_FOR_kiorqi = 1972,
  CODE_FOR_kxorqi = 1973,
  CODE_FOR_kandhi = 1974,
  CODE_FOR_kiorhi = 1975,
  CODE_FOR_kxorhi = 1976,
  CODE_FOR_kandsi = 1977,
  CODE_FOR_kiorsi = 1978,
  CODE_FOR_kxorsi = 1979,
  CODE_FOR_kanddi = 1980,
  CODE_FOR_kiordi = 1981,
  CODE_FOR_kxordi = 1982,
  CODE_FOR_kandnqi = 1983,
  CODE_FOR_kandnhi = 1984,
  CODE_FOR_kandnsi = 1985,
  CODE_FOR_kandndi = 1986,
  CODE_FOR_kxnorqi = 1987,
  CODE_FOR_kxnorhi = 1988,
  CODE_FOR_kxnorsi = 1989,
  CODE_FOR_kxnordi = 1990,
  CODE_FOR_knotqi = 1991,
  CODE_FOR_knothi = 1992,
  CODE_FOR_knotsi = 1993,
  CODE_FOR_knotdi = 1994,
  CODE_FOR_kaddqi = 1996,
  CODE_FOR_kaddhi = 1997,
  CODE_FOR_kaddsi = 1998,
  CODE_FOR_kadddi = 1999,
  CODE_FOR_kashiftqi = 2000,
  CODE_FOR_klshiftrtqi = 2001,
  CODE_FOR_kashifthi = 2002,
  CODE_FOR_klshiftrthi = 2003,
  CODE_FOR_kashiftsi = 2004,
  CODE_FOR_klshiftrtsi = 2005,
  CODE_FOR_kashiftdi = 2006,
  CODE_FOR_klshiftrtdi = 2007,
  CODE_FOR_ktestqi = 2008,
  CODE_FOR_ktesthi = 2009,
  CODE_FOR_ktestsi = 2010,
  CODE_FOR_ktestdi = 2011,
  CODE_FOR_kortestqi = 2012,
  CODE_FOR_kortesthi = 2013,
  CODE_FOR_kortestsi = 2014,
  CODE_FOR_kortestdi = 2015,
  CODE_FOR_kunpckhi = 2016,
  CODE_FOR_kunpcksi = 2017,
  CODE_FOR_kunpckdi = 2018,
  CODE_FOR_avx512fp16_vmaddv8hf3 = 2121,
  CODE_FOR_avx512fp16_vmaddv8hf3_round = 2122,
  CODE_FOR_avx512fp16_vmaddv8hf3_mask = 2123,
  CODE_FOR_avx512fp16_vmaddv8hf3_mask_round = 2124,
  CODE_FOR_avx512fp16_vmsubv8hf3 = 2125,
  CODE_FOR_avx512fp16_vmsubv8hf3_round = 2126,
  CODE_FOR_avx512fp16_vmsubv8hf3_mask = 2127,
  CODE_FOR_avx512fp16_vmsubv8hf3_mask_round = 2128,
  CODE_FOR_sse_vmaddv4sf3 = 2129,
  CODE_FOR_sse_vmaddv4sf3_round = 2130,
  CODE_FOR_sse_vmaddv4sf3_mask = 2131,
  CODE_FOR_sse_vmaddv4sf3_mask_round = 2132,
  CODE_FOR_sse_vmsubv4sf3 = 2133,
  CODE_FOR_sse_vmsubv4sf3_round = 2134,
  CODE_FOR_sse_vmsubv4sf3_mask = 2135,
  CODE_FOR_sse_vmsubv4sf3_mask_round = 2136,
  CODE_FOR_sse2_vmaddv2df3 = 2137,
  CODE_FOR_sse2_vmaddv2df3_round = 2138,
  CODE_FOR_sse2_vmaddv2df3_mask = 2139,
  CODE_FOR_sse2_vmaddv2df3_mask_round = 2140,
  CODE_FOR_sse2_vmsubv2df3 = 2141,
  CODE_FOR_sse2_vmsubv2df3_round = 2142,
  CODE_FOR_sse2_vmsubv2df3_mask = 2143,
  CODE_FOR_sse2_vmsubv2df3_mask_round = 2144,
  CODE_FOR_avx512fp16_vmmulv8hf3 = 2187,
  CODE_FOR_avx512fp16_vmmulv8hf3_round = 2188,
  CODE_FOR_avx512fp16_vmmulv8hf3_mask = 2189,
  CODE_FOR_avx512fp16_vmmulv8hf3_mask_round = 2190,
  CODE_FOR_avx512fp16_vmdivv8hf3 = 2191,
  CODE_FOR_avx512fp16_vmdivv8hf3_round = 2192,
  CODE_FOR_avx512fp16_vmdivv8hf3_mask = 2193,
  CODE_FOR_avx512fp16_vmdivv8hf3_mask_round = 2194,
  CODE_FOR_sse_vmmulv4sf3 = 2195,
  CODE_FOR_sse_vmmulv4sf3_round = 2196,
  CODE_FOR_sse_vmmulv4sf3_mask = 2197,
  CODE_FOR_sse_vmmulv4sf3_mask_round = 2198,
  CODE_FOR_sse_vmdivv4sf3 = 2199,
  CODE_FOR_sse_vmdivv4sf3_round = 2200,
  CODE_FOR_sse_vmdivv4sf3_mask = 2201,
  CODE_FOR_sse_vmdivv4sf3_mask_round = 2202,
  CODE_FOR_sse2_vmmulv2df3 = 2203,
  CODE_FOR_sse2_vmmulv2df3_round = 2204,
  CODE_FOR_sse2_vmmulv2df3_mask = 2205,
  CODE_FOR_sse2_vmmulv2df3_mask_round = 2206,
  CODE_FOR_sse2_vmdivv2df3 = 2207,
  CODE_FOR_sse2_vmdivv2df3_round = 2208,
  CODE_FOR_sse2_vmdivv2df3_mask = 2209,
  CODE_FOR_sse2_vmdivv2df3_mask_round = 2210,
  CODE_FOR_avx512fp16_divv32hf3 = 2211,
  CODE_FOR_avx512fp16_divv32hf3_round = 2212,
  CODE_FOR_avx512fp16_divv32hf3_mask = 2213,
  CODE_FOR_avx512fp16_divv32hf3_mask_round = 2214,
  CODE_FOR_avx512fp16_divv16hf3 = 2215,
   CODE_FOR_avx512fp16_divv16hf3_round = CODE_FOR_nothing,
  CODE_FOR_avx512fp16_divv16hf3_mask = 2216,
   CODE_FOR_avx512fp16_divv16hf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512fp16_divv8hf3 = 2217,
   CODE_FOR_avx512fp16_divv8hf3_round = CODE_FOR_nothing,
  CODE_FOR_avx512fp16_divv8hf3_mask = 2218,
   CODE_FOR_avx512fp16_divv8hf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_divv16sf3 = 2219,
  CODE_FOR_avx512f_divv16sf3_round = 2220,
  CODE_FOR_avx512f_divv16sf3_mask = 2221,
  CODE_FOR_avx512f_divv16sf3_mask_round = 2222,
  CODE_FOR_avx_divv8sf3 = 2223,
   CODE_FOR_avx_divv8sf3_round = CODE_FOR_nothing,
  CODE_FOR_avx_divv8sf3_mask = 2224,
   CODE_FOR_avx_divv8sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_sse_divv4sf3 = 2225,
   CODE_FOR_sse_divv4sf3_round = CODE_FOR_nothing,
  CODE_FOR_sse_divv4sf3_mask = 2226,
   CODE_FOR_sse_divv4sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_divv8df3 = 2227,
  CODE_FOR_avx512f_divv8df3_round = 2228,
  CODE_FOR_avx512f_divv8df3_mask = 2229,
  CODE_FOR_avx512f_divv8df3_mask_round = 2230,
  CODE_FOR_avx_divv4df3 = 2231,
   CODE_FOR_avx_divv4df3_round = CODE_FOR_nothing,
  CODE_FOR_avx_divv4df3_mask = 2232,
   CODE_FOR_avx_divv4df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_sse2_divv2df3 = 2233,
   CODE_FOR_sse2_divv2df3_round = CODE_FOR_nothing,
  CODE_FOR_sse2_divv2df3_mask = 2234,
   CODE_FOR_sse2_divv2df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx_rcpv8sf2 = 2235,
  CODE_FOR_sse_rcpv4sf2 = 2236,
  CODE_FOR_sse_vmrcpv4sf2 = 2237,
  CODE_FOR_avx512fp16_rcpv32hf2 = 2239,
  CODE_FOR_avx512fp16_rcpv32hf2_mask = 2240,
  CODE_FOR_avx512fp16_rcpv16hf2 = 2241,
  CODE_FOR_avx512fp16_rcpv16hf2_mask = 2242,
  CODE_FOR_avx512fp16_rcpv8hf2 = 2243,
  CODE_FOR_avx512fp16_rcpv8hf2_mask = 2244,
  CODE_FOR_avx512fp16_vmrcpv8hf2 = 2245,
  CODE_FOR_avx512fp16_vmrcpv8hf2_mask = 2246,
  CODE_FOR_rcp14v16sf_mask = 2249,
  CODE_FOR_rcp14v8sf_mask = 2251,
  CODE_FOR_rcp14v4sf_mask = 2253,
  CODE_FOR_rcp14v8df_mask = 2255,
  CODE_FOR_rcp14v4df_mask = 2257,
  CODE_FOR_rcp14v2df_mask = 2259,
  CODE_FOR_srcp14v4sf = 2260,
  CODE_FOR_srcp14v2df = 2261,
  CODE_FOR_srcp14v4sf_mask = 2262,
  CODE_FOR_srcp14v2df_mask = 2263,
  CODE_FOR_avx512fp16_sqrtv32hf2 = 2264,
  CODE_FOR_avx512fp16_sqrtv32hf2_round = 2265,
  CODE_FOR_avx512fp16_sqrtv32hf2_mask = 2266,
  CODE_FOR_avx512fp16_sqrtv32hf2_mask_round = 2267,
  CODE_FOR_avx512fp16_sqrtv16hf2 = 2268,
   CODE_FOR_avx512fp16_sqrtv16hf2_round = CODE_FOR_nothing,
  CODE_FOR_avx512fp16_sqrtv16hf2_mask = 2269,
   CODE_FOR_avx512fp16_sqrtv16hf2_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512fp16_sqrtv8hf2 = 2270,
   CODE_FOR_avx512fp16_sqrtv8hf2_round = CODE_FOR_nothing,
  CODE_FOR_avx512fp16_sqrtv8hf2_mask = 2271,
   CODE_FOR_avx512fp16_sqrtv8hf2_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_sqrtv16sf2 = 2272,
  CODE_FOR_avx512f_sqrtv16sf2_round = 2273,
  CODE_FOR_avx512f_sqrtv16sf2_mask = 2274,
  CODE_FOR_avx512f_sqrtv16sf2_mask_round = 2275,
  CODE_FOR_avx_sqrtv8sf2 = 2276,
   CODE_FOR_avx_sqrtv8sf2_round = CODE_FOR_nothing,
  CODE_FOR_avx_sqrtv8sf2_mask = 2277,
   CODE_FOR_avx_sqrtv8sf2_mask_round = CODE_FOR_nothing,
  CODE_FOR_sse_sqrtv4sf2 = 2278,
   CODE_FOR_sse_sqrtv4sf2_round = CODE_FOR_nothing,
  CODE_FOR_sse_sqrtv4sf2_mask = 2279,
   CODE_FOR_sse_sqrtv4sf2_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_sqrtv8df2 = 2280,
  CODE_FOR_avx512f_sqrtv8df2_round = 2281,
  CODE_FOR_avx512f_sqrtv8df2_mask = 2282,
  CODE_FOR_avx512f_sqrtv8df2_mask_round = 2283,
  CODE_FOR_avx_sqrtv4df2 = 2284,
   CODE_FOR_avx_sqrtv4df2_round = CODE_FOR_nothing,
  CODE_FOR_avx_sqrtv4df2_mask = 2285,
   CODE_FOR_avx_sqrtv4df2_mask_round = CODE_FOR_nothing,
  CODE_FOR_sse2_sqrtv2df2 = 2286,
   CODE_FOR_sse2_sqrtv2df2_round = CODE_FOR_nothing,
  CODE_FOR_sse2_sqrtv2df2_mask = 2287,
   CODE_FOR_sse2_sqrtv2df2_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512fp16_vmsqrtv8hf2 = 2288,
  CODE_FOR_avx512fp16_vmsqrtv8hf2_round = 2289,
  CODE_FOR_avx512fp16_vmsqrtv8hf2_mask = 2290,
  CODE_FOR_avx512fp16_vmsqrtv8hf2_mask_round = 2291,
  CODE_FOR_sse_vmsqrtv4sf2 = 2292,
  CODE_FOR_sse_vmsqrtv4sf2_round = 2293,
  CODE_FOR_sse_vmsqrtv4sf2_mask = 2294,
  CODE_FOR_sse_vmsqrtv4sf2_mask_round = 2295,
  CODE_FOR_sse2_vmsqrtv2df2 = 2296,
  CODE_FOR_sse2_vmsqrtv2df2_round = 2297,
  CODE_FOR_sse2_vmsqrtv2df2_mask = 2298,
  CODE_FOR_sse2_vmsqrtv2df2_mask_round = 2299,
  CODE_FOR_avx_rsqrtv8sf2 = 2312,
  CODE_FOR_sse_rsqrtv4sf2 = 2313,
  CODE_FOR_avx512fp16_rsqrtv32hf2 = 2314,
  CODE_FOR_avx512fp16_rsqrtv32hf2_mask = 2315,
  CODE_FOR_avx512fp16_rsqrtv16hf2 = 2316,
  CODE_FOR_avx512fp16_rsqrtv16hf2_mask = 2317,
  CODE_FOR_avx512fp16_rsqrtv8hf2 = 2318,
  CODE_FOR_avx512fp16_rsqrtv8hf2_mask = 2319,
  CODE_FOR_rsqrt14v16sf_mask = 2321,
  CODE_FOR_rsqrt14v8sf_mask = 2323,
  CODE_FOR_rsqrt14v4sf_mask = 2325,
  CODE_FOR_rsqrt14v8df_mask = 2327,
  CODE_FOR_rsqrt14v4df_mask = 2329,
  CODE_FOR_rsqrt14v2df_mask = 2331,
  CODE_FOR_rsqrt14v4sf = 2332,
  CODE_FOR_rsqrt14v2df = 2333,
  CODE_FOR_rsqrt14_v4sf_mask = 2334,
  CODE_FOR_rsqrt14_v2df_mask = 2335,
  CODE_FOR_sse_vmrsqrtv4sf2 = 2336,
  CODE_FOR_avx512fp16_vmrsqrtv8hf2 = 2338,
  CODE_FOR_avx512fp16_vmrsqrtv8hf2_mask = 2339,
  CODE_FOR_ieee_maxv32hf3 = 2390,
  CODE_FOR_ieee_maxv32hf3_round = 2391,
  CODE_FOR_ieee_maxv32hf3_mask = 2392,
  CODE_FOR_ieee_maxv32hf3_mask_round = 2393,
  CODE_FOR_ieee_minv32hf3 = 2394,
  CODE_FOR_ieee_minv32hf3_round = 2395,
  CODE_FOR_ieee_minv32hf3_mask = 2396,
  CODE_FOR_ieee_minv32hf3_mask_round = 2397,
  CODE_FOR_ieee_maxv16hf3 = 2398,
   CODE_FOR_ieee_maxv16hf3_round = CODE_FOR_nothing,
  CODE_FOR_ieee_maxv16hf3_mask = 2399,
   CODE_FOR_ieee_maxv16hf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_ieee_minv16hf3 = 2400,
   CODE_FOR_ieee_minv16hf3_round = CODE_FOR_nothing,
  CODE_FOR_ieee_minv16hf3_mask = 2401,
   CODE_FOR_ieee_minv16hf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_ieee_maxv8hf3 = 2402,
   CODE_FOR_ieee_maxv8hf3_round = CODE_FOR_nothing,
  CODE_FOR_ieee_maxv8hf3_mask = 2403,
   CODE_FOR_ieee_maxv8hf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_ieee_minv8hf3 = 2404,
   CODE_FOR_ieee_minv8hf3_round = CODE_FOR_nothing,
  CODE_FOR_ieee_minv8hf3_mask = 2405,
   CODE_FOR_ieee_minv8hf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_ieee_maxv16sf3 = 2406,
  CODE_FOR_ieee_maxv16sf3_round = 2407,
  CODE_FOR_ieee_maxv16sf3_mask = 2408,
  CODE_FOR_ieee_maxv16sf3_mask_round = 2409,
  CODE_FOR_ieee_minv16sf3 = 2410,
  CODE_FOR_ieee_minv16sf3_round = 2411,
  CODE_FOR_ieee_minv16sf3_mask = 2412,
  CODE_FOR_ieee_minv16sf3_mask_round = 2413,
  CODE_FOR_ieee_maxv8sf3 = 2414,
   CODE_FOR_ieee_maxv8sf3_round = CODE_FOR_nothing,
  CODE_FOR_ieee_maxv8sf3_mask = 2415,
   CODE_FOR_ieee_maxv8sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_ieee_minv8sf3 = 2416,
   CODE_FOR_ieee_minv8sf3_round = CODE_FOR_nothing,
  CODE_FOR_ieee_minv8sf3_mask = 2417,
   CODE_FOR_ieee_minv8sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_ieee_maxv4sf3 = 2418,
   CODE_FOR_ieee_maxv4sf3_round = CODE_FOR_nothing,
  CODE_FOR_ieee_maxv4sf3_mask = 2419,
   CODE_FOR_ieee_maxv4sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_ieee_minv4sf3 = 2420,
   CODE_FOR_ieee_minv4sf3_round = CODE_FOR_nothing,
  CODE_FOR_ieee_minv4sf3_mask = 2421,
   CODE_FOR_ieee_minv4sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_ieee_maxv8df3 = 2422,
  CODE_FOR_ieee_maxv8df3_round = 2423,
  CODE_FOR_ieee_maxv8df3_mask = 2424,
  CODE_FOR_ieee_maxv8df3_mask_round = 2425,
  CODE_FOR_ieee_minv8df3 = 2426,
  CODE_FOR_ieee_minv8df3_round = 2427,
  CODE_FOR_ieee_minv8df3_mask = 2428,
  CODE_FOR_ieee_minv8df3_mask_round = 2429,
  CODE_FOR_ieee_maxv4df3 = 2430,
   CODE_FOR_ieee_maxv4df3_round = CODE_FOR_nothing,
  CODE_FOR_ieee_maxv4df3_mask = 2431,
   CODE_FOR_ieee_maxv4df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_ieee_minv4df3 = 2432,
   CODE_FOR_ieee_minv4df3_round = CODE_FOR_nothing,
  CODE_FOR_ieee_minv4df3_mask = 2433,
   CODE_FOR_ieee_minv4df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_ieee_maxv2df3 = 2434,
   CODE_FOR_ieee_maxv2df3_round = CODE_FOR_nothing,
  CODE_FOR_ieee_maxv2df3_mask = 2435,
   CODE_FOR_ieee_maxv2df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_ieee_minv2df3 = 2436,
   CODE_FOR_ieee_minv2df3_round = CODE_FOR_nothing,
  CODE_FOR_ieee_minv2df3_mask = 2437,
   CODE_FOR_ieee_minv2df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512fp16_vmsmaxv8hf3 = 2444,
  CODE_FOR_avx512fp16_vmsmaxv8hf3_mask = 2445,
  CODE_FOR_avx512fp16_vmsmaxv8hf3_round = 2446,
  CODE_FOR_avx512fp16_vmsmaxv8hf3_mask_round = 2447,
  CODE_FOR_avx512fp16_vmsminv8hf3 = 2448,
  CODE_FOR_avx512fp16_vmsminv8hf3_mask = 2449,
  CODE_FOR_avx512fp16_vmsminv8hf3_round = 2450,
  CODE_FOR_avx512fp16_vmsminv8hf3_mask_round = 2451,
  CODE_FOR_sse_vmsmaxv4sf3 = 2452,
  CODE_FOR_sse_vmsmaxv4sf3_mask = 2453,
  CODE_FOR_sse_vmsmaxv4sf3_round = 2454,
  CODE_FOR_sse_vmsmaxv4sf3_mask_round = 2455,
  CODE_FOR_sse_vmsminv4sf3 = 2456,
  CODE_FOR_sse_vmsminv4sf3_mask = 2457,
  CODE_FOR_sse_vmsminv4sf3_round = 2458,
  CODE_FOR_sse_vmsminv4sf3_mask_round = 2459,
  CODE_FOR_sse2_vmsmaxv2df3 = 2460,
  CODE_FOR_sse2_vmsmaxv2df3_mask = 2461,
  CODE_FOR_sse2_vmsmaxv2df3_round = 2462,
  CODE_FOR_sse2_vmsmaxv2df3_mask_round = 2463,
  CODE_FOR_sse2_vmsminv2df3 = 2464,
  CODE_FOR_sse2_vmsminv2df3_mask = 2465,
  CODE_FOR_sse2_vmsminv2df3_round = 2466,
  CODE_FOR_sse2_vmsminv2df3_mask_round = 2467,
  CODE_FOR_vec_addsubv8sf3 = 2468,
  CODE_FOR_vec_addsubv4sf3 = 2469,
  CODE_FOR_vec_addsubv4df3 = 2470,
  CODE_FOR_vec_addsubv2df3 = 2471,
  CODE_FOR_avx_haddv4df3 = 2472,
  CODE_FOR_avx_hsubv4df3 = 2473,
  CODE_FOR_sse3_hsubv2df3 = 2475,
  CODE_FOR_avx_haddv8sf3 = 2478,
  CODE_FOR_avx_hsubv8sf3 = 2479,
  CODE_FOR_sse3_haddv4sf3 = 2480,
  CODE_FOR_sse3_hsubv4sf3 = 2481,
  CODE_FOR_reducepv32hf_mask = 2484,
  CODE_FOR_reducepv32hf_mask_round = 2485,
  CODE_FOR_reducepv16hf_mask = 2488,
  CODE_FOR_reducepv16hf_mask_round = 2489,
  CODE_FOR_reducepv8hf_mask = 2492,
  CODE_FOR_reducepv8hf_mask_round = 2493,
  CODE_FOR_reducepv16sf_mask = 2496,
  CODE_FOR_reducepv16sf_mask_round = 2497,
  CODE_FOR_reducepv8sf_mask = 2500,
  CODE_FOR_reducepv8sf_mask_round = 2501,
  CODE_FOR_reducepv4sf_mask = 2504,
  CODE_FOR_reducepv4sf_mask_round = 2505,
  CODE_FOR_reducepv8df_mask = 2508,
  CODE_FOR_reducepv8df_mask_round = 2509,
  CODE_FOR_reducepv4df_mask = 2512,
  CODE_FOR_reducepv4df_mask_round = 2513,
  CODE_FOR_reducepv2df_mask = 2516,
  CODE_FOR_reducepv2df_mask_round = 2517,
  CODE_FOR_reducesv8hf = 2518,
  CODE_FOR_reducesv8hf_mask = 2519,
  CODE_FOR_reducesv8hf_round = 2520,
  CODE_FOR_reducesv8hf_mask_round = 2521,
  CODE_FOR_reducesv4sf = 2522,
  CODE_FOR_reducesv4sf_mask = 2523,
  CODE_FOR_reducesv4sf_round = 2524,
  CODE_FOR_reducesv4sf_mask_round = 2525,
  CODE_FOR_reducesv2df = 2526,
  CODE_FOR_reducesv2df_mask = 2527,
  CODE_FOR_reducesv2df_round = 2528,
  CODE_FOR_reducesv2df_mask_round = 2529,
  CODE_FOR_avx_cmpv8sf3 = 2530,
  CODE_FOR_avx_cmpv4sf3 = 2531,
  CODE_FOR_avx_cmpv4df3 = 2532,
  CODE_FOR_avx_cmpv2df3 = 2533,
  CODE_FOR_avx_vmcmpv4sf3 = 2562,
  CODE_FOR_avx_vmcmpv2df3 = 2563,
  CODE_FOR_avx_maskcmpv8sf3 = 2568,
  CODE_FOR_sse_maskcmpv4sf3 = 2569,
  CODE_FOR_avx_maskcmpv4df3 = 2570,
  CODE_FOR_sse2_maskcmpv2df3 = 2571,
  CODE_FOR_sse_vmmaskcmpv4sf3 = 2572,
  CODE_FOR_sse2_vmmaskcmpv2df3 = 2573,
  CODE_FOR_avx512f_cmpv16si3 = 2574,
  CODE_FOR_avx512f_cmpv16si3_round = 2575,
  CODE_FOR_avx512f_cmpv16si3_mask = 2576,
  CODE_FOR_avx512f_cmpv16si3_mask_round = 2577,
  CODE_FOR_avx512vl_cmpv8si3 = 2578,
   CODE_FOR_avx512vl_cmpv8si3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv8si3_mask = 2579,
   CODE_FOR_avx512vl_cmpv8si3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv4si3 = 2580,
   CODE_FOR_avx512vl_cmpv4si3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv4si3_mask = 2581,
   CODE_FOR_avx512vl_cmpv4si3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_cmpv8di3 = 2582,
  CODE_FOR_avx512f_cmpv8di3_round = 2583,
  CODE_FOR_avx512f_cmpv8di3_mask = 2584,
  CODE_FOR_avx512f_cmpv8di3_mask_round = 2585,
  CODE_FOR_avx512vl_cmpv4di3 = 2586,
   CODE_FOR_avx512vl_cmpv4di3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv4di3_mask = 2587,
   CODE_FOR_avx512vl_cmpv4di3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv2di3 = 2588,
   CODE_FOR_avx512vl_cmpv2di3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv2di3_mask = 2589,
   CODE_FOR_avx512vl_cmpv2di3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512bw_cmpv32hf3 = 2590,
  CODE_FOR_avx512bw_cmpv32hf3_round = 2591,
  CODE_FOR_avx512bw_cmpv32hf3_mask = 2592,
  CODE_FOR_avx512bw_cmpv32hf3_mask_round = 2593,
  CODE_FOR_avx512vl_cmpv16hf3 = 2594,
   CODE_FOR_avx512vl_cmpv16hf3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv16hf3_mask = 2595,
   CODE_FOR_avx512vl_cmpv16hf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512fp16_cmpv8hf3 = 2596,
   CODE_FOR_avx512fp16_cmpv8hf3_round = CODE_FOR_nothing,
  CODE_FOR_avx512fp16_cmpv8hf3_mask = 2597,
   CODE_FOR_avx512fp16_cmpv8hf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_cmpv16sf3 = 2598,
  CODE_FOR_avx512f_cmpv16sf3_round = 2599,
  CODE_FOR_avx512f_cmpv16sf3_mask = 2600,
  CODE_FOR_avx512f_cmpv16sf3_mask_round = 2601,
  CODE_FOR_avx512vl_cmpv8sf3 = 2602,
   CODE_FOR_avx512vl_cmpv8sf3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv8sf3_mask = 2603,
   CODE_FOR_avx512vl_cmpv8sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv4sf3 = 2604,
   CODE_FOR_avx512vl_cmpv4sf3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv4sf3_mask = 2605,
   CODE_FOR_avx512vl_cmpv4sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_cmpv8df3 = 2606,
  CODE_FOR_avx512f_cmpv8df3_round = 2607,
  CODE_FOR_avx512f_cmpv8df3_mask = 2608,
  CODE_FOR_avx512f_cmpv8df3_mask_round = 2609,
  CODE_FOR_avx512vl_cmpv4df3 = 2610,
   CODE_FOR_avx512vl_cmpv4df3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv4df3_mask = 2611,
   CODE_FOR_avx512vl_cmpv4df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv2df3 = 2612,
   CODE_FOR_avx512vl_cmpv2df3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv2df3_mask = 2613,
   CODE_FOR_avx512vl_cmpv2df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512bw_cmpv64qi3 = 2716,
  CODE_FOR_avx512bw_cmpv64qi3_mask = 2717,
  CODE_FOR_avx512vl_cmpv16qi3 = 2718,
  CODE_FOR_avx512vl_cmpv16qi3_mask = 2719,
  CODE_FOR_avx512vl_cmpv32qi3 = 2720,
  CODE_FOR_avx512vl_cmpv32qi3_mask = 2721,
  CODE_FOR_avx512bw_cmpv32hi3 = 2722,
  CODE_FOR_avx512bw_cmpv32hi3_mask = 2723,
  CODE_FOR_avx512vl_cmpv16hi3 = 2724,
  CODE_FOR_avx512vl_cmpv16hi3_mask = 2725,
  CODE_FOR_avx512vl_cmpv8hi3 = 2726,
  CODE_FOR_avx512vl_cmpv8hi3_mask = 2727,
  CODE_FOR_avx512bw_ucmpv64qi3 = 2800,
  CODE_FOR_avx512bw_ucmpv64qi3_mask = 2801,
  CODE_FOR_avx512vl_ucmpv16qi3 = 2802,
  CODE_FOR_avx512vl_ucmpv16qi3_mask = 2803,
  CODE_FOR_avx512vl_ucmpv32qi3 = 2804,
  CODE_FOR_avx512vl_ucmpv32qi3_mask = 2805,
  CODE_FOR_avx512bw_ucmpv32hi3 = 2806,
  CODE_FOR_avx512bw_ucmpv32hi3_mask = 2807,
  CODE_FOR_avx512vl_ucmpv16hi3 = 2808,
  CODE_FOR_avx512vl_ucmpv16hi3_mask = 2809,
  CODE_FOR_avx512vl_ucmpv8hi3 = 2810,
  CODE_FOR_avx512vl_ucmpv8hi3_mask = 2811,
  CODE_FOR_avx512f_ucmpv16si3 = 2872,
  CODE_FOR_avx512f_ucmpv16si3_mask = 2873,
  CODE_FOR_avx512vl_ucmpv8si3 = 2874,
  CODE_FOR_avx512vl_ucmpv8si3_mask = 2875,
  CODE_FOR_avx512vl_ucmpv4si3 = 2876,
  CODE_FOR_avx512vl_ucmpv4si3_mask = 2877,
  CODE_FOR_avx512f_ucmpv8di3 = 2878,
  CODE_FOR_avx512f_ucmpv8di3_mask = 2879,
  CODE_FOR_avx512vl_ucmpv4di3 = 2880,
  CODE_FOR_avx512vl_ucmpv4di3_mask = 2881,
  CODE_FOR_avx512vl_ucmpv2di3 = 2882,
  CODE_FOR_avx512vl_ucmpv2di3_mask = 2883,
  CODE_FOR_avx512f_vmcmpv8hf3 = 2938,
  CODE_FOR_avx512f_vmcmpv8hf3_round = 2939,
  CODE_FOR_avx512f_vmcmpv4sf3 = 2940,
  CODE_FOR_avx512f_vmcmpv4sf3_round = 2941,
  CODE_FOR_avx512f_vmcmpv2df3 = 2942,
  CODE_FOR_avx512f_vmcmpv2df3_round = 2943,
  CODE_FOR_avx512f_vmcmpv8hf3_mask = 2944,
  CODE_FOR_avx512f_vmcmpv8hf3_mask_round = 2945,
  CODE_FOR_avx512f_vmcmpv4sf3_mask = 2946,
  CODE_FOR_avx512f_vmcmpv4sf3_mask_round = 2947,
  CODE_FOR_avx512f_vmcmpv2df3_mask = 2948,
  CODE_FOR_avx512f_vmcmpv2df3_mask_round = 2949,
   CODE_FOR_avx512fp16_comi = CODE_FOR_nothing,
   CODE_FOR_avx512fp16_comi_round = CODE_FOR_nothing,
   CODE_FOR_avx512fp16_ucomi = CODE_FOR_nothing,
   CODE_FOR_avx512fp16_ucomi_round = CODE_FOR_nothing,
  CODE_FOR_sse_comi = 2950,
  CODE_FOR_sse_comi_round = 2951,
  CODE_FOR_sse_ucomi = 2952,
  CODE_FOR_sse_ucomi_round = 2953,
  CODE_FOR_sse2_comi = 2954,
  CODE_FOR_sse2_comi_round = 2955,
  CODE_FOR_sse2_ucomi = 2956,
  CODE_FOR_sse2_ucomi_round = 2957,
  CODE_FOR_avx512fp16_andnotv16hf3 = 2958,
   CODE_FOR_avx512fp16_andnotv16hf3_mask = CODE_FOR_nothing,
  CODE_FOR_avx512fp16_andnotv8hf3 = 2959,
   CODE_FOR_avx512fp16_andnotv8hf3_mask = CODE_FOR_nothing,
  CODE_FOR_avx_andnotv8sf3 = 2960,
  CODE_FOR_avx_andnotv8sf3_mask = 2961,
  CODE_FOR_sse_andnotv4sf3 = 2962,
  CODE_FOR_sse_andnotv4sf3_mask = 2963,
  CODE_FOR_avx_andnotv4df3 = 2964,
  CODE_FOR_avx_andnotv4df3_mask = 2965,
  CODE_FOR_sse2_andnotv2df3 = 2966,
  CODE_FOR_sse2_andnotv2df3_mask = 2967,
  CODE_FOR_avx512fp16_andnotv32hf3 = 2968,
   CODE_FOR_avx512fp16_andnotv32hf3_mask = CODE_FOR_nothing,
  CODE_FOR_avx512f_andnotv16sf3 = 2969,
  CODE_FOR_avx512f_andnotv16sf3_mask = 2970,
  CODE_FOR_avx512f_andnotv8df3 = 2971,
  CODE_FOR_avx512f_andnotv8df3_mask = 2972,
  CODE_FOR_andsf3 = 3031,
  CODE_FOR_iorsf3 = 3032,
  CODE_FOR_xorsf3 = 3033,
  CODE_FOR_anddf3 = 3034,
  CODE_FOR_iordf3 = 3035,
  CODE_FOR_xordf3 = 3036,
  CODE_FOR_fma_fmadd_v32hf_maskz_1 = 3047,
  CODE_FOR_fma_fmadd_v32hf_maskz_1_round = 3049,
  CODE_FOR_fma_fmadd_v16hf_maskz_1 = 3051,
   CODE_FOR_fma_fmadd_v16hf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmadd_v8hf_maskz_1 = 3053,
   CODE_FOR_fma_fmadd_v8hf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmadd_v16sf_maskz_1 = 3057,
  CODE_FOR_fma_fmadd_v16sf_maskz_1_round = 3059,
  CODE_FOR_fma_fmadd_v8sf_maskz_1 = 3061,
   CODE_FOR_fma_fmadd_v8sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmadd_v4sf_maskz_1 = 3063,
   CODE_FOR_fma_fmadd_v4sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmadd_v8df_maskz_1 = 3066,
  CODE_FOR_fma_fmadd_v8df_maskz_1_round = 3068,
  CODE_FOR_fma_fmadd_v4df_maskz_1 = 3070,
   CODE_FOR_fma_fmadd_v4df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmadd_v2df_maskz_1 = 3072,
   CODE_FOR_fma_fmadd_v2df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_avx512bw_fmadd_v32hf_mask = 3073,
  CODE_FOR_avx512bw_fmadd_v32hf_mask_round = 3074,
  CODE_FOR_avx512vl_fmadd_v16hf_mask = 3075,
   CODE_FOR_avx512vl_fmadd_v16hf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512fp16_fmadd_v8hf_mask = 3076,
   CODE_FOR_avx512fp16_fmadd_v8hf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fmadd_v16sf_mask = 3077,
  CODE_FOR_avx512f_fmadd_v16sf_mask_round = 3078,
  CODE_FOR_avx512vl_fmadd_v8sf_mask = 3079,
   CODE_FOR_avx512vl_fmadd_v8sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fmadd_v4sf_mask = 3080,
   CODE_FOR_avx512vl_fmadd_v4sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fmadd_v8df_mask = 3081,
  CODE_FOR_avx512f_fmadd_v8df_mask_round = 3082,
  CODE_FOR_avx512vl_fmadd_v4df_mask = 3083,
   CODE_FOR_avx512vl_fmadd_v4df_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fmadd_v2df_mask = 3084,
   CODE_FOR_avx512vl_fmadd_v2df_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512bw_fmadd_v32hf_mask3 = 3085,
  CODE_FOR_avx512bw_fmadd_v32hf_mask3_round = 3086,
  CODE_FOR_avx512vl_fmadd_v16hf_mask3 = 3087,
  CODE_FOR_avx512vl_fmadd_v16hf_mask3_round = 3088,
  CODE_FOR_avx512fp16_fmadd_v8hf_mask3 = 3089,
  CODE_FOR_avx512fp16_fmadd_v8hf_mask3_round = 3090,
  CODE_FOR_avx512f_fmadd_v16sf_mask3 = 3091,
  CODE_FOR_avx512f_fmadd_v16sf_mask3_round = 3092,
  CODE_FOR_avx512vl_fmadd_v8sf_mask3 = 3093,
  CODE_FOR_avx512vl_fmadd_v8sf_mask3_round = 3094,
  CODE_FOR_avx512vl_fmadd_v4sf_mask3 = 3095,
  CODE_FOR_avx512vl_fmadd_v4sf_mask3_round = 3096,
  CODE_FOR_avx512f_fmadd_v8df_mask3 = 3097,
  CODE_FOR_avx512f_fmadd_v8df_mask3_round = 3098,
  CODE_FOR_avx512vl_fmadd_v4df_mask3 = 3099,
  CODE_FOR_avx512vl_fmadd_v4df_mask3_round = 3100,
  CODE_FOR_avx512vl_fmadd_v2df_mask3 = 3101,
  CODE_FOR_avx512vl_fmadd_v2df_mask3_round = 3102,
  CODE_FOR_fma_fmsub_v32hf_maskz_1 = 3110,
  CODE_FOR_fma_fmsub_v32hf_maskz_1_round = 3112,
  CODE_FOR_fma_fmsub_v16hf_maskz_1 = 3114,
   CODE_FOR_fma_fmsub_v16hf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmsub_v8hf_maskz_1 = 3116,
   CODE_FOR_fma_fmsub_v8hf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmsub_v16sf_maskz_1 = 3120,
  CODE_FOR_fma_fmsub_v16sf_maskz_1_round = 3122,
  CODE_FOR_fma_fmsub_v8sf_maskz_1 = 3124,
   CODE_FOR_fma_fmsub_v8sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmsub_v4sf_maskz_1 = 3126,
   CODE_FOR_fma_fmsub_v4sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmsub_v8df_maskz_1 = 3129,
  CODE_FOR_fma_fmsub_v8df_maskz_1_round = 3131,
  CODE_FOR_fma_fmsub_v4df_maskz_1 = 3133,
   CODE_FOR_fma_fmsub_v4df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmsub_v2df_maskz_1 = 3135,
   CODE_FOR_fma_fmsub_v2df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_avx512bw_fmsub_v32hf_mask = 3136,
  CODE_FOR_avx512bw_fmsub_v32hf_mask_round = 3137,
  CODE_FOR_avx512vl_fmsub_v16hf_mask = 3138,
  CODE_FOR_avx512vl_fmsub_v16hf_mask_round = 3139,
  CODE_FOR_avx512fp16_fmsub_v8hf_mask = 3140,
  CODE_FOR_avx512fp16_fmsub_v8hf_mask_round = 3141,
  CODE_FOR_avx512f_fmsub_v16sf_mask = 3142,
  CODE_FOR_avx512f_fmsub_v16sf_mask_round = 3143,
  CODE_FOR_avx512vl_fmsub_v8sf_mask = 3144,
  CODE_FOR_avx512vl_fmsub_v8sf_mask_round = 3145,
  CODE_FOR_avx512vl_fmsub_v4sf_mask = 3146,
  CODE_FOR_avx512vl_fmsub_v4sf_mask_round = 3147,
  CODE_FOR_avx512f_fmsub_v8df_mask = 3148,
  CODE_FOR_avx512f_fmsub_v8df_mask_round = 3149,
  CODE_FOR_avx512vl_fmsub_v4df_mask = 3150,
  CODE_FOR_avx512vl_fmsub_v4df_mask_round = 3151,
  CODE_FOR_avx512vl_fmsub_v2df_mask = 3152,
  CODE_FOR_avx512vl_fmsub_v2df_mask_round = 3153,
  CODE_FOR_avx512bw_fmsub_v32hf_mask3 = 3154,
  CODE_FOR_avx512bw_fmsub_v32hf_mask3_round = 3155,
  CODE_FOR_avx512vl_fmsub_v16hf_mask3 = 3156,
   CODE_FOR_avx512vl_fmsub_v16hf_mask3_round = CODE_FOR_nothing,
  CODE_FOR_avx512fp16_fmsub_v8hf_mask3 = 3157,
   CODE_FOR_avx512fp16_fmsub_v8hf_mask3_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fmsub_v16sf_mask3 = 3158,
  CODE_FOR_avx512f_fmsub_v16sf_mask3_round = 3159,
  CODE_FOR_avx512vl_fmsub_v8sf_mask3 = 3160,
   CODE_FOR_avx512vl_fmsub_v8sf_mask3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fmsub_v4sf_mask3 = 3161,
   CODE_FOR_avx512vl_fmsub_v4sf_mask3_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fmsub_v8df_mask3 = 3162,
  CODE_FOR_avx512f_fmsub_v8df_mask3_round = 3163,
  CODE_FOR_avx512vl_fmsub_v4df_mask3 = 3164,
   CODE_FOR_avx512vl_fmsub_v4df_mask3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fmsub_v2df_mask3 = 3165,
   CODE_FOR_avx512vl_fmsub_v2df_mask3_round = CODE_FOR_nothing,
  CODE_FOR_fma_fnmadd_v32hf_maskz_1 = 3173,
  CODE_FOR_fma_fnmadd_v32hf_maskz_1_round = 3175,
  CODE_FOR_fma_fnmadd_v16hf_maskz_1 = 3177,
   CODE_FOR_fma_fnmadd_v16hf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fnmadd_v8hf_maskz_1 = 3179,
   CODE_FOR_fma_fnmadd_v8hf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fnmadd_v16sf_maskz_1 = 3183,
  CODE_FOR_fma_fnmadd_v16sf_maskz_1_round = 3185,
  CODE_FOR_fma_fnmadd_v8sf_maskz_1 = 3187,
   CODE_FOR_fma_fnmadd_v8sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fnmadd_v4sf_maskz_1 = 3189,
   CODE_FOR_fma_fnmadd_v4sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fnmadd_v8df_maskz_1 = 3192,
  CODE_FOR_fma_fnmadd_v8df_maskz_1_round = 3194,
  CODE_FOR_fma_fnmadd_v4df_maskz_1 = 3196,
   CODE_FOR_fma_fnmadd_v4df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fnmadd_v2df_maskz_1 = 3198,
   CODE_FOR_fma_fnmadd_v2df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_avx512bw_fnmadd_v32hf_mask = 3199,
  CODE_FOR_avx512bw_fnmadd_v32hf_mask_round = 3200,
  CODE_FOR_avx512vl_fnmadd_v16hf_mask = 3201,
   CODE_FOR_avx512vl_fnmadd_v16hf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512fp16_fnmadd_v8hf_mask = 3202,
   CODE_FOR_avx512fp16_fnmadd_v8hf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fnmadd_v16sf_mask = 3203,
  CODE_FOR_avx512f_fnmadd_v16sf_mask_round = 3204,
  CODE_FOR_avx512vl_fnmadd_v8sf_mask = 3205,
   CODE_FOR_avx512vl_fnmadd_v8sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fnmadd_v4sf_mask = 3206,
   CODE_FOR_avx512vl_fnmadd_v4sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fnmadd_v8df_mask = 3207,
  CODE_FOR_avx512f_fnmadd_v8df_mask_round = 3208,
  CODE_FOR_avx512vl_fnmadd_v4df_mask = 3209,
   CODE_FOR_avx512vl_fnmadd_v4df_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fnmadd_v2df_mask = 3210,
   CODE_FOR_avx512vl_fnmadd_v2df_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512bw_fnmadd_v32hf_mask3 = 3211,
  CODE_FOR_avx512bw_fnmadd_v32hf_mask3_round = 3212,
  CODE_FOR_avx512vl_fnmadd_v16hf_mask3 = 3213,
   CODE_FOR_avx512vl_fnmadd_v16hf_mask3_round = CODE_FOR_nothing,
  CODE_FOR_avx512fp16_fnmadd_v8hf_mask3 = 3214,
   CODE_FOR_avx512fp16_fnmadd_v8hf_mask3_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fnmadd_v16sf_mask3 = 3215,
  CODE_FOR_avx512f_fnmadd_v16sf_mask3_round = 3216,
  CODE_FOR_avx512vl_fnmadd_v8sf_mask3 = 3217,
   CODE_FOR_avx512vl_fnmadd_v8sf_mask3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fnmadd_v4sf_mask3 = 3218,
   CODE_FOR_avx512vl_fnmadd_v4sf_mask3_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fnmadd_v8df_mask3 = 3219,
  CODE_FOR_avx512f_fnmadd_v8df_mask3_round = 3220,
  CODE_FOR_avx512vl_fnmadd_v4df_mask3 = 3221,
   CODE_FOR_avx512vl_fnmadd_v4df_mask3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fnmadd_v2df_mask3 = 3222,
   CODE_FOR_avx512vl_fnmadd_v2df_mask3_round = CODE_FOR_nothing,
  CODE_FOR_fma_fnmsub_v32hf_maskz_1 = 3244,
  CODE_FOR_fma_fnmsub_v32hf_maskz_1_round = 3246,
  CODE_FOR_fma_fnmsub_v16hf_maskz_1 = 3248,
   CODE_FOR_fma_fnmsub_v16hf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fnmsub_v8hf_maskz_1 = 3250,
   CODE_FOR_fma_fnmsub_v8hf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fnmsub_v16sf_maskz_1 = 3254,
  CODE_FOR_fma_fnmsub_v16sf_maskz_1_round = 3256,
  CODE_FOR_fma_fnmsub_v8sf_maskz_1 = 3258,
   CODE_FOR_fma_fnmsub_v8sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fnmsub_v4sf_maskz_1 = 3260,
   CODE_FOR_fma_fnmsub_v4sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fnmsub_v8df_maskz_1 = 3263,
  CODE_FOR_fma_fnmsub_v8df_maskz_1_round = 3265,
  CODE_FOR_fma_fnmsub_v4df_maskz_1 = 3267,
   CODE_FOR_fma_fnmsub_v4df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fnmsub_v2df_maskz_1 = 3269,
   CODE_FOR_fma_fnmsub_v2df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_avx512bw_fnmsub_v32hf_mask = 3270,
  CODE_FOR_avx512bw_fnmsub_v32hf_mask_round = 3271,
  CODE_FOR_avx512vl_fnmsub_v16hf_mask = 3272,
   CODE_FOR_avx512vl_fnmsub_v16hf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512fp16_fnmsub_v8hf_mask = 3273,
   CODE_FOR_avx512fp16_fnmsub_v8hf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fnmsub_v16sf_mask = 3274,
  CODE_FOR_avx512f_fnmsub_v16sf_mask_round = 3275,
  CODE_FOR_avx512vl_fnmsub_v8sf_mask = 3276,
   CODE_FOR_avx512vl_fnmsub_v8sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fnmsub_v4sf_mask = 3277,
   CODE_FOR_avx512vl_fnmsub_v4sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fnmsub_v8df_mask = 3278,
  CODE_FOR_avx512f_fnmsub_v8df_mask_round = 3279,
  CODE_FOR_avx512vl_fnmsub_v4df_mask = 3280,
   CODE_FOR_avx512vl_fnmsub_v4df_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fnmsub_v2df_mask = 3281,
   CODE_FOR_avx512vl_fnmsub_v2df_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512bw_fnmsub_v32hf_mask3 = 3282,
  CODE_FOR_avx512bw_fnmsub_v32hf_mask3_round = 3283,
  CODE_FOR_avx512vl_fnmsub_v16hf_mask3 = 3284,
  CODE_FOR_avx512vl_fnmsub_v16hf_mask3_round = 3285,
  CODE_FOR_avx512fp16_fnmsub_v8hf_mask3 = 3286,
  CODE_FOR_avx512fp16_fnmsub_v8hf_mask3_round = 3287,
  CODE_FOR_avx512f_fnmsub_v16sf_mask3 = 3288,
  CODE_FOR_avx512f_fnmsub_v16sf_mask3_round = 3289,
  CODE_FOR_avx512vl_fnmsub_v8sf_mask3 = 3290,
  CODE_FOR_avx512vl_fnmsub_v8sf_mask3_round = 3291,
  CODE_FOR_avx512vl_fnmsub_v4sf_mask3 = 3292,
  CODE_FOR_avx512vl_fnmsub_v4sf_mask3_round = 3293,
  CODE_FOR_avx512f_fnmsub_v8df_mask3 = 3294,
  CODE_FOR_avx512f_fnmsub_v8df_mask3_round = 3295,
  CODE_FOR_avx512vl_fnmsub_v4df_mask3 = 3296,
  CODE_FOR_avx512vl_fnmsub_v4df_mask3_round = 3297,
  CODE_FOR_avx512vl_fnmsub_v2df_mask3 = 3298,
  CODE_FOR_avx512vl_fnmsub_v2df_mask3_round = 3299,
  CODE_FOR_fma_fmaddsub_v32hf_maskz_1 = 3305,
  CODE_FOR_fma_fmaddsub_v32hf_maskz_1_round = 3307,
  CODE_FOR_fma_fmaddsub_v16hf_maskz_1 = 3309,
   CODE_FOR_fma_fmaddsub_v16hf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmaddsub_v8hf_maskz_1 = 3311,
   CODE_FOR_fma_fmaddsub_v8hf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmaddsub_v16sf_maskz_1 = 3315,
  CODE_FOR_fma_fmaddsub_v16sf_maskz_1_round = 3317,
  CODE_FOR_fma_fmaddsub_v8sf_maskz_1 = 3319,
   CODE_FOR_fma_fmaddsub_v8sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmaddsub_v4sf_maskz_1 = 3321,
   CODE_FOR_fma_fmaddsub_v4sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmaddsub_v8df_maskz_1 = 3324,
  CODE_FOR_fma_fmaddsub_v8df_maskz_1_round = 3326,
  CODE_FOR_fma_fmaddsub_v4df_maskz_1 = 3328,
   CODE_FOR_fma_fmaddsub_v4df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmaddsub_v2df_maskz_1 = 3330,
   CODE_FOR_fma_fmaddsub_v2df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_avx512bw_fmaddsub_v32hf_mask = 3331,
  CODE_FOR_avx512bw_fmaddsub_v32hf_mask_round = 3332,
  CODE_FOR_avx512vl_fmaddsub_v16hf_mask = 3333,
  CODE_FOR_avx512vl_fmaddsub_v16hf_mask_round = 3334,
  CODE_FOR_avx512fp16_fmaddsub_v8hf_mask = 3335,
  CODE_FOR_avx512fp16_fmaddsub_v8hf_mask_round = 3336,
  CODE_FOR_avx512f_fmaddsub_v16sf_mask = 3337,
  CODE_FOR_avx512f_fmaddsub_v16sf_mask_round = 3338,
  CODE_FOR_avx512vl_fmaddsub_v8sf_mask = 3339,
  CODE_FOR_avx512vl_fmaddsub_v8sf_mask_round = 3340,
  CODE_FOR_avx512vl_fmaddsub_v4sf_mask = 3341,
  CODE_FOR_avx512vl_fmaddsub_v4sf_mask_round = 3342,
  CODE_FOR_avx512f_fmaddsub_v8df_mask = 3343,
  CODE_FOR_avx512f_fmaddsub_v8df_mask_round = 3344,
  CODE_FOR_avx512vl_fmaddsub_v4df_mask = 3345,
  CODE_FOR_avx512vl_fmaddsub_v4df_mask_round = 3346,
  CODE_FOR_avx512vl_fmaddsub_v2df_mask = 3347,
  CODE_FOR_avx512vl_fmaddsub_v2df_mask_round = 3348,
  CODE_FOR_avx512bw_fmaddsub_v32hf_mask3 = 3349,
  CODE_FOR_avx512bw_fmaddsub_v32hf_mask3_round = 3350,
  CODE_FOR_avx512vl_fmaddsub_v16hf_mask3 = 3351,
  CODE_FOR_avx512vl_fmaddsub_v16hf_mask3_round = 3352,
  CODE_FOR_avx512fp16_fmaddsub_v8hf_mask3 = 3353,
  CODE_FOR_avx512fp16_fmaddsub_v8hf_mask3_round = 3354,
  CODE_FOR_avx512f_fmaddsub_v16sf_mask3 = 3355,
  CODE_FOR_avx512f_fmaddsub_v16sf_mask3_round = 3356,
  CODE_FOR_avx512vl_fmaddsub_v8sf_mask3 = 3357,
  CODE_FOR_avx512vl_fmaddsub_v8sf_mask3_round = 3358,
  CODE_FOR_avx512vl_fmaddsub_v4sf_mask3 = 3359,
  CODE_FOR_avx512vl_fmaddsub_v4sf_mask3_round = 3360,
  CODE_FOR_avx512f_fmaddsub_v8df_mask3 = 3361,
  CODE_FOR_avx512f_fmaddsub_v8df_mask3_round = 3362,
  CODE_FOR_avx512vl_fmaddsub_v4df_mask3 = 3363,
  CODE_FOR_avx512vl_fmaddsub_v4df_mask3_round = 3364,
  CODE_FOR_avx512vl_fmaddsub_v2df_mask3 = 3365,
  CODE_FOR_avx512vl_fmaddsub_v2df_mask3_round = 3366,
  CODE_FOR_fma_fmsubadd_v32hf_maskz_1 = 3372,
  CODE_FOR_fma_fmsubadd_v32hf_maskz_1_round = 3374,
  CODE_FOR_fma_fmsubadd_v16hf_maskz_1 = 3376,
   CODE_FOR_fma_fmsubadd_v16hf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmsubadd_v8hf_maskz_1 = 3378,
   CODE_FOR_fma_fmsubadd_v8hf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmsubadd_v16sf_maskz_1 = 3382,
  CODE_FOR_fma_fmsubadd_v16sf_maskz_1_round = 3384,
  CODE_FOR_fma_fmsubadd_v8sf_maskz_1 = 3386,
   CODE_FOR_fma_fmsubadd_v8sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmsubadd_v4sf_maskz_1 = 3388,
   CODE_FOR_fma_fmsubadd_v4sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmsubadd_v8df_maskz_1 = 3391,
  CODE_FOR_fma_fmsubadd_v8df_maskz_1_round = 3393,
  CODE_FOR_fma_fmsubadd_v4df_maskz_1 = 3395,
   CODE_FOR_fma_fmsubadd_v4df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmsubadd_v2df_maskz_1 = 3397,
   CODE_FOR_fma_fmsubadd_v2df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_avx512bw_fmsubadd_v32hf_mask = 3398,
  CODE_FOR_avx512bw_fmsubadd_v32hf_mask_round = 3399,
  CODE_FOR_avx512vl_fmsubadd_v16hf_mask = 3400,
  CODE_FOR_avx512vl_fmsubadd_v16hf_mask_round = 3401,
  CODE_FOR_avx512fp16_fmsubadd_v8hf_mask = 3402,
  CODE_FOR_avx512fp16_fmsubadd_v8hf_mask_round = 3403,
  CODE_FOR_avx512f_fmsubadd_v16sf_mask = 3404,
  CODE_FOR_avx512f_fmsubadd_v16sf_mask_round = 3405,
  CODE_FOR_avx512vl_fmsubadd_v8sf_mask = 3406,
  CODE_FOR_avx512vl_fmsubadd_v8sf_mask_round = 3407,
  CODE_FOR_avx512vl_fmsubadd_v4sf_mask = 3408,
  CODE_FOR_avx512vl_fmsubadd_v4sf_mask_round = 3409,
  CODE_FOR_avx512f_fmsubadd_v8df_mask = 3410,
  CODE_FOR_avx512f_fmsubadd_v8df_mask_round = 3411,
  CODE_FOR_avx512vl_fmsubadd_v4df_mask = 3412,
  CODE_FOR_avx512vl_fmsubadd_v4df_mask_round = 3413,
  CODE_FOR_avx512vl_fmsubadd_v2df_mask = 3414,
  CODE_FOR_avx512vl_fmsubadd_v2df_mask_round = 3415,
  CODE_FOR_avx512bw_fmsubadd_v32hf_mask3 = 3416,
  CODE_FOR_avx512bw_fmsubadd_v32hf_mask3_round = 3417,
  CODE_FOR_avx512vl_fmsubadd_v16hf_mask3 = 3418,
  CODE_FOR_avx512vl_fmsubadd_v16hf_mask3_round = 3419,
  CODE_FOR_avx512fp16_fmsubadd_v8hf_mask3 = 3420,
  CODE_FOR_avx512fp16_fmsubadd_v8hf_mask3_round = 3421,
  CODE_FOR_avx512f_fmsubadd_v16sf_mask3 = 3422,
  CODE_FOR_avx512f_fmsubadd_v16sf_mask3_round = 3423,
  CODE_FOR_avx512vl_fmsubadd_v8sf_mask3 = 3424,
  CODE_FOR_avx512vl_fmsubadd_v8sf_mask3_round = 3425,
  CODE_FOR_avx512vl_fmsubadd_v4sf_mask3 = 3426,
  CODE_FOR_avx512vl_fmsubadd_v4sf_mask3_round = 3427,
  CODE_FOR_avx512f_fmsubadd_v8df_mask3 = 3428,
  CODE_FOR_avx512f_fmsubadd_v8df_mask3_round = 3429,
  CODE_FOR_avx512vl_fmsubadd_v4df_mask3 = 3430,
  CODE_FOR_avx512vl_fmsubadd_v4df_mask3_round = 3431,
  CODE_FOR_avx512vl_fmsubadd_v2df_mask3 = 3432,
  CODE_FOR_avx512vl_fmsubadd_v2df_mask3_round = 3433,
  CODE_FOR_avx512f_vmfmadd_v8hf_mask = 3458,
  CODE_FOR_avx512f_vmfmadd_v8hf_mask_round = 3459,
  CODE_FOR_avx512f_vmfmadd_v4sf_mask = 3460,
  CODE_FOR_avx512f_vmfmadd_v4sf_mask_round = 3461,
  CODE_FOR_avx512f_vmfmadd_v2df_mask = 3462,
  CODE_FOR_avx512f_vmfmadd_v2df_mask_round = 3463,
  CODE_FOR_avx512f_vmfmadd_v8hf_mask3 = 3464,
  CODE_FOR_avx512f_vmfmadd_v8hf_mask3_round = 3465,
  CODE_FOR_avx512f_vmfmadd_v4sf_mask3 = 3466,
  CODE_FOR_avx512f_vmfmadd_v4sf_mask3_round = 3467,
  CODE_FOR_avx512f_vmfmadd_v2df_mask3 = 3468,
  CODE_FOR_avx512f_vmfmadd_v2df_mask3_round = 3469,
  CODE_FOR_avx512f_vmfmadd_v8hf_maskz_1 = 3470,
  CODE_FOR_avx512f_vmfmadd_v8hf_maskz_1_round = 3471,
  CODE_FOR_avx512f_vmfmadd_v4sf_maskz_1 = 3472,
  CODE_FOR_avx512f_vmfmadd_v4sf_maskz_1_round = 3473,
  CODE_FOR_avx512f_vmfmadd_v2df_maskz_1 = 3474,
  CODE_FOR_avx512f_vmfmadd_v2df_maskz_1_round = 3475,
  CODE_FOR_avx512f_vmfmsub_v8hf_mask3 = 3482,
  CODE_FOR_avx512f_vmfmsub_v8hf_mask3_round = 3483,
  CODE_FOR_avx512f_vmfmsub_v4sf_mask3 = 3484,
  CODE_FOR_avx512f_vmfmsub_v4sf_mask3_round = 3485,
  CODE_FOR_avx512f_vmfmsub_v2df_mask3 = 3486,
  CODE_FOR_avx512f_vmfmsub_v2df_mask3_round = 3487,
  CODE_FOR_avx512f_vmfnmadd_v8hf_mask = 3494,
  CODE_FOR_avx512f_vmfnmadd_v8hf_mask_round = 3495,
  CODE_FOR_avx512f_vmfnmadd_v4sf_mask = 3496,
  CODE_FOR_avx512f_vmfnmadd_v4sf_mask_round = 3497,
  CODE_FOR_avx512f_vmfnmadd_v2df_mask = 3498,
  CODE_FOR_avx512f_vmfnmadd_v2df_mask_round = 3499,
  CODE_FOR_avx512f_vmfnmadd_v8hf_mask3 = 3500,
  CODE_FOR_avx512f_vmfnmadd_v8hf_mask3_round = 3501,
  CODE_FOR_avx512f_vmfnmadd_v4sf_mask3 = 3502,
  CODE_FOR_avx512f_vmfnmadd_v4sf_mask3_round = 3503,
  CODE_FOR_avx512f_vmfnmadd_v2df_mask3 = 3504,
  CODE_FOR_avx512f_vmfnmadd_v2df_mask3_round = 3505,
  CODE_FOR_avx512f_vmfnmadd_v8hf_maskz_1 = 3506,
  CODE_FOR_avx512f_vmfnmadd_v8hf_maskz_1_round = 3507,
  CODE_FOR_avx512f_vmfnmadd_v4sf_maskz_1 = 3508,
  CODE_FOR_avx512f_vmfnmadd_v4sf_maskz_1_round = 3509,
  CODE_FOR_avx512f_vmfnmadd_v2df_maskz_1 = 3510,
  CODE_FOR_avx512f_vmfnmadd_v2df_maskz_1_round = 3511,
  CODE_FOR_fma_fmaddc_v32hf = 3538,
  CODE_FOR_fma_fmaddc_v32hf_round = 3539,
  CODE_FOR_fma_fmaddc_v32hf_maskz_1 = 3540,
  CODE_FOR_fma_fmaddc_v32hf_maskz_1_round = 3541,
  CODE_FOR_fma_fcmaddc_v32hf = 3542,
  CODE_FOR_fma_fcmaddc_v32hf_round = 3543,
  CODE_FOR_fma_fcmaddc_v32hf_maskz_1 = 3544,
  CODE_FOR_fma_fcmaddc_v32hf_maskz_1_round = 3545,
  CODE_FOR_fma_fmaddc_v16hf = 3546,
   CODE_FOR_fma_fmaddc_v16hf_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmaddc_v16hf_maskz_1 = 3547,
   CODE_FOR_fma_fmaddc_v16hf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fcmaddc_v16hf = 3548,
   CODE_FOR_fma_fcmaddc_v16hf_round = CODE_FOR_nothing,
  CODE_FOR_fma_fcmaddc_v16hf_maskz_1 = 3549,
   CODE_FOR_fma_fcmaddc_v16hf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmaddc_v8hf = 3550,
   CODE_FOR_fma_fmaddc_v8hf_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmaddc_v8hf_maskz_1 = 3551,
   CODE_FOR_fma_fmaddc_v8hf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fcmaddc_v8hf = 3552,
   CODE_FOR_fma_fcmaddc_v8hf_round = CODE_FOR_nothing,
  CODE_FOR_fma_fcmaddc_v8hf_maskz_1 = 3553,
   CODE_FOR_fma_fcmaddc_v8hf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_v32hf_fadd_fmul = 3554,
  CODE_FOR_fma_v16hf_fadd_fmul = 3555,
  CODE_FOR_fma_v8hf_fadd_fmul = 3556,
  CODE_FOR_fma_v32hf_fadd_fcmul = 3557,
  CODE_FOR_fma_v16hf_fadd_fcmul = 3558,
  CODE_FOR_fma_v8hf_fadd_fcmul = 3559,
  CODE_FOR_fma_fmaddc_v32hf_fma_zero = 3560,
  CODE_FOR_fma_fcmaddc_v32hf_fma_zero = 3561,
  CODE_FOR_fma_fmaddc_v16hf_fma_zero = 3562,
  CODE_FOR_fma_fcmaddc_v16hf_fma_zero = 3563,
  CODE_FOR_fma_fmaddc_v8hf_fma_zero = 3564,
  CODE_FOR_fma_fcmaddc_v8hf_fma_zero = 3565,
  CODE_FOR_fma_fmaddc_v16sf_pair = 3566,
  CODE_FOR_fma_fcmaddc_v16sf_pair = 3567,
  CODE_FOR_fma_fmaddc_v8sf_pair = 3568,
  CODE_FOR_fma_fcmaddc_v8sf_pair = 3569,
  CODE_FOR_fma_fmaddc_v4sf_pair = 3570,
  CODE_FOR_fma_fcmaddc_v4sf_pair = 3571,
  CODE_FOR_fma_v32hf_fmaddc_bcst = 3572,
  CODE_FOR_fma_v16hf_fmaddc_bcst = 3573,
  CODE_FOR_fma_v8hf_fmaddc_bcst = 3574,
  CODE_FOR_fma_v32hf_fcmaddc_bcst = 3575,
  CODE_FOR_fma_v16hf_fcmaddc_bcst = 3576,
  CODE_FOR_fma_v8hf_fcmaddc_bcst = 3577,
  CODE_FOR_avx512bw_fmaddc_v32hf_mask = 3578,
  CODE_FOR_avx512bw_fmaddc_v32hf_mask_round = 3579,
  CODE_FOR_avx512bw_fcmaddc_v32hf_mask = 3580,
  CODE_FOR_avx512bw_fcmaddc_v32hf_mask_round = 3581,
  CODE_FOR_avx512vl_fmaddc_v16hf_mask = 3582,
   CODE_FOR_avx512vl_fmaddc_v16hf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fcmaddc_v16hf_mask = 3583,
   CODE_FOR_avx512vl_fcmaddc_v16hf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512fp16_fmaddc_v8hf_mask = 3584,
   CODE_FOR_avx512fp16_fmaddc_v8hf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512fp16_fcmaddc_v8hf_mask = 3585,
   CODE_FOR_avx512fp16_fcmaddc_v8hf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512bw_fmulc_v32hf = 3586,
  CODE_FOR_avx512bw_fmulc_v32hf_mask = 3587,
  CODE_FOR_avx512bw_fmulc_v32hf_round = 3588,
  CODE_FOR_avx512bw_fmulc_v32hf_mask_round = 3589,
  CODE_FOR_avx512bw_fcmulc_v32hf = 3590,
  CODE_FOR_avx512bw_fcmulc_v32hf_mask = 3591,
  CODE_FOR_avx512bw_fcmulc_v32hf_round = 3592,
  CODE_FOR_avx512bw_fcmulc_v32hf_mask_round = 3593,
  CODE_FOR_avx512vl_fmulc_v16hf = 3594,
  CODE_FOR_avx512vl_fmulc_v16hf_mask = 3595,
   CODE_FOR_avx512vl_fmulc_v16hf_round = CODE_FOR_nothing,
   CODE_FOR_avx512vl_fmulc_v16hf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fcmulc_v16hf = 3596,
  CODE_FOR_avx512vl_fcmulc_v16hf_mask = 3597,
   CODE_FOR_avx512vl_fcmulc_v16hf_round = CODE_FOR_nothing,
   CODE_FOR_avx512vl_fcmulc_v16hf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512fp16_fmulc_v8hf = 3598,
  CODE_FOR_avx512fp16_fmulc_v8hf_mask = 3599,
   CODE_FOR_avx512fp16_fmulc_v8hf_round = CODE_FOR_nothing,
   CODE_FOR_avx512fp16_fmulc_v8hf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512fp16_fcmulc_v8hf = 3600,
  CODE_FOR_avx512fp16_fcmulc_v8hf_mask = 3601,
   CODE_FOR_avx512fp16_fcmulc_v8hf_round = CODE_FOR_nothing,
   CODE_FOR_avx512fp16_fcmulc_v8hf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512fp16_fma_fmaddcsh_v8hf = 3602,
  CODE_FOR_avx512fp16_fma_fmaddcsh_v8hf_maskz = 3603,
  CODE_FOR_avx512fp16_fma_fmaddcsh_v8hf_round = 3604,
  CODE_FOR_avx512fp16_fma_fmaddcsh_v8hf_maskz_round = 3605,
  CODE_FOR_avx512fp16_fma_fcmaddcsh_v8hf = 3606,
  CODE_FOR_avx512fp16_fma_fcmaddcsh_v8hf_maskz = 3607,
  CODE_FOR_avx512fp16_fma_fcmaddcsh_v8hf_round = 3608,
  CODE_FOR_avx512fp16_fma_fcmaddcsh_v8hf_maskz_round = 3609,
  CODE_FOR_avx512fp16_fmaddcsh_v8hf_mask = 3610,
  CODE_FOR_avx512fp16_fmaddcsh_v8hf_mask_round = 3611,
  CODE_FOR_avx512fp16_fcmaddcsh_v8hf_mask = 3612,
  CODE_FOR_avx512fp16_fcmaddcsh_v8hf_mask_round = 3613,
  CODE_FOR_avx512fp16_fmulcsh_v8hf = 3614,
  CODE_FOR_avx512fp16_fmulcsh_v8hf_round = 3615,
  CODE_FOR_avx512fp16_fmulcsh_v8hf_mask = 3616,
  CODE_FOR_avx512fp16_fmulcsh_v8hf_mask_round = 3617,
  CODE_FOR_avx512fp16_fcmulcsh_v8hf = 3618,
  CODE_FOR_avx512fp16_fcmulcsh_v8hf_round = 3619,
  CODE_FOR_avx512fp16_fcmulcsh_v8hf_mask = 3620,
  CODE_FOR_avx512fp16_fcmulcsh_v8hf_mask_round = 3621,
  CODE_FOR_avx512fp16_vcvtph2uw_v32hi = 3622,
  CODE_FOR_avx512fp16_vcvtph2uw_v32hi_round = 3623,
  CODE_FOR_avx512fp16_vcvtph2uw_v32hi_mask = 3624,
  CODE_FOR_avx512fp16_vcvtph2uw_v32hi_mask_round = 3625,
  CODE_FOR_avx512fp16_vcvtph2w_v32hi = 3626,
  CODE_FOR_avx512fp16_vcvtph2w_v32hi_round = 3627,
  CODE_FOR_avx512fp16_vcvtph2w_v32hi_mask = 3628,
  CODE_FOR_avx512fp16_vcvtph2w_v32hi_mask_round = 3629,
  CODE_FOR_avx512fp16_vcvtph2udq_v16si = 3630,
  CODE_FOR_avx512fp16_vcvtph2udq_v16si_round = 3631,
  CODE_FOR_avx512fp16_vcvtph2udq_v16si_mask = 3632,
  CODE_FOR_avx512fp16_vcvtph2udq_v16si_mask_round = 3633,
  CODE_FOR_avx512fp16_vcvtph2dq_v16si = 3634,
  CODE_FOR_avx512fp16_vcvtph2dq_v16si_round = 3635,
  CODE_FOR_avx512fp16_vcvtph2dq_v16si_mask = 3636,
  CODE_FOR_avx512fp16_vcvtph2dq_v16si_mask_round = 3637,
  CODE_FOR_avx512fp16_vcvtph2uqq_v8di = 3638,
  CODE_FOR_avx512fp16_vcvtph2uqq_v8di_round = 3639,
  CODE_FOR_avx512fp16_vcvtph2uqq_v8di_mask = 3640,
  CODE_FOR_avx512fp16_vcvtph2uqq_v8di_mask_round = 3641,
  CODE_FOR_avx512fp16_vcvtph2qq_v8di = 3642,
  CODE_FOR_avx512fp16_vcvtph2qq_v8di_round = 3643,
  CODE_FOR_avx512fp16_vcvtph2qq_v8di_mask = 3644,
  CODE_FOR_avx512fp16_vcvtph2qq_v8di_mask_round = 3645,
  CODE_FOR_avx512fp16_vcvtph2uw_v16hi = 3646,
  CODE_FOR_avx512fp16_vcvtph2uw_v16hi_round = 3647,
  CODE_FOR_avx512fp16_vcvtph2uw_v16hi_mask = 3648,
  CODE_FOR_avx512fp16_vcvtph2uw_v16hi_mask_round = 3649,
  CODE_FOR_avx512fp16_vcvtph2w_v16hi = 3650,
  CODE_FOR_avx512fp16_vcvtph2w_v16hi_round = 3651,
  CODE_FOR_avx512fp16_vcvtph2w_v16hi_mask = 3652,
  CODE_FOR_avx512fp16_vcvtph2w_v16hi_mask_round = 3653,
  CODE_FOR_avx512fp16_vcvtph2udq_v8si = 3654,
  CODE_FOR_avx512fp16_vcvtph2udq_v8si_round = 3655,
  CODE_FOR_avx512fp16_vcvtph2udq_v8si_mask = 3656,
  CODE_FOR_avx512fp16_vcvtph2udq_v8si_mask_round = 3657,
  CODE_FOR_avx512fp16_vcvtph2dq_v8si = 3658,
  CODE_FOR_avx512fp16_vcvtph2dq_v8si_round = 3659,
  CODE_FOR_avx512fp16_vcvtph2dq_v8si_mask = 3660,
  CODE_FOR_avx512fp16_vcvtph2dq_v8si_mask_round = 3661,
  CODE_FOR_avx512fp16_vcvtph2uqq_v4di = 3662,
  CODE_FOR_avx512fp16_vcvtph2uqq_v4di_round = 3663,
  CODE_FOR_avx512fp16_vcvtph2uqq_v4di_mask = 3664,
  CODE_FOR_avx512fp16_vcvtph2uqq_v4di_mask_round = 3665,
  CODE_FOR_avx512fp16_vcvtph2qq_v4di = 3666,
  CODE_FOR_avx512fp16_vcvtph2qq_v4di_round = 3667,
  CODE_FOR_avx512fp16_vcvtph2qq_v4di_mask = 3668,
  CODE_FOR_avx512fp16_vcvtph2qq_v4di_mask_round = 3669,
  CODE_FOR_avx512fp16_vcvtph2uw_v8hi = 3670,
  CODE_FOR_avx512fp16_vcvtph2uw_v8hi_round = 3671,
  CODE_FOR_avx512fp16_vcvtph2uw_v8hi_mask = 3672,
  CODE_FOR_avx512fp16_vcvtph2uw_v8hi_mask_round = 3673,
  CODE_FOR_avx512fp16_vcvtph2w_v8hi = 3674,
  CODE_FOR_avx512fp16_vcvtph2w_v8hi_round = 3675,
  CODE_FOR_avx512fp16_vcvtph2w_v8hi_mask = 3676,
  CODE_FOR_avx512fp16_vcvtph2w_v8hi_mask_round = 3677,
  CODE_FOR_avx512fp16_vcvtph2udq_v4si = 3678,
  CODE_FOR_avx512fp16_vcvtph2udq_v4si_round = 3679,
  CODE_FOR_avx512fp16_vcvtph2udq_v4si_mask = 3680,
  CODE_FOR_avx512fp16_vcvtph2udq_v4si_mask_round = 3681,
  CODE_FOR_avx512fp16_vcvtph2dq_v4si = 3682,
  CODE_FOR_avx512fp16_vcvtph2dq_v4si_round = 3683,
  CODE_FOR_avx512fp16_vcvtph2dq_v4si_mask = 3684,
  CODE_FOR_avx512fp16_vcvtph2dq_v4si_mask_round = 3685,
  CODE_FOR_avx512fp16_vcvtph2uqq_v2di = 3686,
  CODE_FOR_avx512fp16_vcvtph2uqq_v2di_round = 3687,
  CODE_FOR_avx512fp16_vcvtph2uqq_v2di_mask = 3688,
  CODE_FOR_avx512fp16_vcvtph2uqq_v2di_mask_round = 3689,
  CODE_FOR_avx512fp16_vcvtph2qq_v2di = 3690,
  CODE_FOR_avx512fp16_vcvtph2qq_v2di_round = 3691,
  CODE_FOR_avx512fp16_vcvtph2qq_v2di_mask = 3692,
  CODE_FOR_avx512fp16_vcvtph2qq_v2di_mask_round = 3693,
  CODE_FOR_avx512fp16_vcvtw2ph_v8hi = 3694,
  CODE_FOR_avx512fp16_vcvtw2ph_v8hi_round = 3695,
  CODE_FOR_avx512fp16_vcvtw2ph_v8hi_mask = 3696,
  CODE_FOR_avx512fp16_vcvtw2ph_v8hi_mask_round = 3697,
  CODE_FOR_avx512fp16_vcvtuw2ph_v8hi = 3698,
  CODE_FOR_avx512fp16_vcvtuw2ph_v8hi_round = 3699,
  CODE_FOR_avx512fp16_vcvtuw2ph_v8hi_mask = 3700,
  CODE_FOR_avx512fp16_vcvtuw2ph_v8hi_mask_round = 3701,
  CODE_FOR_avx512fp16_vcvtw2ph_v16hi = 3702,
  CODE_FOR_avx512fp16_vcvtw2ph_v16hi_round = 3703,
  CODE_FOR_avx512fp16_vcvtw2ph_v16hi_mask = 3704,
  CODE_FOR_avx512fp16_vcvtw2ph_v16hi_mask_round = 3705,
  CODE_FOR_avx512fp16_vcvtuw2ph_v16hi = 3706,
  CODE_FOR_avx512fp16_vcvtuw2ph_v16hi_round = 3707,
  CODE_FOR_avx512fp16_vcvtuw2ph_v16hi_mask = 3708,
  CODE_FOR_avx512fp16_vcvtuw2ph_v16hi_mask_round = 3709,
  CODE_FOR_avx512fp16_vcvtw2ph_v32hi = 3710,
  CODE_FOR_avx512fp16_vcvtw2ph_v32hi_round = 3711,
  CODE_FOR_avx512fp16_vcvtw2ph_v32hi_mask = 3712,
  CODE_FOR_avx512fp16_vcvtw2ph_v32hi_mask_round = 3713,
  CODE_FOR_avx512fp16_vcvtuw2ph_v32hi = 3714,
  CODE_FOR_avx512fp16_vcvtuw2ph_v32hi_round = 3715,
  CODE_FOR_avx512fp16_vcvtuw2ph_v32hi_mask = 3716,
  CODE_FOR_avx512fp16_vcvtuw2ph_v32hi_mask_round = 3717,
  CODE_FOR_avx512fp16_vcvtdq2ph_v8si = 3718,
  CODE_FOR_avx512fp16_vcvtdq2ph_v8si_round = 3719,
  CODE_FOR_avx512fp16_vcvtdq2ph_v8si_mask = 3720,
  CODE_FOR_avx512fp16_vcvtdq2ph_v8si_mask_round = 3721,
  CODE_FOR_avx512fp16_vcvtudq2ph_v8si = 3722,
  CODE_FOR_avx512fp16_vcvtudq2ph_v8si_round = 3723,
  CODE_FOR_avx512fp16_vcvtudq2ph_v8si_mask = 3724,
  CODE_FOR_avx512fp16_vcvtudq2ph_v8si_mask_round = 3725,
  CODE_FOR_avx512fp16_vcvtdq2ph_v16si = 3726,
  CODE_FOR_avx512fp16_vcvtdq2ph_v16si_round = 3727,
  CODE_FOR_avx512fp16_vcvtdq2ph_v16si_mask = 3728,
  CODE_FOR_avx512fp16_vcvtdq2ph_v16si_mask_round = 3729,
  CODE_FOR_avx512fp16_vcvtudq2ph_v16si = 3730,
  CODE_FOR_avx512fp16_vcvtudq2ph_v16si_round = 3731,
  CODE_FOR_avx512fp16_vcvtudq2ph_v16si_mask = 3732,
  CODE_FOR_avx512fp16_vcvtudq2ph_v16si_mask_round = 3733,
  CODE_FOR_avx512fp16_vcvtqq2ph_v8di = 3734,
  CODE_FOR_avx512fp16_vcvtqq2ph_v8di_round = 3735,
  CODE_FOR_avx512fp16_vcvtqq2ph_v8di_mask = 3736,
  CODE_FOR_avx512fp16_vcvtqq2ph_v8di_mask_round = 3737,
  CODE_FOR_avx512fp16_vcvtuqq2ph_v8di = 3738,
  CODE_FOR_avx512fp16_vcvtuqq2ph_v8di_round = 3739,
  CODE_FOR_avx512fp16_vcvtuqq2ph_v8di_mask = 3740,
  CODE_FOR_avx512fp16_vcvtuqq2ph_v8di_mask_round = 3741,
  CODE_FOR_avx512fp16_vcvtsh2usi = 3760,
  CODE_FOR_avx512fp16_vcvtsh2usi_round = 3761,
  CODE_FOR_avx512fp16_vcvtsh2si = 3762,
  CODE_FOR_avx512fp16_vcvtsh2si_round = 3763,
  CODE_FOR_avx512fp16_vcvtsh2usiq = 3764,
  CODE_FOR_avx512fp16_vcvtsh2usiq_round = 3765,
  CODE_FOR_avx512fp16_vcvtsh2siq = 3766,
  CODE_FOR_avx512fp16_vcvtsh2siq_round = 3767,
  CODE_FOR_avx512fp16_vcvtsh2usi_2 = 3768,
  CODE_FOR_avx512fp16_vcvtsh2si_2 = 3769,
  CODE_FOR_avx512fp16_vcvtsh2usiq_2 = 3770,
  CODE_FOR_avx512fp16_vcvtsh2siq_2 = 3771,
  CODE_FOR_avx512fp16_vcvtsi2sh = 3772,
  CODE_FOR_avx512fp16_vcvtsi2sh_round = 3773,
  CODE_FOR_avx512fp16_vcvtusi2sh = 3774,
  CODE_FOR_avx512fp16_vcvtusi2sh_round = 3775,
  CODE_FOR_avx512fp16_vcvtsi2shq = 3776,
  CODE_FOR_avx512fp16_vcvtsi2shq_round = 3777,
  CODE_FOR_avx512fp16_vcvtusi2shq = 3778,
  CODE_FOR_avx512fp16_vcvtusi2shq_round = 3779,
  CODE_FOR_avx512fp16_fix_truncv8hi2 = 3780,
  CODE_FOR_avx512fp16_fix_truncv8hi2_round = 3781,
  CODE_FOR_avx512fp16_fix_truncv8hi2_mask = 3782,
  CODE_FOR_avx512fp16_fix_truncv8hi2_mask_round = 3783,
  CODE_FOR_avx512fp16_fixuns_truncv8hi2 = 3784,
  CODE_FOR_avx512fp16_fixuns_truncv8hi2_round = 3785,
  CODE_FOR_avx512fp16_fixuns_truncv8hi2_mask = 3786,
  CODE_FOR_avx512fp16_fixuns_truncv8hi2_mask_round = 3787,
  CODE_FOR_avx512fp16_fix_truncv16hi2 = 3788,
  CODE_FOR_avx512fp16_fix_truncv16hi2_round = 3789,
  CODE_FOR_avx512fp16_fix_truncv16hi2_mask = 3790,
  CODE_FOR_avx512fp16_fix_truncv16hi2_mask_round = 3791,
  CODE_FOR_avx512fp16_fixuns_truncv16hi2 = 3792,
  CODE_FOR_avx512fp16_fixuns_truncv16hi2_round = 3793,
  CODE_FOR_avx512fp16_fixuns_truncv16hi2_mask = 3794,
  CODE_FOR_avx512fp16_fixuns_truncv16hi2_mask_round = 3795,
  CODE_FOR_avx512fp16_fix_truncv32hi2 = 3796,
  CODE_FOR_avx512fp16_fix_truncv32hi2_round = 3797,
  CODE_FOR_avx512fp16_fix_truncv32hi2_mask = 3798,
  CODE_FOR_avx512fp16_fix_truncv32hi2_mask_round = 3799,
  CODE_FOR_avx512fp16_fixuns_truncv32hi2 = 3800,
  CODE_FOR_avx512fp16_fixuns_truncv32hi2_round = 3801,
  CODE_FOR_avx512fp16_fixuns_truncv32hi2_mask = 3802,
  CODE_FOR_avx512fp16_fixuns_truncv32hi2_mask_round = 3803,
  CODE_FOR_avx512fp16_fix_truncv8si2 = 3804,
  CODE_FOR_avx512fp16_fix_truncv8si2_round = 3805,
  CODE_FOR_avx512fp16_fix_truncv8si2_mask = 3806,
  CODE_FOR_avx512fp16_fix_truncv8si2_mask_round = 3807,
  CODE_FOR_avx512fp16_fixuns_truncv8si2 = 3808,
  CODE_FOR_avx512fp16_fixuns_truncv8si2_round = 3809,
  CODE_FOR_avx512fp16_fixuns_truncv8si2_mask = 3810,
  CODE_FOR_avx512fp16_fixuns_truncv8si2_mask_round = 3811,
  CODE_FOR_avx512fp16_fix_truncv16si2 = 3812,
  CODE_FOR_avx512fp16_fix_truncv16si2_round = 3813,
  CODE_FOR_avx512fp16_fix_truncv16si2_mask = 3814,
  CODE_FOR_avx512fp16_fix_truncv16si2_mask_round = 3815,
  CODE_FOR_avx512fp16_fixuns_truncv16si2 = 3816,
  CODE_FOR_avx512fp16_fixuns_truncv16si2_round = 3817,
  CODE_FOR_avx512fp16_fixuns_truncv16si2_mask = 3818,
  CODE_FOR_avx512fp16_fixuns_truncv16si2_mask_round = 3819,
  CODE_FOR_avx512fp16_fix_truncv8di2 = 3820,
  CODE_FOR_avx512fp16_fix_truncv8di2_round = 3821,
  CODE_FOR_avx512fp16_fix_truncv8di2_mask = 3822,
  CODE_FOR_avx512fp16_fix_truncv8di2_mask_round = 3823,
  CODE_FOR_avx512fp16_fixuns_truncv8di2 = 3824,
  CODE_FOR_avx512fp16_fixuns_truncv8di2_round = 3825,
  CODE_FOR_avx512fp16_fixuns_truncv8di2_mask = 3826,
  CODE_FOR_avx512fp16_fixuns_truncv8di2_mask_round = 3827,
  CODE_FOR_avx512fp16_fix_truncv4si2 = 3828,
  CODE_FOR_avx512fp16_fix_truncv4si2_mask = 3829,
  CODE_FOR_avx512fp16_fixuns_truncv4si2 = 3830,
  CODE_FOR_avx512fp16_fixuns_truncv4si2_mask = 3831,
  CODE_FOR_avx512fp16_fix_truncv4di2 = 3832,
  CODE_FOR_avx512fp16_fix_truncv4di2_mask = 3833,
  CODE_FOR_avx512fp16_fixuns_truncv4di2 = 3834,
  CODE_FOR_avx512fp16_fixuns_truncv4di2_mask = 3835,
  CODE_FOR_avx512fp16_fix_truncv2di2 = 3844,
  CODE_FOR_avx512fp16_fix_truncv2di2_mask = 3845,
  CODE_FOR_avx512fp16_fixuns_truncv2di2 = 3846,
  CODE_FOR_avx512fp16_fixuns_truncv2di2_mask = 3847,
  CODE_FOR_avx512fp16_fix_truncsi2 = 3852,
  CODE_FOR_avx512fp16_fix_truncsi2_round = 3853,
  CODE_FOR_avx512fp16_fixuns_truncsi2 = 3854,
  CODE_FOR_avx512fp16_fixuns_truncsi2_round = 3855,
  CODE_FOR_avx512fp16_fix_truncdi2 = 3856,
  CODE_FOR_avx512fp16_fix_truncdi2_round = 3857,
  CODE_FOR_avx512fp16_fixuns_truncdi2 = 3858,
  CODE_FOR_avx512fp16_fixuns_truncdi2_round = 3859,
  CODE_FOR_avx512fp16_fix_truncsi2_mem = 3860,
  CODE_FOR_avx512fp16_fixuns_truncsi2_mem = 3861,
  CODE_FOR_avx512fp16_fix_truncdi2_mem = 3862,
  CODE_FOR_avx512fp16_fixuns_truncdi2_mem = 3863,
  CODE_FOR_avx512fp16_float_extend_phv8df2 = 3864,
  CODE_FOR_avx512fp16_float_extend_phv8df2_round = 3865,
  CODE_FOR_avx512fp16_float_extend_phv8df2_mask = 3866,
  CODE_FOR_avx512fp16_float_extend_phv8df2_mask_round = 3867,
  CODE_FOR_avx512fp16_float_extend_phv16sf2 = 3868,
  CODE_FOR_avx512fp16_float_extend_phv16sf2_round = 3869,
  CODE_FOR_avx512fp16_float_extend_phv16sf2_mask = 3870,
  CODE_FOR_avx512fp16_float_extend_phv16sf2_mask_round = 3871,
  CODE_FOR_avx512fp16_float_extend_phv8sf2 = 3872,
  CODE_FOR_avx512fp16_float_extend_phv8sf2_round = 3873,
  CODE_FOR_avx512fp16_float_extend_phv8sf2_mask = 3874,
  CODE_FOR_avx512fp16_float_extend_phv8sf2_mask_round = 3875,
  CODE_FOR_avx512fp16_float_extend_phv4df2 = 3876,
  CODE_FOR_avx512fp16_float_extend_phv4df2_mask = 3877,
  CODE_FOR_avx512fp16_float_extend_phv4sf2 = 3878,
  CODE_FOR_avx512fp16_float_extend_phv4sf2_mask = 3879,
  CODE_FOR_avx512fp16_float_extend_phv2df2 = 3884,
  CODE_FOR_avx512fp16_float_extend_phv2df2_mask = 3885,
  CODE_FOR_avx512fp16_vcvtpd2ph_v8df = 3888,
  CODE_FOR_avx512fp16_vcvtpd2ph_v8df_round = 3889,
  CODE_FOR_avx512fp16_vcvtpd2ph_v8df_mask = 3890,
  CODE_FOR_avx512fp16_vcvtpd2ph_v8df_mask_round = 3891,
  CODE_FOR_avx512fp16_vcvtps2ph_v16sf = 3892,
  CODE_FOR_avx512fp16_vcvtps2ph_v16sf_round = 3893,
  CODE_FOR_avx512fp16_vcvtps2ph_v16sf_mask = 3894,
  CODE_FOR_avx512fp16_vcvtps2ph_v16sf_mask_round = 3895,
  CODE_FOR_avx512fp16_vcvtps2ph_v8sf = 3896,
  CODE_FOR_avx512fp16_vcvtps2ph_v8sf_round = 3897,
  CODE_FOR_avx512fp16_vcvtps2ph_v8sf_mask = 3898,
  CODE_FOR_avx512fp16_vcvtps2ph_v8sf_mask_round = 3899,
  CODE_FOR_avx512fp16_vcvtsh2sd = 3909,
  CODE_FOR_avx512fp16_vcvtsh2sd_mask = 3910,
  CODE_FOR_avx512fp16_vcvtsh2sd_round = 3911,
  CODE_FOR_avx512fp16_vcvtsh2sd_mask_round = 3912,
  CODE_FOR_avx512fp16_vcvtsh2ss = 3913,
  CODE_FOR_avx512fp16_vcvtsh2ss_mask = 3914,
  CODE_FOR_avx512fp16_vcvtsh2ss_round = 3915,
  CODE_FOR_avx512fp16_vcvtsh2ss_mask_round = 3916,
  CODE_FOR_avx512fp16_vcvtsh2sd_mem = 3917,
  CODE_FOR_avx512fp16_vcvtsh2sd_mask_mem = 3918,
  CODE_FOR_avx512fp16_vcvtsh2ss_mem = 3919,
  CODE_FOR_avx512fp16_vcvtsh2ss_mask_mem = 3920,
  CODE_FOR_avx512fp16_vcvtsd2sh = 3921,
  CODE_FOR_avx512fp16_vcvtsd2sh_round = 3922,
  CODE_FOR_avx512fp16_vcvtsd2sh_mask = 3923,
  CODE_FOR_avx512fp16_vcvtsd2sh_mask_round = 3924,
  CODE_FOR_avx512fp16_vcvtss2sh = 3925,
  CODE_FOR_avx512fp16_vcvtss2sh_round = 3926,
  CODE_FOR_avx512fp16_vcvtss2sh_mask = 3927,
  CODE_FOR_avx512fp16_vcvtss2sh_mask_round = 3928,
  CODE_FOR_avx512fp16_vcvtss2sh_mem = 3929,
  CODE_FOR_avx512fp16_vcvtss2sh_mask_mem = 3930,
  CODE_FOR_avx512fp16_vcvtsd2sh_mem = 3931,
  CODE_FOR_avx512fp16_vcvtsd2sh_mask_mem = 3932,
  CODE_FOR_sse_cvtpi2ps = 3933,
  CODE_FOR_sse_cvtps2pi = 3934,
  CODE_FOR_sse_cvttps2pi = 3935,
  CODE_FOR_sse_cvtsi2ss = 3936,
  CODE_FOR_sse_cvtsi2ss_round = 3937,
  CODE_FOR_sse_cvtsi2ssq = 3938,
  CODE_FOR_sse_cvtsi2ssq_round = 3939,
  CODE_FOR_sse_cvtss2si = 3940,
  CODE_FOR_sse_cvtss2si_round = 3941,
  CODE_FOR_sse_cvtss2siq = 3942,
  CODE_FOR_sse_cvtss2siq_round = 3943,
  CODE_FOR_sse_cvtss2si_2 = 3944,
  CODE_FOR_sse_cvtss2siq_2 = 3945,
  CODE_FOR_sse_cvttss2si = 3946,
  CODE_FOR_sse_cvttss2si_round = 3947,
  CODE_FOR_sse_cvttss2siq = 3948,
  CODE_FOR_sse_cvttss2siq_round = 3949,
  CODE_FOR_cvtusi2ss32 = 3950,
  CODE_FOR_cvtusi2ss32_round = 3951,
  CODE_FOR_cvtusi2sd32 = 3952,
   CODE_FOR_cvtusi2sd32_round = CODE_FOR_nothing,
  CODE_FOR_cvtusi2ss64 = 3953,
  CODE_FOR_cvtusi2ss64_round = 3954,
  CODE_FOR_cvtusi2sd64 = 3955,
  CODE_FOR_cvtusi2sd64_round = 3956,
  CODE_FOR_floatv16siv16sf2 = 3957,
  CODE_FOR_floatv16siv16sf2_round = 3958,
  CODE_FOR_floatv16siv16sf2_mask = 3959,
  CODE_FOR_floatv16siv16sf2_mask_round = 3960,
  CODE_FOR_floatv8siv8sf2 = 3961,
   CODE_FOR_floatv8siv8sf2_round = CODE_FOR_nothing,
  CODE_FOR_floatv8siv8sf2_mask = 3962,
   CODE_FOR_floatv8siv8sf2_mask_round = CODE_FOR_nothing,
  CODE_FOR_floatv4siv4sf2 = 3963,
   CODE_FOR_floatv4siv4sf2_round = CODE_FOR_nothing,
  CODE_FOR_floatv4siv4sf2_mask = 3964,
   CODE_FOR_floatv4siv4sf2_mask_round = CODE_FOR_nothing,
  CODE_FOR_floatunsv16siv16sf2_mask = 3967,
  CODE_FOR_floatunsv16siv16sf2_mask_round = 3968,
  CODE_FOR_floatunsv8siv8sf2_mask = 3971,
  CODE_FOR_floatunsv8siv8sf2_mask_round = 3972,
  CODE_FOR_floatunsv4siv4sf2_mask = 3975,
  CODE_FOR_floatunsv4siv4sf2_mask_round = 3976,
  CODE_FOR_avx_fix_notruncv8sfv8si = 3977,
  CODE_FOR_avx_fix_notruncv8sfv8si_mask = 3978,
  CODE_FOR_sse2_fix_notruncv4sfv4si = 3979,
  CODE_FOR_sse2_fix_notruncv4sfv4si_mask = 3980,
  CODE_FOR_avx512f_fix_notruncv16sfv16si = 3981,
  CODE_FOR_avx512f_fix_notruncv16sfv16si_round = 3982,
  CODE_FOR_avx512f_fix_notruncv16sfv16si_mask = 3983,
  CODE_FOR_avx512f_fix_notruncv16sfv16si_mask_round = 3984,
  CODE_FOR_avx512f_fixuns_notruncv16sfv16si_mask = 3987,
  CODE_FOR_avx512f_fixuns_notruncv16sfv16si_mask_round = 3988,
  CODE_FOR_avx512vl_fixuns_notruncv8sfv8si_mask = 3991,
  CODE_FOR_avx512vl_fixuns_notruncv8sfv8si_mask_round = 3992,
  CODE_FOR_avx512vl_fixuns_notruncv4sfv4si_mask = 3995,
  CODE_FOR_avx512vl_fixuns_notruncv4sfv4si_mask_round = 3996,
  CODE_FOR_avx512dq_cvtps2qqv8di_mask = 3999,
  CODE_FOR_avx512dq_cvtps2qqv8di_mask_round = 4000,
  CODE_FOR_avx512dq_cvtps2qqv4di_mask = 4002,
   CODE_FOR_avx512dq_cvtps2qqv4di_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_cvtps2qqv2di_mask = 4004,
  CODE_FOR_avx512dq_cvtps2uqqv8di_mask = 4007,
  CODE_FOR_avx512dq_cvtps2uqqv8di_mask_round = 4008,
  CODE_FOR_avx512dq_cvtps2uqqv4di_mask = 4010,
   CODE_FOR_avx512dq_cvtps2uqqv4di_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_cvtps2uqqv2di_mask = 4012,
  CODE_FOR_fix_truncv16sfv16si2 = 4013,
  CODE_FOR_fix_truncv16sfv16si2_round = 4014,
  CODE_FOR_fix_truncv16sfv16si2_mask = 4015,
  CODE_FOR_fix_truncv16sfv16si2_mask_round = 4016,
  CODE_FOR_fixuns_truncv16sfv16si2 = 4017,
  CODE_FOR_fixuns_truncv16sfv16si2_round = 4018,
  CODE_FOR_fixuns_truncv16sfv16si2_mask = 4019,
  CODE_FOR_fixuns_truncv16sfv16si2_mask_round = 4020,
  CODE_FOR_fix_truncv8sfv8si2 = 4021,
  CODE_FOR_fix_truncv8sfv8si2_mask = 4022,
  CODE_FOR_fix_truncv4sfv4si2 = 4023,
  CODE_FOR_fix_truncv4sfv4si2_mask = 4024,
  CODE_FOR_sse2_cvtpi2pd = 4025,
  CODE_FOR_floatunsv2siv2df2 = 4026,
  CODE_FOR_sse2_cvtpd2pi = 4027,
  CODE_FOR_sse2_cvttpd2pi = 4028,
  CODE_FOR_fixuns_truncv2dfv2si2 = 4029,
  CODE_FOR_sse2_cvtsi2sd = 4030,
  CODE_FOR_sse2_cvtsi2sdq = 4031,
  CODE_FOR_sse2_cvtsi2sdq_round = 4032,
  CODE_FOR_avx512f_vcvtss2usi = 4033,
  CODE_FOR_avx512f_vcvtss2usi_round = 4034,
  CODE_FOR_avx512f_vcvtss2usiq = 4035,
  CODE_FOR_avx512f_vcvtss2usiq_round = 4036,
  CODE_FOR_avx512f_vcvttss2usi = 4037,
  CODE_FOR_avx512f_vcvttss2usi_round = 4038,
  CODE_FOR_avx512f_vcvttss2usiq = 4039,
  CODE_FOR_avx512f_vcvttss2usiq_round = 4040,
  CODE_FOR_avx512f_vcvtsd2usi = 4041,
  CODE_FOR_avx512f_vcvtsd2usi_round = 4042,
  CODE_FOR_avx512f_vcvtsd2usiq = 4043,
  CODE_FOR_avx512f_vcvtsd2usiq_round = 4044,
  CODE_FOR_avx512f_vcvttsd2usi = 4045,
  CODE_FOR_avx512f_vcvttsd2usi_round = 4046,
  CODE_FOR_avx512f_vcvttsd2usiq = 4047,
  CODE_FOR_avx512f_vcvttsd2usiq_round = 4048,
  CODE_FOR_sse2_cvtsd2si = 4049,
  CODE_FOR_sse2_cvtsd2si_round = 4050,
  CODE_FOR_sse2_cvtsd2siq = 4051,
  CODE_FOR_sse2_cvtsd2siq_round = 4052,
  CODE_FOR_sse2_cvtsd2si_2 = 4053,
  CODE_FOR_sse2_cvtsd2siq_2 = 4054,
  CODE_FOR_sse2_cvttsd2si = 4055,
  CODE_FOR_sse2_cvttsd2si_round = 4056,
  CODE_FOR_sse2_cvttsd2siq = 4057,
  CODE_FOR_sse2_cvttsd2siq_round = 4058,
  CODE_FOR_floatv8siv8df2 = 4059,
  CODE_FOR_floatv8siv8df2_mask = 4060,
  CODE_FOR_floatv4siv4df2 = 4061,
  CODE_FOR_floatv4siv4df2_mask = 4062,
  CODE_FOR_floatv8div8df2 = 4063,
  CODE_FOR_floatv8div8df2_round = 4064,
  CODE_FOR_floatv8div8df2_mask = 4065,
  CODE_FOR_floatv8div8df2_mask_round = 4066,
  CODE_FOR_floatunsv8div8df2 = 4067,
  CODE_FOR_floatunsv8div8df2_round = 4068,
  CODE_FOR_floatunsv8div8df2_mask = 4069,
  CODE_FOR_floatunsv8div8df2_mask_round = 4070,
  CODE_FOR_floatv4div4df2 = 4071,
  CODE_FOR_floatv4div4df2_round = 4072,
  CODE_FOR_floatv4div4df2_mask = 4073,
  CODE_FOR_floatv4div4df2_mask_round = 4074,
  CODE_FOR_floatunsv4div4df2 = 4075,
  CODE_FOR_floatunsv4div4df2_round = 4076,
  CODE_FOR_floatunsv4div4df2_mask = 4077,
  CODE_FOR_floatunsv4div4df2_mask_round = 4078,
  CODE_FOR_floatv2div2df2 = 4079,
  CODE_FOR_floatv2div2df2_round = 4080,
  CODE_FOR_floatv2div2df2_mask = 4081,
  CODE_FOR_floatv2div2df2_mask_round = 4082,
  CODE_FOR_floatunsv2div2df2 = 4083,
  CODE_FOR_floatunsv2div2df2_round = 4084,
  CODE_FOR_floatunsv2div2df2_mask = 4085,
  CODE_FOR_floatunsv2div2df2_mask_round = 4086,
  CODE_FOR_floatv8div8sf2 = 4087,
  CODE_FOR_floatv8div8sf2_round = 4088,
  CODE_FOR_floatv8div8sf2_mask = 4089,
  CODE_FOR_floatv8div8sf2_mask_round = 4090,
  CODE_FOR_floatunsv8div8sf2 = 4091,
  CODE_FOR_floatunsv8div8sf2_round = 4092,
  CODE_FOR_floatunsv8div8sf2_mask = 4093,
  CODE_FOR_floatunsv8div8sf2_mask_round = 4094,
  CODE_FOR_floatv4div4sf2 = 4095,
   CODE_FOR_floatv4div4sf2_round = CODE_FOR_nothing,
  CODE_FOR_floatv4div4sf2_mask = 4096,
   CODE_FOR_floatv4div4sf2_mask_round = CODE_FOR_nothing,
  CODE_FOR_floatunsv4div4sf2 = 4097,
   CODE_FOR_floatunsv4div4sf2_round = CODE_FOR_nothing,
  CODE_FOR_floatunsv4div4sf2_mask = 4098,
   CODE_FOR_floatunsv4div4sf2_mask_round = CODE_FOR_nothing,
  CODE_FOR_floatunsv8siv8df2 = 4105,
  CODE_FOR_floatunsv8siv8df2_mask = 4106,
  CODE_FOR_floatunsv4siv4df2 = 4107,
  CODE_FOR_floatunsv4siv4df2_mask = 4108,
  CODE_FOR_floatunsv2siv2df2_mask = 4110,
  CODE_FOR_avx512f_cvtdq2pd512_2 = 4111,
  CODE_FOR_avx_cvtdq2pd256_2 = 4112,
  CODE_FOR_sse2_cvtdq2pd = 4113,
  CODE_FOR_sse2_cvtdq2pd_mask = 4114,
  CODE_FOR_avx512f_cvtpd2dq512 = 4115,
  CODE_FOR_avx512f_cvtpd2dq512_round = 4116,
  CODE_FOR_avx512f_cvtpd2dq512_mask = 4117,
  CODE_FOR_avx512f_cvtpd2dq512_mask_round = 4118,
  CODE_FOR_avx_cvtpd2dq256 = 4119,
  CODE_FOR_avx_cvtpd2dq256_mask = 4120,
  CODE_FOR_sse2_cvtpd2dq = 4122,
  CODE_FOR_sse2_cvtpd2dq_mask = 4123,
  CODE_FOR_fixuns_notruncv8dfv8si2 = 4125,
  CODE_FOR_fixuns_notruncv8dfv8si2_round = 4126,
  CODE_FOR_fixuns_notruncv8dfv8si2_mask = 4127,
  CODE_FOR_fixuns_notruncv8dfv8si2_mask_round = 4128,
  CODE_FOR_fixuns_notruncv4dfv4si2 = 4129,
  CODE_FOR_fixuns_notruncv4dfv4si2_round = 4130,
  CODE_FOR_fixuns_notruncv4dfv4si2_mask = 4131,
  CODE_FOR_fixuns_notruncv4dfv4si2_mask_round = 4132,
  CODE_FOR_fixuns_notruncv2dfv2si2 = 4133,
  CODE_FOR_fixuns_notruncv2dfv2si2_mask = 4134,
  CODE_FOR_fix_truncv8dfv8si2 = 4136,
  CODE_FOR_fix_truncv8dfv8si2_round = 4137,
  CODE_FOR_fix_truncv8dfv8si2_mask = 4138,
  CODE_FOR_fix_truncv8dfv8si2_mask_round = 4139,
  CODE_FOR_fixuns_truncv8dfv8si2 = 4140,
  CODE_FOR_fixuns_truncv8dfv8si2_round = 4141,
  CODE_FOR_fixuns_truncv8dfv8si2_mask = 4142,
  CODE_FOR_fixuns_truncv8dfv8si2_mask_round = 4143,
  CODE_FOR_fixuns_truncv2dfv2si2_mask = 4145,
  CODE_FOR_fix_truncv4dfv4si2 = 4147,
  CODE_FOR_fix_truncv4dfv4si2_mask = 4148,
  CODE_FOR_fixuns_truncv4dfv4si2 = 4149,
  CODE_FOR_fixuns_truncv4dfv4si2_mask = 4150,
  CODE_FOR_fix_truncv8dfv8di2 = 4151,
  CODE_FOR_fix_truncv8dfv8di2_round = 4152,
  CODE_FOR_fix_truncv8dfv8di2_mask = 4153,
  CODE_FOR_fix_truncv8dfv8di2_mask_round = 4154,
  CODE_FOR_fixuns_truncv8dfv8di2 = 4155,
  CODE_FOR_fixuns_truncv8dfv8di2_round = 4156,
  CODE_FOR_fixuns_truncv8dfv8di2_mask = 4157,
  CODE_FOR_fixuns_truncv8dfv8di2_mask_round = 4158,
  CODE_FOR_fix_truncv4dfv4di2 = 4159,
   CODE_FOR_fix_truncv4dfv4di2_round = CODE_FOR_nothing,
  CODE_FOR_fix_truncv4dfv4di2_mask = 4160,
   CODE_FOR_fix_truncv4dfv4di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_fixuns_truncv4dfv4di2 = 4161,
   CODE_FOR_fixuns_truncv4dfv4di2_round = CODE_FOR_nothing,
  CODE_FOR_fixuns_truncv4dfv4di2_mask = 4162,
   CODE_FOR_fixuns_truncv4dfv4di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_fix_truncv2dfv2di2 = 4163,
   CODE_FOR_fix_truncv2dfv2di2_round = CODE_FOR_nothing,
  CODE_FOR_fix_truncv2dfv2di2_mask = 4164,
   CODE_FOR_fix_truncv2dfv2di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_fixuns_truncv2dfv2di2 = 4165,
   CODE_FOR_fixuns_truncv2dfv2di2_round = CODE_FOR_nothing,
  CODE_FOR_fixuns_truncv2dfv2di2_mask = 4166,
   CODE_FOR_fixuns_truncv2dfv2di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_fix_notruncv8dfv8di2 = 4167,
  CODE_FOR_fix_notruncv8dfv8di2_round = 4168,
  CODE_FOR_fix_notruncv8dfv8di2_mask = 4169,
  CODE_FOR_fix_notruncv8dfv8di2_mask_round = 4170,
  CODE_FOR_fix_notruncv4dfv4di2 = 4171,
   CODE_FOR_fix_notruncv4dfv4di2_round = CODE_FOR_nothing,
  CODE_FOR_fix_notruncv4dfv4di2_mask = 4172,
   CODE_FOR_fix_notruncv4dfv4di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_fix_notruncv2dfv2di2 = 4173,
   CODE_FOR_fix_notruncv2dfv2di2_round = CODE_FOR_nothing,
  CODE_FOR_fix_notruncv2dfv2di2_mask = 4174,
   CODE_FOR_fix_notruncv2dfv2di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_fixuns_notruncv8dfv8di2 = 4175,
  CODE_FOR_fixuns_notruncv8dfv8di2_round = 4176,
  CODE_FOR_fixuns_notruncv8dfv8di2_mask = 4177,
  CODE_FOR_fixuns_notruncv8dfv8di2_mask_round = 4178,
  CODE_FOR_fixuns_notruncv4dfv4di2 = 4179,
   CODE_FOR_fixuns_notruncv4dfv4di2_round = CODE_FOR_nothing,
  CODE_FOR_fixuns_notruncv4dfv4di2_mask = 4180,
   CODE_FOR_fixuns_notruncv4dfv4di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_fixuns_notruncv2dfv2di2 = 4181,
   CODE_FOR_fixuns_notruncv2dfv2di2_round = CODE_FOR_nothing,
  CODE_FOR_fixuns_notruncv2dfv2di2_mask = 4182,
   CODE_FOR_fixuns_notruncv2dfv2di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_fix_truncv8sfv8di2 = 4183,
  CODE_FOR_fix_truncv8sfv8di2_round = 4184,
  CODE_FOR_fix_truncv8sfv8di2_mask = 4185,
  CODE_FOR_fix_truncv8sfv8di2_mask_round = 4186,
  CODE_FOR_fixuns_truncv8sfv8di2 = 4187,
  CODE_FOR_fixuns_truncv8sfv8di2_round = 4188,
  CODE_FOR_fixuns_truncv8sfv8di2_mask = 4189,
  CODE_FOR_fixuns_truncv8sfv8di2_mask_round = 4190,
  CODE_FOR_fix_truncv4sfv4di2 = 4191,
   CODE_FOR_fix_truncv4sfv4di2_round = CODE_FOR_nothing,
  CODE_FOR_fix_truncv4sfv4di2_mask = 4192,
   CODE_FOR_fix_truncv4sfv4di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_fixuns_truncv4sfv4di2 = 4193,
   CODE_FOR_fixuns_truncv4sfv4di2_round = CODE_FOR_nothing,
  CODE_FOR_fixuns_truncv4sfv4di2_mask = 4194,
   CODE_FOR_fixuns_truncv4sfv4di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_fix_truncv2sfv2di2 = 4195,
  CODE_FOR_avx512dq_fix_truncv2sfv2di2_mask = 4196,
  CODE_FOR_avx512dq_fixuns_truncv2sfv2di2 = 4197,
  CODE_FOR_avx512dq_fixuns_truncv2sfv2di2_mask = 4198,
  CODE_FOR_fixuns_truncv8sfv8si2_mask = 4200,
  CODE_FOR_fixuns_truncv4sfv4si2_mask = 4202,
  CODE_FOR_sse2_cvttpd2dq = 4203,
  CODE_FOR_sse2_cvttpd2dq_mask = 4204,
  CODE_FOR_sse2_cvtsd2ss = 4206,
  CODE_FOR_sse2_cvtsd2ss_round = 4207,
  CODE_FOR_sse2_cvtsd2ss_mask = 4208,
  CODE_FOR_sse2_cvtsd2ss_mask_round = 4209,
  CODE_FOR_sse2_cvtss2sd = 4211,
  CODE_FOR_sse2_cvtss2sd_round = 4212,
  CODE_FOR_sse2_cvtss2sd_mask = 4213,
  CODE_FOR_sse2_cvtss2sd_mask_round = 4214,
  CODE_FOR_avx512f_cvtpd2ps512_mask = 4218,
  CODE_FOR_avx512f_cvtpd2ps512_mask_round = 4219,
  CODE_FOR_avx_cvtpd2ps256 = 4220,
  CODE_FOR_avx_cvtpd2ps256_mask = 4221,
  CODE_FOR_truncv2dfv2sf2 = 4223,
  CODE_FOR_avx512f_cvtps2pd512 = 4226,
  CODE_FOR_avx512f_cvtps2pd512_round = 4227,
  CODE_FOR_avx512f_cvtps2pd512_mask = 4228,
  CODE_FOR_avx512f_cvtps2pd512_mask_round = 4229,
  CODE_FOR_avx_cvtps2pd256 = 4230,
   CODE_FOR_avx_cvtps2pd256_round = CODE_FOR_nothing,
  CODE_FOR_avx_cvtps2pd256_mask = 4231,
   CODE_FOR_avx_cvtps2pd256_mask_round = CODE_FOR_nothing,
  CODE_FOR_vec_unpacks_lo_v16sf = 4233,
  CODE_FOR_avx512bw_cvtb2maskv64qi = 4234,
  CODE_FOR_avx512vl_cvtb2maskv16qi = 4235,
  CODE_FOR_avx512vl_cvtb2maskv32qi = 4236,
  CODE_FOR_avx512bw_cvtw2maskv32hi = 4237,
  CODE_FOR_avx512vl_cvtw2maskv16hi = 4238,
  CODE_FOR_avx512vl_cvtw2maskv8hi = 4239,
  CODE_FOR_avx512f_cvtd2maskv16si = 4240,
  CODE_FOR_avx512vl_cvtd2maskv8si = 4241,
  CODE_FOR_avx512vl_cvtd2maskv4si = 4242,
  CODE_FOR_avx512f_cvtq2maskv8di = 4243,
  CODE_FOR_avx512vl_cvtq2maskv4di = 4244,
  CODE_FOR_avx512vl_cvtq2maskv2di = 4245,
  CODE_FOR_sse2_cvtps2pd = 4258,
  CODE_FOR_sse2_cvtps2pd_mask = 4259,
  CODE_FOR_sse2_cvtps2pd_1 = 4260,
  CODE_FOR_sse2_cvtps2pd_mask_1 = 4261,
  CODE_FOR_sse_movhlps = 4262,
  CODE_FOR_sse_movlhps = 4263,
  CODE_FOR_avx512f_unpckhps512_mask = 4265,
  CODE_FOR_avx_unpckhps256 = 4266,
  CODE_FOR_avx_unpckhps256_mask = 4267,
  CODE_FOR_vec_interleave_highv4sf = 4268,
  CODE_FOR_vec_interleave_highv4sf_mask = 4269,
  CODE_FOR_avx512f_unpcklps512_mask = 4271,
  CODE_FOR_avx_unpcklps256 = 4272,
  CODE_FOR_avx_unpcklps256_mask = 4273,
  CODE_FOR_unpcklps128_mask = 4274,
  CODE_FOR_vec_interleave_lowv4sf = 4275,
  CODE_FOR_avx_movshdup256 = 4276,
  CODE_FOR_avx_movshdup256_mask = 4277,
  CODE_FOR_sse3_movshdup = 4278,
  CODE_FOR_sse3_movshdup_mask = 4279,
  CODE_FOR_avx512f_movshdup512_mask = 4281,
  CODE_FOR_avx_movsldup256 = 4282,
  CODE_FOR_avx_movsldup256_mask = 4283,
  CODE_FOR_sse3_movsldup = 4284,
  CODE_FOR_sse3_movsldup_mask = 4285,
  CODE_FOR_avx512f_movsldup512_mask = 4287,
  CODE_FOR_avx_shufps256_1 = 4288,
  CODE_FOR_avx_shufps256_1_mask = 4289,
  CODE_FOR_sse_shufps_v4sf_mask = 4290,
  CODE_FOR_sse_shufps_v4si = 4291,
  CODE_FOR_sse_shufps_v4sf = 4292,
  CODE_FOR_sse_storehps = 4293,
  CODE_FOR_sse_loadhps = 4294,
  CODE_FOR_sse_storelps = 4295,
  CODE_FOR_sse_loadlps = 4296,
  CODE_FOR_sse_movss_v4si = 4297,
  CODE_FOR_sse_movss_v4sf = 4298,
  CODE_FOR_avx2_vec_dupv8sf = 4299,
  CODE_FOR_avx2_vec_dupv4sf = 4300,
  CODE_FOR_avx2_vec_dupv8sf_1 = 4301,
  CODE_FOR_avx512f_vec_dupv16sf_1 = 4302,
  CODE_FOR_avx512f_vec_dupv8df_1 = 4303,
  CODE_FOR_vec_setv4si_0 = 4308,
  CODE_FOR_vec_setv4sf_0 = 4309,
  CODE_FOR_vec_setv8hi_0 = 4310,
  CODE_FOR_vec_setv8hf_0 = 4311,
  CODE_FOR_vec_setv8bf_0 = 4312,
  CODE_FOR_vec_setv16hi_0 = 4313,
  CODE_FOR_vec_setv32hi_0 = 4314,
  CODE_FOR_vec_setv16hf_0 = 4315,
  CODE_FOR_vec_setv32hf_0 = 4316,
  CODE_FOR_vec_setv16bf_0 = 4317,
  CODE_FOR_vec_setv32bf_0 = 4318,
  CODE_FOR_avx512fp16_movv8hf = 4326,
  CODE_FOR_avx512fp16_movv8bf = 4327,
  CODE_FOR_vec_setv8si_0 = 4329,
  CODE_FOR_vec_setv8sf_0 = 4330,
  CODE_FOR_vec_setv16si_0 = 4331,
  CODE_FOR_vec_setv16sf_0 = 4332,
  CODE_FOR_sse4_1_insertps = 4337,
  CODE_FOR_vec_setv2df_0 = 4338,
  CODE_FOR_avx512dq_vextractf64x2_1_mask = 4342,
  CODE_FOR_avx512dq_vextracti64x2_1_mask = 4343,
  CODE_FOR_avx512f_vextractf32x4_1_mask = 4346,
  CODE_FOR_avx512f_vextracti32x4_1_mask = 4347,
  CODE_FOR_vec_extract_lo_v8df_mask = 4350,
  CODE_FOR_vec_extract_lo_v8di_mask = 4351,
  CODE_FOR_vec_extract_lo_v8df = 4352,
  CODE_FOR_vec_extract_lo_v8di = 4353,
  CODE_FOR_vec_extract_hi_v8df_mask = 4354,
  CODE_FOR_vec_extract_hi_v8di_mask = 4355,
  CODE_FOR_vec_extract_hi_v8df = 4356,
  CODE_FOR_vec_extract_hi_v8di = 4357,
  CODE_FOR_vec_extract_hi_v16sf_mask = 4358,
  CODE_FOR_vec_extract_hi_v16si_mask = 4359,
  CODE_FOR_vec_extract_hi_v16sf = 4360,
  CODE_FOR_vec_extract_hi_v16si = 4361,
  CODE_FOR_vec_extract_lo_v16sf_mask = 4362,
  CODE_FOR_vec_extract_lo_v16si_mask = 4363,
  CODE_FOR_vec_extract_lo_v16sf = 4364,
  CODE_FOR_vec_extract_lo_v16si = 4365,
  CODE_FOR_vec_extract_lo_v4di_mask = 4366,
  CODE_FOR_vec_extract_lo_v4df_mask = 4367,
  CODE_FOR_vec_extract_lo_v4di = 4368,
  CODE_FOR_vec_extract_lo_v4df = 4369,
  CODE_FOR_vec_extract_hi_v4di_mask = 4370,
  CODE_FOR_vec_extract_hi_v4df_mask = 4371,
  CODE_FOR_vec_extract_hi_v4di = 4372,
  CODE_FOR_vec_extract_hi_v4df = 4373,
  CODE_FOR_vec_extract_lo_v8si_mask = 4374,
  CODE_FOR_vec_extract_lo_v8sf_mask = 4375,
  CODE_FOR_vec_extract_lo_v8si = 4376,
  CODE_FOR_vec_extract_lo_v8sf = 4377,
  CODE_FOR_vec_extract_hi_v8si_mask = 4378,
  CODE_FOR_vec_extract_hi_v8sf_mask = 4379,
  CODE_FOR_vec_extract_hi_v8si = 4380,
  CODE_FOR_vec_extract_hi_v8sf = 4381,
  CODE_FOR_vec_extract_lo_v32hi = 4382,
  CODE_FOR_vec_extract_lo_v32hf = 4383,
  CODE_FOR_vec_extract_lo_v32bf = 4384,
  CODE_FOR_vec_extract_hi_v32hi = 4385,
  CODE_FOR_vec_extract_hi_v32hf = 4386,
  CODE_FOR_vec_extract_hi_v32bf = 4387,
  CODE_FOR_vec_extract_lo_v16hi = 4388,
  CODE_FOR_vec_extract_lo_v16hf = 4389,
  CODE_FOR_vec_extract_lo_v16bf = 4390,
  CODE_FOR_vec_extract_hi_v16hi = 4391,
  CODE_FOR_vec_extract_hi_v16hf = 4392,
  CODE_FOR_vec_extract_hi_v16bf = 4393,
  CODE_FOR_vec_extract_lo_v64qi = 4394,
  CODE_FOR_vec_extract_hi_v64qi = 4395,
  CODE_FOR_vec_extract_lo_v32qi = 4396,
  CODE_FOR_vec_extract_hi_v32qi = 4397,
  CODE_FOR_avx512f_unpckhpd512_mask = 4407,
  CODE_FOR_avx_unpckhpd256 = 4408,
  CODE_FOR_avx_unpckhpd256_mask = 4409,
  CODE_FOR_avx512vl_unpckhpd128_mask = 4410,
  CODE_FOR_avx512f_movddup512 = 4412,
  CODE_FOR_avx512f_movddup512_mask = 4413,
  CODE_FOR_avx512f_unpcklpd512 = 4414,
  CODE_FOR_avx512f_unpcklpd512_mask = 4415,
  CODE_FOR_avx_movddup256 = 4416,
  CODE_FOR_avx_movddup256_mask = 4417,
  CODE_FOR_avx_unpcklpd256 = 4418,
  CODE_FOR_avx_unpcklpd256_mask = 4419,
  CODE_FOR_avx512vl_unpcklpd128_mask = 4420,
  CODE_FOR_avx512f_vmscalefv8hf = 4422,
  CODE_FOR_avx512f_vmscalefv8hf_round = 4423,
  CODE_FOR_avx512f_vmscalefv8hf_mask = 4424,
  CODE_FOR_avx512f_vmscalefv8hf_mask_round = 4425,
  CODE_FOR_avx512f_vmscalefv4sf = 4426,
  CODE_FOR_avx512f_vmscalefv4sf_round = 4427,
  CODE_FOR_avx512f_vmscalefv4sf_mask = 4428,
  CODE_FOR_avx512f_vmscalefv4sf_mask_round = 4429,
  CODE_FOR_avx512f_vmscalefv2df = 4430,
  CODE_FOR_avx512f_vmscalefv2df_round = 4431,
  CODE_FOR_avx512f_vmscalefv2df_mask = 4432,
  CODE_FOR_avx512f_vmscalefv2df_mask_round = 4433,
  CODE_FOR_avx512bw_scalefv32hf = 4434,
  CODE_FOR_avx512bw_scalefv32hf_round = 4435,
  CODE_FOR_avx512bw_scalefv32hf_mask = 4436,
  CODE_FOR_avx512bw_scalefv32hf_mask_round = 4437,
  CODE_FOR_avx512vl_scalefv16hf = 4438,
  CODE_FOR_avx512vl_scalefv16hf_round = 4439,
  CODE_FOR_avx512vl_scalefv16hf_mask = 4440,
  CODE_FOR_avx512vl_scalefv16hf_mask_round = 4441,
  CODE_FOR_avx512fp16_scalefv8hf = 4442,
  CODE_FOR_avx512fp16_scalefv8hf_round = 4443,
  CODE_FOR_avx512fp16_scalefv8hf_mask = 4444,
  CODE_FOR_avx512fp16_scalefv8hf_mask_round = 4445,
  CODE_FOR_avx512f_scalefv16sf = 4446,
  CODE_FOR_avx512f_scalefv16sf_round = 4447,
  CODE_FOR_avx512f_scalefv16sf_mask = 4448,
  CODE_FOR_avx512f_scalefv16sf_mask_round = 4449,
  CODE_FOR_avx512vl_scalefv8sf = 4450,
  CODE_FOR_avx512vl_scalefv8sf_round = 4451,
  CODE_FOR_avx512vl_scalefv8sf_mask = 4452,
  CODE_FOR_avx512vl_scalefv8sf_mask_round = 4453,
  CODE_FOR_avx512vl_scalefv4sf = 4454,
  CODE_FOR_avx512vl_scalefv4sf_round = 4455,
  CODE_FOR_avx512vl_scalefv4sf_mask = 4456,
  CODE_FOR_avx512vl_scalefv4sf_mask_round = 4457,
  CODE_FOR_avx512f_scalefv8df = 4458,
  CODE_FOR_avx512f_scalefv8df_round = 4459,
  CODE_FOR_avx512f_scalefv8df_mask = 4460,
  CODE_FOR_avx512f_scalefv8df_mask_round = 4461,
  CODE_FOR_avx512vl_scalefv4df = 4462,
  CODE_FOR_avx512vl_scalefv4df_round = 4463,
  CODE_FOR_avx512vl_scalefv4df_mask = 4464,
  CODE_FOR_avx512vl_scalefv4df_mask_round = 4465,
  CODE_FOR_avx512vl_scalefv2df = 4466,
  CODE_FOR_avx512vl_scalefv2df_round = 4467,
  CODE_FOR_avx512vl_scalefv2df_mask = 4468,
  CODE_FOR_avx512vl_scalefv2df_mask_round = 4469,
  CODE_FOR_avx512f_vternlogv16si = 4470,
  CODE_FOR_avx512f_vternlogv16si_maskz_1 = 4471,
  CODE_FOR_avx512vl_vternlogv8si = 4472,
  CODE_FOR_avx512vl_vternlogv8si_maskz_1 = 4473,
  CODE_FOR_avx512vl_vternlogv4si = 4474,
  CODE_FOR_avx512vl_vternlogv4si_maskz_1 = 4475,
  CODE_FOR_avx512f_vternlogv8di = 4476,
  CODE_FOR_avx512f_vternlogv8di_maskz_1 = 4477,
  CODE_FOR_avx512vl_vternlogv4di = 4478,
  CODE_FOR_avx512vl_vternlogv4di_maskz_1 = 4479,
  CODE_FOR_avx512vl_vternlogv2di = 4480,
  CODE_FOR_avx512vl_vternlogv2di_maskz_1 = 4481,
  CODE_FOR_avx512f_vternlogv16si_mask = 5826,
  CODE_FOR_avx512vl_vternlogv8si_mask = 5827,
  CODE_FOR_avx512vl_vternlogv4si_mask = 5828,
  CODE_FOR_avx512f_vternlogv8di_mask = 5829,
  CODE_FOR_avx512vl_vternlogv4di_mask = 5830,
  CODE_FOR_avx512vl_vternlogv2di_mask = 5831,
  CODE_FOR_avx512bw_getexpv32hf = 5832,
  CODE_FOR_avx512bw_getexpv32hf_round = 5833,
  CODE_FOR_avx512bw_getexpv32hf_mask = 5834,
  CODE_FOR_avx512bw_getexpv32hf_mask_round = 5835,
  CODE_FOR_avx512vl_getexpv16hf = 5836,
  CODE_FOR_avx512vl_getexpv16hf_round = 5837,
  CODE_FOR_avx512vl_getexpv16hf_mask = 5838,
  CODE_FOR_avx512vl_getexpv16hf_mask_round = 5839,
  CODE_FOR_avx512fp16_getexpv8hf = 5840,
  CODE_FOR_avx512fp16_getexpv8hf_round = 5841,
  CODE_FOR_avx512fp16_getexpv8hf_mask = 5842,
  CODE_FOR_avx512fp16_getexpv8hf_mask_round = 5843,
  CODE_FOR_avx512f_getexpv16sf = 5844,
  CODE_FOR_avx512f_getexpv16sf_round = 5845,
  CODE_FOR_avx512f_getexpv16sf_mask = 5846,
  CODE_FOR_avx512f_getexpv16sf_mask_round = 5847,
  CODE_FOR_avx512vl_getexpv8sf = 5848,
  CODE_FOR_avx512vl_getexpv8sf_round = 5849,
  CODE_FOR_avx512vl_getexpv8sf_mask = 5850,
  CODE_FOR_avx512vl_getexpv8sf_mask_round = 5851,
  CODE_FOR_avx512vl_getexpv4sf = 5852,
  CODE_FOR_avx512vl_getexpv4sf_round = 5853,
  CODE_FOR_avx512vl_getexpv4sf_mask = 5854,
  CODE_FOR_avx512vl_getexpv4sf_mask_round = 5855,
  CODE_FOR_avx512f_getexpv8df = 5856,
  CODE_FOR_avx512f_getexpv8df_round = 5857,
  CODE_FOR_avx512f_getexpv8df_mask = 5858,
  CODE_FOR_avx512f_getexpv8df_mask_round = 5859,
  CODE_FOR_avx512vl_getexpv4df = 5860,
  CODE_FOR_avx512vl_getexpv4df_round = 5861,
  CODE_FOR_avx512vl_getexpv4df_mask = 5862,
  CODE_FOR_avx512vl_getexpv4df_mask_round = 5863,
  CODE_FOR_avx512vl_getexpv2df = 5864,
  CODE_FOR_avx512vl_getexpv2df_round = 5865,
  CODE_FOR_avx512vl_getexpv2df_mask = 5866,
  CODE_FOR_avx512vl_getexpv2df_mask_round = 5867,
  CODE_FOR_avx512f_sgetexpv8hf = 5868,
  CODE_FOR_avx512f_sgetexpv8hf_mask = 5869,
  CODE_FOR_avx512f_sgetexpv8hf_round = 5870,
  CODE_FOR_avx512f_sgetexpv8hf_mask_round = 5871,
  CODE_FOR_avx512f_sgetexpv4sf = 5872,
  CODE_FOR_avx512f_sgetexpv4sf_mask = 5873,
  CODE_FOR_avx512f_sgetexpv4sf_round = 5874,
  CODE_FOR_avx512f_sgetexpv4sf_mask_round = 5875,
  CODE_FOR_avx512f_sgetexpv2df = 5876,
  CODE_FOR_avx512f_sgetexpv2df_mask = 5877,
  CODE_FOR_avx512f_sgetexpv2df_round = 5878,
  CODE_FOR_avx512f_sgetexpv2df_mask_round = 5879,
  CODE_FOR_avx512f_alignv16si_mask = 5881,
  CODE_FOR_avx512vl_alignv8si_mask = 5883,
  CODE_FOR_avx512vl_alignv4si_mask = 5885,
  CODE_FOR_avx512f_alignv8di_mask = 5887,
  CODE_FOR_avx512vl_alignv4di_mask = 5889,
  CODE_FOR_avx512vl_alignv2di_mask = 5891,
  CODE_FOR_avx512f_fixupimmv16sf = 5900,
  CODE_FOR_avx512f_fixupimmv16sf_round = 5901,
  CODE_FOR_avx512f_fixupimmv16sf_maskz_1 = 5902,
  CODE_FOR_avx512f_fixupimmv16sf_maskz_1_round = 5903,
  CODE_FOR_avx512vl_fixupimmv8sf = 5904,
  CODE_FOR_avx512vl_fixupimmv8sf_round = 5905,
  CODE_FOR_avx512vl_fixupimmv8sf_maskz_1 = 5906,
  CODE_FOR_avx512vl_fixupimmv8sf_maskz_1_round = 5907,
  CODE_FOR_avx512vl_fixupimmv4sf = 5908,
  CODE_FOR_avx512vl_fixupimmv4sf_round = 5909,
  CODE_FOR_avx512vl_fixupimmv4sf_maskz_1 = 5910,
  CODE_FOR_avx512vl_fixupimmv4sf_maskz_1_round = 5911,
  CODE_FOR_avx512f_fixupimmv8df = 5912,
  CODE_FOR_avx512f_fixupimmv8df_round = 5913,
  CODE_FOR_avx512f_fixupimmv8df_maskz_1 = 5914,
  CODE_FOR_avx512f_fixupimmv8df_maskz_1_round = 5915,
  CODE_FOR_avx512vl_fixupimmv4df = 5916,
  CODE_FOR_avx512vl_fixupimmv4df_round = 5917,
  CODE_FOR_avx512vl_fixupimmv4df_maskz_1 = 5918,
  CODE_FOR_avx512vl_fixupimmv4df_maskz_1_round = 5919,
  CODE_FOR_avx512vl_fixupimmv2df = 5920,
  CODE_FOR_avx512vl_fixupimmv2df_round = 5921,
  CODE_FOR_avx512vl_fixupimmv2df_maskz_1 = 5922,
  CODE_FOR_avx512vl_fixupimmv2df_maskz_1_round = 5923,
  CODE_FOR_avx512f_fixupimmv16sf_mask = 5924,
  CODE_FOR_avx512f_fixupimmv16sf_mask_round = 5925,
  CODE_FOR_avx512vl_fixupimmv8sf_mask = 5926,
  CODE_FOR_avx512vl_fixupimmv8sf_mask_round = 5927,
  CODE_FOR_avx512vl_fixupimmv4sf_mask = 5928,
  CODE_FOR_avx512vl_fixupimmv4sf_mask_round = 5929,
  CODE_FOR_avx512f_fixupimmv8df_mask = 5930,
  CODE_FOR_avx512f_fixupimmv8df_mask_round = 5931,
  CODE_FOR_avx512vl_fixupimmv4df_mask = 5932,
  CODE_FOR_avx512vl_fixupimmv4df_mask_round = 5933,
  CODE_FOR_avx512vl_fixupimmv2df_mask = 5934,
  CODE_FOR_avx512vl_fixupimmv2df_mask_round = 5935,
  CODE_FOR_avx512f_sfixupimmv4sf = 5936,
  CODE_FOR_avx512f_sfixupimmv4sf_round = 5937,
  CODE_FOR_avx512f_sfixupimmv4sf_maskz_1 = 5938,
  CODE_FOR_avx512f_sfixupimmv4sf_maskz_1_round = 5939,
  CODE_FOR_avx512f_sfixupimmv2df = 5940,
  CODE_FOR_avx512f_sfixupimmv2df_round = 5941,
  CODE_FOR_avx512f_sfixupimmv2df_maskz_1 = 5942,
  CODE_FOR_avx512f_sfixupimmv2df_maskz_1_round = 5943,
  CODE_FOR_avx512f_sfixupimmv4sf_mask = 5944,
  CODE_FOR_avx512f_sfixupimmv4sf_mask_round = 5945,
  CODE_FOR_avx512f_sfixupimmv2df_mask = 5946,
  CODE_FOR_avx512f_sfixupimmv2df_mask_round = 5947,
  CODE_FOR_avx512bw_rndscalev32hf = 5948,
  CODE_FOR_avx512bw_rndscalev32hf_round = 5949,
  CODE_FOR_avx512bw_rndscalev32hf_mask = 5950,
  CODE_FOR_avx512bw_rndscalev32hf_mask_round = 5951,
  CODE_FOR_avx512vl_rndscalev16hf = 5952,
  CODE_FOR_avx512vl_rndscalev16hf_round = 5953,
  CODE_FOR_avx512vl_rndscalev16hf_mask = 5954,
  CODE_FOR_avx512vl_rndscalev16hf_mask_round = 5955,
  CODE_FOR_avx512fp16_rndscalev8hf = 5956,
  CODE_FOR_avx512fp16_rndscalev8hf_round = 5957,
  CODE_FOR_avx512fp16_rndscalev8hf_mask = 5958,
  CODE_FOR_avx512fp16_rndscalev8hf_mask_round = 5959,
  CODE_FOR_avx512f_rndscalev16sf = 5960,
  CODE_FOR_avx512f_rndscalev16sf_round = 5961,
  CODE_FOR_avx512f_rndscalev16sf_mask = 5962,
  CODE_FOR_avx512f_rndscalev16sf_mask_round = 5963,
  CODE_FOR_avx512vl_rndscalev8sf = 5964,
  CODE_FOR_avx512vl_rndscalev8sf_round = 5965,
  CODE_FOR_avx512vl_rndscalev8sf_mask = 5966,
  CODE_FOR_avx512vl_rndscalev8sf_mask_round = 5967,
  CODE_FOR_avx512vl_rndscalev4sf = 5968,
  CODE_FOR_avx512vl_rndscalev4sf_round = 5969,
  CODE_FOR_avx512vl_rndscalev4sf_mask = 5970,
  CODE_FOR_avx512vl_rndscalev4sf_mask_round = 5971,
  CODE_FOR_avx512f_rndscalev8df = 5972,
  CODE_FOR_avx512f_rndscalev8df_round = 5973,
  CODE_FOR_avx512f_rndscalev8df_mask = 5974,
  CODE_FOR_avx512f_rndscalev8df_mask_round = 5975,
  CODE_FOR_avx512vl_rndscalev4df = 5976,
  CODE_FOR_avx512vl_rndscalev4df_round = 5977,
  CODE_FOR_avx512vl_rndscalev4df_mask = 5978,
  CODE_FOR_avx512vl_rndscalev4df_mask_round = 5979,
  CODE_FOR_avx512vl_rndscalev2df = 5980,
  CODE_FOR_avx512vl_rndscalev2df_round = 5981,
  CODE_FOR_avx512vl_rndscalev2df_mask = 5982,
  CODE_FOR_avx512vl_rndscalev2df_mask_round = 5983,
  CODE_FOR_avx512f_rndscalev8hf = 5984,
  CODE_FOR_avx512f_rndscalev8hf_mask = 5985,
  CODE_FOR_avx512f_rndscalev8hf_round = 5986,
  CODE_FOR_avx512f_rndscalev8hf_mask_round = 5987,
  CODE_FOR_avx512f_rndscalev4sf = 5988,
  CODE_FOR_avx512f_rndscalev4sf_mask = 5989,
  CODE_FOR_avx512f_rndscalev4sf_round = 5990,
  CODE_FOR_avx512f_rndscalev4sf_mask_round = 5991,
  CODE_FOR_avx512f_rndscalev2df = 5992,
  CODE_FOR_avx512f_rndscalev2df_mask = 5993,
  CODE_FOR_avx512f_rndscalev2df_round = 5994,
  CODE_FOR_avx512f_rndscalev2df_mask_round = 5995,
  CODE_FOR_avx512f_shufps512_1 = 6002,
  CODE_FOR_avx512f_shufps512_1_mask = 6003,
  CODE_FOR_avx512f_shufpd512_1 = 6004,
  CODE_FOR_avx512f_shufpd512_1_mask = 6005,
  CODE_FOR_avx_shufpd256_1 = 6006,
  CODE_FOR_avx_shufpd256_1_mask = 6007,
  CODE_FOR_sse2_shufpd_v2df_mask = 6008,
  CODE_FOR_avx2_interleave_highv4di = 6009,
  CODE_FOR_avx2_interleave_highv4di_mask = 6010,
  CODE_FOR_avx512f_interleave_highv8di_mask = 6012,
  CODE_FOR_vec_interleave_highv2di = 6013,
  CODE_FOR_vec_interleave_highv2di_mask = 6014,
  CODE_FOR_avx2_interleave_lowv4di = 6015,
  CODE_FOR_avx2_interleave_lowv4di_mask = 6016,
  CODE_FOR_avx512f_interleave_lowv8di_mask = 6018,
  CODE_FOR_vec_interleave_lowv2di = 6019,
  CODE_FOR_vec_interleave_lowv2di_mask = 6020,
  CODE_FOR_sse2_shufpd_v2di = 6021,
  CODE_FOR_sse2_shufpd_v2df = 6022,
  CODE_FOR_sse2_storehpd = 6023,
  CODE_FOR_sse2_storelpd = 6025,
  CODE_FOR_sse2_loadhpd = 6027,
  CODE_FOR_sse2_loadlpd = 6028,
  CODE_FOR_sse2_movsd_v2di = 6029,
  CODE_FOR_sse2_movsd_v2df = 6030,
  CODE_FOR_vec_dupv2df = 6031,
  CODE_FOR_vec_dupv2df_mask = 6032,
  CODE_FOR_vec_concatv2df = 6033,
  CODE_FOR_vec_setv8df_0 = 6034,
  CODE_FOR_vec_setv4df_0 = 6035,
  CODE_FOR_avx512f_ss_truncatev16siv16qi2_mask = 6053,
  CODE_FOR_avx512f_truncatev16siv16qi2_mask = 6054,
  CODE_FOR_avx512f_us_truncatev16siv16qi2_mask = 6055,
  CODE_FOR_avx512f_ss_truncatev16siv16hi2_mask = 6056,
  CODE_FOR_avx512f_truncatev16siv16hi2_mask = 6057,
  CODE_FOR_avx512f_us_truncatev16siv16hi2_mask = 6058,
  CODE_FOR_avx512f_ss_truncatev8div8si2_mask = 6059,
  CODE_FOR_avx512f_truncatev8div8si2_mask = 6060,
  CODE_FOR_avx512f_us_truncatev8div8si2_mask = 6061,
  CODE_FOR_avx512f_ss_truncatev8div8hi2_mask = 6062,
  CODE_FOR_avx512f_truncatev8div8hi2_mask = 6063,
  CODE_FOR_avx512f_us_truncatev8div8hi2_mask = 6064,
  CODE_FOR_avx512bw_ss_truncatev32hiv32qi2 = 6065,
  CODE_FOR_avx512bw_truncatev32hiv32qi2 = 6066,
  CODE_FOR_avx512bw_us_truncatev32hiv32qi2 = 6067,
  CODE_FOR_avx512bw_ss_truncatev32hiv32qi2_mask = 6069,
  CODE_FOR_avx512bw_truncatev32hiv32qi2_mask = 6070,
  CODE_FOR_avx512bw_us_truncatev32hiv32qi2_mask = 6071,
  CODE_FOR_avx512vl_ss_truncatev4div4si2_mask = 6083,
  CODE_FOR_avx512vl_truncatev4div4si2_mask = 6084,
  CODE_FOR_avx512vl_us_truncatev4div4si2_mask = 6085,
  CODE_FOR_avx512vl_ss_truncatev8siv8hi2_mask = 6086,
  CODE_FOR_avx512vl_truncatev8siv8hi2_mask = 6087,
  CODE_FOR_avx512vl_us_truncatev8siv8hi2_mask = 6088,
  CODE_FOR_avx512vl_ss_truncatev16hiv16qi2_mask = 6089,
  CODE_FOR_avx512vl_truncatev16hiv16qi2_mask = 6090,
  CODE_FOR_avx512vl_us_truncatev16hiv16qi2_mask = 6091,
  CODE_FOR_avx512vl_ss_truncatev4div4qi2 = 6092,
  CODE_FOR_avx512vl_truncatev4div4qi2 = 6093,
  CODE_FOR_avx512vl_us_truncatev4div4qi2 = 6094,
  CODE_FOR_avx512vl_ss_truncatev2div2qi2 = 6095,
  CODE_FOR_avx512vl_truncatev2div2qi2 = 6096,
  CODE_FOR_avx512vl_us_truncatev2div2qi2 = 6097,
  CODE_FOR_avx512vl_ss_truncatev8siv8qi2 = 6098,
  CODE_FOR_avx512vl_truncatev8siv8qi2 = 6099,
  CODE_FOR_avx512vl_us_truncatev8siv8qi2 = 6100,
  CODE_FOR_avx512vl_ss_truncatev4siv4qi2 = 6101,
  CODE_FOR_avx512vl_truncatev4siv4qi2 = 6102,
  CODE_FOR_avx512vl_us_truncatev4siv4qi2 = 6103,
  CODE_FOR_avx512vl_ss_truncatev8hiv8qi2 = 6104,
  CODE_FOR_avx512vl_truncatev8hiv8qi2 = 6105,
  CODE_FOR_avx512vl_us_truncatev8hiv8qi2 = 6106,
  CODE_FOR_avx512vl_ss_truncatev2div2qi2_mask = 6114,
  CODE_FOR_avx512vl_truncatev2div2qi2_mask = 6115,
  CODE_FOR_avx512vl_us_truncatev2div2qi2_mask = 6116,
  CODE_FOR_avx512vl_ss_truncatev2div2qi2_mask_store_2 = 6123,
  CODE_FOR_avx512vl_truncatev2div2qi2_mask_store_2 = 6124,
  CODE_FOR_avx512vl_us_truncatev2div2qi2_mask_store_2 = 6125,
  CODE_FOR_avx512vl_ss_truncatev4siv4qi2_mask = 6138,
  CODE_FOR_avx512vl_truncatev4siv4qi2_mask = 6139,
  CODE_FOR_avx512vl_us_truncatev4siv4qi2_mask = 6140,
  CODE_FOR_avx512vl_ss_truncatev4div4qi2_mask = 6141,
  CODE_FOR_avx512vl_truncatev4div4qi2_mask = 6142,
  CODE_FOR_avx512vl_us_truncatev4div4qi2_mask = 6143,
  CODE_FOR_avx512vl_ss_truncatev4siv4qi2_mask_store_2 = 6156,
  CODE_FOR_avx512vl_truncatev4siv4qi2_mask_store_2 = 6157,
  CODE_FOR_avx512vl_us_truncatev4siv4qi2_mask_store_2 = 6158,
  CODE_FOR_avx512vl_ss_truncatev4div4qi2_mask_store_2 = 6159,
  CODE_FOR_avx512vl_truncatev4div4qi2_mask_store_2 = 6160,
  CODE_FOR_avx512vl_us_truncatev4div4qi2_mask_store_2 = 6161,
  CODE_FOR_avx512vl_ss_truncatev8hiv8qi2_mask = 6174,
  CODE_FOR_avx512vl_truncatev8hiv8qi2_mask = 6175,
  CODE_FOR_avx512vl_us_truncatev8hiv8qi2_mask = 6176,
  CODE_FOR_avx512vl_ss_truncatev8siv8qi2_mask = 6177,
  CODE_FOR_avx512vl_truncatev8siv8qi2_mask = 6178,
  CODE_FOR_avx512vl_us_truncatev8siv8qi2_mask = 6179,
  CODE_FOR_avx512vl_ss_truncatev8hiv8qi2_mask_store_2 = 6192,
  CODE_FOR_avx512vl_truncatev8hiv8qi2_mask_store_2 = 6193,
  CODE_FOR_avx512vl_us_truncatev8hiv8qi2_mask_store_2 = 6194,
  CODE_FOR_avx512vl_ss_truncatev8siv8qi2_mask_store_2 = 6195,
  CODE_FOR_avx512vl_truncatev8siv8qi2_mask_store_2 = 6196,
  CODE_FOR_avx512vl_us_truncatev8siv8qi2_mask_store_2 = 6197,
  CODE_FOR_avx512vl_ss_truncatev4div4hi2 = 6198,
  CODE_FOR_avx512vl_truncatev4div4hi2 = 6199,
  CODE_FOR_avx512vl_us_truncatev4div4hi2 = 6200,
  CODE_FOR_avx512vl_ss_truncatev2div2hi2 = 6201,
  CODE_FOR_avx512vl_truncatev2div2hi2 = 6202,
  CODE_FOR_avx512vl_us_truncatev2div2hi2 = 6203,
  CODE_FOR_avx512vl_ss_truncatev4siv4hi2 = 6204,
  CODE_FOR_avx512vl_truncatev4siv4hi2 = 6205,
  CODE_FOR_avx512vl_us_truncatev4siv4hi2 = 6206,
  CODE_FOR_avx512vl_ss_truncatev4siv4hi2_mask = 6220,
  CODE_FOR_avx512vl_truncatev4siv4hi2_mask = 6221,
  CODE_FOR_avx512vl_us_truncatev4siv4hi2_mask = 6222,
  CODE_FOR_avx512vl_ss_truncatev4div4hi2_mask = 6223,
  CODE_FOR_avx512vl_truncatev4div4hi2_mask = 6224,
  CODE_FOR_avx512vl_us_truncatev4div4hi2_mask = 6225,
  CODE_FOR_avx512vl_ss_truncatev4siv4hi2_mask_store_2 = 6238,
  CODE_FOR_avx512vl_truncatev4siv4hi2_mask_store_2 = 6239,
  CODE_FOR_avx512vl_us_truncatev4siv4hi2_mask_store_2 = 6240,
  CODE_FOR_avx512vl_ss_truncatev4div4hi2_mask_store_2 = 6241,
  CODE_FOR_avx512vl_truncatev4div4hi2_mask_store_2 = 6242,
  CODE_FOR_avx512vl_us_truncatev4div4hi2_mask_store_2 = 6243,
  CODE_FOR_avx512vl_ss_truncatev2div2hi2_mask = 6250,
  CODE_FOR_avx512vl_truncatev2div2hi2_mask = 6251,
  CODE_FOR_avx512vl_us_truncatev2div2hi2_mask = 6252,
  CODE_FOR_avx512vl_ss_truncatev2div2hi2_mask_store_2 = 6259,
  CODE_FOR_avx512vl_truncatev2div2hi2_mask_store_2 = 6260,
  CODE_FOR_avx512vl_us_truncatev2div2hi2_mask_store_2 = 6261,
  CODE_FOR_avx512vl_ss_truncatev2div2si2 = 6262,
  CODE_FOR_avx512vl_truncatev2div2si2 = 6263,
  CODE_FOR_avx512vl_us_truncatev2div2si2 = 6264,
  CODE_FOR_avx512vl_ss_truncatev2div2si2_mask = 6272,
  CODE_FOR_avx512vl_truncatev2div2si2_mask = 6273,
  CODE_FOR_avx512vl_us_truncatev2div2si2_mask = 6274,
  CODE_FOR_avx512vl_ss_truncatev2div2si2_mask_store_2 = 6281,
  CODE_FOR_avx512vl_truncatev2div2si2_mask_store_2 = 6282,
  CODE_FOR_avx512vl_us_truncatev2div2si2_mask_store_2 = 6283,
  CODE_FOR_avx512f_ss_truncatev8div16qi2 = 6284,
  CODE_FOR_avx512f_truncatev8div16qi2 = 6285,
  CODE_FOR_avx512f_us_truncatev8div16qi2 = 6286,
  CODE_FOR_avx512f_ss_truncatev8div16qi2_mask = 6293,
  CODE_FOR_avx512f_truncatev8div16qi2_mask = 6294,
  CODE_FOR_avx512f_us_truncatev8div16qi2_mask = 6295,
  CODE_FOR_avx512f_ss_truncatev8div16qi2_mask_store_2 = 6302,
  CODE_FOR_avx512f_truncatev8div16qi2_mask_store_2 = 6303,
  CODE_FOR_avx512f_us_truncatev8div16qi2_mask_store_2 = 6304,
  CODE_FOR_avx512bw_pmaddwd512v32hi = 6431,
  CODE_FOR_avx512bw_pmaddwd512v32hi_mask = 6432,
  CODE_FOR_avx512bw_pmaddwd512v16hi = 6433,
  CODE_FOR_avx512bw_pmaddwd512v16hi_mask = 6434,
  CODE_FOR_avx512bw_pmaddwd512v8hi = 6435,
  CODE_FOR_avx512bw_pmaddwd512v8hi_mask = 6436,
  CODE_FOR_ashrv16hi3_mask = 6452,
  CODE_FOR_ashrv8hi3_mask = 6454,
  CODE_FOR_ashrv8si3_mask = 6456,
  CODE_FOR_ashrv4si3_mask = 6458,
  CODE_FOR_ashrv2di3_mask = 6460,
  CODE_FOR_ashrv16hi3 = 6461,
  CODE_FOR_ashrv8hi3 = 6462,
  CODE_FOR_ashrv8si3 = 6463,
  CODE_FOR_ashrv4si3 = 6464,
  CODE_FOR_ashrv32hi3_mask = 6466,
  CODE_FOR_ashrv4di3_mask = 6468,
  CODE_FOR_ashrv16si3_mask = 6470,
  CODE_FOR_ashrv8di3_mask = 6472,
  CODE_FOR_ashlv16hi3_mask = 6474,
  CODE_FOR_lshrv16hi3_mask = 6476,
  CODE_FOR_ashlv8hi3_mask = 6478,
  CODE_FOR_lshrv8hi3_mask = 6480,
  CODE_FOR_ashlv8si3_mask = 6482,
  CODE_FOR_lshrv8si3_mask = 6484,
  CODE_FOR_ashlv4si3_mask = 6486,
  CODE_FOR_lshrv4si3_mask = 6488,
  CODE_FOR_ashlv4di3_mask = 6490,
  CODE_FOR_lshrv4di3_mask = 6492,
  CODE_FOR_ashlv2di3_mask = 6494,
  CODE_FOR_lshrv2di3_mask = 6496,
  CODE_FOR_ashlv16hi3 = 6497,
  CODE_FOR_lshrv16hi3 = 6498,
  CODE_FOR_ashlv8hi3 = 6499,
  CODE_FOR_lshrv8hi3 = 6500,
  CODE_FOR_ashlv8si3 = 6501,
  CODE_FOR_lshrv8si3 = 6502,
  CODE_FOR_ashlv4si3 = 6503,
  CODE_FOR_lshrv4si3 = 6504,
  CODE_FOR_ashlv4di3 = 6505,
  CODE_FOR_lshrv4di3 = 6506,
  CODE_FOR_ashlv2di3 = 6507,
  CODE_FOR_lshrv2di3 = 6508,
  CODE_FOR_ashlv32hi3 = 6509,
  CODE_FOR_ashlv32hi3_mask = 6510,
  CODE_FOR_lshrv32hi3 = 6511,
  CODE_FOR_lshrv32hi3_mask = 6512,
  CODE_FOR_ashlv16si3 = 6513,
  CODE_FOR_ashlv16si3_mask = 6514,
  CODE_FOR_lshrv16si3 = 6515,
  CODE_FOR_lshrv16si3_mask = 6516,
  CODE_FOR_ashlv8di3 = 6517,
  CODE_FOR_ashlv8di3_mask = 6518,
  CODE_FOR_lshrv8di3 = 6519,
  CODE_FOR_lshrv8di3_mask = 6520,
  CODE_FOR_avx512bw_ashlv4ti3 = 6526,
  CODE_FOR_avx512bw_lshrv4ti3 = 6527,
  CODE_FOR_avx512bw_ashlv2ti3 = 6528,
  CODE_FOR_avx512bw_lshrv2ti3 = 6529,
  CODE_FOR_avx512bw_ashlv1ti3 = 6530,
  CODE_FOR_avx512bw_lshrv1ti3 = 6531,
  CODE_FOR_avx2_ashlv2ti3 = 6532,
  CODE_FOR_avx2_lshrv2ti3 = 6533,
  CODE_FOR_sse2_ashlv1ti3 = 6534,
  CODE_FOR_sse2_lshrv1ti3 = 6535,
  CODE_FOR_avx512f_rolvv16si = 6536,
  CODE_FOR_avx512f_rolvv16si_mask = 6537,
  CODE_FOR_avx512f_rorvv16si = 6538,
  CODE_FOR_avx512f_rorvv16si_mask = 6539,
  CODE_FOR_avx512vl_rolvv8si = 6540,
  CODE_FOR_avx512vl_rolvv8si_mask = 6541,
  CODE_FOR_avx512vl_rorvv8si = 6542,
  CODE_FOR_avx512vl_rorvv8si_mask = 6543,
  CODE_FOR_avx512vl_rolvv4si = 6544,
  CODE_FOR_avx512vl_rolvv4si_mask = 6545,
  CODE_FOR_avx512vl_rorvv4si = 6546,
  CODE_FOR_avx512vl_rorvv4si_mask = 6547,
  CODE_FOR_avx512f_rolvv8di = 6548,
  CODE_FOR_avx512f_rolvv8di_mask = 6549,
  CODE_FOR_avx512f_rorvv8di = 6550,
  CODE_FOR_avx512f_rorvv8di_mask = 6551,
  CODE_FOR_avx512vl_rolvv4di = 6552,
  CODE_FOR_avx512vl_rolvv4di_mask = 6553,
  CODE_FOR_avx512vl_rorvv4di = 6554,
  CODE_FOR_avx512vl_rorvv4di_mask = 6555,
  CODE_FOR_avx512vl_rolvv2di = 6556,
  CODE_FOR_avx512vl_rolvv2di_mask = 6557,
  CODE_FOR_avx512vl_rorvv2di = 6558,
  CODE_FOR_avx512vl_rorvv2di_mask = 6559,
  CODE_FOR_avx512f_rolv16si = 6560,
  CODE_FOR_avx512f_rolv16si_mask = 6561,
  CODE_FOR_avx512f_rorv16si = 6562,
  CODE_FOR_avx512f_rorv16si_mask = 6563,
  CODE_FOR_avx512vl_rolv8si = 6564,
  CODE_FOR_avx512vl_rolv8si_mask = 6565,
  CODE_FOR_avx512vl_rorv8si = 6566,
  CODE_FOR_avx512vl_rorv8si_mask = 6567,
  CODE_FOR_avx512vl_rolv4si = 6568,
  CODE_FOR_avx512vl_rolv4si_mask = 6569,
  CODE_FOR_avx512vl_rorv4si = 6570,
  CODE_FOR_avx512vl_rorv4si_mask = 6571,
  CODE_FOR_avx512f_rolv8di = 6572,
  CODE_FOR_avx512f_rolv8di_mask = 6573,
  CODE_FOR_avx512f_rorv8di = 6574,
  CODE_FOR_avx512f_rorv8di_mask = 6575,
  CODE_FOR_avx512vl_rolv4di = 6576,
  CODE_FOR_avx512vl_rolv4di_mask = 6577,
  CODE_FOR_avx512vl_rorv4di = 6578,
  CODE_FOR_avx512vl_rorv4di_mask = 6579,
  CODE_FOR_avx512vl_rolv2di = 6580,
  CODE_FOR_avx512vl_rolv2di_mask = 6581,
  CODE_FOR_avx512vl_rorv2di = 6582,
  CODE_FOR_avx512vl_rorv2di_mask = 6583,
  CODE_FOR_smaxv64qi3_mask = 6645,
  CODE_FOR_sminv64qi3_mask = 6647,
  CODE_FOR_umaxv64qi3_mask = 6649,
  CODE_FOR_uminv64qi3_mask = 6651,
  CODE_FOR_smaxv16qi3_mask = 6653,
  CODE_FOR_sminv16qi3_mask = 6655,
  CODE_FOR_umaxv16qi3_mask = 6657,
  CODE_FOR_uminv16qi3_mask = 6659,
  CODE_FOR_smaxv32qi3_mask = 6661,
  CODE_FOR_sminv32qi3_mask = 6663,
  CODE_FOR_umaxv32qi3_mask = 6665,
  CODE_FOR_uminv32qi3_mask = 6667,
  CODE_FOR_smaxv32hi3_mask = 6669,
  CODE_FOR_sminv32hi3_mask = 6671,
  CODE_FOR_umaxv32hi3_mask = 6673,
  CODE_FOR_uminv32hi3_mask = 6675,
  CODE_FOR_smaxv16hi3_mask = 6677,
  CODE_FOR_sminv16hi3_mask = 6679,
  CODE_FOR_umaxv16hi3_mask = 6681,
  CODE_FOR_uminv16hi3_mask = 6683,
  CODE_FOR_smaxv8hi3_mask = 6685,
  CODE_FOR_sminv8hi3_mask = 6687,
  CODE_FOR_umaxv8hi3_mask = 6689,
  CODE_FOR_uminv8hi3_mask = 6691,
  CODE_FOR_sse4_2_gtv2di3 = 6742,
  CODE_FOR_avx2_gtv32qi3 = 6743,
  CODE_FOR_avx2_gtv16hi3 = 6744,
  CODE_FOR_avx2_gtv8si3 = 6745,
  CODE_FOR_avx2_gtv4di3 = 6746,
  CODE_FOR_one_cmplv16si2_mask = 6751,
  CODE_FOR_one_cmplv8di2_mask = 6753,
   CODE_FOR_one_cmplv64qi2_mask = CODE_FOR_nothing,
   CODE_FOR_one_cmplv32qi2_mask = CODE_FOR_nothing,
   CODE_FOR_one_cmplv16qi2_mask = CODE_FOR_nothing,
   CODE_FOR_one_cmplv32hi2_mask = CODE_FOR_nothing,
   CODE_FOR_one_cmplv16hi2_mask = CODE_FOR_nothing,
   CODE_FOR_one_cmplv8hi2_mask = CODE_FOR_nothing,
  CODE_FOR_one_cmplv8si2_mask = 6761,
  CODE_FOR_one_cmplv4si2_mask = 6763,
  CODE_FOR_one_cmplv4di2_mask = 6765,
  CODE_FOR_one_cmplv2di2_mask = 6767,
  CODE_FOR_andv1ti3 = 6840,
  CODE_FOR_iorv1ti3 = 6841,
  CODE_FOR_xorv1ti3 = 6842,
  CODE_FOR_avx512bw_testmv64qi3 = 6843,
  CODE_FOR_avx512bw_testmv64qi3_mask = 6844,
  CODE_FOR_avx512vl_testmv32qi3 = 6845,
  CODE_FOR_avx512vl_testmv32qi3_mask = 6846,
  CODE_FOR_avx512vl_testmv16qi3 = 6847,
  CODE_FOR_avx512vl_testmv16qi3_mask = 6848,
  CODE_FOR_avx512bw_testmv32hi3 = 6849,
  CODE_FOR_avx512bw_testmv32hi3_mask = 6850,
  CODE_FOR_avx512vl_testmv16hi3 = 6851,
  CODE_FOR_avx512vl_testmv16hi3_mask = 6852,
  CODE_FOR_avx512vl_testmv8hi3 = 6853,
  CODE_FOR_avx512vl_testmv8hi3_mask = 6854,
  CODE_FOR_avx512f_testmv16si3 = 6855,
  CODE_FOR_avx512f_testmv16si3_mask = 6856,
  CODE_FOR_avx512vl_testmv8si3 = 6857,
  CODE_FOR_avx512vl_testmv8si3_mask = 6858,
  CODE_FOR_avx512vl_testmv4si3 = 6859,
  CODE_FOR_avx512vl_testmv4si3_mask = 6860,
  CODE_FOR_avx512f_testmv8di3 = 6861,
  CODE_FOR_avx512f_testmv8di3_mask = 6862,
  CODE_FOR_avx512vl_testmv4di3 = 6863,
  CODE_FOR_avx512vl_testmv4di3_mask = 6864,
  CODE_FOR_avx512vl_testmv2di3 = 6865,
  CODE_FOR_avx512vl_testmv2di3_mask = 6866,
  CODE_FOR_avx512bw_testnmv64qi3 = 6867,
  CODE_FOR_avx512bw_testnmv64qi3_mask = 6868,
  CODE_FOR_avx512vl_testnmv32qi3 = 6869,
  CODE_FOR_avx512vl_testnmv32qi3_mask = 6870,
  CODE_FOR_avx512vl_testnmv16qi3 = 6871,
  CODE_FOR_avx512vl_testnmv16qi3_mask = 6872,
  CODE_FOR_avx512bw_testnmv32hi3 = 6873,
  CODE_FOR_avx512bw_testnmv32hi3_mask = 6874,
  CODE_FOR_avx512vl_testnmv16hi3 = 6875,
  CODE_FOR_avx512vl_testnmv16hi3_mask = 6876,
  CODE_FOR_avx512vl_testnmv8hi3 = 6877,
  CODE_FOR_avx512vl_testnmv8hi3_mask = 6878,
  CODE_FOR_avx512f_testnmv16si3 = 6879,
  CODE_FOR_avx512f_testnmv16si3_mask = 6880,
  CODE_FOR_avx512vl_testnmv8si3 = 6881,
  CODE_FOR_avx512vl_testnmv8si3_mask = 6882,
  CODE_FOR_avx512vl_testnmv4si3 = 6883,
  CODE_FOR_avx512vl_testnmv4si3_mask = 6884,
  CODE_FOR_avx512f_testnmv8di3 = 6885,
  CODE_FOR_avx512f_testnmv8di3_mask = 6886,
  CODE_FOR_avx512vl_testnmv4di3 = 6887,
  CODE_FOR_avx512vl_testnmv4di3_mask = 6888,
  CODE_FOR_avx512vl_testnmv2di3 = 6889,
  CODE_FOR_avx512vl_testnmv2di3_mask = 6890,
  CODE_FOR_avx512bw_packsswb = 7035,
  CODE_FOR_avx512bw_packsswb_mask = 7036,
  CODE_FOR_avx2_packsswb = 7037,
  CODE_FOR_avx2_packsswb_mask = 7038,
  CODE_FOR_sse2_packsswb = 7039,
  CODE_FOR_sse2_packsswb_mask = 7040,
  CODE_FOR_avx512bw_packssdw = 7041,
  CODE_FOR_avx512bw_packssdw_mask = 7042,
  CODE_FOR_avx2_packssdw = 7043,
  CODE_FOR_avx2_packssdw_mask = 7044,
  CODE_FOR_sse2_packssdw = 7045,
  CODE_FOR_sse2_packssdw_mask = 7046,
  CODE_FOR_avx512bw_packuswb = 7047,
  CODE_FOR_avx512bw_packuswb_mask = 7048,
  CODE_FOR_avx2_packuswb = 7049,
  CODE_FOR_avx2_packuswb_mask = 7050,
  CODE_FOR_sse2_packuswb = 7051,
  CODE_FOR_sse2_packuswb_mask = 7052,
  CODE_FOR_avx512bw_interleave_highv64qi = 7053,
  CODE_FOR_avx512bw_interleave_highv64qi_mask = 7054,
  CODE_FOR_avx2_interleave_highv32qi = 7055,
  CODE_FOR_avx2_interleave_highv32qi_mask = 7056,
  CODE_FOR_vec_interleave_highv16qi = 7057,
  CODE_FOR_vec_interleave_highv16qi_mask = 7058,
  CODE_FOR_avx512bw_interleave_lowv64qi = 7059,
  CODE_FOR_avx512bw_interleave_lowv64qi_mask = 7060,
  CODE_FOR_avx2_interleave_lowv32qi = 7061,
  CODE_FOR_avx2_interleave_lowv32qi_mask = 7062,
  CODE_FOR_vec_interleave_lowv16qi = 7063,
  CODE_FOR_vec_interleave_lowv16qi_mask = 7064,
  CODE_FOR_avx512bw_interleave_highv32hi = 7065,
  CODE_FOR_avx512bw_interleave_highv32hi_mask = 7066,
  CODE_FOR_avx512bw_interleave_highv32hf = 7067,
  CODE_FOR_avx512bw_interleave_highv32hf_mask = 7068,
  CODE_FOR_avx512bw_interleave_highv32bf = 7069,
  CODE_FOR_avx512bw_interleave_highv32bf_mask = 7070,
  CODE_FOR_avx2_interleave_highv16hi = 7071,
  CODE_FOR_avx2_interleave_highv16hi_mask = 7072,
  CODE_FOR_avx2_interleave_highv16hf = 7073,
  CODE_FOR_avx2_interleave_highv16hf_mask = 7074,
  CODE_FOR_avx2_interleave_highv16bf = 7075,
  CODE_FOR_avx2_interleave_highv16bf_mask = 7076,
  CODE_FOR_vec_interleave_highv8hi = 7077,
  CODE_FOR_vec_interleave_highv8hi_mask = 7078,
  CODE_FOR_vec_interleave_highv8hf = 7079,
  CODE_FOR_vec_interleave_highv8hf_mask = 7080,
  CODE_FOR_vec_interleave_highv8bf = 7081,
  CODE_FOR_vec_interleave_highv8bf_mask = 7082,
  CODE_FOR_avx512bw_interleave_lowv32hi_mask = 7084,
  CODE_FOR_avx512bw_interleave_lowv32hf_mask = 7086,
  CODE_FOR_avx512bw_interleave_lowv32bf_mask = 7088,
  CODE_FOR_avx2_interleave_lowv16hi = 7089,
  CODE_FOR_avx2_interleave_lowv16hi_mask = 7090,
  CODE_FOR_avx2_interleave_lowv16hf = 7091,
  CODE_FOR_avx2_interleave_lowv16hf_mask = 7092,
  CODE_FOR_avx2_interleave_lowv16bf = 7093,
  CODE_FOR_avx2_interleave_lowv16bf_mask = 7094,
  CODE_FOR_vec_interleave_lowv8hi = 7095,
  CODE_FOR_vec_interleave_lowv8hi_mask = 7096,
  CODE_FOR_vec_interleave_lowv8hf = 7097,
  CODE_FOR_vec_interleave_lowv8hf_mask = 7098,
  CODE_FOR_vec_interleave_lowv8bf = 7099,
  CODE_FOR_vec_interleave_lowv8bf_mask = 7100,
  CODE_FOR_avx2_interleave_highv8si = 7101,
  CODE_FOR_avx2_interleave_highv8si_mask = 7102,
  CODE_FOR_avx512f_interleave_highv16si_mask = 7104,
  CODE_FOR_vec_interleave_highv4si = 7105,
  CODE_FOR_vec_interleave_highv4si_mask = 7106,
  CODE_FOR_avx2_interleave_lowv8si = 7107,
  CODE_FOR_avx2_interleave_lowv8si_mask = 7108,
  CODE_FOR_avx512f_interleave_lowv16si_mask = 7110,
  CODE_FOR_vec_interleave_lowv4si = 7111,
  CODE_FOR_vec_interleave_lowv4si_mask = 7112,
  CODE_FOR_sse4_1_pinsrb = 7113,
  CODE_FOR_sse2_pinsrw = 7114,
  CODE_FOR_sse2_pinsrph = 7115,
  CODE_FOR_sse2_pinsrbf = 7116,
  CODE_FOR_sse4_1_pinsrd = 7117,
  CODE_FOR_sse4_1_pinsrq = 7118,
  CODE_FOR_avx512dq_vinsertf64x2_1_mask = 7124,
  CODE_FOR_avx512dq_vinserti64x2_1_mask = 7126,
  CODE_FOR_avx512f_vinsertf32x4_1_mask = 7128,
  CODE_FOR_avx512f_vinserti32x4_1_mask = 7130,
  CODE_FOR_vec_set_lo_v16sf = 7131,
  CODE_FOR_vec_set_lo_v16sf_mask = 7132,
  CODE_FOR_vec_set_lo_v16si = 7133,
  CODE_FOR_vec_set_lo_v16si_mask = 7134,
  CODE_FOR_vec_set_hi_v16sf = 7135,
  CODE_FOR_vec_set_hi_v16sf_mask = 7136,
  CODE_FOR_vec_set_hi_v16si = 7137,
  CODE_FOR_vec_set_hi_v16si_mask = 7138,
  CODE_FOR_vec_set_lo_v8df = 7139,
  CODE_FOR_vec_set_lo_v8df_mask = 7140,
  CODE_FOR_vec_set_lo_v8di = 7141,
  CODE_FOR_vec_set_lo_v8di_mask = 7142,
  CODE_FOR_vec_set_hi_v8df = 7143,
  CODE_FOR_vec_set_hi_v8df_mask = 7144,
  CODE_FOR_vec_set_hi_v8di = 7145,
  CODE_FOR_vec_set_hi_v8di_mask = 7146,
  CODE_FOR_avx512dq_shuf_i64x2_1_mask = 7148,
  CODE_FOR_avx512dq_shuf_f64x2_1_mask = 7150,
  CODE_FOR_avx512f_shuf_f64x2_1 = 7151,
  CODE_FOR_avx512f_shuf_f64x2_1_mask = 7152,
  CODE_FOR_avx512f_shuf_i64x2_1 = 7153,
  CODE_FOR_avx512f_shuf_i64x2_1_mask = 7154,
  CODE_FOR_avx512vl_shuf_i32x4_1 = 7159,
  CODE_FOR_avx512vl_shuf_i32x4_1_mask = 7160,
  CODE_FOR_avx512vl_shuf_f32x4_1 = 7161,
  CODE_FOR_avx512vl_shuf_f32x4_1_mask = 7162,
  CODE_FOR_avx512f_shuf_f32x4_1 = 7163,
  CODE_FOR_avx512f_shuf_f32x4_1_mask = 7164,
  CODE_FOR_avx512f_shuf_i32x4_1 = 7165,
  CODE_FOR_avx512f_shuf_i32x4_1_mask = 7166,
  CODE_FOR_avx512f_pshufd_1 = 7171,
  CODE_FOR_avx512f_pshufd_1_mask = 7172,
  CODE_FOR_avx2_pshufd_1 = 7173,
  CODE_FOR_avx2_pshufd_1_mask = 7174,
  CODE_FOR_sse2_pshufd_1 = 7175,
  CODE_FOR_sse2_pshufd_1_mask = 7176,
  CODE_FOR_avx512bw_pshuflwv32hi_mask = 7178,
  CODE_FOR_avx2_pshuflw_1 = 7179,
  CODE_FOR_avx2_pshuflw_1_mask = 7180,
  CODE_FOR_sse2_pshuflw_1 = 7181,
  CODE_FOR_sse2_pshuflw_1_mask = 7182,
  CODE_FOR_avx512bw_pshufhwv32hi_mask = 7184,
  CODE_FOR_avx2_pshufhw_1 = 7185,
  CODE_FOR_avx2_pshufhw_1_mask = 7186,
  CODE_FOR_sse2_pshufhw_1 = 7187,
  CODE_FOR_sse2_pshufhw_1_mask = 7188,
  CODE_FOR_sse2_loadld = 7189,
  CODE_FOR_vec_concatv2di = 7223,
  CODE_FOR_vec_setv8di_0 = 7225,
  CODE_FOR_vec_setv4di_0 = 7226,
  CODE_FOR_avx_movmskps256 = 7242,
  CODE_FOR_sse_movmskps = 7243,
  CODE_FOR_avx_movmskpd256 = 7244,
  CODE_FOR_sse2_movmskpd = 7245,
  CODE_FOR_avx2_pmovmskb = 7278,
  CODE_FOR_sse2_pmovmskb = 7279,
  CODE_FOR_sse_ldmxcsr = 7291,
  CODE_FOR_sse_stmxcsr = 7292,
  CODE_FOR_sse2_clflush = 7293,
  CODE_FOR_sse3_mwait = 7294,
  CODE_FOR_sse3_monitor_si = 7295,
  CODE_FOR_sse3_monitor_di = 7296,
  CODE_FOR_avx2_phaddwv16hi3 = 7297,
  CODE_FOR_avx2_phaddswv16hi3 = 7298,
  CODE_FOR_avx2_phsubwv16hi3 = 7299,
  CODE_FOR_avx2_phsubswv16hi3 = 7300,
  CODE_FOR_ssse3_phaddwv8hi3 = 7301,
  CODE_FOR_ssse3_phaddswv8hi3 = 7302,
  CODE_FOR_ssse3_phsubwv8hi3 = 7303,
  CODE_FOR_ssse3_phsubswv8hi3 = 7304,
  CODE_FOR_ssse3_phaddwv4hi3 = 7305,
  CODE_FOR_ssse3_phaddswv4hi3 = 7306,
  CODE_FOR_ssse3_phsubwv4hi3 = 7307,
  CODE_FOR_ssse3_phsubswv4hi3 = 7308,
  CODE_FOR_avx2_phadddv8si3 = 7309,
  CODE_FOR_avx2_phsubdv8si3 = 7310,
  CODE_FOR_ssse3_phadddv4si3 = 7311,
  CODE_FOR_ssse3_phsubdv4si3 = 7312,
  CODE_FOR_ssse3_phadddv2si3 = 7313,
  CODE_FOR_ssse3_phsubdv2si3 = 7314,
  CODE_FOR_avx2_pmaddubsw256 = 7315,
  CODE_FOR_avx512bw_pmaddubsw512v8hi = 7316,
  CODE_FOR_avx512bw_pmaddubsw512v8hi_mask = 7317,
  CODE_FOR_avx512bw_pmaddubsw512v16hi = 7318,
  CODE_FOR_avx512bw_pmaddubsw512v16hi_mask = 7319,
  CODE_FOR_avx512bw_pmaddubsw512v32hi = 7320,
  CODE_FOR_avx512bw_pmaddubsw512v32hi_mask = 7321,
  CODE_FOR_avx512bw_umulhrswv32hi3 = 7322,
  CODE_FOR_avx512bw_umulhrswv32hi3_mask = 7323,
  CODE_FOR_ssse3_pmaddubsw128 = 7324,
  CODE_FOR_ssse3_pmaddubsw = 7325,
  CODE_FOR_avx512bw_pshufbv64qi3 = 7334,
  CODE_FOR_avx512bw_pshufbv64qi3_mask = 7335,
  CODE_FOR_avx2_pshufbv32qi3 = 7336,
  CODE_FOR_avx2_pshufbv32qi3_mask = 7337,
  CODE_FOR_ssse3_pshufbv16qi3 = 7338,
  CODE_FOR_ssse3_pshufbv16qi3_mask = 7339,
  CODE_FOR_avx2_psignv32qi3 = 7341,
  CODE_FOR_ssse3_psignv16qi3 = 7342,
  CODE_FOR_avx2_psignv16hi3 = 7343,
  CODE_FOR_ssse3_psignv8hi3 = 7344,
  CODE_FOR_avx2_psignv8si3 = 7345,
  CODE_FOR_ssse3_psignv4si3 = 7346,
  CODE_FOR_ssse3_psignv8qi3 = 7347,
  CODE_FOR_ssse3_psignv4hi3 = 7348,
  CODE_FOR_ssse3_psignv2si3 = 7349,
  CODE_FOR_avx512bw_palignrv64qi_mask = 7350,
  CODE_FOR_avx2_palignrv32qi_mask = 7351,
  CODE_FOR_ssse3_palignrv16qi_mask = 7352,
  CODE_FOR_avx512bw_palignrv4ti = 7353,
  CODE_FOR_avx2_palignrv2ti = 7354,
  CODE_FOR_ssse3_palignrv1ti = 7355,
  CODE_FOR_ssse3_palignrdi = 7356,
  CODE_FOR_absv16si2_mask = 7369,
  CODE_FOR_absv8si2_mask = 7370,
  CODE_FOR_absv4si2_mask = 7371,
  CODE_FOR_absv8di2_mask = 7372,
  CODE_FOR_absv4di2_mask = 7373,
  CODE_FOR_absv2di2_mask = 7374,
  CODE_FOR_absv64qi2_mask = 7375,
  CODE_FOR_absv16qi2_mask = 7376,
  CODE_FOR_absv32qi2_mask = 7377,
  CODE_FOR_absv32hi2_mask = 7378,
  CODE_FOR_absv16hi2_mask = 7379,
  CODE_FOR_absv8hi2_mask = 7380,
  CODE_FOR_sse4a_movntsf = 7381,
  CODE_FOR_sse4a_movntdf = 7382,
  CODE_FOR_sse4a_vmmovntv4sf = 7383,
  CODE_FOR_sse4a_vmmovntv2df = 7384,
  CODE_FOR_sse4a_extrqi = 7385,
  CODE_FOR_sse4a_extrq = 7386,
  CODE_FOR_sse4a_insertqi = 7387,
  CODE_FOR_sse4a_insertq = 7388,
  CODE_FOR_avx_blendps256 = 7389,
  CODE_FOR_sse4_1_blendps = 7390,
  CODE_FOR_avx_blendpd256 = 7391,
  CODE_FOR_sse4_1_blendpd = 7392,
  CODE_FOR_avx_blendvps256 = 7393,
  CODE_FOR_sse4_1_blendvps = 7394,
  CODE_FOR_avx_blendvpd256 = 7395,
  CODE_FOR_sse4_1_blendvpd = 7396,
  CODE_FOR_sse4_1_blendvss = 7397,
  CODE_FOR_sse4_1_blendvsd = 7398,
  CODE_FOR_avx_dpps256 = 7411,
  CODE_FOR_sse4_1_dpps = 7412,
  CODE_FOR_avx_dppd256 = 7413,
  CODE_FOR_sse4_1_dppd = 7414,
  CODE_FOR_avx512f_movntdqa = 7415,
  CODE_FOR_avx2_movntdqa = 7416,
  CODE_FOR_sse4_1_movntdqa = 7417,
  CODE_FOR_avx2_mpsadbw = 7418,
  CODE_FOR_sse4_1_mpsadbw = 7419,
  CODE_FOR_avx512bw_packusdw = 7420,
  CODE_FOR_avx512bw_packusdw_mask = 7421,
  CODE_FOR_avx2_packusdw = 7422,
  CODE_FOR_avx2_packusdw_mask = 7423,
  CODE_FOR_sse4_1_packusdw = 7424,
  CODE_FOR_sse4_1_packusdw_mask = 7425,
  CODE_FOR_avx2_pblendvb = 7426,
  CODE_FOR_sse4_1_pblendvb = 7427,
  CODE_FOR_sse4_1_pblendw = 7432,
  CODE_FOR_sse4_1_pblendph = 7433,
  CODE_FOR_sse4_1_pblendbf = 7434,
  CODE_FOR_avx2_pblenddv8si = 7438,
  CODE_FOR_avx2_pblenddv4si = 7439,
  CODE_FOR_sse4_1_phminposuw = 7440,
  CODE_FOR_avx2_sign_extendv16qiv16hi2 = 7441,
  CODE_FOR_avx2_sign_extendv16qiv16hi2_mask = 7442,
  CODE_FOR_avx2_zero_extendv16qiv16hi2 = 7443,
  CODE_FOR_avx2_zero_extendv16qiv16hi2_mask = 7444,
  CODE_FOR_avx512bw_sign_extendv32qiv32hi2 = 7449,
  CODE_FOR_avx512bw_sign_extendv32qiv32hi2_mask = 7450,
  CODE_FOR_avx512bw_zero_extendv32qiv32hi2 = 7451,
  CODE_FOR_avx512bw_zero_extendv32qiv32hi2_mask = 7452,
  CODE_FOR_sse4_1_sign_extendv8qiv8hi2 = 7457,
  CODE_FOR_sse4_1_sign_extendv8qiv8hi2_mask = 7458,
  CODE_FOR_sse4_1_zero_extendv8qiv8hi2 = 7459,
  CODE_FOR_sse4_1_zero_extendv8qiv8hi2_mask = 7460,
  CODE_FOR_avx512f_sign_extendv16qiv16si2_mask = 7474,
  CODE_FOR_avx512f_zero_extendv16qiv16si2_mask = 7476,
  CODE_FOR_avx2_sign_extendv8qiv8si2 = 7477,
  CODE_FOR_avx2_sign_extendv8qiv8si2_mask = 7478,
  CODE_FOR_avx2_zero_extendv8qiv8si2 = 7479,
  CODE_FOR_avx2_zero_extendv8qiv8si2_mask = 7480,
  CODE_FOR_sse4_1_sign_extendv4qiv4si2 = 7489,
  CODE_FOR_sse4_1_sign_extendv4qiv4si2_mask = 7490,
  CODE_FOR_sse4_1_zero_extendv4qiv4si2 = 7491,
  CODE_FOR_sse4_1_zero_extendv4qiv4si2_mask = 7492,
  CODE_FOR_avx512f_sign_extendv16hiv16si2 = 7501,
  CODE_FOR_avx512f_sign_extendv16hiv16si2_mask = 7502,
  CODE_FOR_avx512f_zero_extendv16hiv16si2 = 7503,
  CODE_FOR_avx512f_zero_extendv16hiv16si2_mask = 7504,
  CODE_FOR_avx512f_zero_extendv16hiv16si2_1 = 7505,
  CODE_FOR_avx2_sign_extendv8hiv8si2 = 7509,
  CODE_FOR_avx2_sign_extendv8hiv8si2_mask = 7510,
  CODE_FOR_avx2_zero_extendv8hiv8si2 = 7511,
  CODE_FOR_avx2_zero_extendv8hiv8si2_mask = 7512,
  CODE_FOR_avx2_zero_extendv8hiv8si2_1 = 7513,
  CODE_FOR_sse4_1_sign_extendv4hiv4si2 = 7517,
  CODE_FOR_sse4_1_sign_extendv4hiv4si2_mask = 7518,
  CODE_FOR_sse4_1_zero_extendv4hiv4si2 = 7519,
  CODE_FOR_sse4_1_zero_extendv4hiv4si2_mask = 7520,
  CODE_FOR_avx512f_sign_extendv8qiv8di2 = 7533,
  CODE_FOR_avx512f_sign_extendv8qiv8di2_mask = 7534,
  CODE_FOR_avx512f_zero_extendv8qiv8di2 = 7535,
  CODE_FOR_avx512f_zero_extendv8qiv8di2_mask = 7536,
  CODE_FOR_avx2_sign_extendv4qiv4di2 = 7545,
  CODE_FOR_avx2_sign_extendv4qiv4di2_mask = 7546,
  CODE_FOR_avx2_zero_extendv4qiv4di2 = 7547,
  CODE_FOR_avx2_zero_extendv4qiv4di2_mask = 7548,
  CODE_FOR_sse4_1_sign_extendv2qiv2di2 = 7557,
  CODE_FOR_sse4_1_sign_extendv2qiv2di2_mask = 7558,
  CODE_FOR_sse4_1_zero_extendv2qiv2di2 = 7559,
  CODE_FOR_sse4_1_zero_extendv2qiv2di2_mask = 7560,
  CODE_FOR_avx512f_sign_extendv8hiv8di2 = 7568,
  CODE_FOR_avx512f_sign_extendv8hiv8di2_mask = 7569,
  CODE_FOR_avx512f_zero_extendv8hiv8di2 = 7570,
  CODE_FOR_avx512f_zero_extendv8hiv8di2_mask = 7571,
  CODE_FOR_avx2_sign_extendv4hiv4di2 = 7572,
  CODE_FOR_avx2_sign_extendv4hiv4di2_mask = 7573,
  CODE_FOR_avx2_zero_extendv4hiv4di2 = 7574,
  CODE_FOR_avx2_zero_extendv4hiv4di2_mask = 7575,
  CODE_FOR_sse4_1_sign_extendv2hiv2di2 = 7584,
  CODE_FOR_sse4_1_sign_extendv2hiv2di2_mask = 7585,
  CODE_FOR_sse4_1_zero_extendv2hiv2di2 = 7586,
  CODE_FOR_sse4_1_zero_extendv2hiv2di2_mask = 7587,
  CODE_FOR_avx512f_sign_extendv8siv8di2 = 7596,
  CODE_FOR_avx512f_sign_extendv8siv8di2_mask = 7597,
  CODE_FOR_avx512f_zero_extendv8siv8di2 = 7598,
  CODE_FOR_avx512f_zero_extendv8siv8di2_mask = 7599,
  CODE_FOR_avx2_sign_extendv4siv4di2 = 7602,
  CODE_FOR_avx2_sign_extendv4siv4di2_mask = 7603,
  CODE_FOR_avx2_zero_extendv4siv4di2 = 7604,
  CODE_FOR_avx2_zero_extendv4siv4di2_mask = 7605,
  CODE_FOR_sse4_1_sign_extendv2siv2di2 = 7608,
  CODE_FOR_sse4_1_sign_extendv2siv2di2_mask = 7609,
  CODE_FOR_sse4_1_zero_extendv2siv2di2 = 7610,
  CODE_FOR_sse4_1_zero_extendv2siv2di2_mask = 7611,
  CODE_FOR_avx_vtestps256 = 7622,
  CODE_FOR_avx_vtestps = 7623,
  CODE_FOR_avx_vtestpd256 = 7624,
  CODE_FOR_avx_vtestpd = 7625,
  CODE_FOR_sse4_1_ptestv16qi = 7626,
  CODE_FOR_sse4_1_ptestv8hi = 7627,
  CODE_FOR_sse4_1_ptestv4si = 7628,
  CODE_FOR_sse4_1_ptestv2di = 7629,
  CODE_FOR_sse4_1_ptestv1ti = 7630,
  CODE_FOR_sse4_1_ptestv4sf = 7631,
  CODE_FOR_sse4_1_ptestv2df = 7632,
  CODE_FOR_avx_ptestv32qi = 7633,
  CODE_FOR_avx_ptestv16hi = 7634,
  CODE_FOR_avx_ptestv8si = 7635,
  CODE_FOR_avx_ptestv4di = 7636,
  CODE_FOR_avx_ptestv2ti = 7637,
  CODE_FOR_avx_ptestv8sf = 7638,
  CODE_FOR_avx_ptestv4df = 7639,
  CODE_FOR_ptesttf2 = 7640,
  CODE_FOR_avx_roundps256 = 7641,
  CODE_FOR_sse4_1_roundps = 7642,
  CODE_FOR_avx_roundpd256 = 7643,
  CODE_FOR_sse4_1_roundpd = 7644,
  CODE_FOR_sse4_1_roundss = 7645,
  CODE_FOR_sse4_1_roundsd = 7646,
  CODE_FOR_sse4_2_pcmpestr = 7650,
  CODE_FOR_sse4_2_pcmpestri = 7651,
  CODE_FOR_sse4_2_pcmpestrm = 7652,
  CODE_FOR_sse4_2_pcmpestr_cconly = 7653,
  CODE_FOR_sse4_2_pcmpistr = 7654,
  CODE_FOR_sse4_2_pcmpistri = 7655,
  CODE_FOR_sse4_2_pcmpistrm = 7656,
  CODE_FOR_sse4_2_pcmpistr_cconly = 7657,
  CODE_FOR_avx512er_exp2v16sf = 7674,
  CODE_FOR_avx512er_exp2v16sf_round = 7675,
  CODE_FOR_avx512er_exp2v16sf_mask = 7676,
  CODE_FOR_avx512er_exp2v16sf_mask_round = 7677,
  CODE_FOR_avx512er_exp2v8df = 7678,
  CODE_FOR_avx512er_exp2v8df_round = 7679,
  CODE_FOR_avx512er_exp2v8df_mask = 7680,
  CODE_FOR_avx512er_exp2v8df_mask_round = 7681,
  CODE_FOR_avx512er_rcp28v16sf_mask = 7684,
  CODE_FOR_avx512er_rcp28v16sf_mask_round = 7685,
  CODE_FOR_avx512er_rcp28v8df_mask = 7688,
  CODE_FOR_avx512er_rcp28v8df_mask_round = 7689,
  CODE_FOR_avx512er_vmrcp28v4sf = 7690,
  CODE_FOR_avx512er_vmrcp28v4sf_round = 7691,
  CODE_FOR_avx512er_vmrcp28v4sf_mask = 7692,
  CODE_FOR_avx512er_vmrcp28v4sf_mask_round = 7693,
  CODE_FOR_avx512er_vmrcp28v2df = 7694,
  CODE_FOR_avx512er_vmrcp28v2df_round = 7695,
  CODE_FOR_avx512er_vmrcp28v2df_mask = 7696,
  CODE_FOR_avx512er_vmrcp28v2df_mask_round = 7697,
  CODE_FOR_avx512er_rsqrt28v16sf_mask = 7700,
  CODE_FOR_avx512er_rsqrt28v16sf_mask_round = 7701,
  CODE_FOR_avx512er_rsqrt28v8df_mask = 7704,
  CODE_FOR_avx512er_rsqrt28v8df_mask_round = 7705,
  CODE_FOR_avx512er_vmrsqrt28v4sf = 7706,
  CODE_FOR_avx512er_vmrsqrt28v4sf_round = 7707,
  CODE_FOR_avx512er_vmrsqrt28v4sf_mask = 7708,
  CODE_FOR_avx512er_vmrsqrt28v4sf_mask_round = 7709,
  CODE_FOR_avx512er_vmrsqrt28v2df = 7710,
  CODE_FOR_avx512er_vmrsqrt28v2df_round = 7711,
  CODE_FOR_avx512er_vmrsqrt28v2df_mask = 7712,
  CODE_FOR_avx512er_vmrsqrt28v2df_mask_round = 7713,
  CODE_FOR_xop_pmacsww = 7714,
  CODE_FOR_xop_pmacssww = 7715,
  CODE_FOR_xop_pmacsdd = 7716,
  CODE_FOR_xop_pmacssdd = 7717,
  CODE_FOR_xop_pmacsdql = 7718,
  CODE_FOR_xop_pmacssdql = 7719,
  CODE_FOR_xop_pmacsdqh = 7720,
  CODE_FOR_xop_pmacssdqh = 7721,
  CODE_FOR_xop_pmacswd = 7722,
  CODE_FOR_xop_pmacsswd = 7723,
  CODE_FOR_xop_pmadcswd = 7724,
  CODE_FOR_xop_pmadcsswd = 7725,
  CODE_FOR_xop_pcmov_v32qi256 = 7726,
  CODE_FOR_xop_pcmov_v16qi = 7727,
  CODE_FOR_xop_pcmov_v16hi256 = 7728,
  CODE_FOR_xop_pcmov_v8hi = 7729,
  CODE_FOR_xop_pcmov_v8si256 = 7730,
  CODE_FOR_xop_pcmov_v4si = 7731,
  CODE_FOR_xop_pcmov_v4di256 = 7732,
  CODE_FOR_xop_pcmov_v2di = 7733,
  CODE_FOR_xop_pcmov_v2ti256 = 7734,
  CODE_FOR_xop_pcmov_v1ti = 7735,
  CODE_FOR_xop_pcmov_v16hf256 = 7736,
  CODE_FOR_xop_pcmov_v8hf = 7737,
  CODE_FOR_xop_pcmov_v8sf256 = 7738,
  CODE_FOR_xop_pcmov_v4sf = 7739,
  CODE_FOR_xop_pcmov_v4df256 = 7740,
  CODE_FOR_xop_pcmov_v2df = 7741,
  CODE_FOR_xop_phaddbw = 7742,
  CODE_FOR_xop_phaddubw = 7743,
  CODE_FOR_xop_phaddbd = 7744,
  CODE_FOR_xop_phaddubd = 7745,
  CODE_FOR_xop_phaddbq = 7746,
  CODE_FOR_xop_phaddubq = 7747,
  CODE_FOR_xop_phaddwd = 7748,
  CODE_FOR_xop_phadduwd = 7749,
  CODE_FOR_xop_phaddwq = 7750,
  CODE_FOR_xop_phadduwq = 7751,
  CODE_FOR_xop_phadddq = 7752,
  CODE_FOR_xop_phaddudq = 7753,
  CODE_FOR_xop_phsubbw = 7754,
  CODE_FOR_xop_phsubwd = 7755,
  CODE_FOR_xop_phsubdq = 7756,
  CODE_FOR_xop_pperm = 7757,
  CODE_FOR_xop_pperm_pack_v2di_v4si = 7758,
  CODE_FOR_xop_pperm_pack_v4si_v8hi = 7759,
  CODE_FOR_xop_pperm_pack_v8hi_v16qi = 7760,
  CODE_FOR_xop_rotlv16qi3 = 7761,
  CODE_FOR_xop_rotlv8hi3 = 7762,
  CODE_FOR_xop_rotlv4si3 = 7763,
  CODE_FOR_xop_rotlv2di3 = 7764,
  CODE_FOR_xop_rotrv16qi3 = 7765,
  CODE_FOR_xop_rotrv8hi3 = 7766,
  CODE_FOR_xop_rotrv4si3 = 7767,
  CODE_FOR_xop_rotrv2di3 = 7768,
  CODE_FOR_xop_vrotlv16qi3 = 7769,
  CODE_FOR_xop_vrotlv8hi3 = 7770,
  CODE_FOR_xop_vrotlv4si3 = 7771,
  CODE_FOR_xop_vrotlv2di3 = 7772,
  CODE_FOR_xop_shav16qi3 = 7773,
  CODE_FOR_xop_shav8hi3 = 7774,
  CODE_FOR_xop_shav4si3 = 7775,
  CODE_FOR_xop_shav2di3 = 7776,
  CODE_FOR_xop_shlv16qi3 = 7777,
  CODE_FOR_xop_shlv8hi3 = 7778,
  CODE_FOR_xop_shlv4si3 = 7779,
  CODE_FOR_xop_shlv2di3 = 7780,
  CODE_FOR_xop_frczsf2 = 7781,
  CODE_FOR_xop_frczdf2 = 7782,
  CODE_FOR_xop_frczv4sf2 = 7783,
  CODE_FOR_xop_frczv2df2 = 7784,
  CODE_FOR_xop_frczv8sf2 = 7785,
  CODE_FOR_xop_frczv4df2 = 7786,
  CODE_FOR_xop_maskcmpv16qi3 = 7789,
  CODE_FOR_xop_maskcmpv8hi3 = 7790,
  CODE_FOR_xop_maskcmpv4si3 = 7791,
  CODE_FOR_xop_maskcmpv2di3 = 7792,
  CODE_FOR_xop_maskcmp_unsv16qi3 = 7793,
  CODE_FOR_xop_maskcmp_unsv8hi3 = 7794,
  CODE_FOR_xop_maskcmp_unsv4si3 = 7795,
  CODE_FOR_xop_maskcmp_unsv2di3 = 7796,
  CODE_FOR_xop_maskcmp_uns2v16qi3 = 7797,
  CODE_FOR_xop_maskcmp_uns2v8hi3 = 7798,
  CODE_FOR_xop_maskcmp_uns2v4si3 = 7799,
  CODE_FOR_xop_maskcmp_uns2v2di3 = 7800,
  CODE_FOR_xop_pcom_tfv16qi3 = 7801,
  CODE_FOR_xop_pcom_tfv8hi3 = 7802,
  CODE_FOR_xop_pcom_tfv4si3 = 7803,
  CODE_FOR_xop_pcom_tfv2di3 = 7804,
  CODE_FOR_xop_vpermil2v8sf3 = 7805,
  CODE_FOR_xop_vpermil2v4sf3 = 7806,
  CODE_FOR_xop_vpermil2v4df3 = 7807,
  CODE_FOR_xop_vpermil2v2df3 = 7808,
  CODE_FOR_aesenc = 7809,
  CODE_FOR_aesenclast = 7810,
  CODE_FOR_aesdec = 7811,
  CODE_FOR_aesdeclast = 7812,
  CODE_FOR_aesimc = 7813,
  CODE_FOR_aeskeygenassist = 7814,
  CODE_FOR_pclmulqdq = 7815,
  CODE_FOR_avx_vzeroupper_callee_abi = 7817,
  CODE_FOR_avx2_pbroadcastv16si = 7818,
  CODE_FOR_avx2_pbroadcastv8di = 7819,
  CODE_FOR_avx2_pbroadcastv64qi = 7820,
  CODE_FOR_avx2_pbroadcastv32qi = 7821,
  CODE_FOR_avx2_pbroadcastv16qi = 7822,
  CODE_FOR_avx2_pbroadcastv32hi = 7823,
  CODE_FOR_avx2_pbroadcastv16hi = 7824,
  CODE_FOR_avx2_pbroadcastv8hi = 7825,
  CODE_FOR_avx2_pbroadcastv8si = 7826,
  CODE_FOR_avx2_pbroadcastv4si = 7827,
  CODE_FOR_avx2_pbroadcastv4di = 7828,
  CODE_FOR_avx2_pbroadcastv2di = 7829,
  CODE_FOR_avx2_pbroadcastv32hf = 7830,
  CODE_FOR_avx2_pbroadcastv16hf = 7831,
  CODE_FOR_avx2_pbroadcastv8hf = 7832,
  CODE_FOR_avx2_pbroadcastv32bf = 7833,
  CODE_FOR_avx2_pbroadcastv16bf = 7834,
  CODE_FOR_avx2_pbroadcastv8bf = 7835,
  CODE_FOR_avx2_pbroadcastv32qi_1 = 7836,
  CODE_FOR_avx2_pbroadcastv16hi_1 = 7837,
  CODE_FOR_avx2_pbroadcastv8si_1 = 7838,
  CODE_FOR_avx2_pbroadcastv4di_1 = 7839,
  CODE_FOR_avx2_pbroadcastv16hf_1 = 7840,
  CODE_FOR_avx2_pbroadcastv16bf_1 = 7841,
  CODE_FOR_avx2_permvarv8si = 7842,
  CODE_FOR_avx2_permvarv8si_mask = 7843,
  CODE_FOR_avx2_permvarv8sf = 7844,
  CODE_FOR_avx2_permvarv8sf_mask = 7845,
  CODE_FOR_avx512f_permvarv16si = 7846,
  CODE_FOR_avx512f_permvarv16si_mask = 7847,
  CODE_FOR_avx512f_permvarv16sf = 7848,
  CODE_FOR_avx512f_permvarv16sf_mask = 7849,
  CODE_FOR_avx512f_permvarv8di = 7850,
  CODE_FOR_avx512f_permvarv8di_mask = 7851,
  CODE_FOR_avx512f_permvarv8df = 7852,
  CODE_FOR_avx512f_permvarv8df_mask = 7853,
  CODE_FOR_avx2_permvarv4di = 7854,
  CODE_FOR_avx2_permvarv4di_mask = 7855,
  CODE_FOR_avx2_permvarv4df = 7856,
  CODE_FOR_avx2_permvarv4df_mask = 7857,
  CODE_FOR_avx512bw_permvarv64qi = 7858,
  CODE_FOR_avx512bw_permvarv64qi_mask = 7859,
  CODE_FOR_avx512vl_permvarv16qi = 7860,
  CODE_FOR_avx512vl_permvarv16qi_mask = 7861,
  CODE_FOR_avx512vl_permvarv32qi = 7862,
  CODE_FOR_avx512vl_permvarv32qi_mask = 7863,
  CODE_FOR_avx512vl_permvarv8hi = 7864,
  CODE_FOR_avx512vl_permvarv8hi_mask = 7865,
  CODE_FOR_avx512vl_permvarv16hi = 7866,
  CODE_FOR_avx512vl_permvarv16hi_mask = 7867,
  CODE_FOR_avx512bw_permvarv32hi = 7868,
  CODE_FOR_avx512bw_permvarv32hi_mask = 7869,
  CODE_FOR_avx2_permv4di_1 = 7885,
  CODE_FOR_avx2_permv4di_1_mask = 7886,
  CODE_FOR_avx2_permv4df_1 = 7887,
  CODE_FOR_avx2_permv4df_1_mask = 7888,
  CODE_FOR_avx512f_permv8df_1 = 7889,
  CODE_FOR_avx512f_permv8df_1_mask = 7890,
  CODE_FOR_avx512f_permv8di_1 = 7891,
  CODE_FOR_avx512f_permv8di_1_mask = 7892,
  CODE_FOR_avx2_permv2ti = 7893,
  CODE_FOR_avx2_vec_dupv4df = 7894,
  CODE_FOR_avx512f_vec_dupv16si_1 = 7895,
  CODE_FOR_avx512f_vec_dupv8di_1 = 7896,
  CODE_FOR_avx512bw_vec_dupv32hi_1 = 7897,
  CODE_FOR_avx512bw_vec_dupv64qi_1 = 7898,
  CODE_FOR_avx512bw_vec_dupv32hf_1 = 7899,
  CODE_FOR_avx512bw_vec_dupv32bf_1 = 7900,
  CODE_FOR_avx512f_vec_dupv16si = 7901,
  CODE_FOR_avx512f_vec_dupv16si_mask = 7902,
  CODE_FOR_avx512vl_vec_dupv8si = 7903,
  CODE_FOR_avx512vl_vec_dupv8si_mask = 7904,
  CODE_FOR_avx512vl_vec_dupv4si = 7905,
  CODE_FOR_avx512vl_vec_dupv4si_mask = 7906,
  CODE_FOR_avx512f_vec_dupv8di = 7907,
  CODE_FOR_avx512f_vec_dupv8di_mask = 7908,
  CODE_FOR_avx512vl_vec_dupv4di = 7909,
  CODE_FOR_avx512vl_vec_dupv4di_mask = 7910,
  CODE_FOR_avx512vl_vec_dupv2di = 7911,
  CODE_FOR_avx512vl_vec_dupv2di_mask = 7912,
  CODE_FOR_avx512f_vec_dupv16sf = 7913,
  CODE_FOR_avx512f_vec_dupv16sf_mask = 7914,
  CODE_FOR_avx512vl_vec_dupv8sf = 7915,
  CODE_FOR_avx512vl_vec_dupv8sf_mask = 7916,
  CODE_FOR_avx512vl_vec_dupv4sf = 7917,
  CODE_FOR_avx512vl_vec_dupv4sf_mask = 7918,
  CODE_FOR_avx512f_vec_dupv8df = 7919,
  CODE_FOR_avx512f_vec_dupv8df_mask = 7920,
  CODE_FOR_avx512vl_vec_dupv4df = 7921,
  CODE_FOR_avx512vl_vec_dupv4df_mask = 7922,
  CODE_FOR_avx512vl_vec_dupv2df = 7923,
  CODE_FOR_avx512vl_vec_dupv2df_mask = 7924,
  CODE_FOR_avx512bw_vec_dupv64qi = 7925,
  CODE_FOR_avx512bw_vec_dupv64qi_mask = 7926,
  CODE_FOR_avx512vl_vec_dupv16qi = 7927,
  CODE_FOR_avx512vl_vec_dupv16qi_mask = 7928,
  CODE_FOR_avx512vl_vec_dupv32qi = 7929,
  CODE_FOR_avx512vl_vec_dupv32qi_mask = 7930,
  CODE_FOR_avx512bw_vec_dupv32hi = 7931,
  CODE_FOR_avx512bw_vec_dupv32hi_mask = 7932,
  CODE_FOR_avx512vl_vec_dupv16hi = 7933,
  CODE_FOR_avx512vl_vec_dupv16hi_mask = 7934,
  CODE_FOR_avx512vl_vec_dupv8hi = 7935,
  CODE_FOR_avx512vl_vec_dupv8hi_mask = 7936,
  CODE_FOR_avx512bw_vec_dupv32hf = 7937,
  CODE_FOR_avx512bw_vec_dupv32hf_mask = 7938,
  CODE_FOR_avx512vl_vec_dupv16hf = 7939,
  CODE_FOR_avx512vl_vec_dupv16hf_mask = 7940,
  CODE_FOR_avx512fp16_vec_dupv8hf = 7941,
  CODE_FOR_avx512fp16_vec_dupv8hf_mask = 7942,
  CODE_FOR_avx512bw_vec_dupv32bf = 7943,
  CODE_FOR_avx512bw_vec_dupv32bf_mask = 7944,
  CODE_FOR_avx512vl_vec_dupv16bf = 7945,
  CODE_FOR_avx512vl_vec_dupv16bf_mask = 7946,
  CODE_FOR_avx512vl_vec_dupv8bf = 7947,
  CODE_FOR_avx512vl_vec_dupv8bf_mask = 7948,
  CODE_FOR_avx512f_broadcastv16sf_mask = 7950,
  CODE_FOR_avx512f_broadcastv16si_mask = 7952,
  CODE_FOR_avx512f_broadcastv8df_mask = 7954,
  CODE_FOR_avx512f_broadcastv8di_mask = 7956,
  CODE_FOR_avx512bw_vec_dup_gprv64qi_mask = 7958,
  CODE_FOR_avx512vl_vec_dup_gprv16qi_mask = 7960,
  CODE_FOR_avx512vl_vec_dup_gprv32qi_mask = 7962,
  CODE_FOR_avx512bw_vec_dup_gprv32hi_mask = 7964,
  CODE_FOR_avx512vl_vec_dup_gprv16hi_mask = 7966,
  CODE_FOR_avx512vl_vec_dup_gprv8hi_mask = 7968,
  CODE_FOR_avx512bw_vec_dup_gprv32hf_mask = 7970,
  CODE_FOR_avx512vl_vec_dup_gprv16hf_mask = 7972,
  CODE_FOR_avx512fp16_vec_dup_gprv8hf_mask = 7974,
  CODE_FOR_avx512bw_vec_dup_gprv32bf_mask = 7976,
  CODE_FOR_avx512vl_vec_dup_gprv16bf_mask = 7978,
  CODE_FOR_avx512vl_vec_dup_gprv8bf_mask = 7980,
  CODE_FOR_avx512f_vec_dup_gprv16si_mask = 7982,
  CODE_FOR_avx512vl_vec_dup_gprv8si_mask = 7984,
  CODE_FOR_avx512vl_vec_dup_gprv4si_mask = 7986,
  CODE_FOR_avx512f_vec_dup_gprv8di_mask = 7988,
  CODE_FOR_avx512vl_vec_dup_gprv4di_mask = 7990,
  CODE_FOR_avx512vl_vec_dup_gprv2di_mask = 7992,
  CODE_FOR_avx512f_vec_dup_gprv16sf_mask = 7994,
  CODE_FOR_avx512vl_vec_dup_gprv8sf_mask = 7996,
  CODE_FOR_avx512vl_vec_dup_gprv4sf_mask = 7998,
  CODE_FOR_avx512f_vec_dup_gprv8df_mask = 8000,
  CODE_FOR_avx512vl_vec_dup_gprv4df_mask = 8002,
  CODE_FOR_avx512vl_vec_dup_gprv2df_mask = 8004,
  CODE_FOR_vec_dupv4sf = 8005,
  CODE_FOR_avx2_vbroadcasti128_v32qi = 8008,
  CODE_FOR_avx2_vbroadcasti128_v16hi = 8009,
  CODE_FOR_avx2_vbroadcasti128_v8si = 8010,
  CODE_FOR_avx2_vbroadcasti128_v4di = 8011,
  CODE_FOR_vec_dupv8si = 8022,
  CODE_FOR_vec_dupv8sf = 8023,
  CODE_FOR_vec_dupv4di = 8024,
  CODE_FOR_vec_dupv4df = 8025,
  CODE_FOR_avx_vbroadcastf128_v32qi = 8026,
  CODE_FOR_avx_vbroadcastf128_v16hi = 8027,
  CODE_FOR_avx_vbroadcastf128_v8si = 8028,
  CODE_FOR_avx_vbroadcastf128_v4di = 8029,
  CODE_FOR_avx_vbroadcastf128_v8sf = 8030,
  CODE_FOR_avx_vbroadcastf128_v4df = 8031,
  CODE_FOR_avx_vbroadcastf128_v16hf = 8032,
  CODE_FOR_avx_vbroadcastf128_v16bf = 8033,
  CODE_FOR_avx512dq_broadcastv16si_mask = 8035,
  CODE_FOR_avx512dq_broadcastv8si_mask = 8037,
  CODE_FOR_avx512dq_broadcastv4si_mask = 8039,
  CODE_FOR_avx512dq_broadcastv16sf_mask = 8041,
  CODE_FOR_avx512dq_broadcastv8sf_mask = 8043,
  CODE_FOR_avx512vl_broadcastv8si_mask_1 = 8045,
  CODE_FOR_avx512vl_broadcastv8sf_mask_1 = 8047,
  CODE_FOR_avx512dq_broadcastv16sf_mask_1 = 8049,
  CODE_FOR_avx512dq_broadcastv16si_mask_1 = 8051,
  CODE_FOR_avx512dq_broadcastv8di_mask_1 = 8053,
  CODE_FOR_avx512dq_broadcastv8df_mask_1 = 8055,
  CODE_FOR_avx512dq_broadcastv4di_mask_1 = 8057,
  CODE_FOR_avx512dq_broadcastv4df_mask_1 = 8059,
  CODE_FOR_avx512cd_maskb_vec_dupv8di = 8060,
  CODE_FOR_avx512cd_maskb_vec_dupv4di = 8061,
  CODE_FOR_avx512cd_maskb_vec_dupv2di = 8062,
  CODE_FOR_avx512cd_maskw_vec_dupv16si = 8063,
  CODE_FOR_avx512cd_maskw_vec_dupv8si = 8064,
  CODE_FOR_avx512cd_maskw_vec_dupv4si = 8065,
  CODE_FOR_avx512f_vpermilvarv16sf3 = 8066,
  CODE_FOR_avx512f_vpermilvarv16sf3_mask = 8067,
  CODE_FOR_avx_vpermilvarv8sf3 = 8068,
  CODE_FOR_avx_vpermilvarv8sf3_mask = 8069,
  CODE_FOR_avx_vpermilvarv4sf3 = 8070,
  CODE_FOR_avx_vpermilvarv4sf3_mask = 8071,
  CODE_FOR_avx512f_vpermilvarv8df3 = 8072,
  CODE_FOR_avx512f_vpermilvarv8df3_mask = 8073,
  CODE_FOR_avx_vpermilvarv4df3 = 8074,
  CODE_FOR_avx_vpermilvarv4df3_mask = 8075,
  CODE_FOR_avx_vpermilvarv2df3 = 8076,
  CODE_FOR_avx_vpermilvarv2df3_mask = 8077,
  CODE_FOR_avx512f_vpermt2varv16si3 = 8096,
  CODE_FOR_avx512f_vpermt2varv16si3_maskz_1 = 8097,
  CODE_FOR_avx512f_vpermt2varv16sf3 = 8098,
  CODE_FOR_avx512f_vpermt2varv16sf3_maskz_1 = 8099,
  CODE_FOR_avx512f_vpermt2varv8di3 = 8100,
  CODE_FOR_avx512f_vpermt2varv8di3_maskz_1 = 8101,
  CODE_FOR_avx512f_vpermt2varv8df3 = 8102,
  CODE_FOR_avx512f_vpermt2varv8df3_maskz_1 = 8103,
  CODE_FOR_avx512vl_vpermt2varv8si3 = 8104,
  CODE_FOR_avx512vl_vpermt2varv8si3_maskz_1 = 8105,
  CODE_FOR_avx512vl_vpermt2varv8sf3 = 8106,
  CODE_FOR_avx512vl_vpermt2varv8sf3_maskz_1 = 8107,
  CODE_FOR_avx512vl_vpermt2varv4di3 = 8108,
  CODE_FOR_avx512vl_vpermt2varv4di3_maskz_1 = 8109,
  CODE_FOR_avx512vl_vpermt2varv4df3 = 8110,
  CODE_FOR_avx512vl_vpermt2varv4df3_maskz_1 = 8111,
  CODE_FOR_avx512vl_vpermt2varv4si3 = 8112,
  CODE_FOR_avx512vl_vpermt2varv4si3_maskz_1 = 8113,
  CODE_FOR_avx512vl_vpermt2varv4sf3 = 8114,
  CODE_FOR_avx512vl_vpermt2varv4sf3_maskz_1 = 8115,
  CODE_FOR_avx512vl_vpermt2varv2di3 = 8116,
  CODE_FOR_avx512vl_vpermt2varv2di3_maskz_1 = 8117,
  CODE_FOR_avx512vl_vpermt2varv2df3 = 8118,
  CODE_FOR_avx512vl_vpermt2varv2df3_maskz_1 = 8119,
  CODE_FOR_avx512bw_vpermt2varv32hi3 = 8120,
  CODE_FOR_avx512bw_vpermt2varv32hi3_maskz_1 = 8121,
  CODE_FOR_avx512vl_vpermt2varv16hi3 = 8122,
  CODE_FOR_avx512vl_vpermt2varv16hi3_maskz_1 = 8123,
  CODE_FOR_avx512vl_vpermt2varv8hi3 = 8124,
  CODE_FOR_avx512vl_vpermt2varv8hi3_maskz_1 = 8125,
  CODE_FOR_avx512bw_vpermt2varv64qi3 = 8126,
  CODE_FOR_avx512bw_vpermt2varv64qi3_maskz_1 = 8127,
  CODE_FOR_avx512vl_vpermt2varv32qi3 = 8128,
  CODE_FOR_avx512vl_vpermt2varv32qi3_maskz_1 = 8129,
  CODE_FOR_avx512vl_vpermt2varv16qi3 = 8130,
  CODE_FOR_avx512vl_vpermt2varv16qi3_maskz_1 = 8131,
  CODE_FOR_avx512f_vpermt2varv16si3_mask = 8132,
  CODE_FOR_avx512f_vpermt2varv16sf3_mask = 8133,
  CODE_FOR_avx512f_vpermt2varv8di3_mask = 8134,
  CODE_FOR_avx512f_vpermt2varv8df3_mask = 8135,
  CODE_FOR_avx512vl_vpermt2varv8si3_mask = 8136,
  CODE_FOR_avx512vl_vpermt2varv8sf3_mask = 8137,
  CODE_FOR_avx512vl_vpermt2varv4di3_mask = 8138,
  CODE_FOR_avx512vl_vpermt2varv4df3_mask = 8139,
  CODE_FOR_avx512vl_vpermt2varv4si3_mask = 8140,
  CODE_FOR_avx512vl_vpermt2varv4sf3_mask = 8141,
  CODE_FOR_avx512vl_vpermt2varv2di3_mask = 8142,
  CODE_FOR_avx512vl_vpermt2varv2df3_mask = 8143,
  CODE_FOR_avx512bw_vpermt2varv32hi3_mask = 8144,
  CODE_FOR_avx512vl_vpermt2varv16hi3_mask = 8145,
  CODE_FOR_avx512vl_vpermt2varv8hi3_mask = 8146,
  CODE_FOR_avx512bw_vpermt2varv64qi3_mask = 8147,
  CODE_FOR_avx512vl_vpermt2varv32qi3_mask = 8148,
  CODE_FOR_avx512vl_vpermt2varv16qi3_mask = 8149,
  CODE_FOR_vec_set_lo_v4di = 8162,
  CODE_FOR_vec_set_lo_v4di_mask = 8163,
  CODE_FOR_vec_set_lo_v4df = 8164,
  CODE_FOR_vec_set_lo_v4df_mask = 8165,
  CODE_FOR_vec_set_hi_v4di = 8166,
  CODE_FOR_vec_set_hi_v4di_mask = 8167,
  CODE_FOR_vec_set_hi_v4df = 8168,
  CODE_FOR_vec_set_hi_v4df_mask = 8169,
  CODE_FOR_vec_set_lo_v8si = 8170,
  CODE_FOR_vec_set_lo_v8si_mask = 8171,
  CODE_FOR_vec_set_lo_v8sf = 8172,
  CODE_FOR_vec_set_lo_v8sf_mask = 8173,
  CODE_FOR_vec_set_hi_v8si = 8174,
  CODE_FOR_vec_set_hi_v8si_mask = 8175,
  CODE_FOR_vec_set_hi_v8sf = 8176,
  CODE_FOR_vec_set_hi_v8sf_mask = 8177,
  CODE_FOR_vec_set_lo_v16hi = 8178,
  CODE_FOR_vec_set_lo_v16hf = 8179,
  CODE_FOR_vec_set_lo_v16bf = 8180,
  CODE_FOR_vec_set_hi_v16hi = 8181,
  CODE_FOR_vec_set_hi_v16hf = 8182,
  CODE_FOR_vec_set_hi_v16bf = 8183,
  CODE_FOR_vec_set_lo_v32qi = 8184,
  CODE_FOR_vec_set_hi_v32qi = 8185,
  CODE_FOR_avx_maskloadps = 8186,
  CODE_FOR_avx_maskloadpd = 8187,
  CODE_FOR_avx_maskloadps256 = 8188,
  CODE_FOR_avx_maskloadpd256 = 8189,
  CODE_FOR_avx2_maskloadd = 8190,
  CODE_FOR_avx2_maskloadq = 8191,
  CODE_FOR_avx2_maskloadd256 = 8192,
  CODE_FOR_avx2_maskloadq256 = 8193,
  CODE_FOR_avx_maskstoreps = 8194,
  CODE_FOR_avx_maskstorepd = 8195,
  CODE_FOR_avx_maskstoreps256 = 8196,
  CODE_FOR_avx_maskstorepd256 = 8197,
  CODE_FOR_avx2_maskstored = 8198,
  CODE_FOR_avx2_maskstoreq = 8199,
  CODE_FOR_avx2_maskstored256 = 8200,
  CODE_FOR_avx2_maskstoreq256 = 8201,
  CODE_FOR_avx512f_storev16si_mask = 8202,
  CODE_FOR_avx512vl_storev8si_mask = 8203,
  CODE_FOR_avx512vl_storev4si_mask = 8204,
  CODE_FOR_avx512f_storev8di_mask = 8205,
  CODE_FOR_avx512vl_storev4di_mask = 8206,
  CODE_FOR_avx512vl_storev2di_mask = 8207,
  CODE_FOR_avx512f_storev16sf_mask = 8208,
  CODE_FOR_avx512vl_storev8sf_mask = 8209,
  CODE_FOR_avx512vl_storev4sf_mask = 8210,
  CODE_FOR_avx512f_storev8df_mask = 8211,
  CODE_FOR_avx512vl_storev4df_mask = 8212,
  CODE_FOR_avx512vl_storev2df_mask = 8213,
  CODE_FOR_avx512bw_storev64qi_mask = 8214,
  CODE_FOR_avx512vl_storev16qi_mask = 8215,
  CODE_FOR_avx512vl_storev32qi_mask = 8216,
  CODE_FOR_avx512bw_storev32hi_mask = 8217,
  CODE_FOR_avx512vl_storev16hi_mask = 8218,
  CODE_FOR_avx512vl_storev8hi_mask = 8219,
  CODE_FOR_avx512bw_storev32hf_mask = 8220,
  CODE_FOR_avx512vl_storev16hf_mask = 8221,
  CODE_FOR_avx512fp16_storev8hf_mask = 8222,
  CODE_FOR_avx512bw_storev32bf_mask = 8223,
  CODE_FOR_avx512vl_storev16bf_mask = 8224,
  CODE_FOR_avx512vl_storev8bf_mask = 8225,
  CODE_FOR_avx_si256_si = 8226,
  CODE_FOR_avx_ps256_ps = 8227,
  CODE_FOR_avx_pd256_pd = 8228,
  CODE_FOR_avx2_ashrvv4si = 8229,
  CODE_FOR_avx2_ashrvv4si_mask = 8230,
  CODE_FOR_avx2_ashrvv8si = 8231,
  CODE_FOR_avx2_ashrvv8si_mask = 8232,
  CODE_FOR_avx512f_ashrvv16si = 8233,
  CODE_FOR_avx512f_ashrvv16si_mask = 8234,
  CODE_FOR_avx2_ashrvv2di = 8235,
  CODE_FOR_avx2_ashrvv2di_mask = 8236,
  CODE_FOR_avx2_ashrvv4di = 8237,
  CODE_FOR_avx2_ashrvv4di_mask = 8238,
  CODE_FOR_avx512f_ashrvv8di = 8239,
  CODE_FOR_avx512f_ashrvv8di_mask = 8240,
  CODE_FOR_avx512vl_ashrvv8hi = 8241,
  CODE_FOR_avx512vl_ashrvv8hi_mask = 8242,
  CODE_FOR_avx512vl_ashrvv16hi = 8243,
  CODE_FOR_avx512vl_ashrvv16hi_mask = 8244,
  CODE_FOR_avx512bw_ashrvv32hi = 8245,
  CODE_FOR_avx512bw_ashrvv32hi_mask = 8246,
  CODE_FOR_avx512f_ashlvv16si = 8247,
  CODE_FOR_avx512f_ashlvv16si_mask = 8248,
  CODE_FOR_avx512f_lshrvv16si = 8249,
  CODE_FOR_avx512f_lshrvv16si_mask = 8250,
  CODE_FOR_avx2_ashlvv8si = 8251,
  CODE_FOR_avx2_ashlvv8si_mask = 8252,
  CODE_FOR_avx2_lshrvv8si = 8253,
  CODE_FOR_avx2_lshrvv8si_mask = 8254,
  CODE_FOR_avx2_ashlvv4si = 8255,
  CODE_FOR_avx2_ashlvv4si_mask = 8256,
  CODE_FOR_avx2_lshrvv4si = 8257,
  CODE_FOR_avx2_lshrvv4si_mask = 8258,
  CODE_FOR_avx512f_ashlvv8di = 8259,
  CODE_FOR_avx512f_ashlvv8di_mask = 8260,
  CODE_FOR_avx512f_lshrvv8di = 8261,
  CODE_FOR_avx512f_lshrvv8di_mask = 8262,
  CODE_FOR_avx2_ashlvv4di = 8263,
  CODE_FOR_avx2_ashlvv4di_mask = 8264,
  CODE_FOR_avx2_lshrvv4di = 8265,
  CODE_FOR_avx2_lshrvv4di_mask = 8266,
  CODE_FOR_avx2_ashlvv2di = 8267,
  CODE_FOR_avx2_ashlvv2di_mask = 8268,
  CODE_FOR_avx2_lshrvv2di = 8269,
  CODE_FOR_avx2_lshrvv2di_mask = 8270,
  CODE_FOR_avx512vl_ashlvv8hi = 8271,
  CODE_FOR_avx512vl_ashlvv8hi_mask = 8272,
  CODE_FOR_avx512vl_lshrvv8hi = 8273,
  CODE_FOR_avx512vl_lshrvv8hi_mask = 8274,
  CODE_FOR_avx512vl_ashlvv16hi = 8275,
  CODE_FOR_avx512vl_ashlvv16hi_mask = 8276,
  CODE_FOR_avx512vl_lshrvv16hi = 8277,
  CODE_FOR_avx512vl_lshrvv16hi_mask = 8278,
  CODE_FOR_avx512bw_ashlvv32hi = 8279,
  CODE_FOR_avx512bw_ashlvv32hi_mask = 8280,
  CODE_FOR_avx512bw_lshrvv32hi = 8281,
  CODE_FOR_avx512bw_lshrvv32hi_mask = 8282,
  CODE_FOR_avx_vec_concatv32qi = 8283,
  CODE_FOR_avx_vec_concatv16hi = 8284,
  CODE_FOR_avx_vec_concatv16hf = 8285,
  CODE_FOR_avx_vec_concatv16bf = 8286,
  CODE_FOR_avx_vec_concatv8si = 8287,
  CODE_FOR_avx_vec_concatv4di = 8288,
  CODE_FOR_avx_vec_concatv8sf = 8289,
  CODE_FOR_avx_vec_concatv4df = 8290,
  CODE_FOR_avx_vec_concatv64qi = 8291,
  CODE_FOR_avx_vec_concatv32hi = 8292,
  CODE_FOR_avx_vec_concatv32hf = 8293,
  CODE_FOR_avx_vec_concatv32bf = 8294,
  CODE_FOR_avx_vec_concatv16si = 8295,
  CODE_FOR_avx_vec_concatv8di = 8296,
  CODE_FOR_avx_vec_concatv16sf = 8297,
  CODE_FOR_avx_vec_concatv8df = 8298,
  CODE_FOR_vcvtph2ps = 8320,
  CODE_FOR_vcvtph2ps_mask = 8321,
  CODE_FOR_vcvtph2ps256 = 8324,
  CODE_FOR_vcvtph2ps256_mask = 8325,
  CODE_FOR_avx512f_vcvtph2ps512_mask = 8328,
  CODE_FOR_avx512f_vcvtph2ps512_mask_round = 8329,
  CODE_FOR_vcvtps2ph256 = 8333,
  CODE_FOR_vcvtps2ph256_mask = 8334,
  CODE_FOR_avx512f_vcvtps2ph512_mask = 8339,
  CODE_FOR_avx512f_vcvtps2ph512_mask_round = 8340,
  CODE_FOR_avx512f_compressv16si_mask = 8559,
  CODE_FOR_avx512f_compressv16sf_mask = 8560,
  CODE_FOR_avx512f_compressv8di_mask = 8561,
  CODE_FOR_avx512f_compressv8df_mask = 8562,
  CODE_FOR_avx512vl_compressv8si_mask = 8563,
  CODE_FOR_avx512vl_compressv8sf_mask = 8564,
  CODE_FOR_avx512vl_compressv4di_mask = 8565,
  CODE_FOR_avx512vl_compressv4df_mask = 8566,
  CODE_FOR_avx512vl_compressv4si_mask = 8567,
  CODE_FOR_avx512vl_compressv4sf_mask = 8568,
  CODE_FOR_avx512vl_compressv2di_mask = 8569,
  CODE_FOR_avx512vl_compressv2df_mask = 8570,
  CODE_FOR_compressv64qi_mask = 8571,
  CODE_FOR_compressv16qi_mask = 8572,
  CODE_FOR_compressv32qi_mask = 8573,
  CODE_FOR_compressv32hi_mask = 8574,
  CODE_FOR_compressv16hi_mask = 8575,
  CODE_FOR_compressv8hi_mask = 8576,
  CODE_FOR_avx512f_compressstorev16si_mask = 8577,
  CODE_FOR_avx512f_compressstorev16sf_mask = 8578,
  CODE_FOR_avx512f_compressstorev8di_mask = 8579,
  CODE_FOR_avx512f_compressstorev8df_mask = 8580,
  CODE_FOR_avx512vl_compressstorev8si_mask = 8581,
  CODE_FOR_avx512vl_compressstorev8sf_mask = 8582,
  CODE_FOR_avx512vl_compressstorev4di_mask = 8583,
  CODE_FOR_avx512vl_compressstorev4df_mask = 8584,
  CODE_FOR_avx512vl_compressstorev4si_mask = 8585,
  CODE_FOR_avx512vl_compressstorev4sf_mask = 8586,
  CODE_FOR_avx512vl_compressstorev2di_mask = 8587,
  CODE_FOR_avx512vl_compressstorev2df_mask = 8588,
  CODE_FOR_compressstorev64qi_mask = 8589,
  CODE_FOR_compressstorev16qi_mask = 8590,
  CODE_FOR_compressstorev32qi_mask = 8591,
  CODE_FOR_compressstorev32hi_mask = 8592,
  CODE_FOR_compressstorev16hi_mask = 8593,
  CODE_FOR_compressstorev8hi_mask = 8594,
  CODE_FOR_expandv16si_mask = 8595,
  CODE_FOR_expandv16sf_mask = 8596,
  CODE_FOR_expandv8di_mask = 8597,
  CODE_FOR_expandv8df_mask = 8598,
  CODE_FOR_expandv8si_mask = 8599,
  CODE_FOR_expandv8sf_mask = 8600,
  CODE_FOR_expandv4di_mask = 8601,
  CODE_FOR_expandv4df_mask = 8602,
  CODE_FOR_expandv4si_mask = 8603,
  CODE_FOR_expandv4sf_mask = 8604,
  CODE_FOR_expandv2di_mask = 8605,
  CODE_FOR_expandv2df_mask = 8606,
  CODE_FOR_expandv64qi_mask = 8607,
  CODE_FOR_expandv16qi_mask = 8608,
  CODE_FOR_expandv32qi_mask = 8609,
  CODE_FOR_expandv32hi_mask = 8610,
  CODE_FOR_expandv16hi_mask = 8611,
  CODE_FOR_expandv8hi_mask = 8612,
  CODE_FOR_avx512dq_rangepv16sf = 8631,
  CODE_FOR_avx512dq_rangepv16sf_round = 8632,
  CODE_FOR_avx512dq_rangepv16sf_mask = 8633,
  CODE_FOR_avx512dq_rangepv16sf_mask_round = 8634,
  CODE_FOR_avx512dq_rangepv8sf = 8635,
   CODE_FOR_avx512dq_rangepv8sf_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_rangepv8sf_mask = 8636,
   CODE_FOR_avx512dq_rangepv8sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_rangepv4sf = 8637,
   CODE_FOR_avx512dq_rangepv4sf_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_rangepv4sf_mask = 8638,
   CODE_FOR_avx512dq_rangepv4sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_rangepv8df = 8639,
  CODE_FOR_avx512dq_rangepv8df_round = 8640,
  CODE_FOR_avx512dq_rangepv8df_mask = 8641,
  CODE_FOR_avx512dq_rangepv8df_mask_round = 8642,
  CODE_FOR_avx512dq_rangepv4df = 8643,
   CODE_FOR_avx512dq_rangepv4df_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_rangepv4df_mask = 8644,
   CODE_FOR_avx512dq_rangepv4df_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_rangepv2df = 8645,
   CODE_FOR_avx512dq_rangepv2df_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_rangepv2df_mask = 8646,
   CODE_FOR_avx512dq_rangepv2df_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_rangesv4sf = 8647,
  CODE_FOR_avx512dq_rangesv4sf_mask = 8648,
  CODE_FOR_avx512dq_rangesv4sf_round = 8649,
  CODE_FOR_avx512dq_rangesv4sf_mask_round = 8650,
  CODE_FOR_avx512dq_rangesv2df = 8651,
  CODE_FOR_avx512dq_rangesv2df_mask = 8652,
  CODE_FOR_avx512dq_rangesv2df_round = 8653,
  CODE_FOR_avx512dq_rangesv2df_mask_round = 8654,
  CODE_FOR_avx512dq_fpclassv32hf = 8655,
  CODE_FOR_avx512dq_fpclassv32hf_mask = 8656,
  CODE_FOR_avx512dq_fpclassv16hf = 8657,
  CODE_FOR_avx512dq_fpclassv16hf_mask = 8658,
  CODE_FOR_avx512dq_fpclassv8hf = 8659,
  CODE_FOR_avx512dq_fpclassv8hf_mask = 8660,
  CODE_FOR_avx512dq_fpclassv16sf = 8661,
  CODE_FOR_avx512dq_fpclassv16sf_mask = 8662,
  CODE_FOR_avx512dq_fpclassv8sf = 8663,
  CODE_FOR_avx512dq_fpclassv8sf_mask = 8664,
  CODE_FOR_avx512dq_fpclassv4sf = 8665,
  CODE_FOR_avx512dq_fpclassv4sf_mask = 8666,
  CODE_FOR_avx512dq_fpclassv8df = 8667,
  CODE_FOR_avx512dq_fpclassv8df_mask = 8668,
  CODE_FOR_avx512dq_fpclassv4df = 8669,
  CODE_FOR_avx512dq_fpclassv4df_mask = 8670,
  CODE_FOR_avx512dq_fpclassv2df = 8671,
  CODE_FOR_avx512dq_fpclassv2df_mask = 8672,
  CODE_FOR_avx512dq_vmfpclassv8hf = 8673,
  CODE_FOR_avx512dq_vmfpclassv8hf_mask = 8674,
  CODE_FOR_avx512dq_vmfpclassv4sf = 8675,
  CODE_FOR_avx512dq_vmfpclassv4sf_mask = 8676,
  CODE_FOR_avx512dq_vmfpclassv2df = 8677,
  CODE_FOR_avx512dq_vmfpclassv2df_mask = 8678,
  CODE_FOR_avx512bw_getmantv32hf = 8679,
  CODE_FOR_avx512bw_getmantv32hf_round = 8680,
  CODE_FOR_avx512bw_getmantv32hf_mask = 8681,
  CODE_FOR_avx512bw_getmantv32hf_mask_round = 8682,
  CODE_FOR_avx512vl_getmantv16hf = 8683,
  CODE_FOR_avx512vl_getmantv16hf_round = 8684,
  CODE_FOR_avx512vl_getmantv16hf_mask = 8685,
  CODE_FOR_avx512vl_getmantv16hf_mask_round = 8686,
  CODE_FOR_avx512fp16_getmantv8hf = 8687,
  CODE_FOR_avx512fp16_getmantv8hf_round = 8688,
  CODE_FOR_avx512fp16_getmantv8hf_mask = 8689,
  CODE_FOR_avx512fp16_getmantv8hf_mask_round = 8690,
  CODE_FOR_avx512f_getmantv16sf = 8691,
  CODE_FOR_avx512f_getmantv16sf_round = 8692,
  CODE_FOR_avx512f_getmantv16sf_mask = 8693,
  CODE_FOR_avx512f_getmantv16sf_mask_round = 8694,
  CODE_FOR_avx512vl_getmantv8sf = 8695,
  CODE_FOR_avx512vl_getmantv8sf_round = 8696,
  CODE_FOR_avx512vl_getmantv8sf_mask = 8697,
  CODE_FOR_avx512vl_getmantv8sf_mask_round = 8698,
  CODE_FOR_avx512vl_getmantv4sf = 8699,
  CODE_FOR_avx512vl_getmantv4sf_round = 8700,
  CODE_FOR_avx512vl_getmantv4sf_mask = 8701,
  CODE_FOR_avx512vl_getmantv4sf_mask_round = 8702,
  CODE_FOR_avx512f_getmantv8df = 8703,
  CODE_FOR_avx512f_getmantv8df_round = 8704,
  CODE_FOR_avx512f_getmantv8df_mask = 8705,
  CODE_FOR_avx512f_getmantv8df_mask_round = 8706,
  CODE_FOR_avx512vl_getmantv4df = 8707,
  CODE_FOR_avx512vl_getmantv4df_round = 8708,
  CODE_FOR_avx512vl_getmantv4df_mask = 8709,
  CODE_FOR_avx512vl_getmantv4df_mask_round = 8710,
  CODE_FOR_avx512vl_getmantv2df = 8711,
  CODE_FOR_avx512vl_getmantv2df_round = 8712,
  CODE_FOR_avx512vl_getmantv2df_mask = 8713,
  CODE_FOR_avx512vl_getmantv2df_mask_round = 8714,
  CODE_FOR_avx512f_vgetmantv8hf = 8715,
  CODE_FOR_avx512f_vgetmantv8hf_mask = 8716,
  CODE_FOR_avx512f_vgetmantv8hf_round = 8717,
  CODE_FOR_avx512f_vgetmantv8hf_mask_round = 8718,
  CODE_FOR_avx512f_vgetmantv4sf = 8719,
  CODE_FOR_avx512f_vgetmantv4sf_mask = 8720,
  CODE_FOR_avx512f_vgetmantv4sf_round = 8721,
  CODE_FOR_avx512f_vgetmantv4sf_mask_round = 8722,
  CODE_FOR_avx512f_vgetmantv2df = 8723,
  CODE_FOR_avx512f_vgetmantv2df_mask = 8724,
  CODE_FOR_avx512f_vgetmantv2df_round = 8725,
  CODE_FOR_avx512f_vgetmantv2df_mask_round = 8726,
  CODE_FOR_avx512bw_dbpsadbwv8hi_mask = 8728,
  CODE_FOR_avx512bw_dbpsadbwv16hi_mask = 8730,
  CODE_FOR_avx512bw_dbpsadbwv32hi_mask = 8732,
  CODE_FOR_clzv16si2 = 8733,
  CODE_FOR_clzv16si2_mask = 8734,
  CODE_FOR_clzv8si2 = 8735,
  CODE_FOR_clzv8si2_mask = 8736,
  CODE_FOR_clzv4si2 = 8737,
  CODE_FOR_clzv4si2_mask = 8738,
  CODE_FOR_clzv8di2 = 8739,
  CODE_FOR_clzv8di2_mask = 8740,
  CODE_FOR_clzv4di2 = 8741,
  CODE_FOR_clzv4di2_mask = 8742,
  CODE_FOR_clzv2di2 = 8743,
  CODE_FOR_clzv2di2_mask = 8744,
  CODE_FOR_conflictv16si_mask = 8746,
  CODE_FOR_conflictv8si_mask = 8748,
  CODE_FOR_conflictv4si_mask = 8750,
  CODE_FOR_conflictv8di_mask = 8752,
  CODE_FOR_conflictv4di_mask = 8754,
  CODE_FOR_conflictv2di_mask = 8756,
  CODE_FOR_sha1msg1 = 8757,
  CODE_FOR_sha1msg2 = 8758,
  CODE_FOR_sha1nexte = 8759,
  CODE_FOR_sha1rnds4 = 8760,
  CODE_FOR_sha256msg1 = 8761,
  CODE_FOR_sha256msg2 = 8762,
  CODE_FOR_sha256rnds2 = 8763,
  CODE_FOR_avx512f_si512_si = 8764,
  CODE_FOR_avx512f_ps512_ps = 8765,
  CODE_FOR_avx512f_pd512_pd = 8766,
  CODE_FOR_avx512f_si512_256si = 8767,
  CODE_FOR_avx512f_ps512_256ps = 8768,
  CODE_FOR_avx512f_pd512_256pd = 8769,
  CODE_FOR_vpmadd52luqv8di = 8770,
  CODE_FOR_vpmadd52huqv8di = 8771,
  CODE_FOR_vpmadd52luqv4di = 8772,
  CODE_FOR_vpmadd52huqv4di = 8773,
  CODE_FOR_vpmadd52luqv2di = 8774,
  CODE_FOR_vpmadd52huqv2di = 8775,
  CODE_FOR_vpmadd52luqv8di_maskz_1 = 8776,
  CODE_FOR_vpmadd52huqv8di_maskz_1 = 8777,
  CODE_FOR_vpmadd52luqv4di_maskz_1 = 8778,
  CODE_FOR_vpmadd52huqv4di_maskz_1 = 8779,
  CODE_FOR_vpmadd52luqv2di_maskz_1 = 8780,
  CODE_FOR_vpmadd52huqv2di_maskz_1 = 8781,
  CODE_FOR_vpmadd52luqv8di_mask = 8782,
  CODE_FOR_vpmadd52huqv8di_mask = 8783,
  CODE_FOR_vpmadd52luqv4di_mask = 8784,
  CODE_FOR_vpmadd52huqv4di_mask = 8785,
  CODE_FOR_vpmadd52luqv2di_mask = 8786,
  CODE_FOR_vpmadd52huqv2di_mask = 8787,
  CODE_FOR_vpmultishiftqbv64qi = 8788,
  CODE_FOR_vpmultishiftqbv64qi_mask = 8789,
  CODE_FOR_vpmultishiftqbv16qi = 8790,
  CODE_FOR_vpmultishiftqbv16qi_mask = 8791,
  CODE_FOR_vpmultishiftqbv32qi = 8792,
  CODE_FOR_vpmultishiftqbv32qi_mask = 8793,
  CODE_FOR_avx5124fmaddps_4fmaddps = 8796,
  CODE_FOR_avx5124fmaddps_4fmaddps_mask = 8797,
  CODE_FOR_avx5124fmaddps_4fmaddps_maskz = 8798,
  CODE_FOR_avx5124fmaddps_4fmaddss = 8799,
  CODE_FOR_avx5124fmaddps_4fmaddss_mask = 8800,
  CODE_FOR_avx5124fmaddps_4fmaddss_maskz = 8801,
  CODE_FOR_avx5124fmaddps_4fnmaddps = 8802,
  CODE_FOR_avx5124fmaddps_4fnmaddps_mask = 8803,
  CODE_FOR_avx5124fmaddps_4fnmaddps_maskz = 8804,
  CODE_FOR_avx5124fmaddps_4fnmaddss = 8805,
  CODE_FOR_avx5124fmaddps_4fnmaddss_mask = 8806,
  CODE_FOR_avx5124fmaddps_4fnmaddss_maskz = 8807,
  CODE_FOR_avx5124vnniw_vp4dpwssd = 8808,
  CODE_FOR_avx5124vnniw_vp4dpwssd_mask = 8809,
  CODE_FOR_avx5124vnniw_vp4dpwssd_maskz = 8810,
  CODE_FOR_avx5124vnniw_vp4dpwssds = 8811,
  CODE_FOR_avx5124vnniw_vp4dpwssds_mask = 8812,
  CODE_FOR_avx5124vnniw_vp4dpwssds_maskz = 8813,
  CODE_FOR_vpopcountv16si = 8814,
  CODE_FOR_vpopcountv16si_mask = 8815,
  CODE_FOR_vpopcountv8si = 8816,
  CODE_FOR_vpopcountv8si_mask = 8817,
  CODE_FOR_vpopcountv4si = 8818,
  CODE_FOR_vpopcountv4si_mask = 8819,
  CODE_FOR_vpopcountv8di = 8820,
  CODE_FOR_vpopcountv8di_mask = 8821,
  CODE_FOR_vpopcountv4di = 8822,
  CODE_FOR_vpopcountv4di_mask = 8823,
  CODE_FOR_vpopcountv2di = 8824,
  CODE_FOR_vpopcountv2di_mask = 8825,
  CODE_FOR_vpopcountv64qi = 8834,
  CODE_FOR_vpopcountv64qi_mask = 8835,
  CODE_FOR_vpopcountv16qi = 8836,
  CODE_FOR_vpopcountv16qi_mask = 8837,
  CODE_FOR_vpopcountv32qi = 8838,
  CODE_FOR_vpopcountv32qi_mask = 8839,
  CODE_FOR_vpopcountv32hi = 8840,
  CODE_FOR_vpopcountv32hi_mask = 8841,
  CODE_FOR_vpopcountv16hi = 8842,
  CODE_FOR_vpopcountv16hi_mask = 8843,
  CODE_FOR_vpopcountv8hi = 8844,
  CODE_FOR_vpopcountv8hi_mask = 8845,
  CODE_FOR_vgf2p8affineinvqb_v64qi = 8846,
  CODE_FOR_vgf2p8affineinvqb_v64qi_mask = 8847,
  CODE_FOR_vgf2p8affineinvqb_v32qi = 8848,
  CODE_FOR_vgf2p8affineinvqb_v32qi_mask = 8849,
  CODE_FOR_vgf2p8affineinvqb_v16qi = 8850,
  CODE_FOR_vgf2p8affineinvqb_v16qi_mask = 8851,
  CODE_FOR_vgf2p8affineqb_v64qi = 8852,
  CODE_FOR_vgf2p8affineqb_v64qi_mask = 8853,
  CODE_FOR_vgf2p8affineqb_v32qi = 8854,
  CODE_FOR_vgf2p8affineqb_v32qi_mask = 8855,
  CODE_FOR_vgf2p8affineqb_v16qi = 8856,
  CODE_FOR_vgf2p8affineqb_v16qi_mask = 8857,
  CODE_FOR_vgf2p8mulb_v64qi = 8858,
  CODE_FOR_vgf2p8mulb_v64qi_mask = 8859,
  CODE_FOR_vgf2p8mulb_v32qi = 8860,
  CODE_FOR_vgf2p8mulb_v32qi_mask = 8861,
  CODE_FOR_vgf2p8mulb_v16qi = 8862,
  CODE_FOR_vgf2p8mulb_v16qi_mask = 8863,
  CODE_FOR_vpshrd_v32hi = 8864,
  CODE_FOR_vpshrd_v32hi_mask = 8865,
  CODE_FOR_vpshrd_v16si = 8866,
  CODE_FOR_vpshrd_v16si_mask = 8867,
  CODE_FOR_vpshrd_v8di = 8868,
  CODE_FOR_vpshrd_v8di_mask = 8869,
  CODE_FOR_vpshrd_v16hi = 8870,
  CODE_FOR_vpshrd_v16hi_mask = 8871,
  CODE_FOR_vpshrd_v8si = 8872,
  CODE_FOR_vpshrd_v8si_mask = 8873,
  CODE_FOR_vpshrd_v4di = 8874,
  CODE_FOR_vpshrd_v4di_mask = 8875,
  CODE_FOR_vpshrd_v8hi = 8876,
  CODE_FOR_vpshrd_v8hi_mask = 8877,
  CODE_FOR_vpshrd_v4si = 8878,
  CODE_FOR_vpshrd_v4si_mask = 8879,
  CODE_FOR_vpshrd_v2di = 8880,
  CODE_FOR_vpshrd_v2di_mask = 8881,
  CODE_FOR_vpshld_v32hi = 8882,
  CODE_FOR_vpshld_v32hi_mask = 8883,
  CODE_FOR_vpshld_v16si = 8884,
  CODE_FOR_vpshld_v16si_mask = 8885,
  CODE_FOR_vpshld_v8di = 8886,
  CODE_FOR_vpshld_v8di_mask = 8887,
  CODE_FOR_vpshld_v16hi = 8888,
  CODE_FOR_vpshld_v16hi_mask = 8889,
  CODE_FOR_vpshld_v8si = 8890,
  CODE_FOR_vpshld_v8si_mask = 8891,
  CODE_FOR_vpshld_v4di = 8892,
  CODE_FOR_vpshld_v4di_mask = 8893,
  CODE_FOR_vpshld_v8hi = 8894,
  CODE_FOR_vpshld_v8hi_mask = 8895,
  CODE_FOR_vpshld_v4si = 8896,
  CODE_FOR_vpshld_v4si_mask = 8897,
  CODE_FOR_vpshld_v2di = 8898,
  CODE_FOR_vpshld_v2di_mask = 8899,
  CODE_FOR_vpshrdv_v32hi = 8900,
  CODE_FOR_vpshrdv_v16si = 8901,
  CODE_FOR_vpshrdv_v8di = 8902,
  CODE_FOR_vpshrdv_v16hi = 8903,
  CODE_FOR_vpshrdv_v8si = 8904,
  CODE_FOR_vpshrdv_v4di = 8905,
  CODE_FOR_vpshrdv_v8hi = 8906,
  CODE_FOR_vpshrdv_v4si = 8907,
  CODE_FOR_vpshrdv_v2di = 8908,
  CODE_FOR_vpshrdv_v32hi_mask = 8909,
  CODE_FOR_vpshrdv_v16si_mask = 8910,
  CODE_FOR_vpshrdv_v8di_mask = 8911,
  CODE_FOR_vpshrdv_v16hi_mask = 8912,
  CODE_FOR_vpshrdv_v8si_mask = 8913,
  CODE_FOR_vpshrdv_v4di_mask = 8914,
  CODE_FOR_vpshrdv_v8hi_mask = 8915,
  CODE_FOR_vpshrdv_v4si_mask = 8916,
  CODE_FOR_vpshrdv_v2di_mask = 8917,
  CODE_FOR_vpshrdv_v32hi_maskz_1 = 8918,
  CODE_FOR_vpshrdv_v16si_maskz_1 = 8919,
  CODE_FOR_vpshrdv_v8di_maskz_1 = 8920,
  CODE_FOR_vpshrdv_v16hi_maskz_1 = 8921,
  CODE_FOR_vpshrdv_v8si_maskz_1 = 8922,
  CODE_FOR_vpshrdv_v4di_maskz_1 = 8923,
  CODE_FOR_vpshrdv_v8hi_maskz_1 = 8924,
  CODE_FOR_vpshrdv_v4si_maskz_1 = 8925,
  CODE_FOR_vpshrdv_v2di_maskz_1 = 8926,
  CODE_FOR_vpshldv_v32hi = 8927,
  CODE_FOR_vpshldv_v16si = 8928,
  CODE_FOR_vpshldv_v8di = 8929,
  CODE_FOR_vpshldv_v16hi = 8930,
  CODE_FOR_vpshldv_v8si = 8931,
  CODE_FOR_vpshldv_v4di = 8932,
  CODE_FOR_vpshldv_v8hi = 8933,
  CODE_FOR_vpshldv_v4si = 8934,
  CODE_FOR_vpshldv_v2di = 8935,
  CODE_FOR_vpshldv_v32hi_mask = 8936,
  CODE_FOR_vpshldv_v16si_mask = 8937,
  CODE_FOR_vpshldv_v8di_mask = 8938,
  CODE_FOR_vpshldv_v16hi_mask = 8939,
  CODE_FOR_vpshldv_v8si_mask = 8940,
  CODE_FOR_vpshldv_v4di_mask = 8941,
  CODE_FOR_vpshldv_v8hi_mask = 8942,
  CODE_FOR_vpshldv_v4si_mask = 8943,
  CODE_FOR_vpshldv_v2di_mask = 8944,
  CODE_FOR_vpshldv_v32hi_maskz_1 = 8945,
  CODE_FOR_vpshldv_v16si_maskz_1 = 8946,
  CODE_FOR_vpshldv_v8di_maskz_1 = 8947,
  CODE_FOR_vpshldv_v16hi_maskz_1 = 8948,
  CODE_FOR_vpshldv_v8si_maskz_1 = 8949,
  CODE_FOR_vpshldv_v4di_maskz_1 = 8950,
  CODE_FOR_vpshldv_v8hi_maskz_1 = 8951,
  CODE_FOR_vpshldv_v4si_maskz_1 = 8952,
  CODE_FOR_vpshldv_v2di_maskz_1 = 8953,
  CODE_FOR_vpdpbusd_v16si = 8954,
  CODE_FOR_vpdpbusd_v8si = 8955,
  CODE_FOR_vpdpbusd_v4si = 8956,
  CODE_FOR_vpdpbusd_v16si_mask = 8957,
  CODE_FOR_vpdpbusd_v8si_mask = 8958,
  CODE_FOR_vpdpbusd_v4si_mask = 8959,
  CODE_FOR_vpdpbusd_v16si_maskz_1 = 8960,
  CODE_FOR_vpdpbusd_v8si_maskz_1 = 8961,
  CODE_FOR_vpdpbusd_v4si_maskz_1 = 8962,
  CODE_FOR_vpdpbusds_v16si = 8963,
  CODE_FOR_vpdpbusds_v8si = 8964,
  CODE_FOR_vpdpbusds_v4si = 8965,
  CODE_FOR_vpdpbusds_v16si_mask = 8966,
  CODE_FOR_vpdpbusds_v8si_mask = 8967,
  CODE_FOR_vpdpbusds_v4si_mask = 8968,
  CODE_FOR_vpdpbusds_v16si_maskz_1 = 8969,
  CODE_FOR_vpdpbusds_v8si_maskz_1 = 8970,
  CODE_FOR_vpdpbusds_v4si_maskz_1 = 8971,
  CODE_FOR_vpdpwssd_v16si = 8972,
  CODE_FOR_vpdpwssd_v8si = 8973,
  CODE_FOR_vpdpwssd_v4si = 8974,
  CODE_FOR_vpdpwssd_v16si_mask = 8975,
  CODE_FOR_vpdpwssd_v8si_mask = 8976,
  CODE_FOR_vpdpwssd_v4si_mask = 8977,
  CODE_FOR_vpdpwssd_v16si_maskz_1 = 8978,
  CODE_FOR_vpdpwssd_v8si_maskz_1 = 8979,
  CODE_FOR_vpdpwssd_v4si_maskz_1 = 8980,
  CODE_FOR_vpdpwssds_v16si = 8981,
  CODE_FOR_vpdpwssds_v8si = 8982,
  CODE_FOR_vpdpwssds_v4si = 8983,
  CODE_FOR_vpdpwssds_v16si_mask = 8984,
  CODE_FOR_vpdpwssds_v8si_mask = 8985,
  CODE_FOR_vpdpwssds_v4si_mask = 8986,
  CODE_FOR_vpdpwssds_v16si_maskz_1 = 8987,
  CODE_FOR_vpdpwssds_v8si_maskz_1 = 8988,
  CODE_FOR_vpdpwssds_v4si_maskz_1 = 8989,
  CODE_FOR_vaesdec_v32qi = 8990,
  CODE_FOR_vaesdec_v16qi = 8991,
  CODE_FOR_vaesdec_v64qi = 8992,
  CODE_FOR_vaesdeclast_v32qi = 8993,
  CODE_FOR_vaesdeclast_v16qi = 8994,
  CODE_FOR_vaesdeclast_v64qi = 8995,
  CODE_FOR_vaesenc_v32qi = 8996,
  CODE_FOR_vaesenc_v16qi = 8997,
  CODE_FOR_vaesenc_v64qi = 8998,
  CODE_FOR_vaesenclast_v32qi = 8999,
  CODE_FOR_vaesenclast_v16qi = 9000,
  CODE_FOR_vaesenclast_v64qi = 9001,
  CODE_FOR_vpclmulqdq_v8di = 9002,
  CODE_FOR_vpclmulqdq_v4di = 9003,
  CODE_FOR_vpclmulqdq_v2di = 9004,
  CODE_FOR_avx512vl_vpshufbitqmbv64qi = 9005,
  CODE_FOR_avx512vl_vpshufbitqmbv64qi_mask = 9006,
  CODE_FOR_avx512vl_vpshufbitqmbv32qi = 9007,
  CODE_FOR_avx512vl_vpshufbitqmbv32qi_mask = 9008,
  CODE_FOR_avx512vl_vpshufbitqmbv16qi = 9009,
  CODE_FOR_avx512vl_vpshufbitqmbv16qi_mask = 9010,
  CODE_FOR_avx512vp2intersect_2intersectv8di = 9013,
  CODE_FOR_avx512vp2intersect_2intersectv4di = 9014,
  CODE_FOR_avx512vp2intersect_2intersectv2di = 9015,
  CODE_FOR_avx512vp2intersect_2intersectv8si = 9016,
  CODE_FOR_avx512vp2intersect_2intersectv4si = 9017,
  CODE_FOR_avx512vp2intersect_2intersectv16si = 9018,
  CODE_FOR_avx512f_cvtne2ps2bf16_v32bf = 9019,
  CODE_FOR_avx512f_cvtne2ps2bf16_v32bf_mask = 9020,
  CODE_FOR_avx512f_cvtne2ps2bf16_v16bf = 9021,
  CODE_FOR_avx512f_cvtne2ps2bf16_v16bf_mask = 9022,
  CODE_FOR_avx512f_cvtne2ps2bf16_v8bf = 9023,
  CODE_FOR_avx512f_cvtne2ps2bf16_v8bf_mask = 9024,
  CODE_FOR_avx512f_cvtneps2bf16_v4sf_mask_1 = 9026,
  CODE_FOR_vcvtneps2bf16_v8sf = 9027,
  CODE_FOR_avx512f_cvtneps2bf16_v16sf = 9028,
  CODE_FOR_avx512f_cvtneps2bf16_v16sf_mask = 9029,
  CODE_FOR_avx512f_cvtneps2bf16_v8sf = 9030,
  CODE_FOR_avx512f_cvtneps2bf16_v8sf_mask = 9031,
  CODE_FOR_avx512f_dpbf16ps_v16sf = 9032,
  CODE_FOR_avx512f_dpbf16ps_v16sf_maskz_1 = 9033,
  CODE_FOR_avx512f_dpbf16ps_v8sf = 9034,
  CODE_FOR_avx512f_dpbf16ps_v8sf_maskz_1 = 9035,
  CODE_FOR_avx512f_dpbf16ps_v4sf = 9036,
  CODE_FOR_avx512f_dpbf16ps_v4sf_maskz_1 = 9037,
  CODE_FOR_avx512f_dpbf16ps_v16sf_mask = 9038,
  CODE_FOR_avx512f_dpbf16ps_v8sf_mask = 9039,
  CODE_FOR_avx512f_dpbf16ps_v4sf_mask = 9040,
  CODE_FOR_loadiwkey = 9041,
  CODE_FOR_aesdec128klu8 = 9044,
  CODE_FOR_aesdec256klu8 = 9045,
  CODE_FOR_aesenc128klu8 = 9046,
  CODE_FOR_aesenc256klu8 = 9047,
  CODE_FOR_vpdpbssd_v8si = 9052,
  CODE_FOR_vpdpbssds_v8si = 9053,
  CODE_FOR_vpdpbsud_v8si = 9054,
  CODE_FOR_vpdpbsuds_v8si = 9055,
  CODE_FOR_vpdpbuud_v8si = 9056,
  CODE_FOR_vpdpbuuds_v8si = 9057,
  CODE_FOR_vpdpbssd_v4si = 9058,
  CODE_FOR_vpdpbssds_v4si = 9059,
  CODE_FOR_vpdpbsud_v4si = 9060,
  CODE_FOR_vpdpbsuds_v4si = 9061,
  CODE_FOR_vpdpbuud_v4si = 9062,
  CODE_FOR_vpdpbuuds_v4si = 9063,
  CODE_FOR_vbcstnebf162ps_v8sf = 9064,
  CODE_FOR_vbcstnebf162ps_v4sf = 9065,
  CODE_FOR_vbcstnesh2ps_v8sf = 9066,
  CODE_FOR_vbcstnesh2ps_v4sf = 9067,
  CODE_FOR_vcvtneeph2ps_v8hf = 9068,
  CODE_FOR_vcvtneebf162ps_v8bf = 9069,
  CODE_FOR_vcvtneeph2ps_v16hf = 9070,
  CODE_FOR_vcvtneebf162ps_v16bf = 9071,
  CODE_FOR_vcvtneoph2ps_v8hf = 9072,
  CODE_FOR_vcvtneobf162ps_v8bf = 9073,
  CODE_FOR_vcvtneoph2ps_v16hf = 9074,
  CODE_FOR_vcvtneobf162ps_v16bf = 9075,
  CODE_FOR_mfence_sse2 = 9078,
  CODE_FOR_mfence_nosse = 9079,
  CODE_FOR_atomic_loaddi_fpu = 9080,
  CODE_FOR_atomic_storeqi_1 = 9081,
  CODE_FOR_atomic_storehi_1 = 9082,
  CODE_FOR_atomic_storesi_1 = 9083,
  CODE_FOR_atomic_storedi_1 = 9084,
  CODE_FOR_atomic_storedi_fpu = 9085,
  CODE_FOR_loaddi_via_fpu = 9086,
  CODE_FOR_storedi_via_fpu = 9087,
  CODE_FOR_loaddi_via_sse = 9088,
  CODE_FOR_storedi_via_sse = 9089,
  CODE_FOR_atomic_compare_and_swapdi_doubleword = 9090,
  CODE_FOR_atomic_compare_and_swapti_doubleword = 9091,
  CODE_FOR_atomic_compare_and_swapqi_1 = 9092,
  CODE_FOR_atomic_compare_and_swaphi_1 = 9093,
  CODE_FOR_atomic_compare_and_swapsi_1 = 9094,
  CODE_FOR_atomic_compare_and_swapdi_1 = 9095,
  CODE_FOR_atomic_fetch_addqi = 9096,
  CODE_FOR_atomic_fetch_addhi = 9097,
  CODE_FOR_atomic_fetch_addsi = 9098,
  CODE_FOR_atomic_fetch_adddi = 9099,
  CODE_FOR_atomic_exchangeqi = 9104,
  CODE_FOR_atomic_exchangehi = 9105,
  CODE_FOR_atomic_exchangesi = 9106,
  CODE_FOR_atomic_exchangedi = 9107,
  CODE_FOR_rao_aandsi = 9108,
  CODE_FOR_rao_aorsi = 9109,
  CODE_FOR_rao_axorsi = 9110,
  CODE_FOR_rao_aaddsi = 9111,
  CODE_FOR_rao_aanddi = 9112,
  CODE_FOR_rao_aordi = 9113,
  CODE_FOR_rao_axordi = 9114,
  CODE_FOR_rao_aadddi = 9115,
  CODE_FOR_atomic_addqi = 9116,
  CODE_FOR_atomic_addhi = 9117,
  CODE_FOR_atomic_addsi = 9118,
  CODE_FOR_atomic_adddi = 9119,
  CODE_FOR_atomic_subqi = 9120,
  CODE_FOR_atomic_subhi = 9121,
  CODE_FOR_atomic_subsi = 9122,
  CODE_FOR_atomic_subdi = 9123,
  CODE_FOR_atomic_andqi = 9124,
  CODE_FOR_atomic_orqi = 9125,
  CODE_FOR_atomic_xorqi = 9126,
  CODE_FOR_atomic_andhi = 9127,
  CODE_FOR_atomic_orhi = 9128,
  CODE_FOR_atomic_xorhi = 9129,
  CODE_FOR_atomic_andsi = 9130,
  CODE_FOR_atomic_orsi = 9131,
  CODE_FOR_atomic_xorsi = 9132,
  CODE_FOR_atomic_anddi = 9133,
  CODE_FOR_atomic_ordi = 9134,
  CODE_FOR_atomic_xordi = 9135,
  CODE_FOR_atomic_bit_test_and_sethi_1 = 9136,
  CODE_FOR_atomic_bit_test_and_setsi_1 = 9137,
  CODE_FOR_atomic_bit_test_and_setdi_1 = 9138,
  CODE_FOR_atomic_bit_test_and_complementhi_1 = 9139,
  CODE_FOR_atomic_bit_test_and_complementsi_1 = 9140,
  CODE_FOR_atomic_bit_test_and_complementdi_1 = 9141,
  CODE_FOR_atomic_bit_test_and_resethi_1 = 9142,
  CODE_FOR_atomic_bit_test_and_resetsi_1 = 9143,
  CODE_FOR_atomic_bit_test_and_resetdi_1 = 9144,
  CODE_FOR_atomic_add_fetch_cmp_0qi_1 = 9145,
  CODE_FOR_atomic_add_fetch_cmp_0hi_1 = 9146,
  CODE_FOR_atomic_add_fetch_cmp_0si_1 = 9147,
  CODE_FOR_atomic_add_fetch_cmp_0di_1 = 9148,
  CODE_FOR_atomic_sub_fetch_cmp_0qi_1 = 9149,
  CODE_FOR_atomic_sub_fetch_cmp_0hi_1 = 9150,
  CODE_FOR_atomic_sub_fetch_cmp_0si_1 = 9151,
  CODE_FOR_atomic_sub_fetch_cmp_0di_1 = 9152,
  CODE_FOR_atomic_and_fetch_cmp_0qi_1 = 9153,
  CODE_FOR_atomic_or_fetch_cmp_0qi_1 = 9154,
  CODE_FOR_atomic_xor_fetch_cmp_0qi_1 = 9155,
  CODE_FOR_atomic_and_fetch_cmp_0hi_1 = 9156,
  CODE_FOR_atomic_or_fetch_cmp_0hi_1 = 9157,
  CODE_FOR_atomic_xor_fetch_cmp_0hi_1 = 9158,
  CODE_FOR_atomic_and_fetch_cmp_0si_1 = 9159,
  CODE_FOR_atomic_or_fetch_cmp_0si_1 = 9160,
  CODE_FOR_atomic_xor_fetch_cmp_0si_1 = 9161,
  CODE_FOR_atomic_and_fetch_cmp_0di_1 = 9162,
  CODE_FOR_atomic_or_fetch_cmp_0di_1 = 9163,
  CODE_FOR_atomic_xor_fetch_cmp_0di_1 = 9164,
  CODE_FOR_cmpccxadd_si = 9165,
  CODE_FOR_cmpccxadd_di = 9166,
  CODE_FOR_cbranchqi4 = 9167,
  CODE_FOR_cbranchhi4 = 9168,
  CODE_FOR_cbranchsi4 = 9169,
  CODE_FOR_cbranchdi4 = 9170,
  CODE_FOR_cbranchti4 = 9171,
  CODE_FOR_cbranchoi4 = 9172,
  CODE_FOR_cstoreqi4 = 9173,
  CODE_FOR_cstorehi4 = 9174,
  CODE_FOR_cstoresi4 = 9175,
  CODE_FOR_cstoredi4 = 9176,
  CODE_FOR_cstoreti4 = 9177,
  CODE_FOR_cmpsi_1 = 9178,
  CODE_FOR_cmpdi_1 = 9179,
  CODE_FOR_cmpqi_ext_3 = 9180,
  CODE_FOR_cbranchxf4 = 9181,
  CODE_FOR_cstorexf4 = 9182,
  CODE_FOR_cbranchhf4 = 9183,
  CODE_FOR_cbranchsf4 = 9184,
  CODE_FOR_cbranchdf4 = 9185,
  CODE_FOR_cbranchbf4 = 9186,
  CODE_FOR_cstorehf4 = 9187,
  CODE_FOR_cstorebf4 = 9188,
  CODE_FOR_cstoresf4 = 9189,
  CODE_FOR_cstoredf4 = 9190,
  CODE_FOR_cbranchcc4 = 9191,
  CODE_FOR_cstorecc4 = 9192,
  CODE_FOR_reload_noff_store = 9193,
  CODE_FOR_reload_noff_load = 9194,
  CODE_FOR_movxi = 9195,
  CODE_FOR_movoi = 9196,
  CODE_FOR_movti = 9197,
  CODE_FOR_movcdi = 9198,
  CODE_FOR_movqi = 9199,
  CODE_FOR_movhi = 9200,
  CODE_FOR_movsi = 9201,
  CODE_FOR_movdi = 9202,
  CODE_FOR_movstrictqi = 9203,
  CODE_FOR_movstricthi = 9204,
  CODE_FOR_extvhi = 9205,
  CODE_FOR_extvsi = 9206,
  CODE_FOR_extzvhi = 9207,
  CODE_FOR_extzvsi = 9208,
  CODE_FOR_extzvdi = 9209,
  CODE_FOR_insvhi = 9210,
  CODE_FOR_insvsi = 9211,
  CODE_FOR_insvdi = 9212,
  CODE_FOR_movtf = 9213,
  CODE_FOR_movhf = 9214,
  CODE_FOR_movsf = 9215,
  CODE_FOR_movdf = 9216,
  CODE_FOR_movxf = 9217,
  CODE_FOR_movbf = 9218,
  CODE_FOR_zero_extendsidi2 = 9219,
  CODE_FOR_zero_extendqisi2 = 9220,
  CODE_FOR_zero_extendhisi2 = 9221,
  CODE_FOR_zero_extendqihi2 = 9222,
  CODE_FOR_extendsidi2 = 9223,
  CODE_FOR_extendsfdf2 = 9224,
  CODE_FOR_extendhfsf2 = 9225,
  CODE_FOR_extendhfdf2 = 9226,
  CODE_FOR_extendbfsf2 = 9227,
  CODE_FOR_extendsfxf2 = 9228,
  CODE_FOR_extenddfxf2 = 9229,
  CODE_FOR_truncsfhf2 = 9230,
  CODE_FOR_truncdfhf2 = 9231,
  CODE_FOR_fix_truncxfdi2 = 9232,
  CODE_FOR_fix_truncsfdi2 = 9233,
  CODE_FOR_fix_truncdfdi2 = 9234,
  CODE_FOR_fix_truncxfsi2 = 9235,
  CODE_FOR_fix_truncsfsi2 = 9236,
  CODE_FOR_fix_truncdfsi2 = 9237,
  CODE_FOR_fix_truncsfhi2 = 9238,
  CODE_FOR_fix_truncdfhi2 = 9239,
  CODE_FOR_fix_truncxfhi2 = 9240,
  CODE_FOR_fixuns_truncsfsi2 = 9241,
  CODE_FOR_fixuns_truncdfsi2 = 9242,
  CODE_FOR_fixuns_trunchfhi2 = 9243,
  CODE_FOR_fixuns_truncsfhi2 = 9244,
  CODE_FOR_fixuns_truncdfhi2 = 9245,
  CODE_FOR_floatsisf2 = 9246,
  CODE_FOR_floatdisf2 = 9247,
  CODE_FOR_floatsidf2 = 9248,
  CODE_FOR_floatdidf2 = 9249,
  CODE_FOR_floatunsqisf2 = 9250,
  CODE_FOR_floatunshisf2 = 9251,
  CODE_FOR_floatunsqidf2 = 9252,
  CODE_FOR_floatunshidf2 = 9253,
  CODE_FOR_floatunssisf2 = 9254,
  CODE_FOR_floatunssidf2 = 9255,
  CODE_FOR_floatunssixf2 = 9256,
  CODE_FOR_floatunsdisf2 = 9257,
  CODE_FOR_floatunsdidf2 = 9258,
  CODE_FOR_addqi3 = 9259,
  CODE_FOR_addhi3 = 9260,
  CODE_FOR_addsi3 = 9261,
  CODE_FOR_adddi3 = 9262,
  CODE_FOR_addti3 = 9263,
  CODE_FOR_addqi_ext_1 = 9264,
  CODE_FOR_addvqi4 = 9265,
  CODE_FOR_addvhi4 = 9266,
  CODE_FOR_addvsi4 = 9267,
  CODE_FOR_addvdi4 = 9268,
  CODE_FOR_addvti4 = 9269,
  CODE_FOR_uaddvqi4 = 9270,
  CODE_FOR_uaddvhi4 = 9271,
  CODE_FOR_uaddvsi4 = 9272,
  CODE_FOR_uaddvdi4 = 9273,
  CODE_FOR_uaddvti4 = 9274,
  CODE_FOR_subqi3 = 9275,
  CODE_FOR_subhi3 = 9276,
  CODE_FOR_subsi3 = 9277,
  CODE_FOR_subdi3 = 9278,
  CODE_FOR_subti3 = 9279,
  CODE_FOR_subvqi4 = 9280,
  CODE_FOR_subvhi4 = 9281,
  CODE_FOR_subvsi4 = 9282,
  CODE_FOR_subvdi4 = 9283,
  CODE_FOR_subvti4 = 9284,
  CODE_FOR_usubvqi4 = 9285,
  CODE_FOR_usubvhi4 = 9286,
  CODE_FOR_usubvsi4 = 9287,
  CODE_FOR_usubvdi4 = 9288,
  CODE_FOR_addcarrysi_0 = 9289,
  CODE_FOR_addcarrydi_0 = 9290,
  CODE_FOR_subborrowsi_0 = 9291,
  CODE_FOR_subborrowdi_0 = 9292,
  CODE_FOR_addqi3_cconly_overflow = 9293,
  CODE_FOR_addxf3 = 9294,
  CODE_FOR_subxf3 = 9295,
  CODE_FOR_addhf3 = 9296,
  CODE_FOR_subhf3 = 9297,
  CODE_FOR_addsf3 = 9298,
  CODE_FOR_subsf3 = 9299,
  CODE_FOR_adddf3 = 9300,
  CODE_FOR_subdf3 = 9301,
  CODE_FOR_mulhi3 = 9302,
  CODE_FOR_mulsi3 = 9303,
  CODE_FOR_muldi3 = 9304,
  CODE_FOR_mulqi3 = 9305,
  CODE_FOR_mulvhi4 = 9306,
  CODE_FOR_mulvsi4 = 9307,
  CODE_FOR_mulvdi4 = 9308,
  CODE_FOR_umulvhi4 = 9309,
  CODE_FOR_umulvsi4 = 9310,
  CODE_FOR_umulvdi4 = 9311,
  CODE_FOR_mulvqi4 = 9312,
  CODE_FOR_umulvqi4 = 9313,
  CODE_FOR_mulsidi3 = 9314,
  CODE_FOR_umulsidi3 = 9315,
  CODE_FOR_mulditi3 = 9316,
  CODE_FOR_umulditi3 = 9317,
  CODE_FOR_mulqihi3 = 9318,
  CODE_FOR_umulqihi3 = 9319,
  CODE_FOR_mulxf3 = 9320,
  CODE_FOR_mulhf3 = 9321,
  CODE_FOR_mulsf3 = 9322,
  CODE_FOR_muldf3 = 9323,
  CODE_FOR_divxf3 = 9324,
  CODE_FOR_divhf3 = 9325,
  CODE_FOR_divsf3 = 9326,
  CODE_FOR_divdf3 = 9327,
  CODE_FOR_divmodhi4 = 9328,
  CODE_FOR_udivmodhi4 = 9329,
  CODE_FOR_divmodsi4 = 9330,
  CODE_FOR_udivmodsi4 = 9331,
  CODE_FOR_divmoddi4 = 9332,
  CODE_FOR_udivmoddi4 = 9333,
  CODE_FOR_divmodqi4 = 9334,
  CODE_FOR_udivmodqi4 = 9335,
  CODE_FOR_testsi_ccno_1 = 9336,
  CODE_FOR_testdi_ccno_1 = 9337,
  CODE_FOR_testqi_ccz_1 = 9338,
  CODE_FOR_testqi_ext_1_ccno = 9339,
  CODE_FOR_andqi3 = 9340,
  CODE_FOR_andhi3 = 9341,
  CODE_FOR_andsi3 = 9342,
  CODE_FOR_anddi3 = 9343,
  CODE_FOR_andti3 = 9344,
  CODE_FOR_andqi_ext_1 = 9345,
  CODE_FOR_iorqi3 = 9346,
  CODE_FOR_xorqi3 = 9347,
  CODE_FOR_iorhi3 = 9348,
  CODE_FOR_xorhi3 = 9349,
  CODE_FOR_iorsi3 = 9350,
  CODE_FOR_xorsi3 = 9351,
  CODE_FOR_iordi3 = 9352,
  CODE_FOR_xordi3 = 9353,
  CODE_FOR_iorti3 = 9354,
  CODE_FOR_xorti3 = 9355,
  CODE_FOR_xorqi_ext_1_cc = 9356,
  CODE_FOR_negqi2 = 9357,
  CODE_FOR_neghi2 = 9358,
  CODE_FOR_negsi2 = 9359,
  CODE_FOR_negdi2 = 9360,
  CODE_FOR_negti2 = 9361,
  CODE_FOR_x86_negsi_ccc = 9362,
  CODE_FOR_x86_negdi_ccc = 9363,
  CODE_FOR_negvqi3 = 9364,
  CODE_FOR_negvhi3 = 9365,
  CODE_FOR_negvsi3 = 9366,
  CODE_FOR_negvdi3 = 9367,
  CODE_FOR_absqi2 = 9368,
  CODE_FOR_abshi2 = 9369,
  CODE_FOR_abssi2 = 9370,
  CODE_FOR_absdi2 = 9371,
  CODE_FOR_absti2 = 9372,
  CODE_FOR_abstf2 = 9373,
  CODE_FOR_negtf2 = 9374,
  CODE_FOR_abshf2 = 9375,
  CODE_FOR_neghf2 = 9376,
  CODE_FOR_abssf2 = 9377,
  CODE_FOR_negsf2 = 9378,
  CODE_FOR_absdf2 = 9379,
  CODE_FOR_negdf2 = 9380,
  CODE_FOR_absxf2 = 9381,
  CODE_FOR_negxf2 = 9382,
  CODE_FOR_copysignhf3 = 9383,
  CODE_FOR_copysignsf3 = 9384,
  CODE_FOR_copysigndf3 = 9385,
  CODE_FOR_copysigntf3 = 9386,
  CODE_FOR_xorsignhf3 = 9387,
  CODE_FOR_xorsignsf3 = 9388,
  CODE_FOR_xorsigndf3 = 9389,
  CODE_FOR_one_cmplqi2 = 9390,
  CODE_FOR_one_cmplhi2 = 9391,
  CODE_FOR_one_cmplsi2 = 9392,
  CODE_FOR_one_cmpldi2 = 9393,
  CODE_FOR_one_cmplti2 = 9394,
  CODE_FOR_ashlqi3 = 9395,
  CODE_FOR_ashlhi3 = 9396,
  CODE_FOR_ashlsi3 = 9397,
  CODE_FOR_ashldi3 = 9398,
  CODE_FOR_ashlti3 = 9399,
  CODE_FOR_x86_shiftsi_adj_1 = 9400,
  CODE_FOR_x86_shiftdi_adj_1 = 9401,
  CODE_FOR_x86_shiftsi_adj_2 = 9402,
  CODE_FOR_x86_shiftdi_adj_2 = 9403,
  CODE_FOR_lshrqi3 = 9404,
  CODE_FOR_ashrqi3 = 9405,
  CODE_FOR_lshrhi3 = 9406,
  CODE_FOR_ashrhi3 = 9407,
  CODE_FOR_lshrsi3 = 9408,
  CODE_FOR_ashrsi3 = 9409,
  CODE_FOR_lshrdi3 = 9410,
  CODE_FOR_ashrdi3 = 9411,
  CODE_FOR_lshrti3 = 9412,
  CODE_FOR_ashrti3 = 9413,
  CODE_FOR_x86_shiftsi_adj_3 = 9414,
  CODE_FOR_x86_shiftdi_adj_3 = 9415,
  CODE_FOR_rotlti3 = 9416,
  CODE_FOR_rotrti3 = 9417,
  CODE_FOR_rotldi3 = 9418,
  CODE_FOR_rotrdi3 = 9419,
  CODE_FOR_rotlqi3 = 9420,
  CODE_FOR_rotrqi3 = 9421,
  CODE_FOR_rotlhi3 = 9422,
  CODE_FOR_rotrhi3 = 9423,
  CODE_FOR_rotlsi3 = 9424,
  CODE_FOR_rotrsi3 = 9425,
  CODE_FOR_indirect_jump = 9426,
  CODE_FOR_tablejump = 9427,
  CODE_FOR_call = 9428,
  CODE_FOR_sibcall = 9429,
  CODE_FOR_call_pop = 9430,
  CODE_FOR_call_value = 9431,
  CODE_FOR_sibcall_value = 9432,
  CODE_FOR_call_value_pop = 9433,
  CODE_FOR_untyped_call = 9434,
  CODE_FOR_memory_blockage = 9435,
  CODE_FOR_return = 9436,
  CODE_FOR_simple_return = 9437,
  CODE_FOR_simple_return_indirect_internal = 9438,
  CODE_FOR_prologue = 9439,
  CODE_FOR_set_got = 9440,
  CODE_FOR_set_got_labelled = 9441,
  CODE_FOR_epilogue = 9442,
  CODE_FOR_sibcall_epilogue = 9443,
  CODE_FOR_eh_return = 9444,
  CODE_FOR_leave_si = 9445,
  CODE_FOR_leave_di = 9446,
  CODE_FOR_split_stack_prologue = 9447,
  CODE_FOR_split_stack_space_check = 9448,
  CODE_FOR_ffssi2 = 9449,
  CODE_FOR_ffsdi2 = 9450,
  CODE_FOR_clzsi2 = 9451,
  CODE_FOR_clzdi2 = 9452,
  CODE_FOR_bmi2_bzhi_si3 = 9453,
  CODE_FOR_bmi2_bzhi_di3 = 9454,
  CODE_FOR_bswapdi2 = 9455,
  CODE_FOR_bswapsi2 = 9456,
  CODE_FOR_bswaphi2 = 9457,
  CODE_FOR_paritydi2 = 9458,
  CODE_FOR_paritysi2 = 9459,
  CODE_FOR_parityhi2 = 9460,
  CODE_FOR_parityqi2 = 9461,
  CODE_FOR_tls_global_dynamic_32 = 9462,
  CODE_FOR_tls_global_dynamic_64_si = 9463,
  CODE_FOR_tls_global_dynamic_64_di = 9464,
  CODE_FOR_tls_local_dynamic_base_32 = 9465,
  CODE_FOR_tls_local_dynamic_base_64_si = 9466,
  CODE_FOR_tls_local_dynamic_base_64_di = 9467,
  CODE_FOR_get_thread_pointersi = 9468,
  CODE_FOR_get_thread_pointerdi = 9469,
  CODE_FOR_tls_dynamic_gnu2_32 = 9470,
  CODE_FOR_tls_dynamic_gnu2_64_si = 9471,
  CODE_FOR_tls_dynamic_gnu2_64_di = 9472,
  CODE_FOR_rsqrtsf2 = 9473,
  CODE_FOR_sqrtsf2 = 9474,
  CODE_FOR_sqrtdf2 = 9475,
  CODE_FOR_hypotsf3 = 9476,
  CODE_FOR_hypotdf3 = 9477,
  CODE_FOR_fmodxf3 = 9478,
  CODE_FOR_fmodsf3 = 9479,
  CODE_FOR_fmoddf3 = 9480,
  CODE_FOR_remainderxf3 = 9481,
  CODE_FOR_remaindersf3 = 9482,
  CODE_FOR_remainderdf3 = 9483,
  CODE_FOR_sinsf2 = 9484,
  CODE_FOR_cossf2 = 9485,
  CODE_FOR_sindf2 = 9486,
  CODE_FOR_cosdf2 = 9487,
  CODE_FOR_sincossf3 = 9488,
  CODE_FOR_sincosdf3 = 9489,
  CODE_FOR_tanxf2 = 9490,
  CODE_FOR_tansf2 = 9491,
  CODE_FOR_tandf2 = 9492,
  CODE_FOR_atan2sf3 = 9493,
  CODE_FOR_atan2df3 = 9494,
  CODE_FOR_atanxf2 = 9495,
  CODE_FOR_atansf2 = 9496,
  CODE_FOR_atandf2 = 9497,
  CODE_FOR_asinxf2 = 9498,
  CODE_FOR_asinsf2 = 9499,
  CODE_FOR_asindf2 = 9500,
  CODE_FOR_acosxf2 = 9501,
  CODE_FOR_acossf2 = 9502,
  CODE_FOR_acosdf2 = 9503,
  CODE_FOR_sinhxf2 = 9504,
  CODE_FOR_sinhsf2 = 9505,
  CODE_FOR_sinhdf2 = 9506,
  CODE_FOR_coshxf2 = 9507,
  CODE_FOR_coshsf2 = 9508,
  CODE_FOR_coshdf2 = 9509,
  CODE_FOR_tanhxf2 = 9510,
  CODE_FOR_tanhsf2 = 9511,
  CODE_FOR_tanhdf2 = 9512,
  CODE_FOR_asinhxf2 = 9513,
  CODE_FOR_asinhsf2 = 9514,
  CODE_FOR_asinhdf2 = 9515,
  CODE_FOR_acoshxf2 = 9516,
  CODE_FOR_acoshsf2 = 9517,
  CODE_FOR_acoshdf2 = 9518,
  CODE_FOR_atanhxf2 = 9519,
  CODE_FOR_atanhsf2 = 9520,
  CODE_FOR_atanhdf2 = 9521,
  CODE_FOR_logxf2 = 9522,
  CODE_FOR_logsf2 = 9523,
  CODE_FOR_logdf2 = 9524,
  CODE_FOR_log10xf2 = 9525,
  CODE_FOR_log10sf2 = 9526,
  CODE_FOR_log10df2 = 9527,
  CODE_FOR_log2xf2 = 9528,
  CODE_FOR_log2sf2 = 9529,
  CODE_FOR_log2df2 = 9530,
  CODE_FOR_log1pxf2 = 9531,
  CODE_FOR_log1psf2 = 9532,
  CODE_FOR_log1pdf2 = 9533,
  CODE_FOR_logbxf2 = 9534,
  CODE_FOR_logbsf2 = 9535,
  CODE_FOR_logbdf2 = 9536,
  CODE_FOR_ilogbxf2 = 9537,
  CODE_FOR_ilogbsf2 = 9538,
  CODE_FOR_ilogbdf2 = 9539,
  CODE_FOR_expNcorexf3 = 9540,
  CODE_FOR_expxf2 = 9541,
  CODE_FOR_expsf2 = 9542,
  CODE_FOR_expdf2 = 9543,
  CODE_FOR_exp10xf2 = 9544,
  CODE_FOR_exp10sf2 = 9545,
  CODE_FOR_exp10df2 = 9546,
  CODE_FOR_exp2xf2 = 9547,
  CODE_FOR_exp2sf2 = 9548,
  CODE_FOR_exp2df2 = 9549,
  CODE_FOR_expm1xf2 = 9550,
  CODE_FOR_expm1sf2 = 9551,
  CODE_FOR_expm1df2 = 9552,
  CODE_FOR_ldexpxf3 = 9553,
  CODE_FOR_ldexpsf3 = 9554,
  CODE_FOR_ldexpdf3 = 9555,
  CODE_FOR_scalbxf3 = 9556,
  CODE_FOR_scalbsf3 = 9557,
  CODE_FOR_scalbdf3 = 9558,
  CODE_FOR_significandxf2 = 9559,
  CODE_FOR_significandsf2 = 9560,
  CODE_FOR_significanddf2 = 9561,
  CODE_FOR_rinthf2 = 9562,
  CODE_FOR_rintsf2 = 9563,
  CODE_FOR_rintdf2 = 9564,
  CODE_FOR_nearbyintxf2 = 9565,
  CODE_FOR_nearbyinthf2 = 9566,
  CODE_FOR_nearbyintsf2 = 9567,
  CODE_FOR_nearbyintdf2 = 9568,
  CODE_FOR_roundsf2 = 9569,
  CODE_FOR_rounddf2 = 9570,
  CODE_FOR_roundxf2 = 9571,
  CODE_FOR_lrintsfsi2 = 9572,
  CODE_FOR_lrintsfdi2 = 9573,
  CODE_FOR_lrintdfsi2 = 9574,
  CODE_FOR_lrintdfdi2 = 9575,
  CODE_FOR_lroundsfhi2 = 9576,
  CODE_FOR_lrounddfhi2 = 9577,
  CODE_FOR_lroundxfhi2 = 9578,
  CODE_FOR_lroundsfsi2 = 9579,
  CODE_FOR_lrounddfsi2 = 9580,
  CODE_FOR_lroundxfsi2 = 9581,
  CODE_FOR_lroundsfdi2 = 9582,
  CODE_FOR_lrounddfdi2 = 9583,
  CODE_FOR_lroundxfdi2 = 9584,
  CODE_FOR_roundevenxf2 = 9585,
  CODE_FOR_floorxf2 = 9586,
  CODE_FOR_ceilxf2 = 9587,
  CODE_FOR_btruncxf2 = 9588,
  CODE_FOR_roundevenhf2 = 9589,
  CODE_FOR_floorhf2 = 9590,
  CODE_FOR_ceilhf2 = 9591,
  CODE_FOR_btrunchf2 = 9592,
  CODE_FOR_roundevensf2 = 9593,
  CODE_FOR_floorsf2 = 9594,
  CODE_FOR_ceilsf2 = 9595,
  CODE_FOR_btruncsf2 = 9596,
  CODE_FOR_roundevendf2 = 9597,
  CODE_FOR_floordf2 = 9598,
  CODE_FOR_ceildf2 = 9599,
  CODE_FOR_btruncdf2 = 9600,
  CODE_FOR_lfloorxfhi2 = 9601,
  CODE_FOR_lceilxfhi2 = 9602,
  CODE_FOR_lfloorxfsi2 = 9603,
  CODE_FOR_lceilxfsi2 = 9604,
  CODE_FOR_lfloorxfdi2 = 9605,
  CODE_FOR_lceilxfdi2 = 9606,
  CODE_FOR_lfloorsfsi2 = 9607,
  CODE_FOR_lceilsfsi2 = 9608,
  CODE_FOR_lfloorsfdi2 = 9609,
  CODE_FOR_lceilsfdi2 = 9610,
  CODE_FOR_lfloordfsi2 = 9611,
  CODE_FOR_lceildfsi2 = 9612,
  CODE_FOR_lfloordfdi2 = 9613,
  CODE_FOR_lceildfdi2 = 9614,
  CODE_FOR_signbittf2 = 9615,
  CODE_FOR_signbitxf2 = 9616,
  CODE_FOR_signbitdf2 = 9617,
  CODE_FOR_signbitsf2 = 9618,
  CODE_FOR_cpymemsi = 9619,
  CODE_FOR_cpymemdi = 9620,
  CODE_FOR_strmov = 9621,
  CODE_FOR_strmov_singleop = 9622,
  CODE_FOR_rep_mov = 9623,
  CODE_FOR_setmemsi = 9624,
  CODE_FOR_setmemdi = 9625,
  CODE_FOR_strset = 9626,
  CODE_FOR_strset_singleop = 9627,
  CODE_FOR_rep_stos = 9628,
  CODE_FOR_cmpmemsi = 9629,
  CODE_FOR_cmpstrnsi = 9630,
  CODE_FOR_cmpintqi = 9631,
  CODE_FOR_cmpstrnqi_nz_1 = 9632,
  CODE_FOR_cmpstrnqi_1 = 9633,
  CODE_FOR_strlensi = 9634,
  CODE_FOR_strlendi = 9635,
  CODE_FOR_strlenqi_1 = 9636,
  CODE_FOR_movqicc = 9637,
  CODE_FOR_movhicc = 9638,
  CODE_FOR_movsicc = 9639,
  CODE_FOR_movdicc = 9640,
  CODE_FOR_x86_movsicc_0_m1 = 9641,
  CODE_FOR_x86_movdicc_0_m1 = 9642,
  CODE_FOR_x86_movsicc_0_m1_neg = 9643,
  CODE_FOR_x86_movdicc_0_m1_neg = 9644,
  CODE_FOR_movhfcc = 9645,
  CODE_FOR_movsfcc = 9646,
  CODE_FOR_movdfcc = 9647,
  CODE_FOR_movxfcc = 9648,
  CODE_FOR_addqicc = 9649,
  CODE_FOR_addhicc = 9650,
  CODE_FOR_addsicc = 9651,
  CODE_FOR_adddicc = 9652,
  CODE_FOR_smaxqi3 = 9653,
  CODE_FOR_sminqi3 = 9654,
  CODE_FOR_umaxqi3 = 9655,
  CODE_FOR_uminqi3 = 9656,
  CODE_FOR_smaxhi3 = 9657,
  CODE_FOR_sminhi3 = 9658,
  CODE_FOR_umaxhi3 = 9659,
  CODE_FOR_uminhi3 = 9660,
  CODE_FOR_smaxsi3 = 9661,
  CODE_FOR_sminsi3 = 9662,
  CODE_FOR_umaxsi3 = 9663,
  CODE_FOR_uminsi3 = 9664,
  CODE_FOR_smaxdi3 = 9665,
  CODE_FOR_smindi3 = 9666,
  CODE_FOR_umaxdi3 = 9667,
  CODE_FOR_umindi3 = 9668,
  CODE_FOR_smaxti3 = 9669,
  CODE_FOR_sminti3 = 9670,
  CODE_FOR_umaxti3 = 9671,
  CODE_FOR_uminti3 = 9672,
  CODE_FOR_allocate_stack = 9673,
  CODE_FOR_probe_stack = 9674,
  CODE_FOR_builtin_setjmp_receiver = 9675,
  CODE_FOR_save_stack_nonlocal = 9676,
  CODE_FOR_restore_stack_nonlocal = 9677,
  CODE_FOR_prefetch = 9678,
  CODE_FOR_stack_protect_set = 9679,
  CODE_FOR_stack_protect_test = 9680,
  CODE_FOR_pause = 9681,
  CODE_FOR_xbegin = 9682,
  CODE_FOR_xtest = 9683,
  CODE_FOR_rdpkru = 9684,
  CODE_FOR_wrpkru = 9685,
  CODE_FOR_spaceshipsf3 = 9686,
  CODE_FOR_spaceshipdf3 = 9687,
  CODE_FOR_spaceshipxf3 = 9688,
  CODE_FOR_issignalingxf2 = 9689,
  CODE_FOR_movv8qi = 9690,
  CODE_FOR_movv4hi = 9691,
  CODE_FOR_movv2si = 9692,
  CODE_FOR_movv1di = 9693,
  CODE_FOR_movv2sf = 9694,
  CODE_FOR_movv4hf = 9695,
  CODE_FOR_movv4bf = 9696,
  CODE_FOR_movmisalignv8qi = 9697,
  CODE_FOR_movmisalignv4hi = 9698,
  CODE_FOR_movmisalignv2si = 9699,
  CODE_FOR_movmisalignv1di = 9700,
  CODE_FOR_movmisalignv2sf = 9701,
  CODE_FOR_movmisalignv4hf = 9702,
  CODE_FOR_movmisalignv4bf = 9703,
  CODE_FOR_movv4qi = 9704,
  CODE_FOR_movv2hi = 9705,
  CODE_FOR_movv1si = 9706,
  CODE_FOR_movv2hf = 9707,
  CODE_FOR_movv2bf = 9708,
  CODE_FOR_movmisalignv4qi = 9709,
  CODE_FOR_movmisalignv2hi = 9710,
  CODE_FOR_movmisalignv1si = 9711,
  CODE_FOR_movmisalignv2hf = 9712,
  CODE_FOR_movmisalignv2bf = 9713,
  CODE_FOR_movv2qi = 9714,
  CODE_FOR_movmisalignv2qi = 9715,
  CODE_FOR_absv2sf2 = 9716,
  CODE_FOR_negv2sf2 = 9717,
  CODE_FOR_mmx_addv2sf3 = 9718,
  CODE_FOR_addv2sf3 = 9719,
  CODE_FOR_mmx_subv2sf3 = 9720,
  CODE_FOR_mmx_subrv2sf3 = 9721,
  CODE_FOR_subv2sf3 = 9722,
  CODE_FOR_mmx_mulv2sf3 = 9723,
  CODE_FOR_mulv2sf3 = 9724,
  CODE_FOR_divv2sf3 = 9725,
  CODE_FOR_mmx_smaxv2sf3 = 9726,
  CODE_FOR_mmx_sminv2sf3 = 9727,
  CODE_FOR_smaxv2sf3 = 9728,
  CODE_FOR_sminv2sf3 = 9729,
  CODE_FOR_mmx_haddv2sf3 = 9730,
  CODE_FOR_mmx_haddsubv2sf3 = 9731,
  CODE_FOR_mmx_eqv2sf3 = 9732,
  CODE_FOR_vec_cmpv2sfv2si = 9733,
  CODE_FOR_vcondv2sfv2sf = 9734,
  CODE_FOR_vcondv2siv2sf = 9735,
  CODE_FOR_copysignv2sf3 = 9736,
  CODE_FOR_xorsignv2sf3 = 9737,
  CODE_FOR_signbitv2sf2 = 9738,
  CODE_FOR_fix_truncv2sfv2si2 = 9739,
  CODE_FOR_floatv2siv2sf2 = 9740,
  CODE_FOR_vec_setv2sf = 9741,
  CODE_FOR_vec_extractv2sfsf = 9742,
  CODE_FOR_vec_initv2sfsf = 9743,
  CODE_FOR_nearbyintv2sf2 = 9744,
  CODE_FOR_rintv2sf2 = 9745,
  CODE_FOR_ceilv2sf2 = 9746,
  CODE_FOR_lceilv2sfv2si2 = 9747,
  CODE_FOR_floorv2sf2 = 9748,
  CODE_FOR_lfloorv2sfv2si2 = 9749,
  CODE_FOR_btruncv2sf2 = 9750,
  CODE_FOR_roundv2sf2 = 9751,
  CODE_FOR_lroundv2sfv2si2 = 9752,
  CODE_FOR_negv8qi2 = 9753,
  CODE_FOR_negv4hi2 = 9754,
  CODE_FOR_negv2si2 = 9755,
  CODE_FOR_negv4qi2 = 9756,
  CODE_FOR_negv2hi2 = 9757,
  CODE_FOR_mmx_addv8qi3 = 9758,
  CODE_FOR_mmx_subv8qi3 = 9759,
  CODE_FOR_mmx_addv4hi3 = 9760,
  CODE_FOR_mmx_subv4hi3 = 9761,
  CODE_FOR_mmx_addv2si3 = 9762,
  CODE_FOR_mmx_subv2si3 = 9763,
  CODE_FOR_mmx_addv1di3 = 9764,
  CODE_FOR_mmx_subv1di3 = 9765,
  CODE_FOR_addv8qi3 = 9766,
  CODE_FOR_subv8qi3 = 9767,
  CODE_FOR_addv4hi3 = 9768,
  CODE_FOR_subv4hi3 = 9769,
  CODE_FOR_addv2si3 = 9770,
  CODE_FOR_subv2si3 = 9771,
  CODE_FOR_mmx_ssaddv8qi3 = 9772,
  CODE_FOR_mmx_usaddv8qi3 = 9773,
  CODE_FOR_mmx_sssubv8qi3 = 9774,
  CODE_FOR_mmx_ussubv8qi3 = 9775,
  CODE_FOR_mmx_ssaddv4hi3 = 9776,
  CODE_FOR_mmx_usaddv4hi3 = 9777,
  CODE_FOR_mmx_sssubv4hi3 = 9778,
  CODE_FOR_mmx_ussubv4hi3 = 9779,
  CODE_FOR_mmx_mulv4hi3 = 9780,
  CODE_FOR_mulv4hi3 = 9781,
  CODE_FOR_mmx_smulv4hi3_highpart = 9782,
  CODE_FOR_mmx_umulv4hi3_highpart = 9783,
  CODE_FOR_smulv4hi3_highpart = 9784,
  CODE_FOR_umulv4hi3_highpart = 9785,
  CODE_FOR_mmx_pmaddwd = 9786,
  CODE_FOR_mmx_pmulhrwv4hi3 = 9787,
  CODE_FOR_sse2_umulv1siv1di3 = 9788,
  CODE_FOR_mmx_smaxv4hi3 = 9789,
  CODE_FOR_mmx_sminv4hi3 = 9790,
  CODE_FOR_smaxv4hi3 = 9791,
  CODE_FOR_sminv4hi3 = 9792,
  CODE_FOR_mmx_umaxv8qi3 = 9793,
  CODE_FOR_mmx_uminv8qi3 = 9794,
  CODE_FOR_umaxv8qi3 = 9795,
  CODE_FOR_uminv8qi3 = 9796,
  CODE_FOR_absv8qi2 = 9797,
  CODE_FOR_absv4hi2 = 9798,
  CODE_FOR_absv2si2 = 9799,
  CODE_FOR_ashrv4hi3 = 9800,
  CODE_FOR_ashrv2si3 = 9801,
  CODE_FOR_ashlv4hi3 = 9802,
  CODE_FOR_lshrv4hi3 = 9803,
  CODE_FOR_ashlv2si3 = 9804,
  CODE_FOR_lshrv2si3 = 9805,
  CODE_FOR_mmx_eqv8qi3 = 9806,
  CODE_FOR_mmx_eqv4hi3 = 9807,
  CODE_FOR_mmx_eqv2si3 = 9808,
  CODE_FOR_vec_cmpv8qiv8qi = 9809,
  CODE_FOR_vec_cmpv4hiv4hi = 9810,
  CODE_FOR_vec_cmpv2siv2si = 9811,
  CODE_FOR_vec_cmpv4qiv4qi = 9812,
  CODE_FOR_vec_cmpv2qiv2qi = 9813,
  CODE_FOR_vec_cmpv2hiv2hi = 9814,
  CODE_FOR_vec_cmpuv8qiv8qi = 9815,
  CODE_FOR_vec_cmpuv4hiv4hi = 9816,
  CODE_FOR_vec_cmpuv2siv2si = 9817,
  CODE_FOR_vec_cmpuv4qiv4qi = 9818,
  CODE_FOR_vec_cmpuv2qiv2qi = 9819,
  CODE_FOR_vec_cmpuv2hiv2hi = 9820,
  CODE_FOR_vcondv8qiv8qi = 9821,
  CODE_FOR_vcondv8qiv4hi = 9822,
  CODE_FOR_vcondv8qiv2si = 9823,
  CODE_FOR_vcondv4hiv8qi = 9824,
  CODE_FOR_vcondv4hiv4hi = 9825,
  CODE_FOR_vcondv4hiv2si = 9826,
  CODE_FOR_vcondv2siv8qi = 9827,
  CODE_FOR_vcondv2siv4hi = 9828,
  CODE_FOR_vcondv2siv2si = 9829,
  CODE_FOR_vcondv2sfv8qi = 9830,
  CODE_FOR_vcondv2sfv4hi = 9831,
  CODE_FOR_vcondv2sfv2si = 9832,
  CODE_FOR_vcondv4qiv4qi = 9833,
  CODE_FOR_vcondv2qiv2qi = 9834,
  CODE_FOR_vcondv2hiv2hi = 9835,
  CODE_FOR_vconduv8qiv8qi = 9836,
  CODE_FOR_vconduv8qiv4hi = 9837,
  CODE_FOR_vconduv8qiv2si = 9838,
  CODE_FOR_vconduv4hiv8qi = 9839,
  CODE_FOR_vconduv4hiv4hi = 9840,
  CODE_FOR_vconduv4hiv2si = 9841,
  CODE_FOR_vconduv2siv8qi = 9842,
  CODE_FOR_vconduv2siv4hi = 9843,
  CODE_FOR_vconduv2siv2si = 9844,
  CODE_FOR_vconduv2sfv8qi = 9845,
  CODE_FOR_vconduv2sfv4hi = 9846,
  CODE_FOR_vconduv2sfv2si = 9847,
  CODE_FOR_vconduv4qiv4qi = 9848,
  CODE_FOR_vconduv2qiv2qi = 9849,
  CODE_FOR_vconduv2hiv2hi = 9850,
  CODE_FOR_vcond_mask_v8qiv8qi = 9851,
  CODE_FOR_vcond_mask_v4hiv4hi = 9852,
  CODE_FOR_vcond_mask_v2siv2si = 9853,
  CODE_FOR_vcond_mask_v2sfv2si = 9854,
  CODE_FOR_vcond_mask_v4qiv4qi = 9855,
  CODE_FOR_vcond_mask_v2qiv2qi = 9856,
  CODE_FOR_vcond_mask_v2hiv2hi = 9857,
  CODE_FOR_one_cmplv8qi2 = 9858,
  CODE_FOR_one_cmplv4hi2 = 9859,
  CODE_FOR_one_cmplv2si2 = 9860,
  CODE_FOR_mmx_andv8qi3 = 9861,
  CODE_FOR_mmx_iorv8qi3 = 9862,
  CODE_FOR_mmx_xorv8qi3 = 9863,
  CODE_FOR_mmx_andv4hi3 = 9864,
  CODE_FOR_mmx_iorv4hi3 = 9865,
  CODE_FOR_mmx_xorv4hi3 = 9866,
  CODE_FOR_mmx_andv2si3 = 9867,
  CODE_FOR_mmx_iorv2si3 = 9868,
  CODE_FOR_mmx_xorv2si3 = 9869,
  CODE_FOR_andv8qi3 = 9870,
  CODE_FOR_iorv8qi3 = 9871,
  CODE_FOR_xorv8qi3 = 9872,
  CODE_FOR_andv4hi3 = 9873,
  CODE_FOR_iorv4hi3 = 9874,
  CODE_FOR_xorv4hi3 = 9875,
  CODE_FOR_andv2si3 = 9876,
  CODE_FOR_iorv2si3 = 9877,
  CODE_FOR_xorv2si3 = 9878,
  CODE_FOR_andv4qi3 = 9879,
  CODE_FOR_iorv4qi3 = 9880,
  CODE_FOR_xorv4qi3 = 9881,
  CODE_FOR_andv2qi3 = 9882,
  CODE_FOR_iorv2qi3 = 9883,
  CODE_FOR_xorv2qi3 = 9884,
  CODE_FOR_andv2hi3 = 9885,
  CODE_FOR_iorv2hi3 = 9886,
  CODE_FOR_xorv2hi3 = 9887,
  CODE_FOR_vec_pack_trunc_v4hi = 9888,
  CODE_FOR_vec_pack_trunc_v2si = 9889,
  CODE_FOR_vec_pack_trunc_v2hi = 9890,
  CODE_FOR_vec_unpacks_lo_v8qi = 9891,
  CODE_FOR_vec_unpacks_lo_v4hi = 9892,
  CODE_FOR_vec_unpacks_hi_v8qi = 9893,
  CODE_FOR_vec_unpacks_hi_v4hi = 9894,
  CODE_FOR_vec_unpacku_lo_v8qi = 9895,
  CODE_FOR_vec_unpacku_lo_v4hi = 9896,
  CODE_FOR_vec_unpacku_hi_v8qi = 9897,
  CODE_FOR_vec_unpacku_hi_v4hi = 9898,
  CODE_FOR_vec_unpacks_lo_v4qi = 9899,
  CODE_FOR_vec_unpacks_hi_v4qi = 9900,
  CODE_FOR_vec_unpacku_lo_v4qi = 9901,
  CODE_FOR_vec_unpacku_hi_v4qi = 9902,
  CODE_FOR_mmx_pinsrw = 9903,
  CODE_FOR_mmx_pshufw = 9904,
  CODE_FOR_vec_setv2si = 9905,
  CODE_FOR_vec_extractv2sisi = 9906,
  CODE_FOR_vec_initv2sisi = 9907,
  CODE_FOR_vec_setv4hi = 9908,
  CODE_FOR_vec_extractv4hihi = 9909,
  CODE_FOR_vec_initv4hihi = 9910,
  CODE_FOR_vec_setv8qi = 9911,
  CODE_FOR_vec_extractv8qiqi = 9912,
  CODE_FOR_vec_initv8qiqi = 9913,
  CODE_FOR_vec_setv2hi = 9914,
  CODE_FOR_vec_extractv2hihi = 9915,
  CODE_FOR_vec_setv4qi = 9916,
  CODE_FOR_vec_extractv4qiqi = 9917,
  CODE_FOR_vec_initv2hihi = 9918,
  CODE_FOR_vec_initv4qiqi = 9919,
  CODE_FOR_mmx_uavgv8qi3 = 9920,
  CODE_FOR_mmx_uavgv4hi3 = 9921,
  CODE_FOR_uavgv8qi3_ceil = 9922,
  CODE_FOR_uavgv4hi3_ceil = 9923,
  CODE_FOR_mmx_psadbw = 9924,
  CODE_FOR_reduc_plus_scal_v8qi = 9925,
  CODE_FOR_reduc_plus_scal_v4hi = 9926,
  CODE_FOR_reduc_smax_scal_v4hi = 9927,
  CODE_FOR_reduc_smin_scal_v4hi = 9928,
  CODE_FOR_reduc_smax_scal_v4qi = 9929,
  CODE_FOR_reduc_smin_scal_v4qi = 9930,
  CODE_FOR_reduc_umax_scal_v4hi = 9931,
  CODE_FOR_reduc_umin_scal_v4hi = 9932,
  CODE_FOR_reduc_umax_scal_v4qi = 9933,
  CODE_FOR_reduc_umin_scal_v4qi = 9934,
  CODE_FOR_reduc_plus_scal_v4qi = 9935,
  CODE_FOR_usadv8qi = 9936,
  CODE_FOR_mmx_maskmovq = 9937,
  CODE_FOR_mmx_emms = 9938,
  CODE_FOR_mmx_femms = 9939,
  CODE_FOR_movv64qi = 9940,
  CODE_FOR_movv32qi = 9941,
  CODE_FOR_movv16qi = 9942,
  CODE_FOR_movv32hi = 9943,
  CODE_FOR_movv16hi = 9944,
  CODE_FOR_movv8hi = 9945,
  CODE_FOR_movv16si = 9946,
  CODE_FOR_movv8si = 9947,
  CODE_FOR_movv4si = 9948,
  CODE_FOR_movv8di = 9949,
  CODE_FOR_movv4di = 9950,
  CODE_FOR_movv2di = 9951,
  CODE_FOR_movv4ti = 9952,
  CODE_FOR_movv2ti = 9953,
  CODE_FOR_movv1ti = 9954,
  CODE_FOR_movv32hf = 9955,
  CODE_FOR_movv16hf = 9956,
  CODE_FOR_movv8hf = 9957,
  CODE_FOR_movv32bf = 9958,
  CODE_FOR_movv16bf = 9959,
  CODE_FOR_movv8bf = 9960,
  CODE_FOR_movv16sf = 9961,
  CODE_FOR_movv8sf = 9962,
  CODE_FOR_movv4sf = 9963,
  CODE_FOR_movv8df = 9964,
  CODE_FOR_movv4df = 9965,
  CODE_FOR_movv2df = 9966,
  CODE_FOR_avx512f_loadv16si_mask = 9967,
  CODE_FOR_avx512vl_loadv8si_mask = 9968,
  CODE_FOR_avx512vl_loadv4si_mask = 9969,
  CODE_FOR_avx512f_loadv8di_mask = 9970,
  CODE_FOR_avx512vl_loadv4di_mask = 9971,
  CODE_FOR_avx512vl_loadv2di_mask = 9972,
  CODE_FOR_avx512f_loadv16sf_mask = 9973,
  CODE_FOR_avx512vl_loadv8sf_mask = 9974,
  CODE_FOR_avx512vl_loadv4sf_mask = 9975,
  CODE_FOR_avx512f_loadv8df_mask = 9976,
  CODE_FOR_avx512vl_loadv4df_mask = 9977,
  CODE_FOR_avx512vl_loadv2df_mask = 9978,
  CODE_FOR_avx512bw_loadv64qi_mask = 9979,
  CODE_FOR_avx512vl_loadv16qi_mask = 9980,
  CODE_FOR_avx512vl_loadv32qi_mask = 9981,
  CODE_FOR_avx512bw_loadv32hi_mask = 9982,
  CODE_FOR_avx512vl_loadv16hi_mask = 9983,
  CODE_FOR_avx512vl_loadv8hi_mask = 9984,
  CODE_FOR_avx512f_loadhf_mask = 9985,
  CODE_FOR_avx512f_loadsf_mask = 9986,
  CODE_FOR_avx512f_loaddf_mask = 9987,
  CODE_FOR_sse2_movq128 = 9988,
  CODE_FOR_movmisalignv64qi = 9989,
  CODE_FOR_movmisalignv32qi = 9990,
  CODE_FOR_movmisalignv16qi = 9991,
  CODE_FOR_movmisalignv32hi = 9992,
  CODE_FOR_movmisalignv16hi = 9993,
  CODE_FOR_movmisalignv8hi = 9994,
  CODE_FOR_movmisalignv16si = 9995,
  CODE_FOR_movmisalignv8si = 9996,
  CODE_FOR_movmisalignv4si = 9997,
  CODE_FOR_movmisalignv8di = 9998,
  CODE_FOR_movmisalignv4di = 9999,
  CODE_FOR_movmisalignv2di = 10000,
  CODE_FOR_movmisalignv4ti = 10001,
  CODE_FOR_movmisalignv2ti = 10002,
  CODE_FOR_movmisalignv1ti = 10003,
  CODE_FOR_movmisalignv32hf = 10004,
  CODE_FOR_movmisalignv16hf = 10005,
  CODE_FOR_movmisalignv8hf = 10006,
  CODE_FOR_movmisalignv32bf = 10007,
  CODE_FOR_movmisalignv16bf = 10008,
  CODE_FOR_movmisalignv8bf = 10009,
  CODE_FOR_movmisalignv16sf = 10010,
  CODE_FOR_movmisalignv8sf = 10011,
  CODE_FOR_movmisalignv4sf = 10012,
  CODE_FOR_movmisalignv8df = 10013,
  CODE_FOR_movmisalignv4df = 10014,
  CODE_FOR_movmisalignv2df = 10015,
  CODE_FOR_storentdi = 10016,
  CODE_FOR_storentsi = 10017,
  CODE_FOR_storentsf = 10018,
  CODE_FOR_storentdf = 10019,
  CODE_FOR_storentv8di = 10020,
  CODE_FOR_storentv4di = 10021,
  CODE_FOR_storentv2di = 10022,
  CODE_FOR_storentv16sf = 10023,
  CODE_FOR_storentv8sf = 10024,
  CODE_FOR_storentv4sf = 10025,
  CODE_FOR_storentv8df = 10026,
  CODE_FOR_storentv4df = 10027,
  CODE_FOR_storentv2df = 10028,
  CODE_FOR_kmovb = 10029,
  CODE_FOR_kmovw = 10030,
  CODE_FOR_kmovd = 10031,
  CODE_FOR_kmovq = 10032,
  CODE_FOR_absv32hf2 = 10033,
  CODE_FOR_negv32hf2 = 10034,
  CODE_FOR_absv16hf2 = 10035,
  CODE_FOR_negv16hf2 = 10036,
  CODE_FOR_absv8hf2 = 10037,
  CODE_FOR_negv8hf2 = 10038,
  CODE_FOR_absv16sf2 = 10039,
  CODE_FOR_negv16sf2 = 10040,
  CODE_FOR_absv8sf2 = 10041,
  CODE_FOR_negv8sf2 = 10042,
  CODE_FOR_absv4sf2 = 10043,
  CODE_FOR_negv4sf2 = 10044,
  CODE_FOR_absv8df2 = 10045,
  CODE_FOR_negv8df2 = 10046,
  CODE_FOR_absv4df2 = 10047,
  CODE_FOR_negv4df2 = 10048,
  CODE_FOR_absv2df2 = 10049,
  CODE_FOR_negv2df2 = 10050,
  CODE_FOR_cond_addv32hf = 10051,
  CODE_FOR_cond_subv32hf = 10052,
  CODE_FOR_cond_addv16hf = 10053,
  CODE_FOR_cond_subv16hf = 10054,
  CODE_FOR_cond_addv8hf = 10055,
  CODE_FOR_cond_subv8hf = 10056,
  CODE_FOR_cond_addv16sf = 10057,
  CODE_FOR_cond_subv16sf = 10058,
  CODE_FOR_cond_addv8sf = 10059,
  CODE_FOR_cond_subv8sf = 10060,
  CODE_FOR_cond_addv4sf = 10061,
  CODE_FOR_cond_subv4sf = 10062,
  CODE_FOR_cond_addv8df = 10063,
  CODE_FOR_cond_subv8df = 10064,
  CODE_FOR_cond_addv4df = 10065,
  CODE_FOR_cond_subv4df = 10066,
  CODE_FOR_cond_addv2df = 10067,
  CODE_FOR_cond_subv2df = 10068,
  CODE_FOR_addv32hf3 = 10069,
  CODE_FOR_addv32hf3_round = 10070,
  CODE_FOR_addv32hf3_mask = 10071,
  CODE_FOR_addv32hf3_mask_round = 10072,
  CODE_FOR_subv32hf3 = 10073,
  CODE_FOR_subv32hf3_round = 10074,
  CODE_FOR_subv32hf3_mask = 10075,
  CODE_FOR_subv32hf3_mask_round = 10076,
  CODE_FOR_addv16hf3 = 10077,
   CODE_FOR_addv16hf3_round = CODE_FOR_nothing,
  CODE_FOR_addv16hf3_mask = 10078,
   CODE_FOR_addv16hf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_subv16hf3 = 10079,
   CODE_FOR_subv16hf3_round = CODE_FOR_nothing,
  CODE_FOR_subv16hf3_mask = 10080,
   CODE_FOR_subv16hf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_addv8hf3 = 10081,
   CODE_FOR_addv8hf3_round = CODE_FOR_nothing,
  CODE_FOR_addv8hf3_mask = 10082,
   CODE_FOR_addv8hf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_subv8hf3 = 10083,
   CODE_FOR_subv8hf3_round = CODE_FOR_nothing,
  CODE_FOR_subv8hf3_mask = 10084,
   CODE_FOR_subv8hf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_addv16sf3 = 10085,
  CODE_FOR_addv16sf3_round = 10086,
  CODE_FOR_addv16sf3_mask = 10087,
  CODE_FOR_addv16sf3_mask_round = 10088,
  CODE_FOR_subv16sf3 = 10089,
  CODE_FOR_subv16sf3_round = 10090,
  CODE_FOR_subv16sf3_mask = 10091,
  CODE_FOR_subv16sf3_mask_round = 10092,
  CODE_FOR_addv8sf3 = 10093,
   CODE_FOR_addv8sf3_round = CODE_FOR_nothing,
  CODE_FOR_addv8sf3_mask = 10094,
   CODE_FOR_addv8sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_subv8sf3 = 10095,
   CODE_FOR_subv8sf3_round = CODE_FOR_nothing,
  CODE_FOR_subv8sf3_mask = 10096,
   CODE_FOR_subv8sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_addv4sf3 = 10097,
   CODE_FOR_addv4sf3_round = CODE_FOR_nothing,
  CODE_FOR_addv4sf3_mask = 10098,
   CODE_FOR_addv4sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_subv4sf3 = 10099,
   CODE_FOR_subv4sf3_round = CODE_FOR_nothing,
  CODE_FOR_subv4sf3_mask = 10100,
   CODE_FOR_subv4sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_addv8df3 = 10101,
  CODE_FOR_addv8df3_round = 10102,
  CODE_FOR_addv8df3_mask = 10103,
  CODE_FOR_addv8df3_mask_round = 10104,
  CODE_FOR_subv8df3 = 10105,
  CODE_FOR_subv8df3_round = 10106,
  CODE_FOR_subv8df3_mask = 10107,
  CODE_FOR_subv8df3_mask_round = 10108,
  CODE_FOR_addv4df3 = 10109,
   CODE_FOR_addv4df3_round = CODE_FOR_nothing,
  CODE_FOR_addv4df3_mask = 10110,
   CODE_FOR_addv4df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_subv4df3 = 10111,
   CODE_FOR_subv4df3_round = CODE_FOR_nothing,
  CODE_FOR_subv4df3_mask = 10112,
   CODE_FOR_subv4df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_addv2df3 = 10113,
   CODE_FOR_addv2df3_round = CODE_FOR_nothing,
  CODE_FOR_addv2df3_mask = 10114,
   CODE_FOR_addv2df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_subv2df3 = 10115,
   CODE_FOR_subv2df3_round = CODE_FOR_nothing,
  CODE_FOR_subv2df3_mask = 10116,
   CODE_FOR_subv2df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_cond_mulv32hf = 10117,
  CODE_FOR_cond_mulv16hf = 10118,
  CODE_FOR_cond_mulv8hf = 10119,
  CODE_FOR_cond_mulv16sf = 10120,
  CODE_FOR_cond_mulv8sf = 10121,
  CODE_FOR_cond_mulv4sf = 10122,
  CODE_FOR_cond_mulv8df = 10123,
  CODE_FOR_cond_mulv4df = 10124,
  CODE_FOR_cond_mulv2df = 10125,
  CODE_FOR_mulv32hf3 = 10126,
  CODE_FOR_mulv32hf3_round = 10127,
  CODE_FOR_mulv32hf3_mask = 10128,
  CODE_FOR_mulv32hf3_mask_round = 10129,
  CODE_FOR_mulv16hf3 = 10130,
   CODE_FOR_mulv16hf3_round = CODE_FOR_nothing,
  CODE_FOR_mulv16hf3_mask = 10131,
   CODE_FOR_mulv16hf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_mulv8hf3 = 10132,
   CODE_FOR_mulv8hf3_round = CODE_FOR_nothing,
  CODE_FOR_mulv8hf3_mask = 10133,
   CODE_FOR_mulv8hf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_mulv16sf3 = 10134,
  CODE_FOR_mulv16sf3_round = 10135,
  CODE_FOR_mulv16sf3_mask = 10136,
  CODE_FOR_mulv16sf3_mask_round = 10137,
  CODE_FOR_mulv8sf3 = 10138,
   CODE_FOR_mulv8sf3_round = CODE_FOR_nothing,
  CODE_FOR_mulv8sf3_mask = 10139,
   CODE_FOR_mulv8sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_mulv4sf3 = 10140,
   CODE_FOR_mulv4sf3_round = CODE_FOR_nothing,
  CODE_FOR_mulv4sf3_mask = 10141,
   CODE_FOR_mulv4sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_mulv8df3 = 10142,
  CODE_FOR_mulv8df3_round = 10143,
  CODE_FOR_mulv8df3_mask = 10144,
  CODE_FOR_mulv8df3_mask_round = 10145,
  CODE_FOR_mulv4df3 = 10146,
   CODE_FOR_mulv4df3_round = CODE_FOR_nothing,
  CODE_FOR_mulv4df3_mask = 10147,
   CODE_FOR_mulv4df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_mulv2df3 = 10148,
   CODE_FOR_mulv2df3_round = CODE_FOR_nothing,
  CODE_FOR_mulv2df3_mask = 10149,
   CODE_FOR_mulv2df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_divv8df3 = 10150,
  CODE_FOR_divv4df3 = 10151,
  CODE_FOR_divv2df3 = 10152,
  CODE_FOR_divv32hf3 = 10153,
  CODE_FOR_divv16hf3 = 10154,
  CODE_FOR_divv8hf3 = 10155,
  CODE_FOR_divv16sf3 = 10156,
  CODE_FOR_divv8sf3 = 10157,
  CODE_FOR_divv4sf3 = 10158,
  CODE_FOR_cond_divv32hf = 10159,
  CODE_FOR_cond_divv16hf = 10160,
  CODE_FOR_cond_divv8hf = 10161,
  CODE_FOR_cond_divv16sf = 10162,
  CODE_FOR_cond_divv8sf = 10163,
  CODE_FOR_cond_divv4sf = 10164,
  CODE_FOR_cond_divv8df = 10165,
  CODE_FOR_cond_divv4df = 10166,
  CODE_FOR_cond_divv2df = 10167,
  CODE_FOR_sqrtv32hf2 = 10168,
  CODE_FOR_sqrtv16hf2 = 10169,
  CODE_FOR_sqrtv8hf2 = 10170,
  CODE_FOR_sqrtv8df2 = 10171,
  CODE_FOR_sqrtv4df2 = 10172,
  CODE_FOR_sqrtv2df2 = 10173,
  CODE_FOR_sqrtv16sf2 = 10174,
  CODE_FOR_sqrtv8sf2 = 10175,
  CODE_FOR_sqrtv4sf2 = 10176,
  CODE_FOR_rsqrtv16sf2 = 10177,
  CODE_FOR_rsqrtv8sf2 = 10178,
  CODE_FOR_rsqrtv4sf2 = 10179,
  CODE_FOR_rsqrtv32hf2 = 10180,
  CODE_FOR_rsqrtv16hf2 = 10181,
  CODE_FOR_rsqrtv8hf2 = 10182,
  CODE_FOR_cond_smaxv32hf = 10183,
  CODE_FOR_cond_sminv32hf = 10184,
  CODE_FOR_cond_smaxv16hf = 10185,
  CODE_FOR_cond_sminv16hf = 10186,
  CODE_FOR_cond_smaxv8hf = 10187,
  CODE_FOR_cond_sminv8hf = 10188,
  CODE_FOR_cond_smaxv16sf = 10189,
  CODE_FOR_cond_sminv16sf = 10190,
  CODE_FOR_cond_smaxv8sf = 10191,
  CODE_FOR_cond_sminv8sf = 10192,
  CODE_FOR_cond_smaxv4sf = 10193,
  CODE_FOR_cond_sminv4sf = 10194,
  CODE_FOR_cond_smaxv8df = 10195,
  CODE_FOR_cond_sminv8df = 10196,
  CODE_FOR_cond_smaxv4df = 10197,
  CODE_FOR_cond_sminv4df = 10198,
  CODE_FOR_cond_smaxv2df = 10199,
  CODE_FOR_cond_sminv2df = 10200,
  CODE_FOR_smaxv32hf3 = 10201,
  CODE_FOR_smaxv32hf3_round = 10202,
  CODE_FOR_smaxv32hf3_mask = 10203,
  CODE_FOR_smaxv32hf3_mask_round = 10204,
  CODE_FOR_sminv32hf3 = 10205,
  CODE_FOR_sminv32hf3_round = 10206,
  CODE_FOR_sminv32hf3_mask = 10207,
  CODE_FOR_sminv32hf3_mask_round = 10208,
  CODE_FOR_smaxv16hf3 = 10209,
   CODE_FOR_smaxv16hf3_round = CODE_FOR_nothing,
  CODE_FOR_smaxv16hf3_mask = 10210,
   CODE_FOR_smaxv16hf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_sminv16hf3 = 10211,
   CODE_FOR_sminv16hf3_round = CODE_FOR_nothing,
  CODE_FOR_sminv16hf3_mask = 10212,
   CODE_FOR_sminv16hf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_smaxv8hf3 = 10213,
   CODE_FOR_smaxv8hf3_round = CODE_FOR_nothing,
  CODE_FOR_smaxv8hf3_mask = 10214,
   CODE_FOR_smaxv8hf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_sminv8hf3 = 10215,
   CODE_FOR_sminv8hf3_round = CODE_FOR_nothing,
  CODE_FOR_sminv8hf3_mask = 10216,
   CODE_FOR_sminv8hf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_smaxv16sf3 = 10217,
  CODE_FOR_smaxv16sf3_round = 10218,
  CODE_FOR_smaxv16sf3_mask = 10219,
  CODE_FOR_smaxv16sf3_mask_round = 10220,
  CODE_FOR_sminv16sf3 = 10221,
  CODE_FOR_sminv16sf3_round = 10222,
  CODE_FOR_sminv16sf3_mask = 10223,
  CODE_FOR_sminv16sf3_mask_round = 10224,
  CODE_FOR_smaxv8sf3 = 10225,
   CODE_FOR_smaxv8sf3_round = CODE_FOR_nothing,
  CODE_FOR_smaxv8sf3_mask = 10226,
   CODE_FOR_smaxv8sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_sminv8sf3 = 10227,
   CODE_FOR_sminv8sf3_round = CODE_FOR_nothing,
  CODE_FOR_sminv8sf3_mask = 10228,
   CODE_FOR_sminv8sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_smaxv4sf3 = 10229,
   CODE_FOR_smaxv4sf3_round = CODE_FOR_nothing,
  CODE_FOR_smaxv4sf3_mask = 10230,
   CODE_FOR_smaxv4sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_sminv4sf3 = 10231,
   CODE_FOR_sminv4sf3_round = CODE_FOR_nothing,
  CODE_FOR_sminv4sf3_mask = 10232,
   CODE_FOR_sminv4sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_smaxv8df3 = 10233,
  CODE_FOR_smaxv8df3_round = 10234,
  CODE_FOR_smaxv8df3_mask = 10235,
  CODE_FOR_smaxv8df3_mask_round = 10236,
  CODE_FOR_sminv8df3 = 10237,
  CODE_FOR_sminv8df3_round = 10238,
  CODE_FOR_sminv8df3_mask = 10239,
  CODE_FOR_sminv8df3_mask_round = 10240,
  CODE_FOR_smaxv4df3 = 10241,
   CODE_FOR_smaxv4df3_round = CODE_FOR_nothing,
  CODE_FOR_smaxv4df3_mask = 10242,
   CODE_FOR_smaxv4df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_sminv4df3 = 10243,
   CODE_FOR_sminv4df3_round = CODE_FOR_nothing,
  CODE_FOR_sminv4df3_mask = 10244,
   CODE_FOR_sminv4df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_smaxv2df3 = 10245,
   CODE_FOR_smaxv2df3_round = CODE_FOR_nothing,
  CODE_FOR_smaxv2df3_mask = 10246,
   CODE_FOR_smaxv2df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_sminv2df3 = 10247,
   CODE_FOR_sminv2df3_round = CODE_FOR_nothing,
  CODE_FOR_sminv2df3_mask = 10248,
   CODE_FOR_sminv2df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_sse3_haddv2df3 = 10249,
  CODE_FOR_reduc_plus_scal_v2df = 10250,
  CODE_FOR_reduc_plus_scal_v4sf = 10251,
  CODE_FOR_reduc_plus_scal_v8hf = 10252,
  CODE_FOR_reduc_plus_scal_v16qi = 10253,
  CODE_FOR_reduc_plus_scal_v4df = 10254,
  CODE_FOR_reduc_plus_scal_v8sf = 10255,
  CODE_FOR_reduc_plus_scal_v16hf = 10256,
  CODE_FOR_reduc_plus_scal_v8df = 10257,
  CODE_FOR_reduc_plus_scal_v16sf = 10258,
  CODE_FOR_reduc_plus_scal_v32hf = 10259,
  CODE_FOR_reduc_plus_scal_v32qi = 10260,
  CODE_FOR_reduc_plus_scal_v64qi = 10261,
  CODE_FOR_reduc_smax_scal_v8hf = 10262,
  CODE_FOR_reduc_smin_scal_v8hf = 10263,
  CODE_FOR_reduc_smax_scal_v4sf = 10264,
  CODE_FOR_reduc_smin_scal_v4sf = 10265,
  CODE_FOR_reduc_smax_scal_v2df = 10266,
  CODE_FOR_reduc_smin_scal_v2df = 10267,
  CODE_FOR_reduc_smax_scal_v4si = 10268,
  CODE_FOR_reduc_smin_scal_v4si = 10269,
  CODE_FOR_reduc_smax_scal_v8hi = 10270,
  CODE_FOR_reduc_smin_scal_v8hi = 10271,
  CODE_FOR_reduc_smax_scal_v16qi = 10272,
  CODE_FOR_reduc_smin_scal_v16qi = 10273,
  CODE_FOR_reduc_smax_scal_v2di = 10274,
  CODE_FOR_reduc_smin_scal_v2di = 10275,
  CODE_FOR_reduc_smax_scal_v32qi = 10276,
  CODE_FOR_reduc_smin_scal_v32qi = 10277,
  CODE_FOR_reduc_smax_scal_v16hi = 10278,
  CODE_FOR_reduc_smin_scal_v16hi = 10279,
  CODE_FOR_reduc_smax_scal_v16hf = 10280,
  CODE_FOR_reduc_smin_scal_v16hf = 10281,
  CODE_FOR_reduc_smax_scal_v8si = 10282,
  CODE_FOR_reduc_smin_scal_v8si = 10283,
  CODE_FOR_reduc_smax_scal_v4di = 10284,
  CODE_FOR_reduc_smin_scal_v4di = 10285,
  CODE_FOR_reduc_smax_scal_v8sf = 10286,
  CODE_FOR_reduc_smin_scal_v8sf = 10287,
  CODE_FOR_reduc_smax_scal_v4df = 10288,
  CODE_FOR_reduc_smin_scal_v4df = 10289,
  CODE_FOR_reduc_smax_scal_v64qi = 10290,
  CODE_FOR_reduc_smin_scal_v64qi = 10291,
  CODE_FOR_reduc_smax_scal_v32hf = 10292,
  CODE_FOR_reduc_smin_scal_v32hf = 10293,
  CODE_FOR_reduc_smax_scal_v32hi = 10294,
  CODE_FOR_reduc_smin_scal_v32hi = 10295,
  CODE_FOR_reduc_smax_scal_v16si = 10296,
  CODE_FOR_reduc_smin_scal_v16si = 10297,
  CODE_FOR_reduc_smax_scal_v8di = 10298,
  CODE_FOR_reduc_smin_scal_v8di = 10299,
  CODE_FOR_reduc_smax_scal_v16sf = 10300,
  CODE_FOR_reduc_smin_scal_v16sf = 10301,
  CODE_FOR_reduc_smax_scal_v8df = 10302,
  CODE_FOR_reduc_smin_scal_v8df = 10303,
  CODE_FOR_reduc_umax_scal_v16si = 10304,
  CODE_FOR_reduc_umin_scal_v16si = 10305,
  CODE_FOR_reduc_umax_scal_v8di = 10306,
  CODE_FOR_reduc_umin_scal_v8di = 10307,
  CODE_FOR_reduc_umax_scal_v32hi = 10308,
  CODE_FOR_reduc_umin_scal_v32hi = 10309,
  CODE_FOR_reduc_umax_scal_v64qi = 10310,
  CODE_FOR_reduc_umin_scal_v64qi = 10311,
  CODE_FOR_reduc_umax_scal_v32qi = 10312,
  CODE_FOR_reduc_umin_scal_v32qi = 10313,
  CODE_FOR_reduc_umax_scal_v16hi = 10314,
  CODE_FOR_reduc_umin_scal_v16hi = 10315,
  CODE_FOR_reduc_umax_scal_v8si = 10316,
  CODE_FOR_reduc_umin_scal_v8si = 10317,
  CODE_FOR_reduc_umax_scal_v4di = 10318,
  CODE_FOR_reduc_umin_scal_v4di = 10319,
  CODE_FOR_reduc_umin_scal_v8hi = 10320,
  CODE_FOR_vec_cmpv16sihi = 10321,
  CODE_FOR_vec_cmpv8siqi = 10322,
  CODE_FOR_vec_cmpv4siqi = 10323,
  CODE_FOR_vec_cmpv8diqi = 10324,
  CODE_FOR_vec_cmpv4diqi = 10325,
  CODE_FOR_vec_cmpv2diqi = 10326,
  CODE_FOR_vec_cmpv32hfsi = 10327,
  CODE_FOR_vec_cmpv16hfhi = 10328,
  CODE_FOR_vec_cmpv8hfqi = 10329,
  CODE_FOR_vec_cmpv16sfhi = 10330,
  CODE_FOR_vec_cmpv8sfqi = 10331,
  CODE_FOR_vec_cmpv4sfqi = 10332,
  CODE_FOR_vec_cmpv8dfqi = 10333,
  CODE_FOR_vec_cmpv4dfqi = 10334,
  CODE_FOR_vec_cmpv2dfqi = 10335,
  CODE_FOR_vec_cmpv64qidi = 10336,
  CODE_FOR_vec_cmpv16qihi = 10337,
  CODE_FOR_vec_cmpv32qisi = 10338,
  CODE_FOR_vec_cmpv32hisi = 10339,
  CODE_FOR_vec_cmpv16hihi = 10340,
  CODE_FOR_vec_cmpv8hiqi = 10341,
  CODE_FOR_vec_cmpv32qiv32qi = 10342,
  CODE_FOR_vec_cmpv16hiv16hi = 10343,
  CODE_FOR_vec_cmpv8siv8si = 10344,
  CODE_FOR_vec_cmpv4div4di = 10345,
  CODE_FOR_vec_cmpv16qiv16qi = 10346,
  CODE_FOR_vec_cmpv8hiv8hi = 10347,
  CODE_FOR_vec_cmpv4siv4si = 10348,
  CODE_FOR_vec_cmpv2div2di = 10349,
  CODE_FOR_vec_cmpv8sfv8si = 10350,
  CODE_FOR_vec_cmpv4dfv4di = 10351,
  CODE_FOR_vec_cmpv4sfv4si = 10352,
  CODE_FOR_vec_cmpv2dfv2di = 10353,
  CODE_FOR_vec_cmpuv16sihi = 10354,
  CODE_FOR_vec_cmpuv8siqi = 10355,
  CODE_FOR_vec_cmpuv4siqi = 10356,
  CODE_FOR_vec_cmpuv8diqi = 10357,
  CODE_FOR_vec_cmpuv4diqi = 10358,
  CODE_FOR_vec_cmpuv2diqi = 10359,
  CODE_FOR_vec_cmpuv64qidi = 10360,
  CODE_FOR_vec_cmpuv16qihi = 10361,
  CODE_FOR_vec_cmpuv32qisi = 10362,
  CODE_FOR_vec_cmpuv32hisi = 10363,
  CODE_FOR_vec_cmpuv16hihi = 10364,
  CODE_FOR_vec_cmpuv8hiqi = 10365,
  CODE_FOR_vec_cmpuv32qiv32qi = 10366,
  CODE_FOR_vec_cmpuv16hiv16hi = 10367,
  CODE_FOR_vec_cmpuv8siv8si = 10368,
  CODE_FOR_vec_cmpuv4div4di = 10369,
  CODE_FOR_vec_cmpuv16qiv16qi = 10370,
  CODE_FOR_vec_cmpuv8hiv8hi = 10371,
  CODE_FOR_vec_cmpuv4siv4si = 10372,
  CODE_FOR_vec_cmpuv2div2di = 10373,
  CODE_FOR_vec_cmpeqv2div2di = 10374,
  CODE_FOR_vec_cmpeqv1tiv1ti = 10375,
  CODE_FOR_vcondv64qiv16sf = 10376,
  CODE_FOR_vcondv64qiv8df = 10377,
  CODE_FOR_vcondv32hiv16sf = 10378,
  CODE_FOR_vcondv32hiv8df = 10379,
  CODE_FOR_vcondv16siv16sf = 10380,
  CODE_FOR_vcondv16siv8df = 10381,
  CODE_FOR_vcondv8div16sf = 10382,
  CODE_FOR_vcondv8div8df = 10383,
  CODE_FOR_vcondv16sfv16sf = 10384,
  CODE_FOR_vcondv16sfv8df = 10385,
  CODE_FOR_vcondv8dfv16sf = 10386,
  CODE_FOR_vcondv8dfv8df = 10387,
  CODE_FOR_vcondv32qiv8sf = 10388,
  CODE_FOR_vcondv32qiv4df = 10389,
  CODE_FOR_vcondv16hiv8sf = 10390,
  CODE_FOR_vcondv16hiv4df = 10391,
  CODE_FOR_vcondv8siv8sf = 10392,
  CODE_FOR_vcondv8siv4df = 10393,
  CODE_FOR_vcondv4div8sf = 10394,
  CODE_FOR_vcondv4div4df = 10395,
  CODE_FOR_vcondv8sfv8sf = 10396,
  CODE_FOR_vcondv8sfv4df = 10397,
  CODE_FOR_vcondv4dfv8sf = 10398,
  CODE_FOR_vcondv4dfv4df = 10399,
  CODE_FOR_vcondv16qiv4sf = 10400,
  CODE_FOR_vcondv8hiv4sf = 10401,
  CODE_FOR_vcondv4siv4sf = 10402,
  CODE_FOR_vcondv2div4sf = 10403,
  CODE_FOR_vcondv4sfv4sf = 10404,
  CODE_FOR_vcondv2dfv4sf = 10405,
  CODE_FOR_vcondv16qiv2df = 10406,
  CODE_FOR_vcondv8hiv2df = 10407,
  CODE_FOR_vcondv4siv2df = 10408,
  CODE_FOR_vcondv2div2df = 10409,
  CODE_FOR_vcondv4sfv2df = 10410,
  CODE_FOR_vcondv2dfv2df = 10411,
  CODE_FOR_vcondv32hfv32hf = 10412,
  CODE_FOR_vcondv16hfv16hf = 10413,
  CODE_FOR_vcondv8hfv8hf = 10414,
  CODE_FOR_vcondv32hfv32hi = 10415,
  CODE_FOR_vcondv16hfv16hi = 10416,
  CODE_FOR_vcondv8hfv8hi = 10417,
  CODE_FOR_vcondv32bfv32hi = 10418,
  CODE_FOR_vcondv16bfv16hi = 10419,
  CODE_FOR_vcondv8bfv8hi = 10420,
  CODE_FOR_vcondv32hiv32hf = 10421,
  CODE_FOR_vcondv16hiv16hf = 10422,
  CODE_FOR_vcondv8hiv8hf = 10423,
  CODE_FOR_vcond_mask_v16sihi = 10424,
  CODE_FOR_vcond_mask_v8siqi = 10425,
  CODE_FOR_vcond_mask_v4siqi = 10426,
  CODE_FOR_vcond_mask_v8diqi = 10427,
  CODE_FOR_vcond_mask_v4diqi = 10428,
  CODE_FOR_vcond_mask_v2diqi = 10429,
  CODE_FOR_vcond_mask_v16sfhi = 10430,
  CODE_FOR_vcond_mask_v8sfqi = 10431,
  CODE_FOR_vcond_mask_v4sfqi = 10432,
  CODE_FOR_vcond_mask_v8dfqi = 10433,
  CODE_FOR_vcond_mask_v4dfqi = 10434,
  CODE_FOR_vcond_mask_v2dfqi = 10435,
  CODE_FOR_vcond_mask_v64qidi = 10436,
  CODE_FOR_vcond_mask_v16qihi = 10437,
  CODE_FOR_vcond_mask_v32qisi = 10438,
  CODE_FOR_vcond_mask_v32hisi = 10439,
  CODE_FOR_vcond_mask_v16hihi = 10440,
  CODE_FOR_vcond_mask_v8hiqi = 10441,
  CODE_FOR_vcond_mask_v32hfsi = 10442,
  CODE_FOR_vcond_mask_v16hfhi = 10443,
  CODE_FOR_vcond_mask_v8hfqi = 10444,
  CODE_FOR_vcond_mask_v32bfsi = 10445,
  CODE_FOR_vcond_mask_v16bfhi = 10446,
  CODE_FOR_vcond_mask_v8bfqi = 10447,
  CODE_FOR_vcond_mask_v32qiv32qi = 10448,
  CODE_FOR_vcond_mask_v16hiv16hi = 10449,
  CODE_FOR_vcond_mask_v8siv8si = 10450,
  CODE_FOR_vcond_mask_v4div4di = 10451,
  CODE_FOR_vcond_mask_v16qiv16qi = 10452,
  CODE_FOR_vcond_mask_v8hiv8hi = 10453,
  CODE_FOR_vcond_mask_v4siv4si = 10454,
  CODE_FOR_vcond_mask_v2div2di = 10455,
  CODE_FOR_vcond_mask_v1tiv1ti = 10456,
  CODE_FOR_vcond_mask_v8sfv8si = 10457,
  CODE_FOR_vcond_mask_v4dfv4di = 10458,
  CODE_FOR_vcond_mask_v4sfv4si = 10459,
  CODE_FOR_vcond_mask_v2dfv2di = 10460,
  CODE_FOR_andv16hf3 = 10461,
   CODE_FOR_andv16hf3_mask = CODE_FOR_nothing,
  CODE_FOR_iorv16hf3 = 10462,
   CODE_FOR_iorv16hf3_mask = CODE_FOR_nothing,
  CODE_FOR_xorv16hf3 = 10463,
   CODE_FOR_xorv16hf3_mask = CODE_FOR_nothing,
  CODE_FOR_andv8hf3 = 10464,
   CODE_FOR_andv8hf3_mask = CODE_FOR_nothing,
  CODE_FOR_iorv8hf3 = 10465,
   CODE_FOR_iorv8hf3_mask = CODE_FOR_nothing,
  CODE_FOR_xorv8hf3 = 10466,
   CODE_FOR_xorv8hf3_mask = CODE_FOR_nothing,
  CODE_FOR_andv8sf3 = 10467,
  CODE_FOR_andv8sf3_mask = 10468,
  CODE_FOR_iorv8sf3 = 10469,
  CODE_FOR_iorv8sf3_mask = 10470,
  CODE_FOR_xorv8sf3 = 10471,
  CODE_FOR_xorv8sf3_mask = 10472,
  CODE_FOR_andv4sf3 = 10473,
  CODE_FOR_andv4sf3_mask = 10474,
  CODE_FOR_iorv4sf3 = 10475,
  CODE_FOR_iorv4sf3_mask = 10476,
  CODE_FOR_xorv4sf3 = 10477,
  CODE_FOR_xorv4sf3_mask = 10478,
  CODE_FOR_andv4df3 = 10479,
  CODE_FOR_andv4df3_mask = 10480,
  CODE_FOR_iorv4df3 = 10481,
  CODE_FOR_iorv4df3_mask = 10482,
  CODE_FOR_xorv4df3 = 10483,
  CODE_FOR_xorv4df3_mask = 10484,
  CODE_FOR_andv2df3 = 10485,
  CODE_FOR_andv2df3_mask = 10486,
  CODE_FOR_iorv2df3 = 10487,
  CODE_FOR_iorv2df3_mask = 10488,
  CODE_FOR_xorv2df3 = 10489,
  CODE_FOR_xorv2df3_mask = 10490,
  CODE_FOR_andv32hf3 = 10491,
   CODE_FOR_andv32hf3_mask = CODE_FOR_nothing,
  CODE_FOR_iorv32hf3 = 10492,
   CODE_FOR_iorv32hf3_mask = CODE_FOR_nothing,
  CODE_FOR_xorv32hf3 = 10493,
   CODE_FOR_xorv32hf3_mask = CODE_FOR_nothing,
  CODE_FOR_andv16sf3 = 10494,
  CODE_FOR_andv16sf3_mask = 10495,
  CODE_FOR_iorv16sf3 = 10496,
  CODE_FOR_iorv16sf3_mask = 10497,
  CODE_FOR_xorv16sf3 = 10498,
  CODE_FOR_xorv16sf3_mask = 10499,
  CODE_FOR_andv8df3 = 10500,
  CODE_FOR_andv8df3_mask = 10501,
  CODE_FOR_iorv8df3 = 10502,
  CODE_FOR_iorv8df3_mask = 10503,
  CODE_FOR_xorv8df3 = 10504,
  CODE_FOR_xorv8df3_mask = 10505,
  CODE_FOR_copysignv32hf3 = 10506,
  CODE_FOR_copysignv16hf3 = 10507,
  CODE_FOR_copysignv8hf3 = 10508,
  CODE_FOR_copysignv16sf3 = 10509,
  CODE_FOR_copysignv8sf3 = 10510,
  CODE_FOR_copysignv4sf3 = 10511,
  CODE_FOR_copysignv8df3 = 10512,
  CODE_FOR_copysignv4df3 = 10513,
  CODE_FOR_copysignv2df3 = 10514,
  CODE_FOR_xorsignv32hf3 = 10515,
  CODE_FOR_xorsignv16hf3 = 10516,
  CODE_FOR_xorsignv8hf3 = 10517,
  CODE_FOR_xorsignv16sf3 = 10518,
  CODE_FOR_xorsignv8sf3 = 10519,
  CODE_FOR_xorsignv4sf3 = 10520,
  CODE_FOR_xorsignv8df3 = 10521,
  CODE_FOR_xorsignv4df3 = 10522,
  CODE_FOR_xorsignv2df3 = 10523,
  CODE_FOR_signbitv16sf2 = 10524,
  CODE_FOR_signbitv8sf2 = 10525,
  CODE_FOR_signbitv4sf2 = 10526,
  CODE_FOR_andtf3 = 10527,
  CODE_FOR_iortf3 = 10528,
  CODE_FOR_xortf3 = 10529,
  CODE_FOR_fmasf4 = 10530,
  CODE_FOR_fmadf4 = 10531,
  CODE_FOR_fmav4sf4 = 10532,
  CODE_FOR_fmav2df4 = 10533,
  CODE_FOR_fmav8sf4 = 10534,
  CODE_FOR_fmav4df4 = 10535,
  CODE_FOR_fmav16sf4 = 10536,
  CODE_FOR_fmav8df4 = 10537,
  CODE_FOR_fmahf4 = 10538,
  CODE_FOR_fmav8hf4 = 10539,
  CODE_FOR_fmav16hf4 = 10540,
  CODE_FOR_fmav32hf4 = 10541,
  CODE_FOR_fmssf4 = 10542,
  CODE_FOR_fmsdf4 = 10543,
  CODE_FOR_fmsv4sf4 = 10544,
  CODE_FOR_fmsv2df4 = 10545,
  CODE_FOR_fmsv8sf4 = 10546,
  CODE_FOR_fmsv4df4 = 10547,
  CODE_FOR_fmsv16sf4 = 10548,
  CODE_FOR_fmsv8df4 = 10549,
  CODE_FOR_fmshf4 = 10550,
  CODE_FOR_fmsv8hf4 = 10551,
  CODE_FOR_fmsv16hf4 = 10552,
  CODE_FOR_fmsv32hf4 = 10553,
  CODE_FOR_fnmasf4 = 10554,
  CODE_FOR_fnmadf4 = 10555,
  CODE_FOR_fnmav4sf4 = 10556,
  CODE_FOR_fnmav2df4 = 10557,
  CODE_FOR_fnmav8sf4 = 10558,
  CODE_FOR_fnmav4df4 = 10559,
  CODE_FOR_fnmav16sf4 = 10560,
  CODE_FOR_fnmav8df4 = 10561,
  CODE_FOR_fnmahf4 = 10562,
  CODE_FOR_fnmav8hf4 = 10563,
  CODE_FOR_fnmav16hf4 = 10564,
  CODE_FOR_fnmav32hf4 = 10565,
  CODE_FOR_fnmssf4 = 10566,
  CODE_FOR_fnmsdf4 = 10567,
  CODE_FOR_fnmsv4sf4 = 10568,
  CODE_FOR_fnmsv2df4 = 10569,
  CODE_FOR_fnmsv8sf4 = 10570,
  CODE_FOR_fnmsv4df4 = 10571,
  CODE_FOR_fnmsv16sf4 = 10572,
  CODE_FOR_fnmsv8df4 = 10573,
  CODE_FOR_fnmshf4 = 10574,
  CODE_FOR_fnmsv8hf4 = 10575,
  CODE_FOR_fnmsv16hf4 = 10576,
  CODE_FOR_fnmsv32hf4 = 10577,
  CODE_FOR_fma4i_fmadd_sf = 10578,
  CODE_FOR_fma4i_fmadd_df = 10579,
  CODE_FOR_fma4i_fmadd_v4sf = 10580,
  CODE_FOR_fma4i_fmadd_v2df = 10581,
  CODE_FOR_fma4i_fmadd_v8sf = 10582,
  CODE_FOR_fma4i_fmadd_v4df = 10583,
  CODE_FOR_fma4i_fmadd_v16sf = 10584,
  CODE_FOR_fma4i_fmadd_v8df = 10585,
  CODE_FOR_fma4i_fmsub_sf = 10586,
  CODE_FOR_fma4i_fmsub_df = 10587,
  CODE_FOR_fma4i_fmsub_v4sf = 10588,
  CODE_FOR_fma4i_fmsub_v2df = 10589,
  CODE_FOR_fma4i_fmsub_v8sf = 10590,
  CODE_FOR_fma4i_fmsub_v4df = 10591,
  CODE_FOR_fma4i_fmsub_v16sf = 10592,
  CODE_FOR_fma4i_fmsub_v8df = 10593,
  CODE_FOR_fma4i_fnmadd_sf = 10594,
  CODE_FOR_fma4i_fnmadd_df = 10595,
  CODE_FOR_fma4i_fnmadd_v4sf = 10596,
  CODE_FOR_fma4i_fnmadd_v2df = 10597,
  CODE_FOR_fma4i_fnmadd_v8sf = 10598,
  CODE_FOR_fma4i_fnmadd_v4df = 10599,
  CODE_FOR_fma4i_fnmadd_v16sf = 10600,
  CODE_FOR_fma4i_fnmadd_v8df = 10601,
  CODE_FOR_fma4i_fnmsub_sf = 10602,
  CODE_FOR_fma4i_fnmsub_df = 10603,
  CODE_FOR_fma4i_fnmsub_v4sf = 10604,
  CODE_FOR_fma4i_fnmsub_v2df = 10605,
  CODE_FOR_fma4i_fnmsub_v8sf = 10606,
  CODE_FOR_fma4i_fnmsub_v4df = 10607,
  CODE_FOR_fma4i_fnmsub_v16sf = 10608,
  CODE_FOR_fma4i_fnmsub_v8df = 10609,
  CODE_FOR_avx512bw_fmadd_v32hf_maskz = 10610,
  CODE_FOR_avx512bw_fmadd_v32hf_maskz_round = 10611,
  CODE_FOR_avx512vl_fmadd_v16hf_maskz = 10612,
  CODE_FOR_avx512vl_fmadd_v16hf_maskz_round = 10613,
  CODE_FOR_avx512fp16_fmadd_v8hf_maskz = 10614,
  CODE_FOR_avx512fp16_fmadd_v8hf_maskz_round = 10615,
  CODE_FOR_avx512f_fmadd_v16sf_maskz = 10616,
  CODE_FOR_avx512f_fmadd_v16sf_maskz_round = 10617,
  CODE_FOR_avx512vl_fmadd_v8sf_maskz = 10618,
  CODE_FOR_avx512vl_fmadd_v8sf_maskz_round = 10619,
  CODE_FOR_avx512vl_fmadd_v4sf_maskz = 10620,
  CODE_FOR_avx512vl_fmadd_v4sf_maskz_round = 10621,
  CODE_FOR_avx512f_fmadd_v8df_maskz = 10622,
  CODE_FOR_avx512f_fmadd_v8df_maskz_round = 10623,
  CODE_FOR_avx512vl_fmadd_v4df_maskz = 10624,
  CODE_FOR_avx512vl_fmadd_v4df_maskz_round = 10625,
  CODE_FOR_avx512vl_fmadd_v2df_maskz = 10626,
  CODE_FOR_avx512vl_fmadd_v2df_maskz_round = 10627,
  CODE_FOR_cond_fmav32hf = 10628,
  CODE_FOR_cond_fmav16hf = 10629,
  CODE_FOR_cond_fmav8hf = 10630,
  CODE_FOR_cond_fmav16sf = 10631,
  CODE_FOR_cond_fmav8sf = 10632,
  CODE_FOR_cond_fmav4sf = 10633,
  CODE_FOR_cond_fmav8df = 10634,
  CODE_FOR_cond_fmav4df = 10635,
  CODE_FOR_cond_fmav2df = 10636,
  CODE_FOR_avx512bw_fmsub_v32hf_maskz = 10637,
  CODE_FOR_avx512bw_fmsub_v32hf_maskz_round = 10638,
  CODE_FOR_avx512vl_fmsub_v16hf_maskz = 10639,
  CODE_FOR_avx512vl_fmsub_v16hf_maskz_round = 10640,
  CODE_FOR_avx512fp16_fmsub_v8hf_maskz = 10641,
  CODE_FOR_avx512fp16_fmsub_v8hf_maskz_round = 10642,
  CODE_FOR_avx512f_fmsub_v16sf_maskz = 10643,
  CODE_FOR_avx512f_fmsub_v16sf_maskz_round = 10644,
  CODE_FOR_avx512vl_fmsub_v8sf_maskz = 10645,
  CODE_FOR_avx512vl_fmsub_v8sf_maskz_round = 10646,
  CODE_FOR_avx512vl_fmsub_v4sf_maskz = 10647,
  CODE_FOR_avx512vl_fmsub_v4sf_maskz_round = 10648,
  CODE_FOR_avx512f_fmsub_v8df_maskz = 10649,
  CODE_FOR_avx512f_fmsub_v8df_maskz_round = 10650,
  CODE_FOR_avx512vl_fmsub_v4df_maskz = 10651,
  CODE_FOR_avx512vl_fmsub_v4df_maskz_round = 10652,
  CODE_FOR_avx512vl_fmsub_v2df_maskz = 10653,
  CODE_FOR_avx512vl_fmsub_v2df_maskz_round = 10654,
  CODE_FOR_cond_fmsv32hf = 10655,
  CODE_FOR_cond_fmsv16hf = 10656,
  CODE_FOR_cond_fmsv8hf = 10657,
  CODE_FOR_cond_fmsv16sf = 10658,
  CODE_FOR_cond_fmsv8sf = 10659,
  CODE_FOR_cond_fmsv4sf = 10660,
  CODE_FOR_cond_fmsv8df = 10661,
  CODE_FOR_cond_fmsv4df = 10662,
  CODE_FOR_cond_fmsv2df = 10663,
  CODE_FOR_avx512bw_fnmadd_v32hf_maskz = 10664,
  CODE_FOR_avx512bw_fnmadd_v32hf_maskz_round = 10665,
  CODE_FOR_avx512vl_fnmadd_v16hf_maskz = 10666,
  CODE_FOR_avx512vl_fnmadd_v16hf_maskz_round = 10667,
  CODE_FOR_avx512fp16_fnmadd_v8hf_maskz = 10668,
  CODE_FOR_avx512fp16_fnmadd_v8hf_maskz_round = 10669,
  CODE_FOR_avx512f_fnmadd_v16sf_maskz = 10670,
  CODE_FOR_avx512f_fnmadd_v16sf_maskz_round = 10671,
  CODE_FOR_avx512vl_fnmadd_v8sf_maskz = 10672,
  CODE_FOR_avx512vl_fnmadd_v8sf_maskz_round = 10673,
  CODE_FOR_avx512vl_fnmadd_v4sf_maskz = 10674,
  CODE_FOR_avx512vl_fnmadd_v4sf_maskz_round = 10675,
  CODE_FOR_avx512f_fnmadd_v8df_maskz = 10676,
  CODE_FOR_avx512f_fnmadd_v8df_maskz_round = 10677,
  CODE_FOR_avx512vl_fnmadd_v4df_maskz = 10678,
  CODE_FOR_avx512vl_fnmadd_v4df_maskz_round = 10679,
  CODE_FOR_avx512vl_fnmadd_v2df_maskz = 10680,
  CODE_FOR_avx512vl_fnmadd_v2df_maskz_round = 10681,
  CODE_FOR_cond_fnmav32hf = 10682,
  CODE_FOR_cond_fnmav16hf = 10683,
  CODE_FOR_cond_fnmav8hf = 10684,
  CODE_FOR_cond_fnmav16sf = 10685,
  CODE_FOR_cond_fnmav8sf = 10686,
  CODE_FOR_cond_fnmav4sf = 10687,
  CODE_FOR_cond_fnmav8df = 10688,
  CODE_FOR_cond_fnmav4df = 10689,
  CODE_FOR_cond_fnmav2df = 10690,
  CODE_FOR_avx512bw_fnmsub_v32hf_maskz = 10691,
  CODE_FOR_avx512bw_fnmsub_v32hf_maskz_round = 10692,
  CODE_FOR_avx512vl_fnmsub_v16hf_maskz = 10693,
  CODE_FOR_avx512vl_fnmsub_v16hf_maskz_round = 10694,
  CODE_FOR_avx512fp16_fnmsub_v8hf_maskz = 10695,
  CODE_FOR_avx512fp16_fnmsub_v8hf_maskz_round = 10696,
  CODE_FOR_avx512f_fnmsub_v16sf_maskz = 10697,
  CODE_FOR_avx512f_fnmsub_v16sf_maskz_round = 10698,
  CODE_FOR_avx512vl_fnmsub_v8sf_maskz = 10699,
  CODE_FOR_avx512vl_fnmsub_v8sf_maskz_round = 10700,
  CODE_FOR_avx512vl_fnmsub_v4sf_maskz = 10701,
  CODE_FOR_avx512vl_fnmsub_v4sf_maskz_round = 10702,
  CODE_FOR_avx512f_fnmsub_v8df_maskz = 10703,
  CODE_FOR_avx512f_fnmsub_v8df_maskz_round = 10704,
  CODE_FOR_avx512vl_fnmsub_v4df_maskz = 10705,
  CODE_FOR_avx512vl_fnmsub_v4df_maskz_round = 10706,
  CODE_FOR_avx512vl_fnmsub_v2df_maskz = 10707,
  CODE_FOR_avx512vl_fnmsub_v2df_maskz_round = 10708,
  CODE_FOR_cond_fnmsv32hf = 10709,
  CODE_FOR_cond_fnmsv16hf = 10710,
  CODE_FOR_cond_fnmsv8hf = 10711,
  CODE_FOR_cond_fnmsv16sf = 10712,
  CODE_FOR_cond_fnmsv8sf = 10713,
  CODE_FOR_cond_fnmsv4sf = 10714,
  CODE_FOR_cond_fnmsv8df = 10715,
  CODE_FOR_cond_fnmsv4df = 10716,
  CODE_FOR_cond_fnmsv2df = 10717,
  CODE_FOR_vec_fmaddsubv16sf4 = 10718,
  CODE_FOR_vec_fmaddsubv8sf4 = 10719,
  CODE_FOR_vec_fmaddsubv4sf4 = 10720,
  CODE_FOR_vec_fmaddsubv8df4 = 10721,
  CODE_FOR_vec_fmaddsubv4df4 = 10722,
  CODE_FOR_vec_fmaddsubv2df4 = 10723,
  CODE_FOR_vec_fmsubaddv16sf4 = 10724,
  CODE_FOR_vec_fmsubaddv8sf4 = 10725,
  CODE_FOR_vec_fmsubaddv4sf4 = 10726,
  CODE_FOR_vec_fmsubaddv8df4 = 10727,
  CODE_FOR_vec_fmsubaddv4df4 = 10728,
  CODE_FOR_vec_fmsubaddv2df4 = 10729,
  CODE_FOR_fmaddsub_v16sf = 10730,
  CODE_FOR_fmaddsub_v8sf = 10731,
  CODE_FOR_fmaddsub_v4sf = 10732,
  CODE_FOR_fmaddsub_v8df = 10733,
  CODE_FOR_fmaddsub_v4df = 10734,
  CODE_FOR_fmaddsub_v2df = 10735,
  CODE_FOR_avx512bw_fmaddsub_v32hf_maskz = 10736,
  CODE_FOR_avx512bw_fmaddsub_v32hf_maskz_round = 10737,
  CODE_FOR_avx512vl_fmaddsub_v16hf_maskz = 10738,
  CODE_FOR_avx512vl_fmaddsub_v16hf_maskz_round = 10739,
  CODE_FOR_avx512fp16_fmaddsub_v8hf_maskz = 10740,
  CODE_FOR_avx512fp16_fmaddsub_v8hf_maskz_round = 10741,
  CODE_FOR_avx512f_fmaddsub_v16sf_maskz = 10742,
  CODE_FOR_avx512f_fmaddsub_v16sf_maskz_round = 10743,
  CODE_FOR_avx512vl_fmaddsub_v8sf_maskz = 10744,
  CODE_FOR_avx512vl_fmaddsub_v8sf_maskz_round = 10745,
  CODE_FOR_avx512vl_fmaddsub_v4sf_maskz = 10746,
  CODE_FOR_avx512vl_fmaddsub_v4sf_maskz_round = 10747,
  CODE_FOR_avx512f_fmaddsub_v8df_maskz = 10748,
  CODE_FOR_avx512f_fmaddsub_v8df_maskz_round = 10749,
  CODE_FOR_avx512vl_fmaddsub_v4df_maskz = 10750,
  CODE_FOR_avx512vl_fmaddsub_v4df_maskz_round = 10751,
  CODE_FOR_avx512vl_fmaddsub_v2df_maskz = 10752,
  CODE_FOR_avx512vl_fmaddsub_v2df_maskz_round = 10753,
  CODE_FOR_avx512bw_fmsubadd_v32hf_maskz = 10754,
  CODE_FOR_avx512bw_fmsubadd_v32hf_maskz_round = 10755,
  CODE_FOR_avx512vl_fmsubadd_v16hf_maskz = 10756,
  CODE_FOR_avx512vl_fmsubadd_v16hf_maskz_round = 10757,
  CODE_FOR_avx512fp16_fmsubadd_v8hf_maskz = 10758,
  CODE_FOR_avx512fp16_fmsubadd_v8hf_maskz_round = 10759,
  CODE_FOR_avx512f_fmsubadd_v16sf_maskz = 10760,
  CODE_FOR_avx512f_fmsubadd_v16sf_maskz_round = 10761,
  CODE_FOR_avx512vl_fmsubadd_v8sf_maskz = 10762,
  CODE_FOR_avx512vl_fmsubadd_v8sf_maskz_round = 10763,
  CODE_FOR_avx512vl_fmsubadd_v4sf_maskz = 10764,
  CODE_FOR_avx512vl_fmsubadd_v4sf_maskz_round = 10765,
  CODE_FOR_avx512f_fmsubadd_v8df_maskz = 10766,
  CODE_FOR_avx512f_fmsubadd_v8df_maskz_round = 10767,
  CODE_FOR_avx512vl_fmsubadd_v4df_maskz = 10768,
  CODE_FOR_avx512vl_fmsubadd_v4df_maskz_round = 10769,
  CODE_FOR_avx512vl_fmsubadd_v2df_maskz = 10770,
  CODE_FOR_avx512vl_fmsubadd_v2df_maskz_round = 10771,
  CODE_FOR_fmai_vmfmadd_v8hf = 10772,
  CODE_FOR_fmai_vmfmadd_v8hf_round = 10773,
  CODE_FOR_fmai_vmfmadd_v4sf = 10774,
  CODE_FOR_fmai_vmfmadd_v4sf_round = 10775,
  CODE_FOR_fmai_vmfmadd_v2df = 10776,
  CODE_FOR_fmai_vmfmadd_v2df_round = 10777,
  CODE_FOR_fmai_vmfmsub_v8hf = 10778,
  CODE_FOR_fmai_vmfmsub_v8hf_round = 10779,
  CODE_FOR_fmai_vmfmsub_v4sf = 10780,
  CODE_FOR_fmai_vmfmsub_v4sf_round = 10781,
  CODE_FOR_fmai_vmfmsub_v2df = 10782,
  CODE_FOR_fmai_vmfmsub_v2df_round = 10783,
  CODE_FOR_fmai_vmfnmadd_v8hf = 10784,
  CODE_FOR_fmai_vmfnmadd_v8hf_round = 10785,
  CODE_FOR_fmai_vmfnmadd_v4sf = 10786,
  CODE_FOR_fmai_vmfnmadd_v4sf_round = 10787,
  CODE_FOR_fmai_vmfnmadd_v2df = 10788,
  CODE_FOR_fmai_vmfnmadd_v2df_round = 10789,
  CODE_FOR_fmai_vmfnmsub_v8hf = 10790,
  CODE_FOR_fmai_vmfnmsub_v8hf_round = 10791,
  CODE_FOR_fmai_vmfnmsub_v4sf = 10792,
  CODE_FOR_fmai_vmfnmsub_v4sf_round = 10793,
  CODE_FOR_fmai_vmfnmsub_v2df = 10794,
  CODE_FOR_fmai_vmfnmsub_v2df_round = 10795,
  CODE_FOR_avx512f_vmfmadd_v8hf_maskz = 10796,
  CODE_FOR_avx512f_vmfmadd_v8hf_maskz_round = 10797,
  CODE_FOR_avx512f_vmfmadd_v4sf_maskz = 10798,
  CODE_FOR_avx512f_vmfmadd_v4sf_maskz_round = 10799,
  CODE_FOR_avx512f_vmfmadd_v2df_maskz = 10800,
  CODE_FOR_avx512f_vmfmadd_v2df_maskz_round = 10801,
  CODE_FOR_avx512f_vmfnmadd_v8hf_maskz = 10802,
  CODE_FOR_avx512f_vmfnmadd_v8hf_maskz_round = 10803,
  CODE_FOR_avx512f_vmfnmadd_v4sf_maskz = 10804,
  CODE_FOR_avx512f_vmfnmadd_v4sf_maskz_round = 10805,
  CODE_FOR_avx512f_vmfnmadd_v2df_maskz = 10806,
  CODE_FOR_avx512f_vmfnmadd_v2df_maskz_round = 10807,
  CODE_FOR_fma4i_vmfmadd_v4sf = 10808,
  CODE_FOR_fma4i_vmfmadd_v2df = 10809,
  CODE_FOR_avx512bw_fmaddc_v32hf_mask1 = 10810,
  CODE_FOR_avx512bw_fmaddc_v32hf_mask1_round = 10811,
  CODE_FOR_avx512vl_fmaddc_v16hf_mask1 = 10812,
  CODE_FOR_avx512vl_fmaddc_v16hf_mask1_round = 10813,
  CODE_FOR_avx512fp16_fmaddc_v8hf_mask1 = 10814,
  CODE_FOR_avx512fp16_fmaddc_v8hf_mask1_round = 10815,
  CODE_FOR_avx512bw_fmaddc_v32hf_maskz = 10816,
  CODE_FOR_avx512bw_fmaddc_v32hf_maskz_round = 10817,
  CODE_FOR_avx512vl_fmaddc_v16hf_maskz = 10818,
  CODE_FOR_avx512vl_fmaddc_v16hf_maskz_round = 10819,
  CODE_FOR_avx512fp16_fmaddc_v8hf_maskz = 10820,
  CODE_FOR_avx512fp16_fmaddc_v8hf_maskz_round = 10821,
  CODE_FOR_avx512bw_fcmaddc_v32hf_mask1 = 10822,
  CODE_FOR_avx512bw_fcmaddc_v32hf_mask1_round = 10823,
  CODE_FOR_avx512vl_fcmaddc_v16hf_mask1 = 10824,
  CODE_FOR_avx512vl_fcmaddc_v16hf_mask1_round = 10825,
  CODE_FOR_avx512fp16_fcmaddc_v8hf_mask1 = 10826,
  CODE_FOR_avx512fp16_fcmaddc_v8hf_mask1_round = 10827,
  CODE_FOR_avx512bw_fcmaddc_v32hf_maskz = 10828,
  CODE_FOR_avx512bw_fcmaddc_v32hf_maskz_round = 10829,
  CODE_FOR_avx512vl_fcmaddc_v16hf_maskz = 10830,
  CODE_FOR_avx512vl_fcmaddc_v16hf_maskz_round = 10831,
  CODE_FOR_avx512fp16_fcmaddc_v8hf_maskz = 10832,
  CODE_FOR_avx512fp16_fcmaddc_v8hf_maskz_round = 10833,
  CODE_FOR_cmlav32hf4 = 10834,
  CODE_FOR_cmla_conjv32hf4 = 10835,
  CODE_FOR_cmlav16hf4 = 10836,
  CODE_FOR_cmla_conjv16hf4 = 10837,
  CODE_FOR_cmlav8hf4 = 10838,
  CODE_FOR_cmla_conjv8hf4 = 10839,
  CODE_FOR_cmulv32hf3 = 10840,
  CODE_FOR_cmul_conjv32hf3 = 10841,
  CODE_FOR_cmulv16hf3 = 10842,
  CODE_FOR_cmul_conjv16hf3 = 10843,
  CODE_FOR_cmulv8hf3 = 10844,
  CODE_FOR_cmul_conjv8hf3 = 10845,
  CODE_FOR_avx512fp16_fmaddcsh_v8hf_maskz = 10846,
  CODE_FOR_avx512fp16_fmaddcsh_v8hf_maskz_round = 10847,
  CODE_FOR_avx512fp16_fmaddcsh_v8hf_mask1 = 10848,
  CODE_FOR_avx512fp16_fmaddcsh_v8hf_mask1_round = 10849,
  CODE_FOR_avx512fp16_fcmaddcsh_v8hf_maskz = 10850,
  CODE_FOR_avx512fp16_fcmaddcsh_v8hf_maskz_round = 10851,
  CODE_FOR_avx512fp16_fcmaddcsh_v8hf_mask1 = 10852,
  CODE_FOR_avx512fp16_fcmaddcsh_v8hf_mask1_round = 10853,
  CODE_FOR_avx512fp16_fcmaddcsh_v8hf_mask3 = 10854,
  CODE_FOR_avx512fp16_fcmaddcsh_v8hf_mask3_round = 10855,
  CODE_FOR_avx512fp16_fmaddcsh_v8hf_mask3 = 10856,
  CODE_FOR_avx512fp16_fmaddcsh_v8hf_mask3_round = 10857,
  CODE_FOR_floatv8hiv8hf2 = 10858,
  CODE_FOR_floatunsv8hiv8hf2 = 10859,
  CODE_FOR_floatv16hiv16hf2 = 10860,
  CODE_FOR_floatunsv16hiv16hf2 = 10861,
  CODE_FOR_floatv32hiv32hf2 = 10862,
  CODE_FOR_floatunsv32hiv32hf2 = 10863,
  CODE_FOR_floatv8siv8hf2 = 10864,
  CODE_FOR_floatunsv8siv8hf2 = 10865,
  CODE_FOR_floatv16siv16hf2 = 10866,
  CODE_FOR_floatunsv16siv16hf2 = 10867,
  CODE_FOR_floatv8div8hf2 = 10868,
  CODE_FOR_floatunsv8div8hf2 = 10869,
  CODE_FOR_floatv4siv4hf2 = 10870,
  CODE_FOR_floatunsv4siv4hf2 = 10871,
  CODE_FOR_floatv4div4hf2 = 10872,
  CODE_FOR_floatunsv4div4hf2 = 10873,
  CODE_FOR_avx512fp16_floatv4siv4hf2 = 10874,
  CODE_FOR_avx512fp16_floatunsv4siv4hf2 = 10875,
  CODE_FOR_avx512fp16_floatv4div4hf2 = 10876,
  CODE_FOR_avx512fp16_floatunsv4div4hf2 = 10877,
  CODE_FOR_avx512fp16_vcvtdq2ph_v4si_mask = 10878,
  CODE_FOR_avx512fp16_vcvtudq2ph_v4si_mask = 10879,
  CODE_FOR_avx512fp16_vcvtqq2ph_v4di_mask = 10880,
  CODE_FOR_avx512fp16_vcvtuqq2ph_v4di_mask = 10881,
  CODE_FOR_floatv2div2hf2 = 10882,
  CODE_FOR_floatunsv2div2hf2 = 10883,
  CODE_FOR_avx512fp16_floatv2div2hf2 = 10884,
  CODE_FOR_avx512fp16_floatunsv2div2hf2 = 10885,
  CODE_FOR_avx512fp16_vcvtqq2ph_v2di_mask = 10886,
  CODE_FOR_avx512fp16_vcvtuqq2ph_v2di_mask = 10887,
  CODE_FOR_fix_truncv8hfv8hi2 = 10888,
  CODE_FOR_fixuns_truncv8hfv8hi2 = 10889,
  CODE_FOR_fix_truncv16hfv16hi2 = 10890,
  CODE_FOR_fixuns_truncv16hfv16hi2 = 10891,
  CODE_FOR_fix_truncv32hfv32hi2 = 10892,
  CODE_FOR_fixuns_truncv32hfv32hi2 = 10893,
  CODE_FOR_fix_truncv8hfv8si2 = 10894,
  CODE_FOR_fixuns_truncv8hfv8si2 = 10895,
  CODE_FOR_fix_truncv16hfv16si2 = 10896,
  CODE_FOR_fixuns_truncv16hfv16si2 = 10897,
  CODE_FOR_fix_truncv8hfv8di2 = 10898,
  CODE_FOR_fixuns_truncv8hfv8di2 = 10899,
  CODE_FOR_fix_truncv4hfv4si2 = 10900,
  CODE_FOR_fixuns_truncv4hfv4si2 = 10901,
  CODE_FOR_fix_truncv4hfv4di2 = 10902,
  CODE_FOR_fixuns_truncv4hfv4di2 = 10903,
  CODE_FOR_fix_truncv2hfv2di2 = 10904,
  CODE_FOR_fixuns_truncv2hfv2di2 = 10905,
  CODE_FOR_extendv8hfv8df2 = 10906,
  CODE_FOR_extendv16hfv16sf2 = 10907,
  CODE_FOR_extendv8hfv8sf2 = 10908,
  CODE_FOR_extendv4hfv4df2 = 10909,
  CODE_FOR_extendv4hfv4sf2 = 10910,
  CODE_FOR_extendv2hfv2df2 = 10911,
  CODE_FOR_truncv8dfv8hf2 = 10912,
  CODE_FOR_truncv16sfv16hf2 = 10913,
  CODE_FOR_truncv8sfv8hf2 = 10914,
  CODE_FOR_truncv4dfv4hf2 = 10915,
  CODE_FOR_truncv4sfv4hf2 = 10916,
  CODE_FOR_avx512fp16_truncv4dfv4hf2 = 10917,
  CODE_FOR_avx512fp16_truncv4sfv4hf2 = 10918,
  CODE_FOR_avx512fp16_vcvtpd2ph_v4df_mask = 10919,
  CODE_FOR_avx512fp16_vcvtps2ph_v4sf_mask = 10920,
  CODE_FOR_truncv2dfv2hf2 = 10921,
  CODE_FOR_avx512fp16_truncv2dfv2hf2 = 10922,
  CODE_FOR_avx512fp16_vcvtpd2ph_v2df_mask = 10923,
  CODE_FOR_floatunsv16siv16sf2 = 10924,
  CODE_FOR_floatunsv8siv8sf2 = 10925,
  CODE_FOR_floatunsv4siv4sf2 = 10926,
  CODE_FOR_fixuns_truncv8sfv8si2 = 10927,
  CODE_FOR_fixuns_truncv4sfv4si2 = 10928,
  CODE_FOR_floatv2siv2df2 = 10929,
  CODE_FOR_fix_truncv2dfv2si2 = 10930,
  CODE_FOR_avx512dq_floatv2div2sf2 = 10931,
  CODE_FOR_avx512dq_floatunsv2div2sf2 = 10932,
  CODE_FOR_floatv2div2sf2 = 10933,
  CODE_FOR_floatunsv2div2sf2 = 10934,
  CODE_FOR_vec_packs_float_v8di = 10935,
  CODE_FOR_vec_packu_float_v8di = 10936,
  CODE_FOR_vec_packs_float_v4di = 10937,
  CODE_FOR_vec_packu_float_v4di = 10938,
  CODE_FOR_vec_packs_float_v2di = 10939,
  CODE_FOR_vec_packu_float_v2di = 10940,
  CODE_FOR_floatv2div2sf2_mask = 10941,
  CODE_FOR_floatunsv2div2sf2_mask = 10942,
  CODE_FOR_avx_cvtpd2dq256_2 = 10943,
  CODE_FOR_fix_truncv2sfv2di2 = 10944,
  CODE_FOR_fixuns_truncv2sfv2di2 = 10945,
  CODE_FOR_vec_unpack_sfix_trunc_lo_v16sf = 10946,
  CODE_FOR_vec_unpack_ufix_trunc_lo_v16sf = 10947,
  CODE_FOR_vec_unpack_sfix_trunc_lo_v8sf = 10948,
  CODE_FOR_vec_unpack_ufix_trunc_lo_v8sf = 10949,
  CODE_FOR_vec_unpack_sfix_trunc_lo_v4sf = 10950,
  CODE_FOR_vec_unpack_ufix_trunc_lo_v4sf = 10951,
  CODE_FOR_vec_unpack_sfix_trunc_hi_v16sf = 10952,
  CODE_FOR_vec_unpack_ufix_trunc_hi_v16sf = 10953,
  CODE_FOR_vec_unpack_sfix_trunc_hi_v8sf = 10954,
  CODE_FOR_vec_unpack_ufix_trunc_hi_v8sf = 10955,
  CODE_FOR_vec_unpack_sfix_trunc_hi_v4sf = 10956,
  CODE_FOR_vec_unpack_ufix_trunc_hi_v4sf = 10957,
  CODE_FOR_avx_cvttpd2dq256_2 = 10958,
  CODE_FOR_sse2_cvtpd2ps = 10959,
  CODE_FOR_sse2_cvtpd2ps_mask = 10960,
  CODE_FOR_truncv8dfv8sf2 = 10961,
  CODE_FOR_truncv4dfv4sf2 = 10962,
  CODE_FOR_extendv8sfv8df2 = 10963,
  CODE_FOR_extendv4sfv4df2 = 10964,
  CODE_FOR_avx512bw_cvtmask2bv64qi = 10965,
  CODE_FOR_avx512vl_cvtmask2bv16qi = 10966,
  CODE_FOR_avx512vl_cvtmask2bv32qi = 10967,
  CODE_FOR_avx512bw_cvtmask2wv32hi = 10968,
  CODE_FOR_avx512vl_cvtmask2wv16hi = 10969,
  CODE_FOR_avx512vl_cvtmask2wv8hi = 10970,
  CODE_FOR_avx512f_cvtmask2dv16si = 10971,
  CODE_FOR_avx512vl_cvtmask2dv8si = 10972,
  CODE_FOR_avx512vl_cvtmask2dv4si = 10973,
  CODE_FOR_avx512f_cvtmask2qv8di = 10974,
  CODE_FOR_avx512vl_cvtmask2qv4di = 10975,
  CODE_FOR_avx512vl_cvtmask2qv2di = 10976,
  CODE_FOR_extendv2sfv2df2 = 10977,
  CODE_FOR_vec_unpacks_hi_v4sf = 10978,
  CODE_FOR_vec_unpacks_hi_v8sf = 10979,
  CODE_FOR_vec_unpacks_hi_v16sf = 10980,
  CODE_FOR_vec_unpacks_lo_v4sf = 10981,
  CODE_FOR_vec_unpacks_lo_v8sf = 10982,
  CODE_FOR_vec_unpacks_float_hi_v32hi = 10983,
  CODE_FOR_vec_unpacks_float_hi_v16hi = 10984,
  CODE_FOR_vec_unpacks_float_hi_v8hi = 10985,
  CODE_FOR_vec_unpacks_float_lo_v32hi = 10986,
  CODE_FOR_vec_unpacks_float_lo_v16hi = 10987,
  CODE_FOR_vec_unpacks_float_lo_v8hi = 10988,
  CODE_FOR_vec_unpacku_float_hi_v32hi = 10989,
  CODE_FOR_vec_unpacku_float_hi_v16hi = 10990,
  CODE_FOR_vec_unpacku_float_hi_v8hi = 10991,
  CODE_FOR_vec_unpacku_float_lo_v32hi = 10992,
  CODE_FOR_vec_unpacku_float_lo_v16hi = 10993,
  CODE_FOR_vec_unpacku_float_lo_v8hi = 10994,
  CODE_FOR_vec_unpacks_float_hi_v4si = 10995,
  CODE_FOR_vec_unpacks_float_lo_v4si = 10996,
  CODE_FOR_vec_unpacks_float_hi_v8si = 10997,
  CODE_FOR_vec_unpacks_float_lo_v8si = 10998,
  CODE_FOR_vec_unpacks_float_hi_v16si = 10999,
  CODE_FOR_vec_unpacks_float_lo_v16si = 11000,
  CODE_FOR_vec_unpacku_float_hi_v4si = 11001,
  CODE_FOR_vec_unpacku_float_lo_v4si = 11002,
  CODE_FOR_vec_unpacku_float_hi_v8si = 11003,
  CODE_FOR_vec_unpacku_float_hi_v16si = 11004,
  CODE_FOR_vec_unpacku_float_lo_v8si = 11005,
  CODE_FOR_vec_unpacku_float_lo_v16si = 11006,
  CODE_FOR_vec_pack_trunc_v8df = 11007,
  CODE_FOR_vec_pack_trunc_v4df = 11008,
  CODE_FOR_vec_pack_trunc_v2df = 11009,
  CODE_FOR_vec_pack_sfix_trunc_v8df = 11010,
  CODE_FOR_vec_pack_sfix_trunc_v4df = 11011,
  CODE_FOR_vec_pack_sfix_trunc_v2df = 11012,
  CODE_FOR_vec_pack_ufix_trunc_v8df = 11013,
  CODE_FOR_vec_pack_ufix_trunc_v4df = 11014,
  CODE_FOR_vec_pack_ufix_trunc_v2df = 11015,
  CODE_FOR_avx512f_vec_pack_sfix_v8df = 11016,
  CODE_FOR_vec_pack_sfix_v4df = 11017,
  CODE_FOR_vec_pack_sfix_v2df = 11018,
  CODE_FOR_sse_movhlps_exp = 11019,
  CODE_FOR_sse_movlhps_exp = 11020,
  CODE_FOR_vec_interleave_highv8sf = 11021,
  CODE_FOR_vec_interleave_lowv8sf = 11022,
  CODE_FOR_avx_shufps256 = 11023,
  CODE_FOR_avx_shufps256_mask = 11024,
  CODE_FOR_sse_shufps = 11025,
  CODE_FOR_sse_shufps_mask = 11026,
  CODE_FOR_sse_loadhps_exp = 11027,
  CODE_FOR_sse_loadlps_exp = 11028,
  CODE_FOR_vec_setv16qi = 11029,
  CODE_FOR_vec_setv8hi = 11030,
  CODE_FOR_vec_setv4si = 11031,
  CODE_FOR_vec_setv2di = 11032,
  CODE_FOR_vec_setv4sf = 11033,
  CODE_FOR_vec_setv2df = 11034,
  CODE_FOR_vec_setv8hf = 11035,
  CODE_FOR_vec_setv8bf = 11036,
  CODE_FOR_vec_setv32qi = 11037,
  CODE_FOR_vec_setv16hi = 11038,
  CODE_FOR_vec_setv16hf = 11039,
  CODE_FOR_vec_setv16bf = 11040,
  CODE_FOR_vec_setv8si = 11041,
  CODE_FOR_vec_setv4di = 11042,
  CODE_FOR_vec_setv8sf = 11043,
  CODE_FOR_vec_setv4df = 11044,
  CODE_FOR_vec_setv64qi = 11045,
  CODE_FOR_vec_setv32hi = 11046,
  CODE_FOR_vec_setv32hf = 11047,
  CODE_FOR_vec_setv32bf = 11048,
  CODE_FOR_vec_setv16si = 11049,
  CODE_FOR_vec_setv8di = 11050,
  CODE_FOR_vec_setv16sf = 11051,
  CODE_FOR_vec_setv8df = 11052,
  CODE_FOR_avx512dq_vextractf64x2_mask = 11053,
  CODE_FOR_avx512dq_vextracti64x2_mask = 11054,
  CODE_FOR_avx512f_vextractf32x4_mask = 11055,
  CODE_FOR_avx512f_vextracti32x4_mask = 11056,
  CODE_FOR_avx512dq_vextractf32x8_mask = 11057,
  CODE_FOR_avx512dq_vextracti32x8_mask = 11058,
  CODE_FOR_avx512f_vextractf64x4_mask = 11059,
  CODE_FOR_avx512f_vextracti64x4_mask = 11060,
  CODE_FOR_avx512vl_vextractf128v8si = 11061,
  CODE_FOR_avx512vl_vextractf128v8sf = 11062,
  CODE_FOR_avx512vl_vextractf128v4di = 11063,
  CODE_FOR_avx512vl_vextractf128v4df = 11064,
  CODE_FOR_avx_vextractf128v32qi = 11065,
  CODE_FOR_avx_vextractf128v16hi = 11066,
  CODE_FOR_avx_vextractf128v8si = 11067,
  CODE_FOR_avx_vextractf128v4di = 11068,
  CODE_FOR_avx_vextractf128v8sf = 11069,
  CODE_FOR_avx_vextractf128v4df = 11070,
  CODE_FOR_avx_vextractf128v16hf = 11071,
  CODE_FOR_avx_vextractf128v16bf = 11072,
  CODE_FOR_vec_extractv64qiqi = 11073,
  CODE_FOR_vec_extractv32qiqi = 11074,
  CODE_FOR_vec_extractv16qiqi = 11075,
  CODE_FOR_vec_extractv32hihi = 11076,
  CODE_FOR_vec_extractv16hihi = 11077,
  CODE_FOR_vec_extractv8hihi = 11078,
  CODE_FOR_vec_extractv16sisi = 11079,
  CODE_FOR_vec_extractv8sisi = 11080,
  CODE_FOR_vec_extractv4sisi = 11081,
  CODE_FOR_vec_extractv8didi = 11082,
  CODE_FOR_vec_extractv4didi = 11083,
  CODE_FOR_vec_extractv2didi = 11084,
  CODE_FOR_vec_extractv32hfhf = 11085,
  CODE_FOR_vec_extractv16hfhf = 11086,
  CODE_FOR_vec_extractv8hfhf = 11087,
  CODE_FOR_vec_extractv32bfbf = 11088,
  CODE_FOR_vec_extractv16bfbf = 11089,
  CODE_FOR_vec_extractv8bfbf = 11090,
  CODE_FOR_vec_extractv16sfsf = 11091,
  CODE_FOR_vec_extractv8sfsf = 11092,
  CODE_FOR_vec_extractv4sfsf = 11093,
  CODE_FOR_vec_extractv8dfdf = 11094,
  CODE_FOR_vec_extractv4dfdf = 11095,
  CODE_FOR_vec_extractv2dfdf = 11096,
  CODE_FOR_vec_extractv4titi = 11097,
  CODE_FOR_vec_extractv2titi = 11098,
  CODE_FOR_vec_extractv32qiv16qi = 11099,
  CODE_FOR_vec_extractv16hiv8hi = 11100,
  CODE_FOR_vec_extractv16hfv8hf = 11101,
  CODE_FOR_vec_extractv16bfv8bf = 11102,
  CODE_FOR_vec_extractv8siv4si = 11103,
  CODE_FOR_vec_extractv4div2di = 11104,
  CODE_FOR_vec_extractv8sfv4sf = 11105,
  CODE_FOR_vec_extractv4dfv2df = 11106,
  CODE_FOR_vec_extractv64qiv32qi = 11107,
  CODE_FOR_vec_extractv32hiv16hi = 11108,
  CODE_FOR_vec_extractv32hfv16hf = 11109,
  CODE_FOR_vec_extractv32bfv16bf = 11110,
  CODE_FOR_vec_extractv16siv8si = 11111,
  CODE_FOR_vec_extractv8div4di = 11112,
  CODE_FOR_vec_extractv16sfv8sf = 11113,
  CODE_FOR_vec_extractv8dfv4df = 11114,
  CODE_FOR_vec_interleave_highv4df = 11115,
  CODE_FOR_vec_interleave_highv2df = 11116,
  CODE_FOR_vec_interleave_lowv4df = 11117,
  CODE_FOR_vec_interleave_lowv2df = 11118,
  CODE_FOR_avx512f_vternlogv16si_maskz = 11119,
  CODE_FOR_avx512vl_vternlogv8si_maskz = 11120,
  CODE_FOR_avx512vl_vternlogv4si_maskz = 11121,
  CODE_FOR_avx512f_vternlogv8di_maskz = 11122,
  CODE_FOR_avx512vl_vternlogv4di_maskz = 11123,
  CODE_FOR_avx512vl_vternlogv2di_maskz = 11124,
  CODE_FOR_avx512f_shufps512_mask = 11125,
  CODE_FOR_avx512f_fixupimmv16sf_maskz = 11126,
  CODE_FOR_avx512f_fixupimmv16sf_maskz_round = 11127,
  CODE_FOR_avx512vl_fixupimmv8sf_maskz = 11128,
  CODE_FOR_avx512vl_fixupimmv8sf_maskz_round = 11129,
  CODE_FOR_avx512vl_fixupimmv4sf_maskz = 11130,
  CODE_FOR_avx512vl_fixupimmv4sf_maskz_round = 11131,
  CODE_FOR_avx512f_fixupimmv8df_maskz = 11132,
  CODE_FOR_avx512f_fixupimmv8df_maskz_round = 11133,
  CODE_FOR_avx512vl_fixupimmv4df_maskz = 11134,
  CODE_FOR_avx512vl_fixupimmv4df_maskz_round = 11135,
  CODE_FOR_avx512vl_fixupimmv2df_maskz = 11136,
  CODE_FOR_avx512vl_fixupimmv2df_maskz_round = 11137,
  CODE_FOR_avx512f_sfixupimmv4sf_maskz = 11138,
  CODE_FOR_avx512f_sfixupimmv4sf_maskz_round = 11139,
  CODE_FOR_avx512f_sfixupimmv2df_maskz = 11140,
  CODE_FOR_avx512f_sfixupimmv2df_maskz_round = 11141,
  CODE_FOR_avx512f_shufpd512_mask = 11142,
  CODE_FOR_avx_shufpd256 = 11143,
  CODE_FOR_avx_shufpd256_mask = 11144,
  CODE_FOR_sse2_shufpd = 11145,
  CODE_FOR_sse2_shufpd_mask = 11146,
  CODE_FOR_sse2_loadhpd_exp = 11147,
  CODE_FOR_sse2_loadlpd_exp = 11148,
  CODE_FOR_truncv16siv16qi2 = 11149,
  CODE_FOR_truncv16siv16hi2 = 11150,
  CODE_FOR_truncv8div8si2 = 11151,
  CODE_FOR_truncv8div8hi2 = 11152,
  CODE_FOR_avx512f_ss_truncatev16siv16qi2_mask_store = 11153,
  CODE_FOR_avx512f_truncatev16siv16qi2_mask_store = 11154,
  CODE_FOR_avx512f_us_truncatev16siv16qi2_mask_store = 11155,
  CODE_FOR_avx512f_ss_truncatev16siv16hi2_mask_store = 11156,
  CODE_FOR_avx512f_truncatev16siv16hi2_mask_store = 11157,
  CODE_FOR_avx512f_us_truncatev16siv16hi2_mask_store = 11158,
  CODE_FOR_avx512f_ss_truncatev8div8si2_mask_store = 11159,
  CODE_FOR_avx512f_truncatev8div8si2_mask_store = 11160,
  CODE_FOR_avx512f_us_truncatev8div8si2_mask_store = 11161,
  CODE_FOR_avx512f_ss_truncatev8div8hi2_mask_store = 11162,
  CODE_FOR_avx512f_truncatev8div8hi2_mask_store = 11163,
  CODE_FOR_avx512f_us_truncatev8div8hi2_mask_store = 11164,
  CODE_FOR_truncv32hiv32qi2 = 11165,
  CODE_FOR_avx512bw_ss_truncatev32hiv32qi2_mask_store = 11166,
  CODE_FOR_avx512bw_truncatev32hiv32qi2_mask_store = 11167,
  CODE_FOR_avx512bw_us_truncatev32hiv32qi2_mask_store = 11168,
  CODE_FOR_truncv4div4si2 = 11169,
  CODE_FOR_truncv8siv8hi2 = 11170,
  CODE_FOR_truncv16hiv16qi2 = 11171,
  CODE_FOR_avx512vl_ss_truncatev4div4si2_mask_store = 11172,
  CODE_FOR_avx512vl_truncatev4div4si2_mask_store = 11173,
  CODE_FOR_avx512vl_us_truncatev4div4si2_mask_store = 11174,
  CODE_FOR_avx512vl_ss_truncatev8siv8hi2_mask_store = 11175,
  CODE_FOR_avx512vl_truncatev8siv8hi2_mask_store = 11176,
  CODE_FOR_avx512vl_us_truncatev8siv8hi2_mask_store = 11177,
  CODE_FOR_avx512vl_ss_truncatev16hiv16qi2_mask_store = 11178,
  CODE_FOR_avx512vl_truncatev16hiv16qi2_mask_store = 11179,
  CODE_FOR_avx512vl_us_truncatev16hiv16qi2_mask_store = 11180,
  CODE_FOR_truncv4div4qi2 = 11181,
  CODE_FOR_truncv2div2qi2 = 11182,
  CODE_FOR_truncv8siv8qi2 = 11183,
  CODE_FOR_truncv4siv4qi2 = 11184,
  CODE_FOR_truncv8hiv8qi2 = 11185,
  CODE_FOR_truncv4div4hi2 = 11186,
  CODE_FOR_truncv2div2hi2 = 11187,
  CODE_FOR_truncv4siv4hi2 = 11188,
  CODE_FOR_truncv2div2si2 = 11189,
  CODE_FOR_truncv8div8qi2 = 11190,
  CODE_FOR_negv64qi2 = 11191,
  CODE_FOR_negv32qi2 = 11192,
  CODE_FOR_negv16qi2 = 11193,
  CODE_FOR_negv32hi2 = 11194,
  CODE_FOR_negv16hi2 = 11195,
  CODE_FOR_negv8hi2 = 11196,
  CODE_FOR_negv16si2 = 11197,
  CODE_FOR_negv8si2 = 11198,
  CODE_FOR_negv4si2 = 11199,
  CODE_FOR_negv8di2 = 11200,
  CODE_FOR_negv4di2 = 11201,
  CODE_FOR_negv2di2 = 11202,
  CODE_FOR_addv64qi3 = 11203,
  CODE_FOR_subv64qi3 = 11204,
  CODE_FOR_addv32qi3 = 11205,
  CODE_FOR_subv32qi3 = 11206,
  CODE_FOR_addv16qi3 = 11207,
  CODE_FOR_subv16qi3 = 11208,
  CODE_FOR_addv32hi3 = 11209,
  CODE_FOR_subv32hi3 = 11210,
  CODE_FOR_addv16hi3 = 11211,
  CODE_FOR_subv16hi3 = 11212,
  CODE_FOR_addv8hi3 = 11213,
  CODE_FOR_subv8hi3 = 11214,
  CODE_FOR_addv16si3 = 11215,
  CODE_FOR_subv16si3 = 11216,
  CODE_FOR_addv8si3 = 11217,
  CODE_FOR_subv8si3 = 11218,
  CODE_FOR_addv4si3 = 11219,
  CODE_FOR_subv4si3 = 11220,
  CODE_FOR_addv8di3 = 11221,
  CODE_FOR_subv8di3 = 11222,
  CODE_FOR_addv4di3 = 11223,
  CODE_FOR_subv4di3 = 11224,
  CODE_FOR_addv2di3 = 11225,
  CODE_FOR_subv2di3 = 11226,
  CODE_FOR_cond_addv64qi = 11227,
  CODE_FOR_cond_subv64qi = 11228,
  CODE_FOR_cond_addv32qi = 11229,
  CODE_FOR_cond_subv32qi = 11230,
  CODE_FOR_cond_addv16qi = 11231,
  CODE_FOR_cond_subv16qi = 11232,
  CODE_FOR_cond_addv32hi = 11233,
  CODE_FOR_cond_subv32hi = 11234,
  CODE_FOR_cond_addv16hi = 11235,
  CODE_FOR_cond_subv16hi = 11236,
  CODE_FOR_cond_addv8hi = 11237,
  CODE_FOR_cond_subv8hi = 11238,
  CODE_FOR_cond_addv16si = 11239,
  CODE_FOR_cond_subv16si = 11240,
  CODE_FOR_cond_addv8si = 11241,
  CODE_FOR_cond_subv8si = 11242,
  CODE_FOR_cond_addv4si = 11243,
  CODE_FOR_cond_subv4si = 11244,
  CODE_FOR_cond_addv8di = 11245,
  CODE_FOR_cond_subv8di = 11246,
  CODE_FOR_cond_addv4di = 11247,
  CODE_FOR_cond_subv4di = 11248,
  CODE_FOR_cond_addv2di = 11249,
  CODE_FOR_cond_subv2di = 11250,
  CODE_FOR_addv16si3_mask = 11251,
  CODE_FOR_subv16si3_mask = 11252,
  CODE_FOR_addv8si3_mask = 11253,
  CODE_FOR_subv8si3_mask = 11254,
  CODE_FOR_addv4si3_mask = 11255,
  CODE_FOR_subv4si3_mask = 11256,
  CODE_FOR_addv8di3_mask = 11257,
  CODE_FOR_subv8di3_mask = 11258,
  CODE_FOR_addv4di3_mask = 11259,
  CODE_FOR_subv4di3_mask = 11260,
  CODE_FOR_addv2di3_mask = 11261,
  CODE_FOR_subv2di3_mask = 11262,
  CODE_FOR_addv64qi3_mask = 11263,
  CODE_FOR_subv64qi3_mask = 11264,
  CODE_FOR_addv16qi3_mask = 11265,
  CODE_FOR_subv16qi3_mask = 11266,
  CODE_FOR_addv32qi3_mask = 11267,
  CODE_FOR_subv32qi3_mask = 11268,
  CODE_FOR_addv32hi3_mask = 11269,
  CODE_FOR_subv32hi3_mask = 11270,
  CODE_FOR_addv16hi3_mask = 11271,
  CODE_FOR_subv16hi3_mask = 11272,
  CODE_FOR_addv8hi3_mask = 11273,
  CODE_FOR_subv8hi3_mask = 11274,
  CODE_FOR_avx512bw_ssaddv64qi3 = 11275,
  CODE_FOR_avx512bw_ssaddv64qi3_mask = 11276,
  CODE_FOR_avx512bw_usaddv64qi3 = 11277,
  CODE_FOR_avx512bw_usaddv64qi3_mask = 11278,
  CODE_FOR_avx512bw_sssubv64qi3 = 11279,
  CODE_FOR_avx512bw_sssubv64qi3_mask = 11280,
  CODE_FOR_avx512bw_ussubv64qi3 = 11281,
  CODE_FOR_avx512bw_ussubv64qi3_mask = 11282,
  CODE_FOR_avx2_ssaddv32qi3 = 11283,
  CODE_FOR_avx2_ssaddv32qi3_mask = 11284,
  CODE_FOR_avx2_usaddv32qi3 = 11285,
  CODE_FOR_avx2_usaddv32qi3_mask = 11286,
  CODE_FOR_avx2_sssubv32qi3 = 11287,
  CODE_FOR_avx2_sssubv32qi3_mask = 11288,
  CODE_FOR_avx2_ussubv32qi3 = 11289,
  CODE_FOR_avx2_ussubv32qi3_mask = 11290,
  CODE_FOR_sse2_ssaddv16qi3 = 11291,
  CODE_FOR_sse2_ssaddv16qi3_mask = 11292,
  CODE_FOR_sse2_usaddv16qi3 = 11293,
  CODE_FOR_sse2_usaddv16qi3_mask = 11294,
  CODE_FOR_sse2_sssubv16qi3 = 11295,
  CODE_FOR_sse2_sssubv16qi3_mask = 11296,
  CODE_FOR_sse2_ussubv16qi3 = 11297,
  CODE_FOR_sse2_ussubv16qi3_mask = 11298,
  CODE_FOR_avx512bw_ssaddv32hi3 = 11299,
  CODE_FOR_avx512bw_ssaddv32hi3_mask = 11300,
  CODE_FOR_avx512bw_usaddv32hi3 = 11301,
  CODE_FOR_avx512bw_usaddv32hi3_mask = 11302,
  CODE_FOR_avx512bw_sssubv32hi3 = 11303,
  CODE_FOR_avx512bw_sssubv32hi3_mask = 11304,
  CODE_FOR_avx512bw_ussubv32hi3 = 11305,
  CODE_FOR_avx512bw_ussubv32hi3_mask = 11306,
  CODE_FOR_avx2_ssaddv16hi3 = 11307,
  CODE_FOR_avx2_ssaddv16hi3_mask = 11308,
  CODE_FOR_avx2_usaddv16hi3 = 11309,
  CODE_FOR_avx2_usaddv16hi3_mask = 11310,
  CODE_FOR_avx2_sssubv16hi3 = 11311,
  CODE_FOR_avx2_sssubv16hi3_mask = 11312,
  CODE_FOR_avx2_ussubv16hi3 = 11313,
  CODE_FOR_avx2_ussubv16hi3_mask = 11314,
  CODE_FOR_sse2_ssaddv8hi3 = 11315,
  CODE_FOR_sse2_ssaddv8hi3_mask = 11316,
  CODE_FOR_sse2_usaddv8hi3 = 11317,
  CODE_FOR_sse2_usaddv8hi3_mask = 11318,
  CODE_FOR_sse2_sssubv8hi3 = 11319,
  CODE_FOR_sse2_sssubv8hi3_mask = 11320,
  CODE_FOR_sse2_ussubv8hi3 = 11321,
  CODE_FOR_sse2_ussubv8hi3_mask = 11322,
  CODE_FOR_mulv8qi3 = 11323,
  CODE_FOR_mulv64qi3 = 11324,
  CODE_FOR_mulv32qi3 = 11325,
  CODE_FOR_mulv16qi3 = 11326,
  CODE_FOR_cond_mulv8hi = 11327,
  CODE_FOR_cond_mulv16hi = 11328,
  CODE_FOR_cond_mulv32hi = 11329,
  CODE_FOR_mulv32hi3 = 11330,
  CODE_FOR_mulv32hi3_mask = 11331,
  CODE_FOR_mulv16hi3 = 11332,
  CODE_FOR_mulv16hi3_mask = 11333,
  CODE_FOR_mulv8hi3 = 11334,
  CODE_FOR_mulv8hi3_mask = 11335,
  CODE_FOR_smulv32hi3_highpart = 11336,
  CODE_FOR_smulv32hi3_highpart_mask = 11337,
  CODE_FOR_umulv32hi3_highpart = 11338,
  CODE_FOR_umulv32hi3_highpart_mask = 11339,
  CODE_FOR_smulv16hi3_highpart = 11340,
  CODE_FOR_smulv16hi3_highpart_mask = 11341,
  CODE_FOR_umulv16hi3_highpart = 11342,
  CODE_FOR_umulv16hi3_highpart_mask = 11343,
  CODE_FOR_smulv8hi3_highpart = 11344,
  CODE_FOR_smulv8hi3_highpart_mask = 11345,
  CODE_FOR_umulv8hi3_highpart = 11346,
  CODE_FOR_umulv8hi3_highpart_mask = 11347,
  CODE_FOR_vec_widen_umult_even_v16si = 11348,
  CODE_FOR_vec_widen_umult_even_v16si_mask = 11349,
  CODE_FOR_vec_widen_umult_even_v8si = 11350,
  CODE_FOR_vec_widen_umult_even_v8si_mask = 11351,
  CODE_FOR_vec_widen_umult_even_v4si = 11352,
  CODE_FOR_vec_widen_umult_even_v4si_mask = 11353,
  CODE_FOR_vec_widen_smult_even_v16si = 11354,
  CODE_FOR_vec_widen_smult_even_v16si_mask = 11355,
  CODE_FOR_vec_widen_smult_even_v8si = 11356,
  CODE_FOR_vec_widen_smult_even_v8si_mask = 11357,
  CODE_FOR_sse4_1_mulv2siv2di3 = 11358,
  CODE_FOR_sse4_1_mulv2siv2di3_mask = 11359,
  CODE_FOR_avx2_pmaddwd = 11360,
  CODE_FOR_sse2_pmaddwd = 11361,
  CODE_FOR_cond_mulv8di = 11362,
  CODE_FOR_cond_mulv4di = 11363,
  CODE_FOR_cond_mulv2di = 11364,
  CODE_FOR_avx512dq_mulv8di3 = 11365,
  CODE_FOR_avx512dq_mulv8di3_mask = 11366,
  CODE_FOR_avx512dq_mulv4di3 = 11367,
  CODE_FOR_avx512dq_mulv4di3_mask = 11368,
  CODE_FOR_avx512dq_mulv2di3 = 11369,
  CODE_FOR_avx512dq_mulv2di3_mask = 11370,
  CODE_FOR_cond_mulv16si = 11371,
  CODE_FOR_cond_mulv8si = 11372,
  CODE_FOR_cond_mulv4si = 11373,
  CODE_FOR_mulv16si3 = 11374,
  CODE_FOR_mulv16si3_mask = 11375,
  CODE_FOR_mulv8si3 = 11376,
  CODE_FOR_mulv8si3_mask = 11377,
  CODE_FOR_mulv4si3 = 11378,
  CODE_FOR_mulv4si3_mask = 11379,
  CODE_FOR_mulv8di3 = 11380,
  CODE_FOR_mulv4di3 = 11381,
  CODE_FOR_mulv2di3 = 11382,
  CODE_FOR_vec_widen_smult_hi_v32qi = 11383,
  CODE_FOR_vec_widen_umult_hi_v32qi = 11384,
  CODE_FOR_vec_widen_smult_hi_v16qi = 11385,
  CODE_FOR_vec_widen_umult_hi_v16qi = 11386,
  CODE_FOR_vec_widen_smult_hi_v16hi = 11387,
  CODE_FOR_vec_widen_umult_hi_v16hi = 11388,
  CODE_FOR_vec_widen_smult_hi_v8hi = 11389,
  CODE_FOR_vec_widen_umult_hi_v8hi = 11390,
  CODE_FOR_vec_widen_smult_hi_v8si = 11391,
  CODE_FOR_vec_widen_umult_hi_v8si = 11392,
  CODE_FOR_vec_widen_smult_hi_v4si = 11393,
  CODE_FOR_vec_widen_umult_hi_v4si = 11394,
  CODE_FOR_vec_widen_smult_lo_v32qi = 11395,
  CODE_FOR_vec_widen_umult_lo_v32qi = 11396,
  CODE_FOR_vec_widen_smult_lo_v16qi = 11397,
  CODE_FOR_vec_widen_umult_lo_v16qi = 11398,
  CODE_FOR_vec_widen_smult_lo_v16hi = 11399,
  CODE_FOR_vec_widen_umult_lo_v16hi = 11400,
  CODE_FOR_vec_widen_smult_lo_v8hi = 11401,
  CODE_FOR_vec_widen_umult_lo_v8hi = 11402,
  CODE_FOR_vec_widen_smult_lo_v8si = 11403,
  CODE_FOR_vec_widen_umult_lo_v8si = 11404,
  CODE_FOR_vec_widen_smult_lo_v4si = 11405,
  CODE_FOR_vec_widen_umult_lo_v4si = 11406,
  CODE_FOR_vec_widen_smult_even_v4si = 11407,
  CODE_FOR_vec_widen_smult_odd_v16si = 11408,
  CODE_FOR_vec_widen_umult_odd_v16si = 11409,
  CODE_FOR_vec_widen_smult_odd_v8si = 11410,
  CODE_FOR_vec_widen_umult_odd_v8si = 11411,
  CODE_FOR_vec_widen_smult_odd_v4si = 11412,
  CODE_FOR_vec_widen_umult_odd_v4si = 11413,
  CODE_FOR_sdot_prodv32hi = 11414,
  CODE_FOR_sdot_prodv16hi = 11415,
  CODE_FOR_sdot_prodv8hi = 11416,
  CODE_FOR_sdot_prodv4si = 11417,
  CODE_FOR_uavgv64qi3_ceil = 11418,
  CODE_FOR_uavgv32qi3_ceil = 11419,
  CODE_FOR_uavgv16qi3_ceil = 11420,
  CODE_FOR_uavgv32hi3_ceil = 11421,
  CODE_FOR_uavgv16hi3_ceil = 11422,
  CODE_FOR_uavgv8hi3_ceil = 11423,
  CODE_FOR_usadv16qi = 11424,
  CODE_FOR_usadv32qi = 11425,
  CODE_FOR_usadv64qi = 11426,
  CODE_FOR_ashrv32hi3 = 11427,
  CODE_FOR_ashrv16si3 = 11428,
  CODE_FOR_ashrv8di3 = 11429,
  CODE_FOR_ashrv4di3 = 11430,
  CODE_FOR_vec_shl_v16qi = 11431,
  CODE_FOR_vec_shl_v8hi = 11432,
  CODE_FOR_vec_shl_v4si = 11433,
  CODE_FOR_vec_shl_v2di = 11434,
  CODE_FOR_vec_shl_v4sf = 11435,
  CODE_FOR_vec_shl_v2df = 11436,
  CODE_FOR_vec_shr_v16qi = 11437,
  CODE_FOR_vec_shr_v8hi = 11438,
  CODE_FOR_vec_shr_v4si = 11439,
  CODE_FOR_vec_shr_v2di = 11440,
  CODE_FOR_vec_shr_v4sf = 11441,
  CODE_FOR_vec_shr_v2df = 11442,
  CODE_FOR_ashlv1ti3 = 11443,
  CODE_FOR_lshrv1ti3 = 11444,
  CODE_FOR_ashrv1ti3 = 11445,
  CODE_FOR_rotlv1ti3 = 11446,
  CODE_FOR_rotrv1ti3 = 11447,
  CODE_FOR_smaxv32qi3 = 11448,
  CODE_FOR_sminv32qi3 = 11449,
  CODE_FOR_umaxv32qi3 = 11450,
  CODE_FOR_uminv32qi3 = 11451,
  CODE_FOR_smaxv16hi3 = 11452,
  CODE_FOR_sminv16hi3 = 11453,
  CODE_FOR_umaxv16hi3 = 11454,
  CODE_FOR_uminv16hi3 = 11455,
  CODE_FOR_smaxv8si3 = 11456,
  CODE_FOR_sminv8si3 = 11457,
  CODE_FOR_umaxv8si3 = 11458,
  CODE_FOR_uminv8si3 = 11459,
  CODE_FOR_smaxv64qi3 = 11460,
  CODE_FOR_sminv64qi3 = 11461,
  CODE_FOR_umaxv64qi3 = 11462,
  CODE_FOR_uminv64qi3 = 11463,
  CODE_FOR_smaxv32hi3 = 11464,
  CODE_FOR_sminv32hi3 = 11465,
  CODE_FOR_umaxv32hi3 = 11466,
  CODE_FOR_uminv32hi3 = 11467,
  CODE_FOR_smaxv16si3 = 11468,
  CODE_FOR_sminv16si3 = 11469,
  CODE_FOR_umaxv16si3 = 11470,
  CODE_FOR_uminv16si3 = 11471,
  CODE_FOR_cond_smaxv64qi = 11472,
  CODE_FOR_cond_sminv64qi = 11473,
  CODE_FOR_cond_umaxv64qi = 11474,
  CODE_FOR_cond_uminv64qi = 11475,
  CODE_FOR_cond_smaxv32qi = 11476,
  CODE_FOR_cond_sminv32qi = 11477,
  CODE_FOR_cond_umaxv32qi = 11478,
  CODE_FOR_cond_uminv32qi = 11479,
  CODE_FOR_cond_smaxv16qi = 11480,
  CODE_FOR_cond_sminv16qi = 11481,
  CODE_FOR_cond_umaxv16qi = 11482,
  CODE_FOR_cond_uminv16qi = 11483,
  CODE_FOR_cond_smaxv32hi = 11484,
  CODE_FOR_cond_sminv32hi = 11485,
  CODE_FOR_cond_umaxv32hi = 11486,
  CODE_FOR_cond_uminv32hi = 11487,
  CODE_FOR_cond_smaxv16hi = 11488,
  CODE_FOR_cond_sminv16hi = 11489,
  CODE_FOR_cond_umaxv16hi = 11490,
  CODE_FOR_cond_uminv16hi = 11491,
  CODE_FOR_cond_smaxv8hi = 11492,
  CODE_FOR_cond_sminv8hi = 11493,
  CODE_FOR_cond_umaxv8hi = 11494,
  CODE_FOR_cond_uminv8hi = 11495,
  CODE_FOR_cond_smaxv16si = 11496,
  CODE_FOR_cond_sminv16si = 11497,
  CODE_FOR_cond_umaxv16si = 11498,
  CODE_FOR_cond_uminv16si = 11499,
  CODE_FOR_cond_smaxv8si = 11500,
  CODE_FOR_cond_sminv8si = 11501,
  CODE_FOR_cond_umaxv8si = 11502,
  CODE_FOR_cond_uminv8si = 11503,
  CODE_FOR_cond_smaxv4si = 11504,
  CODE_FOR_cond_sminv4si = 11505,
  CODE_FOR_cond_umaxv4si = 11506,
  CODE_FOR_cond_uminv4si = 11507,
  CODE_FOR_cond_smaxv8di = 11508,
  CODE_FOR_cond_sminv8di = 11509,
  CODE_FOR_cond_umaxv8di = 11510,
  CODE_FOR_cond_uminv8di = 11511,
  CODE_FOR_cond_smaxv4di = 11512,
  CODE_FOR_cond_sminv4di = 11513,
  CODE_FOR_cond_umaxv4di = 11514,
  CODE_FOR_cond_uminv4di = 11515,
  CODE_FOR_cond_smaxv2di = 11516,
  CODE_FOR_cond_sminv2di = 11517,
  CODE_FOR_cond_umaxv2di = 11518,
  CODE_FOR_cond_uminv2di = 11519,
  CODE_FOR_smaxv16si3_mask = 11520,
  CODE_FOR_sminv16si3_mask = 11521,
  CODE_FOR_umaxv16si3_mask = 11522,
  CODE_FOR_uminv16si3_mask = 11523,
  CODE_FOR_smaxv8si3_mask = 11524,
  CODE_FOR_sminv8si3_mask = 11525,
  CODE_FOR_umaxv8si3_mask = 11526,
  CODE_FOR_uminv8si3_mask = 11527,
  CODE_FOR_smaxv4si3_mask = 11528,
  CODE_FOR_sminv4si3_mask = 11529,
  CODE_FOR_umaxv4si3_mask = 11530,
  CODE_FOR_uminv4si3_mask = 11531,
  CODE_FOR_smaxv8di3_mask = 11532,
  CODE_FOR_sminv8di3_mask = 11533,
  CODE_FOR_umaxv8di3_mask = 11534,
  CODE_FOR_uminv8di3_mask = 11535,
  CODE_FOR_smaxv4di3_mask = 11536,
  CODE_FOR_sminv4di3_mask = 11537,
  CODE_FOR_umaxv4di3_mask = 11538,
  CODE_FOR_uminv4di3_mask = 11539,
  CODE_FOR_smaxv2di3_mask = 11540,
  CODE_FOR_sminv2di3_mask = 11541,
  CODE_FOR_umaxv2di3_mask = 11542,
  CODE_FOR_uminv2di3_mask = 11543,
  CODE_FOR_smaxv8di3 = 11544,
  CODE_FOR_sminv8di3 = 11545,
  CODE_FOR_umaxv8di3 = 11546,
  CODE_FOR_uminv8di3 = 11547,
  CODE_FOR_smaxv4di3 = 11548,
  CODE_FOR_sminv4di3 = 11549,
  CODE_FOR_umaxv4di3 = 11550,
  CODE_FOR_uminv4di3 = 11551,
  CODE_FOR_smaxv2di3 = 11552,
  CODE_FOR_sminv2di3 = 11553,
  CODE_FOR_umaxv2di3 = 11554,
  CODE_FOR_uminv2di3 = 11555,
  CODE_FOR_smaxv16qi3 = 11556,
  CODE_FOR_sminv16qi3 = 11557,
  CODE_FOR_smaxv8hi3 = 11558,
  CODE_FOR_sminv8hi3 = 11559,
  CODE_FOR_smaxv4si3 = 11560,
  CODE_FOR_sminv4si3 = 11561,
  CODE_FOR_umaxv16qi3 = 11562,
  CODE_FOR_uminv16qi3 = 11563,
  CODE_FOR_umaxv8hi3 = 11564,
  CODE_FOR_uminv8hi3 = 11565,
  CODE_FOR_umaxv4si3 = 11566,
  CODE_FOR_uminv4si3 = 11567,
  CODE_FOR_avx512bw_eqv64qi3 = 11568,
  CODE_FOR_avx512bw_eqv64qi3_mask = 11569,
  CODE_FOR_avx512vl_eqv16qi3 = 11570,
  CODE_FOR_avx512vl_eqv16qi3_mask = 11571,
  CODE_FOR_avx512vl_eqv32qi3 = 11572,
  CODE_FOR_avx512vl_eqv32qi3_mask = 11573,
  CODE_FOR_avx512bw_eqv32hi3 = 11574,
  CODE_FOR_avx512bw_eqv32hi3_mask = 11575,
  CODE_FOR_avx512vl_eqv16hi3 = 11576,
  CODE_FOR_avx512vl_eqv16hi3_mask = 11577,
  CODE_FOR_avx512vl_eqv8hi3 = 11578,
  CODE_FOR_avx512vl_eqv8hi3_mask = 11579,
  CODE_FOR_avx512f_eqv16si3 = 11580,
  CODE_FOR_avx512f_eqv16si3_mask = 11581,
  CODE_FOR_avx512vl_eqv8si3 = 11582,
  CODE_FOR_avx512vl_eqv8si3_mask = 11583,
  CODE_FOR_avx512vl_eqv4si3 = 11584,
  CODE_FOR_avx512vl_eqv4si3_mask = 11585,
  CODE_FOR_avx512f_eqv8di3 = 11586,
  CODE_FOR_avx512f_eqv8di3_mask = 11587,
  CODE_FOR_avx512vl_eqv4di3 = 11588,
  CODE_FOR_avx512vl_eqv4di3_mask = 11589,
  CODE_FOR_avx512vl_eqv2di3 = 11590,
  CODE_FOR_avx512vl_eqv2di3_mask = 11591,
  CODE_FOR_avx512f_gtv16si3 = 11592,
  CODE_FOR_avx512f_gtv16si3_mask = 11593,
  CODE_FOR_avx512vl_gtv8si3 = 11594,
  CODE_FOR_avx512vl_gtv8si3_mask = 11595,
  CODE_FOR_avx512vl_gtv4si3 = 11596,
  CODE_FOR_avx512vl_gtv4si3_mask = 11597,
  CODE_FOR_avx512f_gtv8di3 = 11598,
  CODE_FOR_avx512f_gtv8di3_mask = 11599,
  CODE_FOR_avx512vl_gtv4di3 = 11600,
  CODE_FOR_avx512vl_gtv4di3_mask = 11601,
  CODE_FOR_avx512vl_gtv2di3 = 11602,
  CODE_FOR_avx512vl_gtv2di3_mask = 11603,
  CODE_FOR_avx512bw_gtv64qi3 = 11604,
  CODE_FOR_avx512bw_gtv64qi3_mask = 11605,
  CODE_FOR_avx512vl_gtv16qi3 = 11606,
  CODE_FOR_avx512vl_gtv16qi3_mask = 11607,
  CODE_FOR_avx512vl_gtv32qi3 = 11608,
  CODE_FOR_avx512vl_gtv32qi3_mask = 11609,
  CODE_FOR_avx512bw_gtv32hi3 = 11610,
  CODE_FOR_avx512bw_gtv32hi3_mask = 11611,
  CODE_FOR_avx512vl_gtv16hi3 = 11612,
  CODE_FOR_avx512vl_gtv16hi3_mask = 11613,
  CODE_FOR_avx512vl_gtv8hi3 = 11614,
  CODE_FOR_avx512vl_gtv8hi3_mask = 11615,
  CODE_FOR_vcondv64qiv16si = 11616,
  CODE_FOR_vcondv64qiv8di = 11617,
  CODE_FOR_vcondv64qiv32hi = 11618,
  CODE_FOR_vcondv64qiv64qi = 11619,
  CODE_FOR_vcondv32hiv16si = 11620,
  CODE_FOR_vcondv32hiv8di = 11621,
  CODE_FOR_vcondv32hiv32hi = 11622,
  CODE_FOR_vcondv32hiv64qi = 11623,
  CODE_FOR_vcondv16siv16si = 11624,
  CODE_FOR_vcondv16siv8di = 11625,
  CODE_FOR_vcondv16siv32hi = 11626,
  CODE_FOR_vcondv16siv64qi = 11627,
  CODE_FOR_vcondv8div16si = 11628,
  CODE_FOR_vcondv8div8di = 11629,
  CODE_FOR_vcondv8div32hi = 11630,
  CODE_FOR_vcondv8div64qi = 11631,
  CODE_FOR_vcondv16sfv16si = 11632,
  CODE_FOR_vcondv16sfv8di = 11633,
  CODE_FOR_vcondv16sfv32hi = 11634,
  CODE_FOR_vcondv16sfv64qi = 11635,
  CODE_FOR_vcondv8dfv16si = 11636,
  CODE_FOR_vcondv8dfv8di = 11637,
  CODE_FOR_vcondv8dfv32hi = 11638,
  CODE_FOR_vcondv8dfv64qi = 11639,
  CODE_FOR_vcondv32qiv32qi = 11640,
  CODE_FOR_vcondv32qiv16hi = 11641,
  CODE_FOR_vcondv32qiv8si = 11642,
  CODE_FOR_vcondv32qiv4di = 11643,
  CODE_FOR_vcondv16hiv32qi = 11644,
  CODE_FOR_vcondv16hiv16hi = 11645,
  CODE_FOR_vcondv16hiv8si = 11646,
  CODE_FOR_vcondv16hiv4di = 11647,
  CODE_FOR_vcondv8siv32qi = 11648,
  CODE_FOR_vcondv8siv16hi = 11649,
  CODE_FOR_vcondv8siv8si = 11650,
  CODE_FOR_vcondv8siv4di = 11651,
  CODE_FOR_vcondv4div32qi = 11652,
  CODE_FOR_vcondv4div16hi = 11653,
  CODE_FOR_vcondv4div8si = 11654,
  CODE_FOR_vcondv4div4di = 11655,
  CODE_FOR_vcondv8sfv32qi = 11656,
  CODE_FOR_vcondv8sfv16hi = 11657,
  CODE_FOR_vcondv8sfv8si = 11658,
  CODE_FOR_vcondv8sfv4di = 11659,
  CODE_FOR_vcondv4dfv32qi = 11660,
  CODE_FOR_vcondv4dfv16hi = 11661,
  CODE_FOR_vcondv4dfv8si = 11662,
  CODE_FOR_vcondv4dfv4di = 11663,
  CODE_FOR_vcondv16qiv16qi = 11664,
  CODE_FOR_vcondv8hiv16qi = 11665,
  CODE_FOR_vcondv4siv16qi = 11666,
  CODE_FOR_vcondv2div16qi = 11667,
  CODE_FOR_vcondv4sfv16qi = 11668,
  CODE_FOR_vcondv2dfv16qi = 11669,
  CODE_FOR_vcondv16qiv8hi = 11670,
  CODE_FOR_vcondv8hiv8hi = 11671,
  CODE_FOR_vcondv4siv8hi = 11672,
  CODE_FOR_vcondv2div8hi = 11673,
  CODE_FOR_vcondv4sfv8hi = 11674,
  CODE_FOR_vcondv2dfv8hi = 11675,
  CODE_FOR_vcondv16qiv4si = 11676,
  CODE_FOR_vcondv8hiv4si = 11677,
  CODE_FOR_vcondv4siv4si = 11678,
  CODE_FOR_vcondv2div4si = 11679,
  CODE_FOR_vcondv4sfv4si = 11680,
  CODE_FOR_vcondv2dfv4si = 11681,
  CODE_FOR_vcondv2div2di = 11682,
  CODE_FOR_vcondv2dfv2di = 11683,
  CODE_FOR_vconduv64qiv16si = 11684,
  CODE_FOR_vconduv64qiv8di = 11685,
  CODE_FOR_vconduv64qiv32hi = 11686,
  CODE_FOR_vconduv64qiv64qi = 11687,
  CODE_FOR_vconduv32hiv16si = 11688,
  CODE_FOR_vconduv32hiv8di = 11689,
  CODE_FOR_vconduv32hiv32hi = 11690,
  CODE_FOR_vconduv32hiv64qi = 11691,
  CODE_FOR_vconduv16siv16si = 11692,
  CODE_FOR_vconduv16siv8di = 11693,
  CODE_FOR_vconduv16siv32hi = 11694,
  CODE_FOR_vconduv16siv64qi = 11695,
  CODE_FOR_vconduv8div16si = 11696,
  CODE_FOR_vconduv8div8di = 11697,
  CODE_FOR_vconduv8div32hi = 11698,
  CODE_FOR_vconduv8div64qi = 11699,
  CODE_FOR_vconduv16sfv16si = 11700,
  CODE_FOR_vconduv16sfv8di = 11701,
  CODE_FOR_vconduv16sfv32hi = 11702,
  CODE_FOR_vconduv16sfv64qi = 11703,
  CODE_FOR_vconduv8dfv16si = 11704,
  CODE_FOR_vconduv8dfv8di = 11705,
  CODE_FOR_vconduv8dfv32hi = 11706,
  CODE_FOR_vconduv8dfv64qi = 11707,
  CODE_FOR_vconduv32qiv32qi = 11708,
  CODE_FOR_vconduv32qiv16hi = 11709,
  CODE_FOR_vconduv32qiv8si = 11710,
  CODE_FOR_vconduv32qiv4di = 11711,
  CODE_FOR_vconduv16hiv32qi = 11712,
  CODE_FOR_vconduv16hiv16hi = 11713,
  CODE_FOR_vconduv16hiv8si = 11714,
  CODE_FOR_vconduv16hiv4di = 11715,
  CODE_FOR_vconduv8siv32qi = 11716,
  CODE_FOR_vconduv8siv16hi = 11717,
  CODE_FOR_vconduv8siv8si = 11718,
  CODE_FOR_vconduv8siv4di = 11719,
  CODE_FOR_vconduv4div32qi = 11720,
  CODE_FOR_vconduv4div16hi = 11721,
  CODE_FOR_vconduv4div8si = 11722,
  CODE_FOR_vconduv4div4di = 11723,
  CODE_FOR_vconduv8sfv32qi = 11724,
  CODE_FOR_vconduv8sfv16hi = 11725,
  CODE_FOR_vconduv8sfv8si = 11726,
  CODE_FOR_vconduv8sfv4di = 11727,
  CODE_FOR_vconduv4dfv32qi = 11728,
  CODE_FOR_vconduv4dfv16hi = 11729,
  CODE_FOR_vconduv4dfv8si = 11730,
  CODE_FOR_vconduv4dfv4di = 11731,
  CODE_FOR_vconduv16qiv16qi = 11732,
  CODE_FOR_vconduv8hiv16qi = 11733,
  CODE_FOR_vconduv4siv16qi = 11734,
  CODE_FOR_vconduv2div16qi = 11735,
  CODE_FOR_vconduv4sfv16qi = 11736,
  CODE_FOR_vconduv2dfv16qi = 11737,
  CODE_FOR_vconduv16qiv8hi = 11738,
  CODE_FOR_vconduv8hiv8hi = 11739,
  CODE_FOR_vconduv4siv8hi = 11740,
  CODE_FOR_vconduv2div8hi = 11741,
  CODE_FOR_vconduv4sfv8hi = 11742,
  CODE_FOR_vconduv2dfv8hi = 11743,
  CODE_FOR_vconduv16qiv4si = 11744,
  CODE_FOR_vconduv8hiv4si = 11745,
  CODE_FOR_vconduv4siv4si = 11746,
  CODE_FOR_vconduv2div4si = 11747,
  CODE_FOR_vconduv4sfv4si = 11748,
  CODE_FOR_vconduv2dfv4si = 11749,
  CODE_FOR_vconduv2div2di = 11750,
  CODE_FOR_vconduv2dfv2di = 11751,
  CODE_FOR_vconduv32hfv32hi = 11752,
  CODE_FOR_vconduv16hfv16hi = 11753,
  CODE_FOR_vconduv8hfv8hi = 11754,
  CODE_FOR_vcondeqv2div2di = 11755,
  CODE_FOR_vcondeqv2dfv2di = 11756,
  CODE_FOR_vec_permv16qi = 11757,
  CODE_FOR_vec_permv8hi = 11758,
  CODE_FOR_vec_permv4si = 11759,
  CODE_FOR_vec_permv2di = 11760,
  CODE_FOR_vec_permv4sf = 11761,
  CODE_FOR_vec_permv2df = 11762,
  CODE_FOR_vec_permv8hf = 11763,
  CODE_FOR_vec_permv32qi = 11764,
  CODE_FOR_vec_permv16hi = 11765,
  CODE_FOR_vec_permv8si = 11766,
  CODE_FOR_vec_permv4di = 11767,
  CODE_FOR_vec_permv8sf = 11768,
  CODE_FOR_vec_permv4df = 11769,
  CODE_FOR_vec_permv16hf = 11770,
  CODE_FOR_vec_permv16sf = 11771,
  CODE_FOR_vec_permv8df = 11772,
  CODE_FOR_vec_permv16si = 11773,
  CODE_FOR_vec_permv8di = 11774,
  CODE_FOR_vec_permv32hi = 11775,
  CODE_FOR_vec_permv64qi = 11776,
  CODE_FOR_vec_permv32hf = 11777,
  CODE_FOR_one_cmplv16si2 = 11778,
  CODE_FOR_one_cmplv8di2 = 11779,
  CODE_FOR_one_cmplv64qi2 = 11780,
  CODE_FOR_one_cmplv32qi2 = 11781,
  CODE_FOR_one_cmplv16qi2 = 11782,
  CODE_FOR_one_cmplv32hi2 = 11783,
  CODE_FOR_one_cmplv16hi2 = 11784,
  CODE_FOR_one_cmplv8hi2 = 11785,
  CODE_FOR_one_cmplv8si2 = 11786,
  CODE_FOR_one_cmplv4si2 = 11787,
  CODE_FOR_one_cmplv4di2 = 11788,
  CODE_FOR_one_cmplv2di2 = 11789,
  CODE_FOR_avx512bw_andnotv64qi3 = 11790,
  CODE_FOR_avx2_andnotv32qi3 = 11791,
  CODE_FOR_sse2_andnotv16qi3 = 11792,
  CODE_FOR_avx512bw_andnotv32hi3 = 11793,
  CODE_FOR_avx2_andnotv16hi3 = 11794,
  CODE_FOR_sse2_andnotv8hi3 = 11795,
  CODE_FOR_avx512f_andnotv16si3 = 11796,
  CODE_FOR_avx2_andnotv8si3 = 11797,
  CODE_FOR_sse2_andnotv4si3 = 11798,
  CODE_FOR_avx512f_andnotv8di3 = 11799,
  CODE_FOR_avx2_andnotv4di3 = 11800,
  CODE_FOR_sse2_andnotv2di3 = 11801,
  CODE_FOR_avx512f_andnotv16si3_mask = 11802,
  CODE_FOR_avx2_andnotv8si3_mask = 11803,
  CODE_FOR_sse2_andnotv4si3_mask = 11804,
  CODE_FOR_avx512f_andnotv8di3_mask = 11805,
  CODE_FOR_avx2_andnotv4di3_mask = 11806,
  CODE_FOR_sse2_andnotv2di3_mask = 11807,
  CODE_FOR_andv16si3 = 11808,
  CODE_FOR_iorv16si3 = 11809,
  CODE_FOR_xorv16si3 = 11810,
  CODE_FOR_andv8di3 = 11811,
  CODE_FOR_iorv8di3 = 11812,
  CODE_FOR_xorv8di3 = 11813,
  CODE_FOR_andv64qi3 = 11814,
  CODE_FOR_iorv64qi3 = 11815,
  CODE_FOR_xorv64qi3 = 11816,
  CODE_FOR_andv32qi3 = 11817,
  CODE_FOR_iorv32qi3 = 11818,
  CODE_FOR_xorv32qi3 = 11819,
  CODE_FOR_andv16qi3 = 11820,
  CODE_FOR_iorv16qi3 = 11821,
  CODE_FOR_xorv16qi3 = 11822,
  CODE_FOR_andv32hi3 = 11823,
  CODE_FOR_iorv32hi3 = 11824,
  CODE_FOR_xorv32hi3 = 11825,
  CODE_FOR_andv16hi3 = 11826,
  CODE_FOR_iorv16hi3 = 11827,
  CODE_FOR_xorv16hi3 = 11828,
  CODE_FOR_andv8hi3 = 11829,
  CODE_FOR_iorv8hi3 = 11830,
  CODE_FOR_xorv8hi3 = 11831,
  CODE_FOR_andv8si3 = 11832,
  CODE_FOR_iorv8si3 = 11833,
  CODE_FOR_xorv8si3 = 11834,
  CODE_FOR_andv4si3 = 11835,
  CODE_FOR_iorv4si3 = 11836,
  CODE_FOR_xorv4si3 = 11837,
  CODE_FOR_andv4di3 = 11838,
  CODE_FOR_iorv4di3 = 11839,
  CODE_FOR_xorv4di3 = 11840,
  CODE_FOR_andv2di3 = 11841,
  CODE_FOR_iorv2di3 = 11842,
  CODE_FOR_xorv2di3 = 11843,
  CODE_FOR_cond_andv16si = 11844,
  CODE_FOR_cond_iorv16si = 11845,
  CODE_FOR_cond_xorv16si = 11846,
  CODE_FOR_cond_andv8si = 11847,
  CODE_FOR_cond_iorv8si = 11848,
  CODE_FOR_cond_xorv8si = 11849,
  CODE_FOR_cond_andv4si = 11850,
  CODE_FOR_cond_iorv4si = 11851,
  CODE_FOR_cond_xorv4si = 11852,
  CODE_FOR_cond_andv8di = 11853,
  CODE_FOR_cond_iorv8di = 11854,
  CODE_FOR_cond_xorv8di = 11855,
  CODE_FOR_cond_andv4di = 11856,
  CODE_FOR_cond_iorv4di = 11857,
  CODE_FOR_cond_xorv4di = 11858,
  CODE_FOR_cond_andv2di = 11859,
  CODE_FOR_cond_iorv2di = 11860,
  CODE_FOR_cond_xorv2di = 11861,
  CODE_FOR_andv16si3_mask = 11862,
  CODE_FOR_iorv16si3_mask = 11863,
  CODE_FOR_xorv16si3_mask = 11864,
  CODE_FOR_andv8si3_mask = 11865,
  CODE_FOR_iorv8si3_mask = 11866,
  CODE_FOR_xorv8si3_mask = 11867,
  CODE_FOR_andv4si3_mask = 11868,
  CODE_FOR_iorv4si3_mask = 11869,
  CODE_FOR_xorv4si3_mask = 11870,
  CODE_FOR_andv8di3_mask = 11871,
  CODE_FOR_iorv8di3_mask = 11872,
  CODE_FOR_xorv8di3_mask = 11873,
  CODE_FOR_andv4di3_mask = 11874,
  CODE_FOR_iorv4di3_mask = 11875,
  CODE_FOR_xorv4di3_mask = 11876,
  CODE_FOR_andv2di3_mask = 11877,
  CODE_FOR_iorv2di3_mask = 11878,
  CODE_FOR_xorv2di3_mask = 11879,
  CODE_FOR_one_cmplv1ti2 = 11880,
  CODE_FOR_vec_pack_trunc_v32hi = 11881,
  CODE_FOR_vec_pack_trunc_v16hi = 11882,
  CODE_FOR_vec_pack_trunc_v8hi = 11883,
  CODE_FOR_vec_pack_trunc_v16si = 11884,
  CODE_FOR_vec_pack_trunc_v8si = 11885,
  CODE_FOR_vec_pack_trunc_v4si = 11886,
  CODE_FOR_vec_pack_trunc_v8di = 11887,
  CODE_FOR_vec_pack_trunc_v4di = 11888,
  CODE_FOR_vec_pack_trunc_v2di = 11889,
  CODE_FOR_vec_pack_trunc_qi = 11890,
  CODE_FOR_vec_pack_trunc_hi = 11891,
  CODE_FOR_vec_pack_trunc_si = 11892,
  CODE_FOR_vec_pack_sbool_trunc_qi = 11893,
  CODE_FOR_vec_interleave_highv32qi = 11894,
  CODE_FOR_vec_interleave_highv16hi = 11895,
  CODE_FOR_vec_interleave_highv8si = 11896,
  CODE_FOR_vec_interleave_highv4di = 11897,
  CODE_FOR_vec_interleave_lowv32qi = 11898,
  CODE_FOR_vec_interleave_lowv16hi = 11899,
  CODE_FOR_vec_interleave_lowv8si = 11900,
  CODE_FOR_vec_interleave_lowv4di = 11901,
  CODE_FOR_avx512dq_vinsertf64x2_mask = 11902,
  CODE_FOR_avx512dq_vinserti64x2_mask = 11903,
  CODE_FOR_avx512f_vinsertf32x4_mask = 11904,
  CODE_FOR_avx512f_vinserti32x4_mask = 11905,
  CODE_FOR_avx512dq_vinsertf32x8_mask = 11906,
  CODE_FOR_avx512dq_vinserti32x8_mask = 11907,
  CODE_FOR_avx512f_vinsertf64x4_mask = 11908,
  CODE_FOR_avx512f_vinserti64x4_mask = 11909,
  CODE_FOR_avx512dq_shuf_i64x2_mask = 11910,
  CODE_FOR_avx512dq_shuf_f64x2_mask = 11911,
  CODE_FOR_avx512f_shuf_f64x2_mask = 11912,
  CODE_FOR_avx512f_shuf_i64x2_mask = 11913,
  CODE_FOR_avx512vl_shuf_i32x4_mask = 11914,
  CODE_FOR_avx512vl_shuf_f32x4_mask = 11915,
  CODE_FOR_avx512f_shuf_f32x4_mask = 11916,
  CODE_FOR_avx512f_shuf_i32x4_mask = 11917,
  CODE_FOR_avx512f_pshufdv3_mask = 11918,
  CODE_FOR_avx512vl_pshufdv3_mask = 11919,
  CODE_FOR_avx2_pshufdv3 = 11920,
  CODE_FOR_avx512vl_pshufd_mask = 11921,
  CODE_FOR_sse2_pshufd = 11922,
  CODE_FOR_avx512vl_pshuflwv3_mask = 11923,
  CODE_FOR_avx2_pshuflwv3 = 11924,
  CODE_FOR_avx512vl_pshuflw_mask = 11925,
  CODE_FOR_sse2_pshuflw = 11926,
  CODE_FOR_avx2_pshufhwv3 = 11927,
  CODE_FOR_avx512vl_pshufhwv3_mask = 11928,
  CODE_FOR_avx512vl_pshufhw_mask = 11929,
  CODE_FOR_sse2_pshufhw = 11930,
  CODE_FOR_sse2_loadd = 11931,
  CODE_FOR_vec_unpacks_lo_v64qi = 11932,
  CODE_FOR_vec_unpacks_lo_v32qi = 11933,
  CODE_FOR_vec_unpacks_lo_v16qi = 11934,
  CODE_FOR_vec_unpacks_lo_v32hi = 11935,
  CODE_FOR_vec_unpacks_lo_v16hi = 11936,
  CODE_FOR_vec_unpacks_lo_v8hi = 11937,
  CODE_FOR_vec_unpacks_lo_v16si = 11938,
  CODE_FOR_vec_unpacks_lo_v8si = 11939,
  CODE_FOR_vec_unpacks_lo_v4si = 11940,
  CODE_FOR_vec_unpacks_hi_v64qi = 11941,
  CODE_FOR_vec_unpacks_hi_v32qi = 11942,
  CODE_FOR_vec_unpacks_hi_v16qi = 11943,
  CODE_FOR_vec_unpacks_hi_v32hi = 11944,
  CODE_FOR_vec_unpacks_hi_v16hi = 11945,
  CODE_FOR_vec_unpacks_hi_v8hi = 11946,
  CODE_FOR_vec_unpacks_hi_v16si = 11947,
  CODE_FOR_vec_unpacks_hi_v8si = 11948,
  CODE_FOR_vec_unpacks_hi_v4si = 11949,
  CODE_FOR_vec_unpacku_lo_v64qi = 11950,
  CODE_FOR_vec_unpacku_lo_v32qi = 11951,
  CODE_FOR_vec_unpacku_lo_v16qi = 11952,
  CODE_FOR_vec_unpacku_lo_v32hi = 11953,
  CODE_FOR_vec_unpacku_lo_v16hi = 11954,
  CODE_FOR_vec_unpacku_lo_v8hi = 11955,
  CODE_FOR_vec_unpacku_lo_v16si = 11956,
  CODE_FOR_vec_unpacku_lo_v8si = 11957,
  CODE_FOR_vec_unpacku_lo_v4si = 11958,
  CODE_FOR_vec_unpacks_sbool_lo_qi = 11959,
  CODE_FOR_vec_unpacks_lo_hi = 11960,
  CODE_FOR_vec_unpacks_lo_si = 11961,
  CODE_FOR_vec_unpacks_lo_di = 11962,
  CODE_FOR_vec_unpacku_hi_v64qi = 11963,
  CODE_FOR_vec_unpacku_hi_v32qi = 11964,
  CODE_FOR_vec_unpacku_hi_v16qi = 11965,
  CODE_FOR_vec_unpacku_hi_v32hi = 11966,
  CODE_FOR_vec_unpacku_hi_v16hi = 11967,
  CODE_FOR_vec_unpacku_hi_v8hi = 11968,
  CODE_FOR_vec_unpacku_hi_v16si = 11969,
  CODE_FOR_vec_unpacku_hi_v8si = 11970,
  CODE_FOR_vec_unpacku_hi_v4si = 11971,
  CODE_FOR_vec_unpacks_sbool_hi_qi = 11972,
  CODE_FOR_vec_unpacks_hi_hi = 11973,
  CODE_FOR_vec_unpacks_hi_si = 11974,
  CODE_FOR_vec_unpacks_hi_di = 11975,
  CODE_FOR_avx512bw_uavgv64qi3 = 11976,
  CODE_FOR_avx512bw_uavgv64qi3_mask = 11977,
  CODE_FOR_avx2_uavgv32qi3 = 11978,
  CODE_FOR_avx2_uavgv32qi3_mask = 11979,
  CODE_FOR_sse2_uavgv16qi3 = 11980,
  CODE_FOR_sse2_uavgv16qi3_mask = 11981,
  CODE_FOR_avx512bw_uavgv32hi3 = 11982,
  CODE_FOR_avx512bw_uavgv32hi3_mask = 11983,
  CODE_FOR_avx2_uavgv16hi3 = 11984,
  CODE_FOR_avx2_uavgv16hi3_mask = 11985,
  CODE_FOR_sse2_uavgv8hi3 = 11986,
  CODE_FOR_sse2_uavgv8hi3_mask = 11987,
  CODE_FOR_avx512f_psadbw = 11988,
  CODE_FOR_avx2_psadbw = 11989,
  CODE_FOR_sse2_psadbw = 11990,
  CODE_FOR_sse2_maskmovdqu = 11991,
  CODE_FOR_ssse3_pmulhrswv8hi3_mask = 11992,
  CODE_FOR_avx2_pmulhrswv16hi3_mask = 11993,
  CODE_FOR_ssse3_pmulhrswv8hi3 = 11994,
  CODE_FOR_avx2_pmulhrswv16hi3 = 11995,
  CODE_FOR_smulhrsv32hi3 = 11996,
  CODE_FOR_smulhrsv16hi3 = 11997,
  CODE_FOR_smulhrsv8hi3 = 11998,
  CODE_FOR_smulhrsv4hi3 = 11999,
  CODE_FOR_ssse3_pmulhrswv4hi3 = 12000,
  CODE_FOR_smulhrsv2hi3 = 12001,
  CODE_FOR_ssse3_pshufbv8qi3 = 12002,
  CODE_FOR_absv64qi2 = 12003,
  CODE_FOR_absv32qi2 = 12004,
  CODE_FOR_absv16qi2 = 12005,
  CODE_FOR_absv32hi2 = 12006,
  CODE_FOR_absv16hi2 = 12007,
  CODE_FOR_absv8hi2 = 12008,
  CODE_FOR_absv16si2 = 12009,
  CODE_FOR_absv8si2 = 12010,
  CODE_FOR_absv4si2 = 12011,
  CODE_FOR_absv8di2 = 12012,
  CODE_FOR_absv4di2 = 12013,
  CODE_FOR_absv2di2 = 12014,
  CODE_FOR_avx2_pblendw = 12015,
  CODE_FOR_avx2_pblendph = 12016,
  CODE_FOR_avx2_pblendbf = 12017,
  CODE_FOR_avx2_pblendw_1 = 12018,
  CODE_FOR_avx2_pblendph_1 = 12019,
  CODE_FOR_avx2_pblendbf_1 = 12020,
  CODE_FOR_extendv16qiv16hi2 = 12021,
  CODE_FOR_zero_extendv16qiv16hi2 = 12022,
  CODE_FOR_extendv32qiv32hi2 = 12023,
  CODE_FOR_zero_extendv32qiv32hi2 = 12024,
  CODE_FOR_extendv8qiv8hi2 = 12025,
  CODE_FOR_zero_extendv8qiv8hi2 = 12026,
  CODE_FOR_extendv16qiv16si2 = 12027,
  CODE_FOR_zero_extendv16qiv16si2 = 12028,
  CODE_FOR_extendv8qiv8si2 = 12029,
  CODE_FOR_zero_extendv8qiv8si2 = 12030,
  CODE_FOR_extendv4qiv4si2 = 12031,
  CODE_FOR_zero_extendv4qiv4si2 = 12032,
  CODE_FOR_extendv16hiv16si2 = 12033,
  CODE_FOR_zero_extendv16hiv16si2 = 12034,
  CODE_FOR_extendv8hiv8si2 = 12035,
  CODE_FOR_zero_extendv8hiv8si2 = 12036,
  CODE_FOR_extendv4hiv4si2 = 12037,
  CODE_FOR_zero_extendv4hiv4si2 = 12038,
  CODE_FOR_extendv8qiv8di2 = 12039,
  CODE_FOR_zero_extendv8qiv8di2 = 12040,
  CODE_FOR_extendv4qiv4di2 = 12041,
  CODE_FOR_zero_extendv4qiv4di2 = 12042,
  CODE_FOR_extendv2qiv2di2 = 12043,
  CODE_FOR_zero_extendv2qiv2di2 = 12044,
  CODE_FOR_extendv8hiv8di2 = 12045,
  CODE_FOR_zero_extendv8hiv8di2 = 12046,
  CODE_FOR_extendv4hiv4di2 = 12047,
  CODE_FOR_zero_extendv4hiv4di2 = 12048,
  CODE_FOR_extendv2hiv2di2 = 12049,
  CODE_FOR_zero_extendv2hiv2di2 = 12050,
  CODE_FOR_extendv8siv8di2 = 12051,
  CODE_FOR_zero_extendv8siv8di2 = 12052,
  CODE_FOR_extendv4siv4di2 = 12053,
  CODE_FOR_zero_extendv4siv4di2 = 12054,
  CODE_FOR_extendv2siv2di2 = 12055,
  CODE_FOR_zero_extendv2siv2di2 = 12056,
  CODE_FOR_nearbyintv32hf2 = 12057,
  CODE_FOR_nearbyintv16hf2 = 12058,
  CODE_FOR_nearbyintv8hf2 = 12059,
  CODE_FOR_nearbyintv16sf2 = 12060,
  CODE_FOR_nearbyintv8sf2 = 12061,
  CODE_FOR_nearbyintv4sf2 = 12062,
  CODE_FOR_nearbyintv8df2 = 12063,
  CODE_FOR_nearbyintv4df2 = 12064,
  CODE_FOR_nearbyintv2df2 = 12065,
  CODE_FOR_rintv32hf2 = 12066,
  CODE_FOR_rintv16hf2 = 12067,
  CODE_FOR_rintv8hf2 = 12068,
  CODE_FOR_rintv16sf2 = 12069,
  CODE_FOR_rintv8sf2 = 12070,
  CODE_FOR_rintv4sf2 = 12071,
  CODE_FOR_rintv8df2 = 12072,
  CODE_FOR_rintv4df2 = 12073,
  CODE_FOR_rintv2df2 = 12074,
  CODE_FOR_lrintv16sfv16si2 = 12075,
  CODE_FOR_lrintv8sfv8si2 = 12076,
  CODE_FOR_lrintv4sfv4si2 = 12077,
  CODE_FOR_lrintv8dfv8di2 = 12078,
  CODE_FOR_lrintv4dfv4di2 = 12079,
  CODE_FOR_lrintv2dfv2di2 = 12080,
  CODE_FOR_avx_roundps_sfix256 = 12081,
  CODE_FOR_sse4_1_roundps_sfix = 12082,
  CODE_FOR_avx512f_roundps512 = 12083,
  CODE_FOR_avx512f_roundpd512 = 12084,
  CODE_FOR_avx512f_roundps512_sfix = 12085,
  CODE_FOR_avx512f_roundpd_vec_pack_sfix512 = 12086,
  CODE_FOR_avx_roundpd_vec_pack_sfix256 = 12087,
  CODE_FOR_sse4_1_roundpd_vec_pack_sfix = 12088,
  CODE_FOR_floorv32hf2 = 12089,
  CODE_FOR_floorv16hf2 = 12090,
  CODE_FOR_floorv8hf2 = 12091,
  CODE_FOR_floorv16sf2 = 12092,
  CODE_FOR_floorv8sf2 = 12093,
  CODE_FOR_floorv4sf2 = 12094,
  CODE_FOR_floorv8df2 = 12095,
  CODE_FOR_floorv4df2 = 12096,
  CODE_FOR_floorv2df2 = 12097,
  CODE_FOR_lfloorv16sfv16si2 = 12098,
  CODE_FOR_lfloorv8sfv8si2 = 12099,
  CODE_FOR_lfloorv4sfv4si2 = 12100,
  CODE_FOR_lfloorv8dfv8di2 = 12101,
  CODE_FOR_lfloorv4dfv4di2 = 12102,
  CODE_FOR_lfloorv2dfv2di2 = 12103,
  CODE_FOR_ceilv32hf2 = 12104,
  CODE_FOR_ceilv16hf2 = 12105,
  CODE_FOR_ceilv8hf2 = 12106,
  CODE_FOR_ceilv16sf2 = 12107,
  CODE_FOR_ceilv8sf2 = 12108,
  CODE_FOR_ceilv4sf2 = 12109,
  CODE_FOR_ceilv8df2 = 12110,
  CODE_FOR_ceilv4df2 = 12111,
  CODE_FOR_ceilv2df2 = 12112,
  CODE_FOR_lceilv16sfv16si2 = 12113,
  CODE_FOR_lceilv8sfv8si2 = 12114,
  CODE_FOR_lceilv4sfv4si2 = 12115,
  CODE_FOR_lceilv8dfv8di2 = 12116,
  CODE_FOR_lceilv4dfv4di2 = 12117,
  CODE_FOR_lceilv2dfv2di2 = 12118,
  CODE_FOR_btruncv32hf2 = 12119,
  CODE_FOR_btruncv16hf2 = 12120,
  CODE_FOR_btruncv8hf2 = 12121,
  CODE_FOR_btruncv16sf2 = 12122,
  CODE_FOR_btruncv8sf2 = 12123,
  CODE_FOR_btruncv4sf2 = 12124,
  CODE_FOR_btruncv8df2 = 12125,
  CODE_FOR_btruncv4df2 = 12126,
  CODE_FOR_btruncv2df2 = 12127,
  CODE_FOR_roundv16sf2 = 12128,
  CODE_FOR_roundv8sf2 = 12129,
  CODE_FOR_roundv4sf2 = 12130,
  CODE_FOR_roundv8df2 = 12131,
  CODE_FOR_roundv4df2 = 12132,
  CODE_FOR_roundv2df2 = 12133,
  CODE_FOR_lroundv16sfv16si2 = 12134,
  CODE_FOR_lroundv8sfv8si2 = 12135,
  CODE_FOR_lroundv4sfv4si2 = 12136,
  CODE_FOR_lroundv8dfv8di2 = 12137,
  CODE_FOR_lroundv4dfv4di2 = 12138,
  CODE_FOR_lroundv2dfv2di2 = 12139,
  CODE_FOR_roundv16sf2_sfix = 12140,
  CODE_FOR_roundv8sf2_sfix = 12141,
  CODE_FOR_roundv4sf2_sfix = 12142,
  CODE_FOR_roundv8df2_vec_pack_sfix = 12143,
  CODE_FOR_roundv4df2_vec_pack_sfix = 12144,
  CODE_FOR_roundv2df2_vec_pack_sfix = 12145,
  CODE_FOR_avx512pf_gatherpfv16sisf = 12146,
  CODE_FOR_avx512pf_gatherpfv8disf = 12147,
  CODE_FOR_avx512pf_gatherpfv8sidf = 12148,
  CODE_FOR_avx512pf_gatherpfv8didf = 12149,
  CODE_FOR_avx512pf_scatterpfv16sisf = 12150,
  CODE_FOR_avx512pf_scatterpfv8disf = 12151,
  CODE_FOR_avx512pf_scatterpfv8sidf = 12152,
  CODE_FOR_avx512pf_scatterpfv8didf = 12153,
  CODE_FOR_exp2v16sf2 = 12154,
  CODE_FOR_exp2v8df2 = 12155,
  CODE_FOR_rotlv16qi3 = 12156,
  CODE_FOR_rotlv8hi3 = 12157,
  CODE_FOR_rotlv4si3 = 12158,
  CODE_FOR_rotlv2di3 = 12159,
  CODE_FOR_rotrv16qi3 = 12160,
  CODE_FOR_rotrv8hi3 = 12161,
  CODE_FOR_rotrv4si3 = 12162,
  CODE_FOR_rotrv2di3 = 12163,
  CODE_FOR_vrotrv16qi3 = 12164,
  CODE_FOR_vrotrv8hi3 = 12165,
  CODE_FOR_vrotrv4si3 = 12166,
  CODE_FOR_vrotrv2di3 = 12167,
  CODE_FOR_vrotlv16qi3 = 12168,
  CODE_FOR_vrotlv8hi3 = 12169,
  CODE_FOR_vrotlv4si3 = 12170,
  CODE_FOR_vrotlv2di3 = 12171,
  CODE_FOR_vlshrv16qi3 = 12172,
  CODE_FOR_vlshrv8hi3 = 12173,
  CODE_FOR_vlshrv4si3 = 12174,
  CODE_FOR_vlshrv2di3 = 12175,
  CODE_FOR_vashlv64qi3 = 12176,
  CODE_FOR_vlshrv64qi3 = 12177,
  CODE_FOR_vashrv64qi3 = 12178,
  CODE_FOR_vashlv32qi3 = 12179,
  CODE_FOR_vlshrv32qi3 = 12180,
  CODE_FOR_vashrv32qi3 = 12181,
  CODE_FOR_vashlv32hi3 = 12182,
  CODE_FOR_vlshrv32hi3 = 12183,
  CODE_FOR_vashrv32hi3 = 12184,
  CODE_FOR_vashlv16hi3 = 12185,
  CODE_FOR_vlshrv16hi3 = 12186,
  CODE_FOR_vashrv16hi3 = 12187,
  CODE_FOR_vashlv8qi3 = 12188,
  CODE_FOR_vlshrv8qi3 = 12189,
  CODE_FOR_vashrv8qi3 = 12190,
  CODE_FOR_vlshrv16si3 = 12191,
  CODE_FOR_vlshrv8di3 = 12192,
  CODE_FOR_vlshrv8si3 = 12193,
  CODE_FOR_vlshrv4di3 = 12194,
  CODE_FOR_vashrv8di3 = 12195,
  CODE_FOR_vashrv4di3 = 12196,
  CODE_FOR_vashrv16qi3 = 12197,
  CODE_FOR_vashrv8hi3 = 12198,
  CODE_FOR_vashrv2di3 = 12199,
  CODE_FOR_vashrv4si3 = 12200,
  CODE_FOR_vashrv16si3 = 12201,
  CODE_FOR_vashrv8si3 = 12202,
  CODE_FOR_vashlv16qi3 = 12203,
  CODE_FOR_vashlv8hi3 = 12204,
  CODE_FOR_vashlv4si3 = 12205,
  CODE_FOR_vashlv2di3 = 12206,
  CODE_FOR_vashlv16si3 = 12207,
  CODE_FOR_vashlv8di3 = 12208,
  CODE_FOR_vashlv8si3 = 12209,
  CODE_FOR_vashlv4di3 = 12210,
  CODE_FOR_ashlv64qi3 = 12211,
  CODE_FOR_lshrv64qi3 = 12212,
  CODE_FOR_ashrv64qi3 = 12213,
  CODE_FOR_ashlv32qi3 = 12214,
  CODE_FOR_lshrv32qi3 = 12215,
  CODE_FOR_ashrv32qi3 = 12216,
  CODE_FOR_ashlv16qi3 = 12217,
  CODE_FOR_lshrv16qi3 = 12218,
  CODE_FOR_ashrv16qi3 = 12219,
  CODE_FOR_ashrv2di3 = 12220,
  CODE_FOR_xop_vmfrczv4sf2 = 12221,
  CODE_FOR_xop_vmfrczv2df2 = 12222,
  CODE_FOR_avx_vzeroall = 12223,
  CODE_FOR_avx_vzeroupper = 12224,
  CODE_FOR_avx512f_vpermilv8df = 12225,
  CODE_FOR_avx512f_vpermilv8df_mask = 12226,
  CODE_FOR_avx_vpermilv4df = 12227,
  CODE_FOR_avx_vpermilv4df_mask = 12228,
  CODE_FOR_avx_vpermilv2df = 12229,
  CODE_FOR_avx_vpermilv2df_mask = 12230,
  CODE_FOR_avx512f_vpermilv16sf = 12231,
  CODE_FOR_avx512f_vpermilv16sf_mask = 12232,
  CODE_FOR_avx_vpermilv8sf = 12233,
  CODE_FOR_avx_vpermilv8sf_mask = 12234,
  CODE_FOR_avx_vpermilv4sf = 12235,
  CODE_FOR_avx_vpermilv4sf_mask = 12236,
  CODE_FOR_avx2_permv4di = 12237,
  CODE_FOR_avx2_permv4df = 12238,
  CODE_FOR_avx512vl_permv4di_mask = 12239,
  CODE_FOR_avx512vl_permv4df_mask = 12240,
  CODE_FOR_avx512f_permv8df = 12241,
  CODE_FOR_avx512f_permv8di = 12242,
  CODE_FOR_avx512f_permv8df_mask = 12243,
  CODE_FOR_avx512f_permv8di_mask = 12244,
  CODE_FOR_avx512f_vpermi2varv16si3_mask = 12245,
  CODE_FOR_avx512f_vpermi2varv16sf3_mask = 12246,
  CODE_FOR_avx512f_vpermi2varv8di3_mask = 12247,
  CODE_FOR_avx512f_vpermi2varv8df3_mask = 12248,
  CODE_FOR_avx512vl_vpermi2varv8si3_mask = 12249,
  CODE_FOR_avx512vl_vpermi2varv8sf3_mask = 12250,
  CODE_FOR_avx512vl_vpermi2varv4di3_mask = 12251,
  CODE_FOR_avx512vl_vpermi2varv4df3_mask = 12252,
  CODE_FOR_avx512vl_vpermi2varv4si3_mask = 12253,
  CODE_FOR_avx512vl_vpermi2varv4sf3_mask = 12254,
  CODE_FOR_avx512vl_vpermi2varv2di3_mask = 12255,
  CODE_FOR_avx512vl_vpermi2varv2df3_mask = 12256,
  CODE_FOR_avx512bw_vpermi2varv32hi3_mask = 12257,
  CODE_FOR_avx512vl_vpermi2varv16hi3_mask = 12258,
  CODE_FOR_avx512vl_vpermi2varv8hi3_mask = 12259,
  CODE_FOR_avx512bw_vpermi2varv64qi3_mask = 12260,
  CODE_FOR_avx512vl_vpermi2varv32qi3_mask = 12261,
  CODE_FOR_avx512vl_vpermi2varv16qi3_mask = 12262,
  CODE_FOR_avx512f_vpermt2varv16si3_maskz = 12263,
  CODE_FOR_avx512f_vpermt2varv16sf3_maskz = 12264,
  CODE_FOR_avx512f_vpermt2varv8di3_maskz = 12265,
  CODE_FOR_avx512f_vpermt2varv8df3_maskz = 12266,
  CODE_FOR_avx512vl_vpermt2varv8si3_maskz = 12267,
  CODE_FOR_avx512vl_vpermt2varv8sf3_maskz = 12268,
  CODE_FOR_avx512vl_vpermt2varv4di3_maskz = 12269,
  CODE_FOR_avx512vl_vpermt2varv4df3_maskz = 12270,
  CODE_FOR_avx512vl_vpermt2varv4si3_maskz = 12271,
  CODE_FOR_avx512vl_vpermt2varv4sf3_maskz = 12272,
  CODE_FOR_avx512vl_vpermt2varv2di3_maskz = 12273,
  CODE_FOR_avx512vl_vpermt2varv2df3_maskz = 12274,
  CODE_FOR_avx512bw_vpermt2varv32hi3_maskz = 12275,
  CODE_FOR_avx512vl_vpermt2varv16hi3_maskz = 12276,
  CODE_FOR_avx512vl_vpermt2varv8hi3_maskz = 12277,
  CODE_FOR_avx512bw_vpermt2varv64qi3_maskz = 12278,
  CODE_FOR_avx512vl_vpermt2varv32qi3_maskz = 12279,
  CODE_FOR_avx512vl_vpermt2varv16qi3_maskz = 12280,
  CODE_FOR_avx_vperm2f128v8si3 = 12281,
  CODE_FOR_avx_vperm2f128v8sf3 = 12282,
  CODE_FOR_avx_vperm2f128v4df3 = 12283,
  CODE_FOR_avx512vl_vinsertv8si = 12284,
  CODE_FOR_avx512vl_vinsertv8sf = 12285,
  CODE_FOR_avx512vl_vinsertv4di = 12286,
  CODE_FOR_avx512vl_vinsertv4df = 12287,
  CODE_FOR_avx_vinsertf128v32qi = 12288,
  CODE_FOR_avx_vinsertf128v16hi = 12289,
  CODE_FOR_avx_vinsertf128v8si = 12290,
  CODE_FOR_avx_vinsertf128v4di = 12291,
  CODE_FOR_avx_vinsertf128v8sf = 12292,
  CODE_FOR_avx_vinsertf128v4df = 12293,
  CODE_FOR_maskloadv4sfv4si = 12294,
  CODE_FOR_maskloadv2dfv2di = 12295,
  CODE_FOR_maskloadv8sfv8si = 12296,
  CODE_FOR_maskloadv4dfv4di = 12297,
  CODE_FOR_maskloadv4siv4si = 12298,
  CODE_FOR_maskloadv2div2di = 12299,
  CODE_FOR_maskloadv8siv8si = 12300,
  CODE_FOR_maskloadv4div4di = 12301,
  CODE_FOR_maskloadv16sihi = 12302,
  CODE_FOR_maskloadv8siqi = 12303,
  CODE_FOR_maskloadv4siqi = 12304,
  CODE_FOR_maskloadv8diqi = 12305,
  CODE_FOR_maskloadv4diqi = 12306,
  CODE_FOR_maskloadv2diqi = 12307,
  CODE_FOR_maskloadv16sfhi = 12308,
  CODE_FOR_maskloadv8sfqi = 12309,
  CODE_FOR_maskloadv4sfqi = 12310,
  CODE_FOR_maskloadv8dfqi = 12311,
  CODE_FOR_maskloadv4dfqi = 12312,
  CODE_FOR_maskloadv2dfqi = 12313,
  CODE_FOR_maskloadv64qidi = 12314,
  CODE_FOR_maskloadv16qihi = 12315,
  CODE_FOR_maskloadv32qisi = 12316,
  CODE_FOR_maskloadv32hisi = 12317,
  CODE_FOR_maskloadv16hihi = 12318,
  CODE_FOR_maskloadv8hiqi = 12319,
  CODE_FOR_maskloadv32hfsi = 12320,
  CODE_FOR_maskloadv16hfhi = 12321,
  CODE_FOR_maskloadv8hfqi = 12322,
  CODE_FOR_maskloadv32bfsi = 12323,
  CODE_FOR_maskloadv16bfhi = 12324,
  CODE_FOR_maskloadv8bfqi = 12325,
  CODE_FOR_maskstorev4sfv4si = 12326,
  CODE_FOR_maskstorev2dfv2di = 12327,
  CODE_FOR_maskstorev8sfv8si = 12328,
  CODE_FOR_maskstorev4dfv4di = 12329,
  CODE_FOR_maskstorev4siv4si = 12330,
  CODE_FOR_maskstorev2div2di = 12331,
  CODE_FOR_maskstorev8siv8si = 12332,
  CODE_FOR_maskstorev4div4di = 12333,
  CODE_FOR_maskstorev16sihi = 12334,
  CODE_FOR_maskstorev8siqi = 12335,
  CODE_FOR_maskstorev4siqi = 12336,
  CODE_FOR_maskstorev8diqi = 12337,
  CODE_FOR_maskstorev4diqi = 12338,
  CODE_FOR_maskstorev2diqi = 12339,
  CODE_FOR_maskstorev16sfhi = 12340,
  CODE_FOR_maskstorev8sfqi = 12341,
  CODE_FOR_maskstorev4sfqi = 12342,
  CODE_FOR_maskstorev8dfqi = 12343,
  CODE_FOR_maskstorev4dfqi = 12344,
  CODE_FOR_maskstorev2dfqi = 12345,
  CODE_FOR_maskstorev64qidi = 12346,
  CODE_FOR_maskstorev16qihi = 12347,
  CODE_FOR_maskstorev32qisi = 12348,
  CODE_FOR_maskstorev32hisi = 12349,
  CODE_FOR_maskstorev16hihi = 12350,
  CODE_FOR_maskstorev8hiqi = 12351,
  CODE_FOR_maskstorev32hfsi = 12352,
  CODE_FOR_maskstorev16hfhi = 12353,
  CODE_FOR_maskstorev8hfqi = 12354,
  CODE_FOR_maskstorev32bfsi = 12355,
  CODE_FOR_maskstorev16bfhi = 12356,
  CODE_FOR_maskstorev8bfqi = 12357,
  CODE_FOR_cbranchv4si4 = 12358,
  CODE_FOR_cbranchv2di4 = 12359,
  CODE_FOR_cbranchv8si4 = 12360,
  CODE_FOR_cbranchv4di4 = 12361,
  CODE_FOR_vec_initv64qiqi = 12362,
  CODE_FOR_vec_initv32qiqi = 12363,
  CODE_FOR_vec_initv16qiqi = 12364,
  CODE_FOR_vec_initv32hihi = 12365,
  CODE_FOR_vec_initv16hihi = 12366,
  CODE_FOR_vec_initv8hihi = 12367,
  CODE_FOR_vec_initv16sisi = 12368,
  CODE_FOR_vec_initv8sisi = 12369,
  CODE_FOR_vec_initv4sisi = 12370,
  CODE_FOR_vec_initv8didi = 12371,
  CODE_FOR_vec_initv4didi = 12372,
  CODE_FOR_vec_initv2didi = 12373,
  CODE_FOR_vec_initv32hfhf = 12374,
  CODE_FOR_vec_initv16hfhf = 12375,
  CODE_FOR_vec_initv8hfhf = 12376,
  CODE_FOR_vec_initv32bfbf = 12377,
  CODE_FOR_vec_initv16bfbf = 12378,
  CODE_FOR_vec_initv8bfbf = 12379,
  CODE_FOR_vec_initv16sfsf = 12380,
  CODE_FOR_vec_initv8sfsf = 12381,
  CODE_FOR_vec_initv4sfsf = 12382,
  CODE_FOR_vec_initv8dfdf = 12383,
  CODE_FOR_vec_initv4dfdf = 12384,
  CODE_FOR_vec_initv2dfdf = 12385,
  CODE_FOR_vec_initv4titi = 12386,
  CODE_FOR_vec_initv2titi = 12387,
  CODE_FOR_vec_initv64qiv32qi = 12388,
  CODE_FOR_vec_initv32qiv16qi = 12389,
  CODE_FOR_vec_initv16qiv8qi = 12390,
  CODE_FOR_vec_initv32hiv16hi = 12391,
  CODE_FOR_vec_initv16hiv8hi = 12392,
  CODE_FOR_vec_initv8hiv4hi = 12393,
  CODE_FOR_vec_initv16siv8si = 12394,
  CODE_FOR_vec_initv8siv4si = 12395,
  CODE_FOR_vec_initv4siv2si = 12396,
  CODE_FOR_vec_initv8div4di = 12397,
  CODE_FOR_vec_initv4div2di = 12398,
  CODE_FOR_vec_initv32hfv16hf = 12399,
  CODE_FOR_vec_initv16hfv8hf = 12400,
  CODE_FOR_vec_initv8hfv4hf = 12401,
  CODE_FOR_vec_initv32bfv16bf = 12402,
  CODE_FOR_vec_initv16bfv8bf = 12403,
  CODE_FOR_vec_initv8bfv4bf = 12404,
  CODE_FOR_vec_initv16sfv8sf = 12405,
  CODE_FOR_vec_initv8sfv4sf = 12406,
  CODE_FOR_vec_initv4sfv2sf = 12407,
  CODE_FOR_vec_initv8dfv4df = 12408,
  CODE_FOR_vec_initv4dfv2df = 12409,
  CODE_FOR_vec_initv4tiv2ti = 12410,
  CODE_FOR_cond_ashlv32hi = 12411,
  CODE_FOR_cond_lshrv32hi = 12412,
  CODE_FOR_cond_ashrv32hi = 12413,
  CODE_FOR_cond_ashlv16hi = 12414,
  CODE_FOR_cond_lshrv16hi = 12415,
  CODE_FOR_cond_ashrv16hi = 12416,
  CODE_FOR_cond_ashlv8hi = 12417,
  CODE_FOR_cond_lshrv8hi = 12418,
  CODE_FOR_cond_ashrv8hi = 12419,
  CODE_FOR_cond_ashlv16si = 12420,
  CODE_FOR_cond_lshrv16si = 12421,
  CODE_FOR_cond_ashrv16si = 12422,
  CODE_FOR_cond_ashlv8si = 12423,
  CODE_FOR_cond_lshrv8si = 12424,
  CODE_FOR_cond_ashrv8si = 12425,
  CODE_FOR_cond_ashlv4si = 12426,
  CODE_FOR_cond_lshrv4si = 12427,
  CODE_FOR_cond_ashrv4si = 12428,
  CODE_FOR_cond_ashlv8di = 12429,
  CODE_FOR_cond_lshrv8di = 12430,
  CODE_FOR_cond_ashrv8di = 12431,
  CODE_FOR_cond_ashlv4di = 12432,
  CODE_FOR_cond_lshrv4di = 12433,
  CODE_FOR_cond_ashrv4di = 12434,
  CODE_FOR_cond_ashlv2di = 12435,
  CODE_FOR_cond_lshrv2di = 12436,
  CODE_FOR_cond_ashrv2di = 12437,
  CODE_FOR_vcvtps2ph_mask = 12438,
  CODE_FOR_vcvtps2ph = 12439,
  CODE_FOR_avx512f_vcvtps2ph512_mask_sae = 12440,
  CODE_FOR_avx2_gathersiv2di = 12441,
  CODE_FOR_avx2_gathersiv2df = 12442,
  CODE_FOR_avx2_gathersiv4di = 12443,
  CODE_FOR_avx2_gathersiv4df = 12444,
  CODE_FOR_avx2_gathersiv4si = 12445,
  CODE_FOR_avx2_gathersiv4sf = 12446,
  CODE_FOR_avx2_gathersiv8si = 12447,
  CODE_FOR_avx2_gathersiv8sf = 12448,
  CODE_FOR_avx2_gatherdiv2di = 12449,
  CODE_FOR_avx2_gatherdiv2df = 12450,
  CODE_FOR_avx2_gatherdiv4di = 12451,
  CODE_FOR_avx2_gatherdiv4df = 12452,
  CODE_FOR_avx2_gatherdiv4si = 12453,
  CODE_FOR_avx2_gatherdiv4sf = 12454,
  CODE_FOR_avx2_gatherdiv8si = 12455,
  CODE_FOR_avx2_gatherdiv8sf = 12456,
  CODE_FOR_avx512f_gathersiv16si = 12457,
  CODE_FOR_avx512f_gathersiv16sf = 12458,
  CODE_FOR_avx512f_gathersiv8di = 12459,
  CODE_FOR_avx512f_gathersiv8df = 12460,
  CODE_FOR_avx512vl_gathersiv8si = 12461,
  CODE_FOR_avx512vl_gathersiv8sf = 12462,
  CODE_FOR_avx512vl_gathersiv4di = 12463,
  CODE_FOR_avx512vl_gathersiv4df = 12464,
  CODE_FOR_avx512vl_gathersiv4si = 12465,
  CODE_FOR_avx512vl_gathersiv4sf = 12466,
  CODE_FOR_avx512vl_gathersiv2di = 12467,
  CODE_FOR_avx512vl_gathersiv2df = 12468,
  CODE_FOR_avx512f_gatherdiv16si = 12469,
  CODE_FOR_avx512f_gatherdiv16sf = 12470,
  CODE_FOR_avx512f_gatherdiv8di = 12471,
  CODE_FOR_avx512f_gatherdiv8df = 12472,
  CODE_FOR_avx512vl_gatherdiv8si = 12473,
  CODE_FOR_avx512vl_gatherdiv8sf = 12474,
  CODE_FOR_avx512vl_gatherdiv4di = 12475,
  CODE_FOR_avx512vl_gatherdiv4df = 12476,
  CODE_FOR_avx512vl_gatherdiv4si = 12477,
  CODE_FOR_avx512vl_gatherdiv4sf = 12478,
  CODE_FOR_avx512vl_gatherdiv2di = 12479,
  CODE_FOR_avx512vl_gatherdiv2df = 12480,
  CODE_FOR_avx512f_scattersiv16si = 12481,
  CODE_FOR_avx512f_scattersiv16sf = 12482,
  CODE_FOR_avx512f_scattersiv8di = 12483,
  CODE_FOR_avx512f_scattersiv8df = 12484,
  CODE_FOR_avx512vl_scattersiv8si = 12485,
  CODE_FOR_avx512vl_scattersiv8sf = 12486,
  CODE_FOR_avx512vl_scattersiv4di = 12487,
  CODE_FOR_avx512vl_scattersiv4df = 12488,
  CODE_FOR_avx512vl_scattersiv4si = 12489,
  CODE_FOR_avx512vl_scattersiv4sf = 12490,
  CODE_FOR_avx512vl_scattersiv2di = 12491,
  CODE_FOR_avx512vl_scattersiv2df = 12492,
  CODE_FOR_avx512f_scatterdiv16si = 12493,
  CODE_FOR_avx512f_scatterdiv16sf = 12494,
  CODE_FOR_avx512f_scatterdiv8di = 12495,
  CODE_FOR_avx512f_scatterdiv8df = 12496,
  CODE_FOR_avx512vl_scatterdiv8si = 12497,
  CODE_FOR_avx512vl_scatterdiv8sf = 12498,
  CODE_FOR_avx512vl_scatterdiv4di = 12499,
  CODE_FOR_avx512vl_scatterdiv4df = 12500,
  CODE_FOR_avx512vl_scatterdiv4si = 12501,
  CODE_FOR_avx512vl_scatterdiv4sf = 12502,
  CODE_FOR_avx512vl_scatterdiv2di = 12503,
  CODE_FOR_avx512vl_scatterdiv2df = 12504,
  CODE_FOR_avx512f_expandv16si_maskz = 12505,
  CODE_FOR_avx512f_expandv16sf_maskz = 12506,
  CODE_FOR_avx512f_expandv8di_maskz = 12507,
  CODE_FOR_avx512f_expandv8df_maskz = 12508,
  CODE_FOR_avx512vl_expandv8si_maskz = 12509,
  CODE_FOR_avx512vl_expandv8sf_maskz = 12510,
  CODE_FOR_avx512vl_expandv4di_maskz = 12511,
  CODE_FOR_avx512vl_expandv4df_maskz = 12512,
  CODE_FOR_avx512vl_expandv4si_maskz = 12513,
  CODE_FOR_avx512vl_expandv4sf_maskz = 12514,
  CODE_FOR_avx512vl_expandv2di_maskz = 12515,
  CODE_FOR_avx512vl_expandv2df_maskz = 12516,
  CODE_FOR_expandv64qi_maskz = 12517,
  CODE_FOR_expandv16qi_maskz = 12518,
  CODE_FOR_expandv32qi_maskz = 12519,
  CODE_FOR_expandv32hi_maskz = 12520,
  CODE_FOR_expandv16hi_maskz = 12521,
  CODE_FOR_expandv8hi_maskz = 12522,
  CODE_FOR_vpmadd52huqv8di_maskz = 12523,
  CODE_FOR_vpmadd52huqv4di_maskz = 12524,
  CODE_FOR_vpmadd52huqv2di_maskz = 12525,
  CODE_FOR_vpmadd52luqv8di_maskz = 12526,
  CODE_FOR_vpmadd52luqv4di_maskz = 12527,
  CODE_FOR_vpmadd52luqv2di_maskz = 12528,
  CODE_FOR_movv64sf = 12529,
  CODE_FOR_movv64si = 12530,
  CODE_FOR_popcountv16si2 = 12531,
  CODE_FOR_popcountv8si2 = 12532,
  CODE_FOR_popcountv4si2 = 12533,
  CODE_FOR_popcountv8di2 = 12534,
  CODE_FOR_popcountv4di2 = 12535,
  CODE_FOR_popcountv2di2 = 12536,
  CODE_FOR_popcountv64qi2 = 12537,
  CODE_FOR_popcountv16qi2 = 12538,
  CODE_FOR_popcountv32qi2 = 12539,
  CODE_FOR_popcountv32hi2 = 12540,
  CODE_FOR_popcountv16hi2 = 12541,
  CODE_FOR_popcountv8hi2 = 12542,
  CODE_FOR_vpshrdv_v32hi_maskz = 12543,
  CODE_FOR_vpshrdv_v16si_maskz = 12544,
  CODE_FOR_vpshrdv_v8di_maskz = 12545,
  CODE_FOR_vpshrdv_v16hi_maskz = 12546,
  CODE_FOR_vpshrdv_v8si_maskz = 12547,
  CODE_FOR_vpshrdv_v4di_maskz = 12548,
  CODE_FOR_vpshrdv_v8hi_maskz = 12549,
  CODE_FOR_vpshrdv_v4si_maskz = 12550,
  CODE_FOR_vpshrdv_v2di_maskz = 12551,
  CODE_FOR_vpshldv_v32hi_maskz = 12552,
  CODE_FOR_vpshldv_v16si_maskz = 12553,
  CODE_FOR_vpshldv_v8di_maskz = 12554,
  CODE_FOR_vpshldv_v16hi_maskz = 12555,
  CODE_FOR_vpshldv_v8si_maskz = 12556,
  CODE_FOR_vpshldv_v4di_maskz = 12557,
  CODE_FOR_vpshldv_v8hi_maskz = 12558,
  CODE_FOR_vpshldv_v4si_maskz = 12559,
  CODE_FOR_vpshldv_v2di_maskz = 12560,
  CODE_FOR_usdot_prodv64qi = 12561,
  CODE_FOR_usdot_prodv32qi = 12562,
  CODE_FOR_usdot_prodv16qi = 12563,
  CODE_FOR_vpdpbusd_v16si_maskz = 12564,
  CODE_FOR_vpdpbusd_v8si_maskz = 12565,
  CODE_FOR_vpdpbusd_v4si_maskz = 12566,
  CODE_FOR_vpdpbusds_v16si_maskz = 12567,
  CODE_FOR_vpdpbusds_v8si_maskz = 12568,
  CODE_FOR_vpdpbusds_v4si_maskz = 12569,
  CODE_FOR_vpdpwssd_v16si_maskz = 12570,
  CODE_FOR_vpdpwssd_v8si_maskz = 12571,
  CODE_FOR_vpdpwssd_v4si_maskz = 12572,
  CODE_FOR_vpdpwssds_v16si_maskz = 12573,
  CODE_FOR_vpdpwssds_v8si_maskz = 12574,
  CODE_FOR_vpdpwssds_v4si_maskz = 12575,
  CODE_FOR_movp2qi = 12576,
  CODE_FOR_movp2hi = 12577,
  CODE_FOR_avx512f_cvtne2ps2bf16_v32bf_maskz = 12578,
  CODE_FOR_avx512f_cvtne2ps2bf16_v16bf_maskz = 12579,
  CODE_FOR_avx512f_cvtne2ps2bf16_v8bf_maskz = 12580,
  CODE_FOR_vcvtneps2bf16_v4sf = 12581,
  CODE_FOR_avx512f_cvtneps2bf16_v4sf_maskz = 12582,
  CODE_FOR_avx512f_cvtneps2bf16_v4sf_mask = 12583,
  CODE_FOR_avx512f_cvtneps2bf16_v16sf_maskz = 12584,
  CODE_FOR_avx512f_cvtneps2bf16_v8sf_maskz = 12585,
  CODE_FOR_avx512f_dpbf16ps_v16sf_maskz = 12586,
  CODE_FOR_avx512f_dpbf16ps_v8sf_maskz = 12587,
  CODE_FOR_avx512f_dpbf16ps_v4sf_maskz = 12588,
  CODE_FOR_encodekey128u32 = 12589,
  CODE_FOR_encodekey256u32 = 12590,
  CODE_FOR_aesdecwide128klu8 = 12591,
  CODE_FOR_aesdecwide256klu8 = 12592,
  CODE_FOR_aesencwide128klu8 = 12593,
  CODE_FOR_aesencwide256klu8 = 12594,
  CODE_FOR_vec_duplicatev64qi = 12595,
  CODE_FOR_vec_duplicatev32qi = 12596,
  CODE_FOR_vec_duplicatev16qi = 12597,
  CODE_FOR_vec_duplicatev32hi = 12598,
  CODE_FOR_vec_duplicatev16hi = 12599,
  CODE_FOR_vec_duplicatev8hi = 12600,
  CODE_FOR_vec_duplicatev16si = 12601,
  CODE_FOR_vec_duplicatev8si = 12602,
  CODE_FOR_vec_duplicatev4si = 12603,
  CODE_FOR_vec_duplicatev8di = 12604,
  CODE_FOR_vec_duplicatev4di = 12605,
  CODE_FOR_vec_duplicatev2di = 12606,
  CODE_FOR_sdot_prodv32qi = 12607,
  CODE_FOR_sdot_prodv16qi = 12608,
  CODE_FOR_udot_prodv32qi = 12609,
  CODE_FOR_udot_prodv16qi = 12610,
  CODE_FOR_sse2_lfence = 12611,
  CODE_FOR_sse_sfence = 12612,
  CODE_FOR_sse2_mfence = 12613,
  CODE_FOR_mem_thread_fence = 12614,
  CODE_FOR_atomic_loadqi = 12615,
  CODE_FOR_atomic_loadhi = 12616,
  CODE_FOR_atomic_loadsi = 12617,
  CODE_FOR_atomic_loaddi = 12618,
  CODE_FOR_atomic_storeqi = 12619,
  CODE_FOR_atomic_storehi = 12620,
  CODE_FOR_atomic_storesi = 12621,
  CODE_FOR_atomic_storedi = 12622,
  CODE_FOR_atomic_compare_and_swapqi = 12623,
  CODE_FOR_atomic_compare_and_swaphi = 12624,
  CODE_FOR_atomic_compare_and_swapsi = 12625,
  CODE_FOR_atomic_compare_and_swapdi = 12626,
  CODE_FOR_atomic_compare_and_swapti = 12627,
  CODE_FOR_atomic_fetch_andqi = 12628,
  CODE_FOR_atomic_fetch_orqi = 12629,
  CODE_FOR_atomic_fetch_xorqi = 12630,
  CODE_FOR_atomic_fetch_andhi = 12631,
  CODE_FOR_atomic_fetch_orhi = 12632,
  CODE_FOR_atomic_fetch_xorhi = 12633,
  CODE_FOR_atomic_fetch_andsi = 12634,
  CODE_FOR_atomic_fetch_orsi = 12635,
  CODE_FOR_atomic_fetch_xorsi = 12636,
  CODE_FOR_atomic_and_fetchqi = 12637,
  CODE_FOR_atomic_or_fetchqi = 12638,
  CODE_FOR_atomic_xor_fetchqi = 12639,
  CODE_FOR_atomic_and_fetchhi = 12640,
  CODE_FOR_atomic_or_fetchhi = 12641,
  CODE_FOR_atomic_xor_fetchhi = 12642,
  CODE_FOR_atomic_and_fetchsi = 12643,
  CODE_FOR_atomic_or_fetchsi = 12644,
  CODE_FOR_atomic_xor_fetchsi = 12645,
  CODE_FOR_atomic_fetch_nandqi = 12646,
  CODE_FOR_atomic_fetch_nandhi = 12647,
  CODE_FOR_atomic_fetch_nandsi = 12648,
  CODE_FOR_atomic_nand_fetchqi = 12649,
  CODE_FOR_atomic_nand_fetchhi = 12650,
  CODE_FOR_atomic_nand_fetchsi = 12651,
  CODE_FOR_atomic_fetch_anddi = 12652,
  CODE_FOR_atomic_fetch_ordi = 12653,
  CODE_FOR_atomic_fetch_xordi = 12654,
  CODE_FOR_atomic_fetch_andti = 12655,
  CODE_FOR_atomic_fetch_orti = 12656,
  CODE_FOR_atomic_fetch_xorti = 12657,
  CODE_FOR_atomic_and_fetchdi = 12658,
  CODE_FOR_atomic_or_fetchdi = 12659,
  CODE_FOR_atomic_xor_fetchdi = 12660,
  CODE_FOR_atomic_and_fetchti = 12661,
  CODE_FOR_atomic_or_fetchti = 12662,
  CODE_FOR_atomic_xor_fetchti = 12663,
  CODE_FOR_atomic_fetch_nanddi = 12664,
  CODE_FOR_atomic_fetch_nandti = 12665,
  CODE_FOR_atomic_nand_fetchdi = 12666,
  CODE_FOR_atomic_nand_fetchti = 12667,
  CODE_FOR_atomic_bit_test_and_sethi = 12668,
  CODE_FOR_atomic_bit_test_and_setsi = 12669,
  CODE_FOR_atomic_bit_test_and_setdi = 12670,
  CODE_FOR_atomic_bit_test_and_complementhi = 12671,
  CODE_FOR_atomic_bit_test_and_complementsi = 12672,
  CODE_FOR_atomic_bit_test_and_complementdi = 12673,
  CODE_FOR_atomic_bit_test_and_resethi = 12674,
  CODE_FOR_atomic_bit_test_and_resetsi = 12675,
  CODE_FOR_atomic_bit_test_and_resetdi = 12676,
  CODE_FOR_atomic_add_fetch_cmp_0qi = 12677,
  CODE_FOR_atomic_sub_fetch_cmp_0qi = 12678,
  CODE_FOR_atomic_add_fetch_cmp_0hi = 12679,
  CODE_FOR_atomic_sub_fetch_cmp_0hi = 12680,
  CODE_FOR_atomic_add_fetch_cmp_0si = 12681,
  CODE_FOR_atomic_sub_fetch_cmp_0si = 12682,
  CODE_FOR_atomic_add_fetch_cmp_0di = 12683,
  CODE_FOR_atomic_sub_fetch_cmp_0di = 12684,
  CODE_FOR_atomic_and_fetch_cmp_0qi = 12685,
  CODE_FOR_atomic_or_fetch_cmp_0qi = 12686,
  CODE_FOR_atomic_xor_fetch_cmp_0qi = 12687,
  CODE_FOR_atomic_and_fetch_cmp_0hi = 12688,
  CODE_FOR_atomic_or_fetch_cmp_0hi = 12689,
  CODE_FOR_atomic_xor_fetch_cmp_0hi = 12690,
  CODE_FOR_atomic_and_fetch_cmp_0si = 12691,
  CODE_FOR_atomic_or_fetch_cmp_0si = 12692,
  CODE_FOR_atomic_xor_fetch_cmp_0si = 12693,
  CODE_FOR_atomic_and_fetch_cmp_0di = 12694,
  CODE_FOR_atomic_or_fetch_cmp_0di = 12695,
  CODE_FOR_atomic_xor_fetch_cmp_0di = 12696
};

const unsigned int NUM_INSN_CODES = 12697;
#endif /* GCC_INSN_CODES_H */
